! =================================================================================================================================
! MODULE        : stomate_wind
!
! CONTACT	: Ferenc.Pasztor@lsce.ipsl.fr
!
! LICENCE      	: IPSL (2006). This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF        : Calculates critical wind speed for tree damage for each half-hourly period of the simulation.
!                 This wind speed can be compared to the actual wind speed to decide whether a tree damage occurs.
!!
!!\streamlining_n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! SVN           :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_windfall.f90 $ 
!! $Date: 2015               -01-22 13:41:13 +0100 (mer., 22 janv. 2014) $
!! $Revision:                   1831 $
!! \streamlining_n
!_ ================================================================================================================================

MODULE stomate_windfall

  ! modules used:
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE function_library

  IMPLICIT NONE

  ! private & public routines

  PRIVATE 
  PUBLIC wind_damage
!!$  wind_clear, streamlining, calculate_force, calculate_bending_moment, &
!!$       calculate_cws, calculate_wind_process, elevate

  LOGICAL, SAVE                   :: firstcall = .TRUE.        !! first call 

CONTAINS


!! ================================================================================================================================
!!  SUBROUTINE   : wind_clear 
!!
!>\BRIEF        Set the firstcall flag to .TRUE. and activate initialization
!! 
!_ ================================================================================================================================

  SUBROUTINE wind_clear
    firstcall = .TRUE.
  END SUBROUTINE wind_clear


!! ================================================================================================================================
!! SUBROUTINE     : wind_damage
!!
!>\BRIEF          This subroutine calculates the critical wind speeds for both uprooting and stem breakage
!!                for each half-hourly period of the simulation. The algorithm follows Barry Gardiner's wind 
!!                damage risk model called GALES (Hale et al. 2015).
!! 
!! DESCRIPTION	: The purpose of this module is to describe the actual sensitivity of a given PFT to the wind that is present
!! at the same pixel. For each circumference class in each PFT, two critical wind speed values (CWS) are calculated in every
!! half-hourly time step by this subroutine, following the approach of differentiating two types of wind damage of trees:
!! - uprooting (the whole tree is removed from the soil together with its root plate)
!! - stem breakage (the root plate and the bottom part of the stem remains attached to the ground).
!! Whichever threshold (i.e. CWS) is reached first for a tree, the according damage type occurs in the model.
!!
!! The module completes the following tasks:
!! 1. Calculates...
!! 
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): :: Critical wind speed for stem breakage 10 m above the zero-plane displacement (m/s): U_Break_10
!!                             Critical wind speed for tree overturn 10 m above the zero-plane displacement (m/s): U_Overturn_10
!!
!! REFERENCES    :
!! - Hale, S., Gardiner, B., Nicoll, B., Taylor, P., and Pizzirani, S. (2015). Comparison and validation of three versions
!! of a forest wind risk model. Environmental Modelling and Software. In Press.
!! ...
!!
!! FLOWCHART     : None
!! \streamlining_n 
!_ ================================================================================================================================
  
  SUBROUTINE wind_damage (npts, njsc, circ_class_biomass, snow, &
       veget_max, circ_class_n)
    !root_dens, senescence, &
    !   circ_class_biomass, circ_class_n, snowdepth, sph, gap_size, snow_depth) 
    ! OUTPUT VARIABLES SHOULD BE LISTED HERE, AS WELL.

  IMPLICIT NONE



 !! 0. Variable declarations

 !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                       :: npts                         !! Domain size @tex $(unitless)$ @endtex
    INTEGER(i_std), DIMENSION(:), INTENT(in)         :: njsc                         !! Index of the dominant soil textural class in
                                                                                     !! the grid cell @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(in)    :: circ_class_biomass           !! Biomass in of an individual tree in each circ class
                                                                                     !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)        :: circ_class_n                 !! Number of trees within each circumference
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: veget_max                    !! "maximal" coverage fraction of a PFT on the ground
                                                                                     !! May sum to less than unity if the pixel has nobio 
                                                                                     !! area. (unitless, 0-1)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: snow                         !! Snow mass @tex $ (kg m^{-2})$ @endtex
   
  

 !! 0.2 Output variables
    
    ! OUTPUT VARIABLES TO BE DECLARED HERE


 !! 0.3 Modified variables (INTENT(inout))


 !! 0.4 Local variables
    
    INTEGER(i_std)                                   :: ibdl,ipts,ivm,icir,i         !! Index for the loops @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc,60)                 :: section_height               !! The height of the bottom of the 1-m sections 
                                                                                     !! of the mean tree @tex $(m)$ @endtex. 60 is the
                                                                                     !! max number of sections thus max tree height is 60m
    REAL(r_std), DIMENSION(ncirc,60)                 :: section_diam                 !! The diameter of the 1-m sections of the mean
                                                                                     !! tree @tex $(m)$ @endtex. max 60 sections. 
    REAL(r_std), DIMENSION(ncirc,60)                 :: branch_width                 !! Width of canopy) of the 1-m sections of the mean 
                                                                                     !! tree @tex $(m)$ @endtex 60 is the max number of 
                                                                                     !! sections thus max tree height is 60m
    REAL(r_std), DIMENSION(nbdl,nvm)                 :: root_dens                    !! Root density per soil layer (what proportion
                                                                                     !! of the root system is found at the given layer)
    REAL(r_std), DIMENSION(nbdl+1)                   :: z_soil	                     !! Variable to store depth of the different
                                                                                     !! soil layers (m)
    REAL(r_std), DIMENSION(nvm)                      :: rpc                          !! scaling factor for integral
    REAL(r_std), DIMENSION(npts)                     :: soil_type                    !! Soil type (Free-draining mineral soils;
                                                                                     !! Gleyed mineral soils; Peaty mineral soils; and
                                                                                     !! deep peat soils. See constantes_soil_var.f90 
                                                                                     !! for the definitions
    REAL(r_std), DIMENSION(ncirc)                    :: mean_dbh                     !! Diameter at breast height (dbh) of the mean tree
                                                                                     !! of the circumference-class @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: mean_height                  !! Height of the mean tree of the given dbh-class 
                                                                                     !! @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: lai                          !! Leaf area index @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: canopy_breadth               !! Canopy breadth (width of canopy) @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: canopy_depth                 !! Canopy depth (height of canopy) @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: canopy_height                !! The height of the starting point of the canopy from 
                                                                                     !! below @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: mid_canopy                   !! The middle height of the canopy @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: mean_dbh_height              !! The height of the diameter (by default it is at 
                                                                                     !! breast height = 1.3 m) @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: snow_mass                    !! Weight of the snow on the tree @tex $(kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: diameter_c2                  !! Variable for calculating diameters 
                                                                                     !! @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: diameter_canopy_base         !! Diameter at the base of the canopy 
                                                                                     !! @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: diameter_c1                  !! Variable for calculating diameters 
                                                                                     !! @tex $(unitless)$ @endtex
!!$    REAL(r_std), DIMENSION(ncirc)                    :: no_of_sections               !! The number of one-meter sections that the mean tree 
!!$                                                                                     !! has @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: crown_volume                 !! Crown volume of the mean tree @tex $(m^{3})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: crown_mass                   !! Mass of the mean tree crown @tex $(kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: stem_mass                    !! Mass of the mean tree stem @tex $(kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: current_spacing              !! Spacing between trees in the given dbh-class 
                                                                                     !! @tex $(m)$ @endtex
    REAL(r_std)                                      :: rooting_depth                !! Rooting depth of the mean tree (three categories;
                                                                                     !! 1: shallow, 2: deep, 3:average 
                                                                                     !! See constantes_soil_var.f90 for the definitions
!!$    REAL(r_std)                                      :: canopy_density               !! Density of the tree canopy, i.e. of the
!!$                                                                                     !! branches and leaves @tex $(kg/m^{3})$ @endtex
    REAL(r_std)                                      :: streamlining_c               !! Streamlining parameter. @tex $(unitless)$ @endtex. 
                                                                                     !! Streamlining is the change of shape of the crowns 
                                                                                     !! due to wind.
    REAL(r_std)                                      :: streamlining_n               !! Streamlining parameter. @tex $(unitless)$ @endtex. 
                                                                                     !! Streamlining is the change of shape of the crowns 
                                                                                     !! due to wind.
    REAL(r_std)                                      :: streamlining_rb              !! Streamlining parameter. @tex $(unitless)$ @endtex. 
                                                                                     !! Streamlining is the change of shape of the crowns 
                                                                                     !! due to wind.    
    

   


    REAL(r_std), DIMENSION(ncirc)                :: area_timber_removals_5_years !! Area of the total timber removal in the
                                                                                          !! previous 5 year @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: area_icir                    !! The total area of the circumference class @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: tree_heights_from_edge       !! Width of the forest edge area, i.e. area_closer (in number of tree heights)
                                                                                          !! @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: n_gap                        !! The number of gaps with an area of clear_cut_max @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: length_side_gap              !! The length of one side of the square-shaped gap @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: area_around_gap              !! The framing edge area around the gap with different gustiness
                                                                                          !! @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: area_total_closer            !! The total area of forest edge areas affected by wind @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: area_total_further           !! The total area of forest without edge areas @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: ratio_spacing_height         !! Ratio of tree height and tree spacing @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: gap_size                     !! The length (width of the cross-section) of the gap @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: ratio_gap_height             !! Ratio of gap size and tree height @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_gap10                   !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: max_gap10                    !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_gap_factor              !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: max_gap_factor               !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_bm_closer               !! Mean bending moment for the forest edge area @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: max_bm_closer                !! Maximum bending moment for the forest edge area @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_bm_further              !! Mean bending moment for the forest area @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: max_bm_further               !! Maximum bending moment for the forest area @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_bm_edge                 !! Mean bending moment for the forest edge @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: max_bm_edge                  !! Maximum bending moment for the forest edge @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_bm_edge_gap             !! Mean bending moment for the forest edge - gap factor considered
                                                                                          !! @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: max_bm_edge_gap              !! Maximum bending moment for the forest edge - gap factor considered
                                                                                          !! @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: term_a                       !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: term_b                       !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: old_gust_closer              !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: old_gust_further             !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: old_gust_edge                !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: old_gust_edge_gap            !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: new_gust_closer              !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: new_gust_further             !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: new_gust_edge                !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: new_gust_edge_gap_closer     !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: new_gust_edge_gap_further    !! Variable used for calculating gustiness @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_bm_gust_factor_closer   !! Mean bending moment of the forest edge area (for calculating
                                                                                 !! the edge factor) @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: mean_bm_gust_factor_further  !! Mean bending moment of the inner forest area (for calculating
                                                                                 !! the edge factor) @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: edge_factor                  !! The ratio of the mean bending moment in the forest edge area 
                                                                                          !! and mean bending moment in the inner forest area
                                                                                          !! @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: gust_factor_closer           !! Gust factor for calculating the critical wind speed of the forest
                                                                                          !! edge area @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: gust_factor_further          !! Gust factor for calculating the critical wind speed of the inner
                                                                                          !! forest area @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: g_closer                     !! Gustiness of the forest edge area @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: g_further                    !! Gustiness of the inner forest area @tex $(unitless)$ @endtex
    REAL(r_std), DIMENSION(nvm)                      :: overturning_moment_multiplier!! Overturning moment multiplier representing the effects of species,
                                                                                          !! soil type, soil depths and senescence @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: max_overturning_moment       !! The maximum overturning moment the mean tree can withstand
                                                                                          !! @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: max_breaking_moment          !! The maximum breaking moment the mean tree can withstand
                                                                                          !! @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: overturning_moment_closer    !! The maximum overturning moment the mean tree can withstand in the
                                                                                          !! forest edge area with gustiness taken into account @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: overturning_moment_further   !! The maximum overturning moment the mean tree can withstand in the
                                                                                          !! inner forest area with gustiness taken into account @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: breaking_moment_closer       !! The maximum breaking moment the mean tree can withstand in the
                                                                                          !! forest edge area with gustiness taken into account @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: breaking_moment_further      !! The maximum breaking moment the mean tree can withstand in the
                                                                                          !! inner forest area with gustiness taken into account @tex $(Nm/kg)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: cws_break_closer             !! Critical wind speed for stem breakage in the forest edge area
                                                                                          !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: cws_break_closer_10          !! Critical wind speed for stem breakage in the forest edge area
                                                                                          !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: cws_overturn_closer          !! Critical wind speed for overturning in the forest edge area
                                                                                          !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: cws_overturn_closer_10       !! Critical wind speed for overturning in the forest edge area
                                                                                          !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: cws_final_closer             !! The critical wind speed of the final form of damage (breakage / uprooting)
                                                                                          !! in the forest edge area @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(npts)                          :: wind_speed_actual            !! The actual wind speed over the pixel
                                                                                          !! @tex $(m/s; mean hourly wind speed)$ @endtex
    LOGICAL                                      :: wind_damage_type_uprooting_closer      !! The final form of damage (breakage / uprooting) in the forest edge area
    REAL(r_std), DIMENSION(ncirc)                :: wind_damage_rate_closer      !! Damage rate (%) in the forest edge area in case of damage
                                                                                          !! @tex $(0-1)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: cws_break_further            !! Critical wind speed for stem breakage in the inner forest area
                                                                                          !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: cws_break_further_10         !! Critical wind speed for stem breakage in the inner forest area
                                                                                          !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: cws_overturn_further         !! Critical wind speed for overturning in the inner forest area
                                                                                          !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: cws_overturn_further_10      !! Critical wind speed for overturning in the inner forest area
                                                                                          !! @tex $(m/s; mean hourly wind speed)$ @endtex
    REAL(r_std), DIMENSION(ncirc)                :: cws_final_further            !! The critical wind speed of the final form of damage (breakage / uprooting)
                                                                                          !! in the inner forest area @tex $(m/s; mean hourly wind speed)$ @endtex
    LOGICAL                                      :: wind_damage_type_uprooting_further     !! The final form of damage (breakage / uprooting) in the inner forest area
    REAL(r_std), DIMENSION(ncirc)                :: wind_damage_rate_further     !! Damage rate (%) in the forest edge area in case
                                                                                          !! of damage in the inner forest area @tex $(0-1)$ @endtex

!_ ============================================================================================================================================================================

    !+++CHECK+++
    ! We could split this routine and calculate the sections 
    ! only once a day and then calculate the critical wind 
    ! speeds every half hour. If if the co
    !+++++++++++

 !! 1. Getting the input variables in the required form for the actual sub-routine 

    ! Looping over pixels
    DO ipts = 1, npts

       !! 1.1 Calculate root profile 
       z_soil(1) = 0.
       z_soil(2:nbdl+1) = diaglev(1:nbdl)

       ! Scaling factor for integration
       rpc(:) = un / ( un - EXP( -z_soil(nbdl) / humcste(:) ) )

       ! Calculate root density per layer per pft
       ! Loop over # soil layers
       DO ibdl = 1, nbdl

          root_dens(ibdl,:) = rpc(:) * &
               ( EXP( -z_soil(ibdl)/humcste(:)) - &
               EXP( -z_soil(ibdl+1)/humcste(:)) )

       ENDDO ! Loop over # soil layers
 
       !! 1.2 Defining the soil type
       ! This sub-routine uses 4 types of soil for calculating the 
       ! critical wind speed, so 12 different soil types present in 
       ! other modules of ORCHIDEE need to be grouped into these
       ! 4 generic types.

       ! +++CHECK+++
       ! For the moment only 3 soil types are distinguished.
       ! Needs to be checked with all 12 USDA soil types. 
       ! NOTE: your initial approach to use text is correct
       ! but makes it more difficult to use IF statements.
       ! I implemented the classic work around. Defined in
       ! constantes_soil_var.f90. Update the documentation
       ! in that routine.
       SELECT CASE (njsc(ipts))
       CASE (1:3) 
          soil_type(ipts) = ifree_draining
       CASE (4:6)
          soil_type(ipts) = igleyed
       CASE (7:9)
          soil_type(ipts) = ipeaty
       CASE (10:12)
          soil_type(ipts) = ipeat 
       CASE DEFAULT
          CALL ipslerr_p(3, 'stomate_windfall.f90',&
               'soil type is not defined','','')
       END SELECT
       !++++++++++++
      
       ! Looping over PFT-s
       DO ivm = 2, nvm

          ! Don't do the calculations if there is no vegetation
          ! or the vegetation is bare soil, grass or crop
          IF ((veget_max(ipts,ivm).LT.min_stomate) .OR. .NOT.is_tree(ivm)) THEN
             CYCLE
          ENDIF

          !! 1.3 Calculating the rooting depth
          !  If root density (proportion of root mass) is higher 
          !  than 3% in the 11th soil layer (1.024-2.056 m) below 
          !  the soil surface, then we treat rooting depth as 
          !  "deep". Otherwise, it's "shallow". Note that the 
          !  threshold of 3% is arbitrary and it might be needed to
          !  justify at some point in the future.
          
          ! +++CHECK+++
          ! It was hard coded (index = 11). We don't like hard coded
          ! even if it is unlikely that the 11-layers will change
          ! any time soon. The depth of the 11th layer depends on
          ! the total depth of the soil (dpu_max). Would be better
          ! to use ::dpu_max rather than assuming the 2 m.
          ! Also, there is new scheme being tested where nbdl may 
          ! have changed. CHECK when using the new scheme.
          IF (root_dens(nbdl,ivm) > 0.03) THEN

             rooting_depth = ideep

          ELSE

             rooting_depth = ishallow

          END IF

          !!  1.4 Set crown parameters
          !   To set crown parameters leaf mass needs to be available
          IF (SUM(circ_class_biomass(ipts,ivm,:,ileaf,icarbon)) .GT. min_stomate) THEN
  
             ! There is a canopy
             ! +++CHECK+++
             ! We can calculate the canopy density internally. We
             ! have leaf mass and crown volume. Adding part of the
             ! branches would be more challenging but possible
!!$             canopy_density = canopy_density_leaf(ivm)
             streamlining_c = streamlining_c_leaf(ivm)
             streamlining_n = streamlining_n_leaf(ivm)
             streamlining_rb = streamlining_rb_leaf(ivm)
             ! +++++++++++

          ELSE
       
             ! If the tree is leafless, the following parameters 
             ! become different. The weight of leaves will be absent.
             ! +++CHECK+++
             ! the definition says this includes leaves and branches. 
             ! If set to zero (as is the case here) there are no 
             ! branches!
!!$             canopy_density = canopy_density_leafless(ivm)
             ! +++++++++++
             
             ! Streamlining changes as the surface of leaves is absent.
             streamlining_c = streamlining_c_leafless(ivm)
             streamlining_n = streamlining_n_leafless(ivm)
             streamlining_rb = streamlining_rb_leafless(ivm)

          END IF

          !---TEMP---
          IF(ld_windfall)THEN
             WRITE(numout,*) 'Initialisation of critical windspeed'
             WRITE(numout,*) 'ipts, ivm, ',ipts, ivm
             WRITE(numout,*) 'root_dens(ibdl,ivm), ',root_dens(:,ivm)
             WRITE(numout,*) 'soil_type(ipts), ',soil_type(ipts)
             WRITE(numout,*) 'rooting_depth, ',rooting_depth
             WRITE(numout,*) 'streamlining_c, ', streamlining_c
             WRITE(numout,*) 'streamlining_n, ', streamlining_n
             WRITE(numout,*) 'streamlining_rb, ', streamlining_rb
          ENDIF
          !----------

          !! 1.5 Calculate stand characteristics
          !  Calculating diameter at breast height (dbh) in m of the
          !  mean tree of all circ classes.
          mean_dbh(:) = wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)

          ! Calculating the height in m of the mean tree of the given 
          ! dbh-class using
          mean_height(:) = wood_to_height(&
               circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)
          
          ! Stem mass is calculated as the aboveground woody biomass
          ! minus the branches. circ_class_biomass is the biomass of
          ! an individual tree (gC tree-1)
          stem_mass(:) = (circ_class_biomass(ipts,ivm,:,isapabove,icarbon) + &
               circ_class_biomass(ipts,ivm,:,iheartabove,icarbon)) * &
               (un - branch_ratio(ivm))

          ! Crown_mass is calculated as the leaf mass and the branch mass)
          crown_mass(:) = circ_class_biomass(ipts,ivm,:,ileaf,icarbon) + &
               (circ_class_biomass(ipts,ivm,:,isapabove,icarbon) + &
               circ_class_biomass(ipts,ivm,:,iheartabove,icarbon)) * &
               branch_ratio(ivm)
   

          DO icir = ncirc,1,-1

             !+++CHECK+++
             ! add a while loop. if CWS is not exceed for the
             ! largest diameter class then there is no need
             ! to repeat these calculations for the other 
             ! dbh classes
             !+++++++++++

             ! Calculating leaf area index
             lai(icir) = biomass_to_lai(&
                  circ_class_biomass(ipts,ivm,icir,ileaf,icarbon),ivm)
             
             ! Calculating the current tree spacing in the PFT
             ! The current tree spacing between tree stems are calculated 
             ! from the number of stems per hectare. circ_class_n is the 
             ! number of stems per m2; it's converted to stems per hectare 
             ! with m2_to_ha which has a value of 10000.
             ! +++CHECK+++
             ! this does not account for competiton/overlap
             ! expecially for the largest dbh classes the
             ! spacing will be very large. Calculating a
             ! single current spacing at the stand level 
             ! probably makes more sense
             IF (circ_class_n(ipts,ivm,icir) .GT. min_stomate) THEN
                current_spacing(icir) = 100.0 / &
                     SQRT(m2_to_ha * circ_class_n(ipts,ivm,icir))
             ELSE
                current_spacing(icir) = zero
             ENDIF
             ! +++++++++++

          ENDDO
          
          ! At this point, we have the weight of the stem and the crown, which 
          ! is used in calculating the resistance force of the tree against wind.          
          !---TEMP---
          IF(ld_windfall)THEN
             WRITE(numout,*) 'Calculated stem sections'
             WRITE(numout,*) 'mean dbh, ', mean_dbh(:)
             WRITE(numout,*) 'mean height, ', mean_height(:)
             WRITE(numout,*) 'stem_mass, ', stem_mass(:)
             WRITE(numout,*) 'crown_volume, ',crown_volume(:)
             WRITE(numout,*) 'crown_mass, ',crown_mass(:)
             WRITE(numout,*) 'lai, ',lai(:)
             WRITE(numout,*) 'current_spacing, ',current_spacing(:)
          ENDIF
          !----------

 


 !! 3. Calculating the gustiness of wind in the PFT

!!$        ! IMPORTANT: AS WE DO NOT HAVE INFORMATION ON STAND EDGES IN THE 
!!$        ! ORCHIDEE SIMULATIONS, A POSSIBLE PROXY TO USE HERE IS A VARIABLE 
!!$        ! THAT REPRESENTS EARLIER HARVESTS IN THE PREVIOUS 5 YEARS. AFTER 
!!$        ! SUCH A PERIOD, THE FOREST EDGE IS ASSUMED TO GET ACCLIMATIZED
!!$        ! TO THE NEW CIRCUMSTANCES, AND IT'S VULNERABILITY DIMINISHES 
!!$        ! (see Persson, 1975; Valinger and Fridman, 2011)
!!$          
!!$        ! There is a big increase in gustiness after a certain distance from 
!!$        ! the edge of the forest, hence we divide the area of the PFT into two 
!!$        ! smaller areas:
!!$        ! - the area which is further than X tree heights from any edge that 
!!$        ! was created in the previous five years (area_total_further)
!!$        ! - the area which is closer than X tree heights from any edge that 
!!$        ! was created in the previous five years (area_total_closer).
!!$
!!$        ! Of course, these smaller areas depend on the fragmentation of the 
!!$        ! gaps (created in the last five years). As we have no information 
!!$        ! on this, we assume a gap size for each country depending on the 
!!$        ! maximum allowed size of clear cuts (clear_cut_max) and that the  
!!$        ! gaps are square-shaped. Then, we divide the area of harvest in the 
!!$        ! previous five years (area_timber_removals_5_years) by clear_cut_max. 
!!$        ! Hence, from the number of gaps we can calculate the area_total_closer. 
!!$        ! As each circumference class will have its forest area divided, we  
!!$        ! calculate two gustiness values and hence, two critical wind speeds.
!!$
!!$        ! NOTE: We have to add a forest management map that contains the maximum 
!!$        ! allowed areas of clear fellings. As this is not available yet, we use  
!!$        ! an area of 2 ha as gap size for all calculations.
!!$
!!$        ! The value of X depends on the leaf area index of the PFT and is calculated as follows:
!!$        tree_heights_from_edge(:) = 28.0 * mean_height(icir) /&
!!$        lai(icir)
!!$        ! This comes from the equation of calculating canopy penetration depth in Pan et al. 2015.
!!$   
!!$        ! Calculating the total area of timber removals in the previous five years:
!!$        ! NOTE: Could you help with this?
!!$        ! +++CHECK+++
!!$        ! needs to become a SAVE in stomate. Just gave it a value so that we can compile
!!$        area_timber_removals_5_years(icir) = 1.0
!!$        ! +++++++++++
!!$
!!$        ! Calculating the total forest area belonging to the circumference class:
!!$        ! NOTE: Could you help with this?
!!$        !+++CHECK+++
!!$        ! We already discussed this by mail. Area per circ_class is not easy 
!!$        ! because the crowns are allowed to overlap. We can find a solution.
!!$        ! Area of the whole age class is given by
!!$        ! area = resolution_x(ipts) * resolution_y(ipts) * veget_max(ipts,ivm)
!!$        ! Need to discuss why this is needed. Just want to get the code
!!$        ! compiling for now so I gave it a neutral value
!!$        area_icir(icir) = 1.0
!!$        !++++++++++++
!!$
!!$        ! Number of gaps with the area of the maximum allowed clearfelling:
!!$        n_gap(icir) = INT(area_timber_removals_5_years(icir) / clear_cut_max)
!!$        ! The length of the sides of the squared-shaped gaps:
!!$        length_side_gap(icir) = SQRT(clear_cut_max)
!!$        ! The frame-shaped area around a 2-ha gap with X as the width of the frame:
!!$        area_around_gap(icir) = ((length_side_gap(icir) +&
!!$        tree_heights_from_edge(icir) * mean_height(icir))**2.0) - clear_cut_max
!!$        ! Total area of the "frames" in the particular dbh-class of the particular PFT:
!!$        ! The multiplication by 0.25 is because of the assumption that wind direction does not
!!$        ! change during the half-hourly time step, therefore only one of the four sides of the
!!$        ! gaps are affected.
!!$        area_total_closer(icir) = n_gap(icir) * area_around_gap(icir) * 0.25
!!$        ! The total area which is nor gap neither "frame":
!!$        area_total_further(icir) = area_icir(icir) - area_total_closer(icir) -&
!!$        (n_gap(icir) * clear_cut_max)
!!$
!!$
!!$        ! At this point, we know how areas with different gustiness will look like,
!!$        ! now we calculate gustiness for these areas.
!!$
!!$        ! Values of the ratio of mean tree spacing and height are limited
!!$        ! between 0.075 and 0.45. Bigger values don't make any difference,
!!$        ! smaller values are not really possible in real life.
!!$        IF (current_spacing(icir) / mean_height(icir) < 0.075) THEN
!!$           ratio_spacing_height(icir) = 0.075
!!$           ELSE
!!$           IF (current_spacing(icir) / mean_height(icir) > 0.45) THEN
!!$              ratio_spacing_height(icir) = 0.45
!!$              ELSE 
!!$              ratio_spacing_height(icir) = current_spacing(icir) / &
!!$              mean_height(icir)
!!$           END IF
!!$        END IF
!!$
!!$        
!!$        ! Calculating gap characteristics:
!!$
!!$        ! An originally input variable in GALES. We use the length of the edge of the square-shaped
!!$        ! gaps calculated in the previous section.
!!$        gap_size(icir) = length_side_gap(icir)  
!!$
!!$
!!$        ! If the gap is wider than 10 times the tree height,
!!$        ! then wind loading on the stand edge does not increase any more.
!!$        IF (gap_size(icir) > 10.0 * mean_height(icir)) THEN
!!$           gap_size(icir) = 10.0 * mean_height(icir)
!!$           ELSE
!!$           gap_size(icir) = gap_size(icir)
!!$        END IF
!!$        
!!$        ! Calculating factors for calculating bending moments:
!!$        ratio_gap_height(icir) = gap_size(icir) / mean_height(icir)
!!$        mean_gap10(icir) = 0.001 * 10.0**0.562    
!!$        max_gap10(icir) = 0.0064 * 10.0**0.3467 
!!$        mean_gap_factor(icir) = (0.001 * ratio_gap_height(icir)**0.562) / &
!!$        mean_gap10(icir)
!!$        max_gap_factor(icir) = (0.0064 * ratio_gap_height(icir)**0.3467) / &
!!$        max_gap10(icir)
!!$
!!$        ! In the following section, there are two types of terms for forest edges:
!!$        ! the edge and the edge area.
!!$        ! The first one represents the very edge of the forest (0 m from the gap),
!!$        ! the second one represents the area around the gap (area_around_gap) where
!!$        ! gustiness is different than in the rest of the forest.
!!$        
!!$        ! In the original code of GALES, there are two methods to calculate gustiness inside
!!$        ! the forest stand and they are both used, depending on the values of a few particular
!!$        ! variables. That's the reason for the existence of variables e.g. old_gust_edge
!!$        ! and new_gust_edge.
!!$        
!!$        ! The next bit corrects the gustiness for the effect of an upwind gap. First, it calculates
!!$        ! the gustiness well inside the forest and at the edge, with a 10-tree gap size and with
!!$        ! the specified gap size. It does that using the old method of calculating the gustiness
!!$        ! because the gap factors are designed for this method. It then uses these to correct the
!!$        ! gustiness at the edge calculated using the new method to account for a gap. Gustiness
!!$        ! inside the forest is proportionally adjusted to reflect how far back you are (either X or
!!$        ! 9 tree heights).
!!$        ! At the edge of the forest, with a very large gap you should have less gustiness than well
!!$        ! inside the forest.
!!$        ! Gustiness is calculated for both the closer and further areas from the gaps,
!!$        ! the critical wind speed will differ accordingly.
!!$        ! Zeroes in the power term are only used for completeness.
!!$        mean_bm_closer(icir) = (0.68 * ratio_spacing_height(icir) - 0.0385) + &
!!$        (-0.68 * ratio_spacing_height(icir) + 0.4785) * (1.7239 * &
!!$        ratio_spacing_height(icir) + 0.0316)**tree_heights_from_edge(icir)
!!$
!!$        max_bm_closer(icir) = (2.7193 * ratio_spacing_height(icir) - 0.061) + &
!!$        (-1.273 * ratio_spacing_height(icir) + 0.9701) * (1.1127 * &
!!$        ratio_spacing_height(icir) + 0.0311)**tree_heights_from_edge(icir)
!!$
!!$        mean_bm_further(icir) = (0.68 * ratio_spacing_height(icir) - 0.0385) + &
!!$        (-0.68 * ratio_spacing_height(icir) + 0.4785) * (1.7239 * &
!!$        ratio_spacing_height(icir) + 0.0316)**9.0
!!$
!!$        max_bm_further(icir) = (2.7193 * ratio_spacing_height(icir) - 0.061) + &
!!$        (-1.273 * ratio_spacing_height(icir) + 0.9701) * (1.1127 * &
!!$        ratio_spacing_height(icir) + 0.0311)**9.0
!!$
!!$        mean_bm_edge(icir) = (0.68 * ratio_spacing_height(icir) - 0.0385) + &
!!$        (-0.68 * ratio_spacing_height(icir) + 0.4785) * (1.7239 * &
!!$        ratio_spacing_height(icir) + 0.0316)**0.0
!!$
!!$        max_bm_edge(icir) = (2.7193 * ratio_spacing_height(icir) - 0.061) + &
!!$        (-1.273 * ratio_spacing_height(icir) + 0.9701) * (1.1127 * &
!!$        ratio_spacing_height(icir) + 0.0311)**0.0
!!$
!!$        mean_bm_edge_gap(icir) = (0.68 * ratio_spacing_height(icir) - 0.0385) + &
!!$        (-0.68 * ratio_spacing_height(icir) + 0.4785) * ((1.7239 * &
!!$        ratio_spacing_height(icir) + 0.0316)**0.0) * mean_gap_factor(icir)
!!$
!!$        max_bm_edge_gap(icir) = (2.7193 * ratio_spacing_height(icir) - 0.061) + &
!!$        (-1.273 * ratio_spacing_height(icir) + 0.9701) * ((1.1127 * &
!!$        ratio_spacing_height(icir) + 0.0311)**0.0) * max_gap_factor(icir)
!!$
!!$        ! Calculating term_a and term_b for the new gust method (there are two methods):
!!$        IF (-2.1 * ratio_spacing_height(icir) + 0.91 < zero) THEN
!!$           term_a(icir) = zero 
!!$           ELSE
!!$           term_a(icir) = -2.1 * ratio_spacing_height(icir) + 0.91
!!$        END IF
!!$        term_b(icir) = 1.0611 * LOG(ratio_spacing_height(icir)) + 4.2
!!$
!!$        ! Old gust factors:
!!$        old_gust_closer(icir) = max_bm_closer(icir) / mean_bm_closer(icir)
!!$        old_gust_further(icir) = max_bm_further(icir) / mean_bm_further(icir)
!!$        old_gust_edge(icir) = max_bm_edge(icir) / mean_bm_edge(icir)
!!$        old_gust_edge_gap(icir) = max_bm_edge_gap(icir) / mean_bm_edge_gap(icir)
!!$
!!$        ! New gust factors:
!!$        new_gust_closer(icir) = term_a(icir) * tree_heights_from_edge(icir) + &
!!$        term_b(icir)
!!$        new_gust_further(icir) = term_a(icir) * 9.0 + term_b(icir)
!!$        new_gust_edge(icir) = term_a(icir) * zero + term_b(icir)
!!$
!!$        ! Calculating the new gustiness at the edge accounting for the gap. This is done using the
!!$        ! argument that the ratios between old and new gustiness values stay the same:
!!$        IF ((old_gust_closer(icir) - old_gust_edge(icir)) > zero) THEN
!!$           new_gust_edge_gap_closer(icir) = (old_gust_edge_gap(icir) - &
!!$           old_gust_edge(icir)) * (new_gust_closer(icir) - &
!!$           new_gust_edge(icir)) / (old_gust_closer(icir) - &
!!$           old_gust_edge(icir)) + new_gust_edge(icir)
!!$           ELSE
!!$           new_gust_edge_gap_closer(icir) = new_gust_edge(icir)
!!$        END IF
!!$
!!$        IF ((old_gust_further(icir) - old_gust_edge(icir)) > zero) THEN
!!$           new_gust_edge_gap_further(icir) = (old_gust_edge_gap(icir) - &
!!$           old_gust_edge(icir)) * (new_gust_further(icir) - &
!!$           new_gust_edge(icir)) / (old_gust_further(icir) - &
!!$           old_gust_edge(icir)) + new_gust_edge(icir)
!!$           ELSE
!!$           new_gust_edge_gap_further(icir) = new_gust_edge(icir)
!!$        END IF
!!$
!!$        ! Calculating gustiness (g):
!!$        g_closer(icir) = ((mean_height(icir) * tree_heights_from_edge(icir) / &
!!$        (mean_height(icir) * 9.0)) * (new_gust_closer(icir) - new_gust_edge_gap_closer(icir))) +&
!!$        new_gust_edge_gap_closer(icir)
!!$
!!$        g_further(icir) = ((mean_height(icir) * 9.0 / (mean_height(icir) * 9.0)) * &
!!$        (new_gust_further(icir) - new_gust_edge_gap_further(icir))) +&
!!$        new_gust_edge_gap_further(icir)
!!$
!!$        ! The following section is used to calculate the edge_factor. This section calculates
!!$        ! how the mean loading on a tree changes as a function of tree height, spacing, distance
!!$        ! from edge and size of any upwind gap. An upwind gap greater than 10 tree heights
!!$        ! is assumed to be an infinite gap. The output (edge_factor) is used to modify the
!!$        ! calculated wind loading on the tree, which assumes a steady wind and a position well
!!$        ! inside the forest.
!!$        mean_bm_gust_factor_closer(icir) = (0.68 * ratio_spacing_height(icir) - &
!!$        0.0385) + (-0.68 * ratio_spacing_height(icir) + 0.4785) * ((1.7239 * &
!!$        ratio_spacing_height(icir) + 0.0316)**tree_heights_from_edge(icir)) * &
!!$        mean_gap_factor(icir)
!!$
!!$        mean_bm_gust_factor_further(icir) = (0.68 * ratio_spacing_height(icir) - &
!!$        0.0385) + (-0.68 * ratio_spacing_height(icir) + 0.4785) * ((1.7239 * &
!!$        ratio_spacing_height(icir) + 0.0316)**9.0) * mean_gap_factor(icir)
!!$
!!$        edge_factor(icir) = mean_bm_gust_factor_closer(icir) / &
!!$        mean_bm_gust_factor_further(icir)
!!$
!!$        ! Calculating gust_factor acounting for gustiness and edge_factor:
!!$        gust_factor_closer(icir) = g_closer(icir) * edge_factor(icir)
!!$        gust_factor_further(icir) = g_further(icir) * edge_factor(icir)
!!$
!!$
!!$
!!$ !! 5. Calculating critical moments (tree mechanics section)
!!$
!!$        ! overturning_moment_multiplier: this comes form the species parameters file and is related to the
!!$        ! depth and type of the soil.
!!$        ! Rooting depth has three categories: Shallow (less then 0.8 m deep); Deep (deeper than 0.8 m);
!!$        ! Average (average values to be used when rooting depth is unknown).
!!$        ! Soil type has four categories: Free-draining mineral soils; Gleyed mineral soils; Peaty mineral
!!$        ! soils; Deep peats
!!$        ! As these are generic soil types, a modified soil map of Europe would be needed for the optimal use
!!$        ! of WINDFALL.
!!$
!!$        IF (soil_type(ipts) == 'free_draining' .AND. rooting_depth(ipts,ivm) == 'shallow' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_shallow
!!$   
!!$           ELSE IF (soil_type(ipts) == 'free_draining' .AND. rooting_depth(ipts,ivm) == 'shallow' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_shallow_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'free_draining' .AND. rooting_depth(ipts,ivm) == 'deep' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_deep
!!$   
!!$           ELSE IF (soil_type(ipts) == 'free_draining' .AND. rooting_depth(ipts,ivm) == 'deep' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_deep_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'free_draining' .AND. rooting_depth(ipts,ivm) == 'average' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_average
!!$   
!!$           ELSE IF (soil_type(ipts) == 'free_draining' .AND. rooting_depth(ipts,ivm) == 'average' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_free_draining_average_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'gleyed' .AND. rooting_depth(ipts,ivm) == 'shallow' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_shallow
!!$   
!!$           ELSE IF (soil_type(ipts) == 'gleyed' .AND. rooting_depth(ipts,ivm) == 'shallow' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_shallow_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'gleyed' .AND. rooting_depth(ipts,ivm) == 'deep' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_deep
!!$   
!!$           ELSE IF (soil_type(ipts) == 'gleyed' .AND. rooting_depth(ipts,ivm) == 'deep' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_deep_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'gleyed' .AND. rooting_depth(ipts,ivm) == 'average' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_average
!!$   
!!$           ELSE IF (soil_type(ipts) == 'gleyed' .AND. rooting_depth(ipts,ivm) == 'average' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_gleyed_average_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peaty' .AND. rooting_depth(ipts,ivm) == 'shallow' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peaty_shallow
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peaty' .AND. rooting_depth(ipts,ivm) == 'shallow' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peaty_shallow_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peaty' .AND. rooting_depth(ipts,ivm) == 'deep' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peaty_deep
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peaty' .AND. rooting_depth(ipts,ivm) == 'deep' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peaty_deep_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peaty' .AND. rooting_depth(ipts,ivm) == 'average' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peaty_average
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peaty' .AND. rooting_depth(ipts,ivm) == 'average' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peaty_average_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peaty' .AND. rooting_depth(ipts,ivm) == 'shallow' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peat_shallow
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peat' .AND. rooting_depth(ipts,ivm) == 'shallow' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peat_shallow_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peat' .AND. rooting_depth(ipts,ivm) == 'deep' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peat_deep
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peat' .AND. rooting_depth(ipts,ivm) == 'deep' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peat_deep_leafless
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peat' .AND. rooting_depth(ipts,ivm) == 'average' &
!!$           .AND. senescence(ipts,ivm) .EQ. .FALSE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peat_average
!!$   
!!$           ELSE IF (soil_type(ipts) == 'peat' .AND. rooting_depth(ipts,ivm) == 'average' &
!!$           .AND. senescence(ipts,ivm) .EQ. .TRUE.) THEN
!!$           overturning_moment_multiplier(ipts,ivm) = overturning_peat_average_leafless
!!$   
!!$           ELSE
!!$           overturning_moment_multiplier(ipts,ivm) = 125.0
!!$           ! average value if parameters are missing
!!$        END IF
!!$
!!$     ! Maximum moment calculated for tree overturning.
!!$        max_overturning_moment(ipts,ivm,icir) = overturning_moment_multiplier(ipts,ivm) * &
!!$        stem_mass(ipts,ivm,icir)
!!$
!!$     ! Maximum moment calculated for stem breakage.
!!$        max_breaking_moment(ipts,ivm,icir) = modulus_rupture * f_knot * pi * &
!!$        (section_diam(ipts,ivm,icir,1)**3.0) / 32.0
!!$
!!$
!!$     ! This is where the gustiness of wind is "filtered out"
!!$     ! so that hourly mean wind speeds can be used in the simulations.
!!$     ! In the end, moments and hence, critical wind speeds are calculated
!!$     ! for both areas (closer & further) with different gustiness.
!!$
!!$        overturning_moment_closer(ipts,ivm,icir) = max_overturning_moment(ipts,ivm,icir) / &
!!$        gust_factor_closer(ipts,ivm,icir)
!!$   
!!$        overturning_moment_further(ipts,ivm,icir) = max_overturning_moment(ipts,ivm,icir) / &
!!$        gust_factor_further(ipts,ivm,icir)
!!$   
!!$        breaking_moment_closer(ipts,ivm,icir) = max_breaking_moment(ipts,ivm,icir) / &
!!$        gust_factor_closer(ipts,ivm,icir)
!!$   
!!$        breaking_moment_further(ipts,ivm,icir) = max_breaking_moment(ipts,ivm,icir) / &
!!$        gust_factor_further(ipts,ivm,icir)

!!$
!!$
!!$ !! 7. Calculating critical wind speed and damage rate
!!$     !! 7.1 Calculating critical wind speed in the forest and 10 m above the zero-plane displacement
!!$     ! for the edge area
!!$
!!$     ! Critical wind speed for stem breakage in the edge area (m/s)
!!$     call calculate_wind_process(breaking_moment_closer(ipts,ivm,icir),cws_break_closer(ipts,ivm,icir))
!!$     ! Critical wind speed for stem breakage 10 m above the zero-plane displacement in the edge area
!!$     ! (m/s) 
!!$     call elevate(cws_break_closer(ipts,ivm,icir), z0, d, cws_break_closer_10(ipts,ivm,icir))
!!$
!!$     ! Critical wind speed for tree overturning in the edge area (m/s)
!!$     call calculate_wind_process(overturning_moment_closer(ipts,ivm,icir),cws_overturn_closer(ipts,ivm,icir))
!!$     ! Critical wind speed for tree overturning 10 m above the zero-plane displacement in the edge area
!!$     ! (m/s)
!!$     call elevate(cws_overturn_closer(ipts,ivm,icir), z0, d, cws_overturn_closer_10)
!!$
!!$
!!$     ! Calculating damage severity in case any of the critical wind speeds is reached in the area
!!$     ! around gaps (forest edge area):
!!$
!!$     ! Until a better method is found, we kill all the trees in the given dbh-class of the PFT
!!$     ! in case of damage.
!!$
!!$     ! Whichever of the two damage types (overturning, stem breakage) has the lowest critical wind speed,
!!$     ! it will be the occurring one.
!!$     IF (cws_overturn_closer_10(ipts,ivm,icir) < cws_break_closer_10(ipts,ivm,icir)) THEN
!!$        cws_final_closer(ipts,ivm,icir) = cws_overturn_closer_10(ipts,ivm,icir)
!!$        ELSE
!!$        cws_final_closer(ipts,ivm,icir) = cws_break_closer_10(ipts,ivm,icir)
!!$     END IF
!!$
!!$     IF (wind_speed_actual(ipts) >= cws_final_closer(ipts,ivm,icir)) THEN
!!$        IF (cws_overturn_closer_10(ipts,ivm,icir) < cws_break_closer_10(ipts,ivm,icir)) THEN
!!$           wind_damage_type_closer(ipts,ivm,icir) = overturning
!!$           ELSE
!!$           wind_damage_type_closer(ipts,ivm,icir) = breakage
!!$        END IF
!!$        wind_damage_rate_closer(ipts,ivm,icir) = 1.0
!!$        ELSE
!!$        wind_damage_rate_closer(ipts,ivm,icir) = zero
!!$       ! Damage rates should be dependent on the actual difference between cws and actual wind speed.
!!$       ! However, as literature is lacking, we use 100% mortality rates at the moment (as in GALES).
!!$     END IF
!!$
!!$
!!$     !! 7.2 Calculating critical wind speed in the forest and 10 m above the zero-plane displacement
!!$     ! for the inner forest area
!!$
!!$     ! These are only calculated if there was a damage rate higher than zero in the area around the gaps
!!$     ! (forest edge area).
!!$
!!$     IF (wind_damage_rate_closer(ipts,ivm,icir) > zero) THEN
!!$
!!$        ! Whichever of the two damage types (overturning, stem breakage) has the lowest critical wind
!!$        ! speed, it will be the occurring one.
!!$
!!$        ! Critical wind speed for stem breakage in the inner forest area (m/s)
!!$        call calculate_wind_process(breaking_moment_further(ipts,ivm,icir),cws_break_further(ipts,ivm,icir))
!!$        ! Critical wind speed for stem breakage 10 m above the zero-plane displacement in the inner
!!$        ! forest area (m/s)
!!$        call elevate(cws_break_further(ipts,ivm,icir), surf_rough(ipts,ivm,icir), &
!!$             zp_displ(ipts,ivm,icir), cws_break_further_10(ipts,ivm,icir))
!!$
!!$        ! Critical wind speed for tree overturning in the inner forest area (m/s)
!!$        call calculate_wind_process(overturning_moment_further(ipts,ivm,icir),&
!!$             cws_overturn_further(ipts,ivm,icir))
!!$        ! Critical wind speed for tree overturning 10 m above the zero-plane displacement in the
!!$        ! inner forest area (m/s)
!!$        call elevate(cws_overturn_further(ipts,ivm,icir), surf_rough(ipts,ivm,icir), &
!!$             zp_displ(ipts,ivm,icir), cws_overturn_further_10(ipts,ivm,icir))
!!$
!!$        ! Whichever of the two damage types (overturning, stem breakage) has the lowest critical wind
!!$        ! speed, it will be the occurring one.
!!$        IF (cws_overturn_further_10(ipts,ivm,icir) < cws_break_further_10(ipts,ivm,icir)) THEN
!!$           cws_final_further(ipts,ivm,icir) = cws_overturn_further_10(ipts,ivm,icir)
!!$           ELSE
!!$           cws_final_further(ipts,ivm,icir) = cws_break_further_10(ipts,ivm,icir)
!!$        END IF
!!$
!!$        IF (wind_speed_actual(ipts) >= cws_final_further(ipts,ivm,icir)) THEN
!!$           IF (cws_overturn_further_10(ipts,ivm,icir) < cws_break_closer_10(ipts,ivm,icir)) THEN
!!$              wind_damage_type_further(ipts,ivm,icir) = overturning
!!$              ELSE
!!$              wind_damage_type_further(ipts,ivm,icir) = breakage
!!$           END IF
!!$           wind_damage_rate_further(ipts,ivm,icir) = 1.0
!!$           ELSE
!!$           wind_damage_rate_further(ipts,ivm,icir) = zero
!!$        END IF
!!$
!!$        ELSE
!!$        wind_damage_rate_further(ipts,ivm,icir) = zero
!!$     END IF
!!$
!!$
!!$
!!$     ! NOTE: this is the point where damage should be taken into account in other
!!$     ! modules of ORCHIDEE. Output variables should be calculated here.
!!$
!!$
!!$
!!$          END DO ! loop ncirc

       END DO ! loop nvm

    END DO ! loop npts



!!$ !! 9. Writing out history variables
!!$
!!$     ! NOTE: THIS IS ADOPTED FROM stomate_fire AND HASN'T CHANGED YET.
!!$     firefrac(:,:) = firefrac(:,:) / dt
!!$
!!$     CALL xios_orchidee_send_field("FIREFRAC",firefrac(:,:))
!!$     CALL xios_orchidee_send_field("FIREDEATH",firedeath(:,:))
!!$
!!$     CALL histwrite_p (hist_id_stomate, 'FIREFRAC', itime, &
!!$          firefrac(:,:), npts*nvm, horipft_index)
!!$     CALL histwrite_p (hist_id_stomate, 'FIREDEATH', itime, &
!!$          firedeath(:,:), npts*nvm, horipft_index)
!!$
!!$     IF (bavard.GE.4) WRITE(numout,*) 'Leaving fire'
!!$        ELSE
!!$        ! This is for bavard.
!!$     END IF



END SUBROUTINE wind_damage

!!$
!!$
!!$!! ===========================================================================================
!!$!! SUBROUTINE      : streamlining
!!$!!
!!$!>\BRIEF           : This subroutine calculates the streamlining of tree crowns. Streamlining is
!!$!!                   the change of shape of the crowns due to wind.
!!$!! 
!!$!! DESCRIPTION	   : None
!!$!!
!!$!! RECENT CHANGE(S): None
!!$!!
!!$!! RETURN VALUE    : Surface roughness and zero-plane displacement among other variables that
!!$!!                   are used in the other subroutines
!!$!! REFERENCES      :
!!$!!
!!$!! FLOWCHART       : None
!!$!! \streamlining_n 
!!$!_ ============================================================================================
!!$   
!!$  SUBROUTINE streamlining(a_guess_wind_speed, porosity_sub, lambda_sub, lambda_capital_sub, &
!!$             gamma_solved_sub, zp_displ_sub, psih_sub, surf_rough_sub)
!!$
!!$  IMPLICIT NONE
!!$
!!$!! 0. Variable and parameter declaration
!!$
!!$    !! 0.1 Input variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)   :: a_guess_wind_speed
!!$
!!$    !! 0.2 Output variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out)  :: porosity_sub
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out)  :: lambda_sub
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out)  :: lambda_capital_sub
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out)  :: gamma_solved_sub
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out)  :: zp_displ_sub
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out)  :: psih_sub
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out)  :: surf_rough_sub
!!$
!!$    !! 0.3 Modified variables
!!$
!!$    !! 0.4 Local variables
!!$
!!$!_ =============================================================================================
!!$
!!$!! 1. Subroutine
!!$
!!$   IF (a_guess_wind_speed(ipts,ivm,icir) > 25.0) THEN
!!$   ! Above 25 m/s, tree crowns cannot streamline any further.
!!$      porosity_sub(ipts,ivm,icir) = streamlining_c * 25.0**(-streamlining_n)            
!!$      ELSE
!!$      porosity_sub(ipts,ivm,icir) = streamlining_c * &
!!$      a_guess_wind_speed(ipts,ivm,icir)**(-streamlining_n)
!!$   END IF
!!$
!!$   lambda_sub(ipts,ivm,icir) = (canopy_breadth(ipts,ivm,icir)/2.0) * canopy_depth(ipts,ivm,icir) * &
!!$   porosity_sub(ipts,ivm,icir) / current_spacing(ipts,ivm,icir)**2.0
!!$   lambda_capital_sub(ipts,ivm,icir) = 2.0 * lambda_sub(ipts,ivm,icir)
!!$
!!$   IF (lambda_capital_sub(ipts,ivm,icir) > 0.6) THEN
!!$      ! Gamma is needed for the calculation of the roughness length.
!!$      gamma_solved_sub(ipts,ivm,icir) = 1.0 / ((c_surface + c_drag * 0.3)**0.5)
!!$      ELSE
!!$      gamma_solved_sub(ipts,ivm,icir) = 1.0 / ((c_surface + c_drag * &
!!$      lambda_capital_sub(ipts,ivm,icir) / 2.0)**0.5)
!!$   END IF
!!$
!!$   ! Calculation of the zero-plane displacement:
!!$   zp_displ_sub(ipts,ivm,icir) = mean_height(ipts,ivm,icir) * (1.0 - ((1.0 - EXP(-(c_displacement * &
!!$   lambda_capital_sub(ipts,ivm,icir))**0.5)) / &
!!$   (c_displacement * lambda_capital_sub(ipts,ivm,icir))**0.5))
!!$    
!!$   ! Calculation of the aerodynamic roughness length:
!!$
!!$   ! psih = "Profile influence function" at h (height of the roughness elements).
!!$   psih_sub(ipts,ivm,icir) = LOG(c_roughness) -1.0 + c_roughness**(-1.0)
!!$   surf_rough_sub(ipts,ivm,icir) = (mean_height(ipts,ivm,icir) - zp_displ_sub(ipts,ivm,icir)) * &
!!$   EXP(psih_sub(ipts,ivm,icir) - ct_karman * gamma_solved_sub(ipts,ivm,icir))
!!$
!!$  END SUBROUTINE streamlining
!!$
!!$
!!$
!!$!! ============================================================================================
!!$!! SUBROUTINE      : calculate_force
!!$!!
!!$!>\BRIEF           : This subroutine calculates the force affecting the tree.
!!$!! 
!!$!! DESCRIPTION	   : None
!!$!!
!!$!! RECENT CHANGE(S): None
!!$!!
!!$!! RETURN VALUE    : 
!!$!!
!!$!! REFERENCES      :
!!$!!
!!$!! FLOWCHART       : None
!!$!! \streamlining_n 
!!$!_ ============================================================================================
!!$   
!!$  SUBROUTINE calculate_force(a_guess_wind_speed, a_gamma_solved, a_zp_displ, force_on_tree_sub, &
!!$             height_of_force_sub)
!!$
!!$  IMPLICIT NONE
!!$
!!$!! 0. Variable and parameter declaration
!!$
!!$    !! 0.1 Input variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: a_guess_wind_speed
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: a_gamma_solved
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: a_zp_displ
!!$
!!$    !! 0.2 Output variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out) :: force_on_tree_sub
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out) :: height_of_force_sub
!!$
!!$    !! 0.3 Modified variables
!!$
!!$    !! 0.4 Local variables
!!$
!!$!_ =============================================================================================
!!$
!!$!! 1. Subroutine
!!$
!!$   force_on_tree_sub(ipts,ivm,icir) = air_density * (a_guess_wind_speed(ipts,ivm,icir) * &
!!$   current_spacing(ipts,ivm,icir) / a_gamma_solved(ipts,ivm,icir))**2.0
!!$   
!!$   height_of_force_sub(ipts,ivm,icir) = a_zp_displ(ipts,ivm,icir)
!!$
!!$   ! The maximum of the zero-plane displacement height is locked at 0.8 tree height.
!!$   IF (height_of_force_sub(ipts,ivm,icir) > 0.8 * mean_height(ipts,ivm,icir)) THEN
!!$      height_of_force_sub(ipts,ivm,icir) = 0.8 * mean_height(ipts,ivm,icir)
!!$   END IF
!!$
!!$  END subroutine calculate_force
!!$
!!$
!!$!! =============================================================================================
!!$!! SUBROUTINE      : calculate_bending_moment
!!$!!
!!$!>\BRIEF           : Subroutine to calculate the bending moment
!!$!!                   (to compare with critical moments in the iterative calculation of the critical
!!$!!                   wind speeds)
!!$!! 
!!$!! DESCRIPTION	   : None
!!$!!
!!$!! RECENT CHANGE(S): None
!!$!!
!!$!! RETURN VALUE    : 
!!$!!
!!$!! REFERENCES      :
!!$!!
!!$!! FLOWCHART       : None
!!$!! \streamlining_n 
!!$!_ =============================================================================================
!!$   
!!$  SUBROUTINE calculate_bending_moment (a_force_on_tree, a_height_of_force, bending_moment_sub)
!!$
!!$  IMPLICIT NONE
!!$
!!$!! 0. Variable and parameter declaration
!!$
!!$    !! 0.1 Input variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: a_force_on_tree 
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: a_height_of_force
!!$
!!$    !! 0.2 Output variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out) :: bending_moment_sub
!!$
!!$    !! 0.3 Modified variables
!!$
!!$    !! 0.4 Local variables
!!$
!!$!_ =============================================================================================
!!$
!!$!! 1. Subroutine
!!$
!!$   bending_moment_sub(ipts,ivm,icir) = a_force_on_tree(ipts,ivm,icir) * &
!!$   a_height_of_force(ipts,ivm,icir) * f_crown_mass
!!$
!!$  END SUBROUTINE calculate_bending_moment
!!$
!!$
!!$
!!$!! =============================================================================================
!!$!! SUBROUTINE      : calculate_cws
!!$!!
!!$!>\BRIEF           : Subroutine to calculate critical wind speed
!!$!!                   (to compare with wind speeds used for calculating streamlining in the
!!$!!                   iterative calculation of the final critical wind speed)
!!$!! 
!!$!! DESCRIPTION	   : None
!!$!!
!!$!! RECENT CHANGE(S): None
!!$!!
!!$!! RETURN VALUE    : 
!!$!!
!!$!! REFERENCES      :
!!$!!
!!$!! FLOWCHART       : None
!!$!! \streamlining_n 
!!$!_ =============================================================================================
!!$   
!!$  SUBROUTINE calculate_cws(critical_moment_sub, zp_displ_sub, gamma_solved_sub, &
!!$             current_spacing_sub, crit_wind_speed_sub)
!!$
!!$  IMPLICIT NONE
!!$
!!$!! 0. Variable and parameter declaration
!!$
!!$    !! 0.1 Input variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: critical_moment_sub
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: zp_displ_sub
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: gamma_solved_sub
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: current_spacing_sub
!!$    ! f_crown_mass and rho are parameters.
!!$
!!$    !! 0.2 Output variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out) :: crit_wind_speed_sub
!!$
!!$    !! 0.3 Modified variables
!!$
!!$    !! 0.4 Local variables
!!$
!!$!_ =============================================================================================
!!$
!!$!! 1. Subroutine
!!$
!!$   crit_wind_speed_sub(ipts,ivm,icir) = (gamma_solved_sub(ipts,ivm,icir) / &
!!$   current_spacing_sub(ipts,ivm,icir)) * (critical_moment_sub(ipts,ivm,icir) / &
!!$   (f_crown_mass * air_density * zp_displ_sub(ipts,ivm,icir)))**0.5
!!$
!!$  END SUBROUTINE calculate_cws
!!$
!!$
!!$
!!$!! =============================================================================================
!!$!! SUBROUTINE      : calculate_wind_process
!!$!!
!!$!>\BRIEF           : This subroutine calculates the breaking and overturning critical wind
!!$!!                   speeds
!!$!! 
!!$!! DESCRIPTION	   : None
!!$!!
!!$!! RECENT CHANGE(S): None
!!$!!
!!$!! RETURN VALUE    : 
!!$!!
!!$!! REFERENCES      :
!!$!!
!!$!! FLOWCHART       : None
!!$!! \streamlining_n 
!!$!_ =============================================================================================
!!$   
!!$  SUBROUTINE calculate_wind_process(critical_moment, guess_wind_speed_sub)
!!$
!!$  IMPLICIT NONE
!!$
!!$!! 0. Variable and parameter declaration
!!$
!!$    !! 0.1 Input variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: critical_moment
!!$
!!$    !! 0.2 Output variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out) :: guess_wind_speed_sub !! A guess for the
!!$                                                    !! critical wind speed
!!$
!!$    !! 0.3 Modified variables
!!$
!!$    !! 0.4 Local variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc)              :: delta !! Difference between the guessed
!!$                                                    !! and the calculated wind speed
!!$    REAL, DIMENSION(npts,nvm,ncirc)              :: wind_precision !! How close we should
!!$                                                    !! get with the iterations
!!$
!!$!_ =============================================================================================
!!$
!!$!! 1. Subroutine
!!$
!!$! This bit belongs to the original form of doing the iterations. For more info, see the notes below.
!!$!  guess_wind_speed_sub = 64.0
!!$!  delta = guess_wind_speed_sub / 2.0
!!$
!!$   guess_wind_speed_sub(ipts,ivm,icir) = 25.0
!!$
!!$   wind_precision(ipts,ivm,icir) = 0.1
!!$
!!$   DO WHILE (delta(ipts,ivm,icir) > wind_precision(ipts,ivm,icir))
!!$      CALL streamlining(guess_wind_speed_sub(ipts,ivm,icir), porosity(ipts,ivm,icir), &
!!$           lambda(ipts,ivm,icir), lambda_capital(ipts,ivm,icir), gamma_solved(ipts,ivm,icir), &
!!$           zp_displ(ipts,ivm,icir), psih(ipts,ivm,icir), surf_rough(ipts,ivm,icir))
!!$
!!$! The following lines belong to the original way of calculating the final critical wind speed
!!$! with multiple iterations, as streamlining depends on wind speed and critical wind speed
!!$! depends on streamlining. Originally, iterations started with a wind speed value of 64 m/s and
!!$! then if bending moment was below the critical moment, then a wind speed value of 32 m/s was
!!$! used. If bending moment was above critical moment afterwards, then a wind speed value of 48
!!$! m/s was used, and so on, until the difference between the critical moment and bending moment
!!$! was less than the pre-defined precision.
!!$! The new version uses a shorter iterating process instead.
!!$! Now the critical wind speed is calculated instead of the moments, and the iterating process
!!$! starts from 25 m/s (no further streamlining above that value) and uses the calculated critical
!!$! wind speed as the wind speed used for calculating streamlining after that first value.
!!$! Then, with the new streamlining values a new critical wind speed is calculated, and so on,
!!$! until the difference between the two wind speeds (guess_wind_speed_sub and crit_wind_speed)
!!$! is less than the pre-defined precision.
!!$
!!$!     call calculate_force(guess_wind_speed_sub, gammaSolved, d, force_on_tree, height_of_force)
!!$!     gammaSolved and d come from call streamlining.
!!$!     call calculate_bending_moment(force_on_tree, height_of_force, bending_moment)
!!$!     force_on_tree and height_of_force come from call calculate_force.
!!$!     IF (bending_moment > critical_moment) THEN
!!$!        guess_wind_speed_sub = guess_wind_speed_sub - delta
!!$!        ELSE
!!$!        guess_wind_speed_sub = guess_wind_speed_sub + delta
!!$!     END IF
!!$!     delta = delta / 2.0 
!!$!     END DO
!!$
!!$      ! The input variable of the subroutine is passed towards the call of another subroutine:
!!$      critical_moment_cws(ipts,ivm,icir) = critical_moment(ipts,ivm,icir)
!!$
!!$      CALL calculate_cws(critical_moment_cws(ipts,ivm,icir), zp_displ(ipts,ivm,icir), &
!!$           gamma_solved(ipts,ivm,icir), current_spacing(ipts,ivm,icir), &
!!$           crit_wind_speed(ipts,ivm,icir))
!!$
!!$      delta(ipts,ivm,icir) = guess_wind_speed_sub(ipts,ivm,icir) - &
!!$      crit_wind_speed(ipts,ivm,icir)
!!$
!!$      guess_wind_speed_sub(ipts,ivm,icir) = crit_wind_speed(ipts,ivm,icir)
!!$   END DO
!!$
!!$  END SUBROUTINE calculate_wind_process
!!$
!!$
!!$
!!$!! =============================================================================================
!!$!! SUBROUTINE      : elevate
!!$!!
!!$!>\BRIEF           : This subroutine converts the critical wind speeds calculated above to the
!!$!!                   wind speed 10 meters above the zero-plane displacement
!!$!! 
!!$!! DESCRIPTION	   : None
!!$!!
!!$!! RECENT CHANGE(S): None
!!$!!
!!$!! RETURN VALUE    : 
!!$!!
!!$!! REFERENCES      :
!!$!!
!!$!! FLOWCHART       : None
!!$!! \streamlining_n 
!!$!_ =============================================================================================
!!$   
!!$  SUBROUTINE elevate(uh_speed, the_surf_rough, the_zp_displ, elevate_result_sub)
!!$
!!$  IMPLICIT NONE
!!$
!!$!! 0. Variable and parameter declaration
!!$
!!$    !! 0.1 Input variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: uh_speed
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: the_surf_rough
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(in)  :: the_zp_displ
!!$
!!$    !! 0.2 Output variables
!!$
!!$    REAL, DIMENSION(npts,nvm,ncirc), INTENT(out) :: elevate_result_sub
!!$
!!$    !! 0.3 Modified variables
!!$
!!$    !! 0.4 Local variables
!!$
!!$!_ ==============================================================================================
!!$
!!$!! 1. Subroutine
!!$
!!$   elevate_result_sub(ipts,ivm,icir) = uh_speed(ipts,ivm,icir) * log(10 / &
!!$   the_surf_rough(ipts,ivm,icir)) / log((mean_height(ipts,ivm,icir) - the_zp_displ(ipts,ivm,icir)) / &
!!$   the_surf_rough(ipts,ivm,icir))
!!$
!!$  END SUBROUTINE elevate



END MODULE stomate_windfall
