! =================================================================================================================================
! MODULE : routing_reg
!
! CONTACT : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF This module routes the water over the continents into the oceans and computes the water
!! stored in floodplains or taken for irrigation.
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S) :
!!
!! SVN :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DRIVER/ORCHIDEE/src_sechiba/routing.f90 $
!! $Date: 2015-06-10 10:34:42 +0200 (Wed, 10 Jun 2015) $
!! $Revision: 2751 $
!! \n
!_ ================================================================================================================================

!
!
! Histoire Salee
!---------------
! La douce riviere
! Sortant de son lit
! S'est jetee ma chere
! dans les bras mais oui
! du beau fleuve
!
! L'eau coule sous les ponts
! Et puis les flots s'emeuvent
! - N'etes vous pas au courant ?
! Il parait que la riviere
! Va devenir mer
! Roland Bacri
!
! "... So close, no matter how far
! Couldn't be much more from the heart ..." [Metallica]
! 

MODULE routing_reg

  USE ioipsl
  USE xios_orchidee
  USE ioipsl_para
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE sechiba_io
  USE interpol_help
  USE grid_var
  USE grid
  USE haversine
  USE mod_orchidee_para
  USE timer

  USE routing_tools

!  USE mod_orchidee_para_var
#ifdef XIOS
  USE xios
#endif
  USE constantes_var 

  IMPLICIT NONE
  PRIVATE
  PUBLIC :: routing_reg_main, routing_reg_initialize, routing_reg_finalize, routing_reg_clear, &
       &    routing_reg_riverloc, routing_reg_riverflow

!! PARAMETERS
  INTEGER(i_std) :: nbasmax=200 !! The maximum number of basins we wish to have per grid box (truncation of the model) (unitless)
!$OMP THREADPRIVATE(nbasmax)
  INTEGER(i_std), SAVE :: nbvmax=440 !! The maximum number of basins we can handle at any time during the generation of the maps (unitless)
!$OMP THREADPRIVATE(nbvmax)
  INTEGER(i_std), SAVE :: nbxmax=63 !! The maximum number of points in one direction (NS or WE) using in routing_reg_getgrid (unitless)
!$OMP THREADPRIVATE(nbxmax)
  INTEGER(i_std), SAVE :: maxpercent=2 !! The maximum area percentage of a sub-basin in a grib-box (default = 2%)
!$OMP THREADPRIVATE(maxpercent)
  REAL(r_std), PARAMETER :: slow_tcst_cwrr = 7.0 !! Property of the slow reservoir, when CWRR hydrology is activated (day/m)
  REAL(r_std), PARAMETER :: fast_tcst_cwrr = 0.5 !! Property of the fast reservoir, when CWRR hydrology is activated (day/m)
  REAL(r_std), PARAMETER :: stream_tcst_cwrr = 0.01 !! Property of the stream reservoir, when CWRR hydrology is activated (day/m)
  REAL(r_std), PARAMETER :: flood_tcst_cwrr = 4.0 !! Property of the floodplains reservoir, when CWRR hydrology is activated (day/m)
  REAL(r_std), PARAMETER :: swamp_cst_cwrr = 0.2 !! Fraction of the river transport that flows to the swamps, when CWRR hydrology is activated (unitless;0-1)
  !
  REAL(r_std), PARAMETER :: slow_tcst_chois = 25.0 !! Property of the slow reservoir, when Choisnel hydrology is activated (day/m)
  REAL(r_std), PARAMETER :: fast_tcst_chois = 3.0 !! Property of the fast reservoir, when Choisnel hydrology is activated (day/m)
  REAL(r_std), PARAMETER :: stream_tcst_chois = 0.24 !! Property of the stream reservoir, when Choisnel hydrology is activated (day/m)
  REAL(r_std), PARAMETER :: flood_tcst_chois = 4.0 !! Property of the floodplains reservoir, when Choisnel hydrology is activated (day/m)
  REAL(r_std), PARAMETER :: swamp_cst_chois = 0.2 !! Fraction of the river transport that flows to the swamps, when Choisnel hydrology is activated (unitless;0-1)
  REAL(r_std), PARAMETER :: low_reservoir_warning = -1.0 !! From which value do we start to warn the user that something is wrong
                                                         !! and the slow reservoir has large negative values.
  !
  REAL(r_std), SAVE :: fast_tcst !! Property of the fast reservoir, (day/m)
!$OMP THREADPRIVATE(fast_tcst)
  REAL(r_std), SAVE :: slow_tcst !! Property of the slow reservoir, (day/m)
!$OMP THREADPRIVATE(slow_tcst)
  REAL(r_std), SAVE :: stream_tcst !! Property of the stream reservoir, (day/m)
!$OMP THREADPRIVATE(stream_tcst)
  REAL(r_std), SAVE :: flood_tcst !! Property of the floodplains reservoir, (day/m)
!$OMP THREADPRIVATE(flood_tcst)
  REAL(r_std), SAVE :: swamp_cst !! Fraction of the river transport that flows to the swamps (unitless;0-1)
!$OMP THREADPRIVATE(swamp_cst)
  !
  ! Relation between volume and fraction of floodplains
  !
  REAL(r_std), SAVE :: beta = 2.0 !! Parameter to fix the shape of the floodplain (>1 for convex edges, <1 for concave edges) (unitless)
!$OMP THREADPRIVATE(beta)
  REAL(r_std), SAVE :: betap = 0.5 !! Ratio of the basin surface intercepted by ponds and the maximum surface of ponds (unitless;0-1)
!$OMP THREADPRIVATE(betap)
  REAL(r_std), SAVE :: floodcri = 2000.0 !! Potential height for which all the basin is flooded (mm)
!$OMP THREADPRIVATE(floodcri)
  !
  ! Relation between maximum surface of ponds and basin surface, and drainage (mm/j) to the slow_res
  !
  REAL(r_std), PARAMETER :: pond_bas = 50.0 !! [DISPENSABLE] - not used
  REAL(r_std), SAVE :: pondcri = 2000.0 !! Potential height for which all the basin is a pond (mm)
!$OMP THREADPRIVATE(pondcri)
  !
  REAL(r_std), PARAMETER :: maxevap_lake = 7.5/86400. !! Maximum evaporation rate from lakes (kg/m^2/s)
  !
  REAL(r_std),SAVE :: dt_routing !! Routing time step (s)
!$OMP THREADPRIVATE(dt_routing)
  !
  INTEGER(i_std), SAVE :: diagunit = 87 !! Diagnostic file unit (unitless)
!$OMP THREADPRIVATE(diagunit)
  !
  ! Logicals to control model configuration
  !
  LOGICAL, SAVE :: dofloodinfilt = .FALSE. !! Logical to choose if floodplains infiltration is activated or not (true/false)
!$OMP THREADPRIVATE(dofloodinfilt)
  LOGICAL, SAVE :: doswamps = .FALSE. !! Logical to choose if swamps are activated or not (true/false)
!$OMP THREADPRIVATE(doswamps)
  LOGICAL, SAVE :: doponds = .FALSE. !! Logical to choose if ponds are activated or not (true/false)
!$OMP THREADPRIVATE(doponds)
  !
  ! The variables describing the basins and their routing, need to be in the restart file.
  !
  INTEGER(i_std), SAVE :: num_largest !! Number of largest river basins which should be treated as independently as rivers
                                                                                            !! (not flow into ocean as diffusion coastal flow) (unitless)
!$OMP THREADPRIVATE(num_largest)
  REAL(r_std), SAVE :: time_counter !! Time counter (s)
!$OMP THREADPRIVATE(time_counter)
  REAL(r_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: routing_area_loc !! Surface of basin (m^2)
!$OMP THREADPRIVATE(routing_area_loc)
  REAL(r_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: topo_resid_loc !! Topographic index of the retention time (m)
!$OMP THREADPRIVATE(topo_resid_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: route_togrid_loc !! Grid into which the basin flows (unitless)
!$OMP THREADPRIVATE(route_togrid_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: route_tobasin_loc !! Basin in to which the water goes (unitless)
!$OMP THREADPRIVATE(route_tobasin_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:) :: route_nbintobas_loc !! Number of basin into current one (unitless)
!$OMP THREADPRIVATE(route_nbintobas_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:) :: origin_nbintobas_loc !! Number of sub-grid basin into current one before truncation (unitless)
!$OMP THREADPRIVATE(origin_nbintobas_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: global_basinid_loc !! ID of basin (unitless)
!$OMP THREADPRIVATE(global_basinid_loc)
  REAL(r_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:,:) :: route_outlet_loc !! Coordinate of outlet (-)
!$OMP THREADPRIVATE(route_outlet_loc)
  REAL(r_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: route_type_loc !! Coordinate of outlet (-)
!$OMP THREADPRIVATE(route_type_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: hydrodiag_loc !! Variable to diagnose the hydrographs
!$OMP THREADPRIVATE(hydrodiag_loc)
  REAL(r_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:) :: hydroupbasin_loc !! The area upstream of the gauging station (m^2)
  !$OMP THREADPRIVATE(hydroupbasin_loc)
  INTEGER(i_std), SAVE :: outflow_nb_loc, outflow_offset_loc
  !$OMP THREADPRIVATE(outflow_nb_loc, outflow_offset_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: outflow_pt_loc      !! Indicis on the processor of the largest rivers
  !$OMP THREADPRIVATE(outflow_pt_loc)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)    :: outflow_lalo_loc    !! Coordinates of the outflow points of the lage river
  !$OMP THREADPRIVATE(outflow_lalo_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:)   :: outflow_type_loc    !! Type of outlow for the rivers
  !$OMP THREADPRIVATE(outflow_type_loc)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)      :: outflow_loc         !! Outflow for the num_largest rivers.
  !$OMP THREADPRIVATE(outflow_loc)
  !
  ! parallelism
  REAL(r_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: routing_area_glo !! Surface of basin (m^2)
!$OMP THREADPRIVATE(routing_area_glo)
  REAL(r_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: topo_resid_glo !! Topographic index of the retention time (m)
!$OMP THREADPRIVATE(topo_resid_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: route_togrid_glo !! Grid into which the basin flows (unitless)
!$OMP THREADPRIVATE(route_togrid_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: route_tobasin_glo !! Basin in to which the water goes (unitless)
!$OMP THREADPRIVATE(route_tobasin_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:) :: route_nbintobas_glo !! Number of basin into current one (unitless)
!$OMP THREADPRIVATE(route_nbintobas_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:) :: origin_nbintobas_glo !! Number of sub-grid basin into current one before truncation (unitless)
!$OMP THREADPRIVATE(origin_nbintobas_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: global_basinid_glo !! ID of basin (unitless)
!$OMP THREADPRIVATE(global_basinid_glo)
  REAL(r_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:,:) :: route_outlet_glo !! Coordinate of outlet (-)
!$OMP THREADPRIVATE(route_outlet_glo)
  REAL(r_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: route_type_glo !! Coordinate of outlet (-)
!$OMP THREADPRIVATE(route_type_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:,:) :: hydrodiag_glo !! Variable to diagnose the hydrographs
!$OMP THREADPRIVATE(hydrodiag_glo)
  REAL(r_std), SAVE, ALLOCATABLE, TARGET, DIMENSION(:) :: hydroupbasin_glo !! The area upstream of the gauging station (m^2)
!$OMP THREADPRIVATE(hydroupbasin_glo)
  !
    REAL(r_std), SAVE, POINTER, DIMENSION(:,:)                 :: routing_area                !! Surface of basin (m^2)
!$OMP THREADPRIVATE(routing_area)
  REAL(r_std), SAVE, POINTER, DIMENSION(:,:)                 :: topo_resid                  !! Topographic index of the retention time (m)
!$OMP THREADPRIVATE(topo_resid)
  INTEGER(i_std), SAVE, POINTER, DIMENSION(:,:)              :: route_togrid                !! Grid into which the basin flows (unitless)
!$OMP THREADPRIVATE(route_togrid)
  INTEGER(i_std), SAVE, POINTER, DIMENSION(:,:)              :: route_tobasin               !! Basin in to which the water goes (unitless)
!$OMP THREADPRIVATE(route_tobasin)
  INTEGER(i_std), SAVE, POINTER, DIMENSION(:)                :: route_nbintobas             !! Number of basin into current one (unitless)
!$OMP THREADPRIVATE(route_nbintobas)
  INTEGER(i_std), SAVE, POINTER, DIMENSION(:,:)              :: global_basinid              !! ID of basin (unitless)
!$OMP THREADPRIVATE(global_basinid)
  INTEGER(i_std), SAVE, POINTER, DIMENSION(:,:)              :: hydrodiag                   !! Variable to diagnose the hydrographs
!$OMP THREADPRIVATE(hydrodiag)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)               :: slowflow_diag               !! Diagnostic slow flow hydrographs (kg/dt)
!$OMP THREADPRIVATE(slowflow_diag)  
  REAL(r_std), SAVE, POINTER, DIMENSION(:)                   :: hydroupbasin                !! The area upstream of the gauging station (m^2)
!$OMP THREADPRIVATE(hydroupbasin)
  !
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: irrigated !! Area equipped for irrigation in each grid box (m^2)
!$OMP THREADPRIVATE(irrigated)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: floodplains !! Maximal surface which can be inundated in each grid box (m^2)
!$OMP THREADPRIVATE(floodplains)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: swamp !! Maximal surface of swamps in each grid box (m^2)
!$OMP THREADPRIVATE(swamp)
  !
  ! The reservoirs, also to be put into the restart file.
  !
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: fast_reservoir !! Water amount in the fast reservoir (kg)
!$OMP THREADPRIVATE(fast_reservoir)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: slow_reservoir !! Water amount in the slow reservoir (kg)
!$OMP THREADPRIVATE(slow_reservoir)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: stream_reservoir !! Water amount in the stream reservoir (kg)
!$OMP THREADPRIVATE(stream_reservoir)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: flood_reservoir !! Water amount in the floodplains reservoir (kg)
!$OMP THREADPRIVATE(flood_reservoir)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: lake_reservoir !! Water amount in the lake reservoir (kg)
!$OMP THREADPRIVATE(lake_reservoir)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: pond_reservoir !! Water amount in the pond reservoir (kg)
!$OMP THREADPRIVATE(pond_reservoir)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: flood_frac_bas !! Flooded fraction per basin (unitless;0-1)
!$OMP THREADPRIVATE(flood_frac_bas)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: pond_frac !! Pond fraction per grid box (unitless;0-1)
!$OMP THREADPRIVATE(pond_frac)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: flood_height !! Floodplain height (mm)
!$OMP THREADPRIVATE(flood_height)
  !
  ! The accumulated fluxes.
  !
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: floodout_mean !! Accumulated flow out of floodplains (kg/m^2/dt)
!$OMP THREADPRIVATE(floodout_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: runoff_mean !! Accumulated runoff (kg/m^2/dt)
!$OMP THREADPRIVATE(runoff_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: drainage_mean !! Accumulated drainage (kg/m^2/dt)
!$OMP THREADPRIVATE(drainage_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: transpot_mean !! Mean potential transpiration from the plants (kg/m^2/dt)
!$OMP THREADPRIVATE(transpot_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: precip_mean !! Accumulated precipitation (kg/m^2/dt)
!$OMP THREADPRIVATE(precip_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: humrel_mean !! Mean soil moisture stress, mean root extraction potential (unitless)
!$OMP THREADPRIVATE(humrel_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: totnobio_mean !! Mean last total fraction of no bio (unitless;0-1)
!$OMP THREADPRIVATE(totnobio_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: vegtot_mean !! Mean potentially vegetated fraction (unitless;0-1)
!$OMP THREADPRIVATE(vegtot_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: k_litt_mean !! Mean averaged conductivity for saturated infiltration in the 'litter' layer (kg/m^2/dt)
!$OMP THREADPRIVATE(k_litt_mean)
  !
  ! The averaged outflow fluxes.
  !
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: lakeinflow_mean !! Mean lake inflow (kg/m^2/dt)
!$OMP THREADPRIVATE(lakeinflow_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: returnflow_mean !! Mean water flow from lakes and swamps which returns to the grid box.
                                                                                             !! This water will go back into the hydrol or hydrolc module to allow re-evaporation (kg/m^2/dt)
!$OMP THREADPRIVATE(returnflow_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: reinfiltration_mean !! Mean water flow which returns to the grid box (kg/m^2/dt)
!$OMP THREADPRIVATE(reinfiltration_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: irrigation_mean !! Mean irrigation flux.
                                                                                             !! This is the water taken from the reservoirs and beeing put into the upper layers of the soil (kg/m^2/dt)
!$OMP THREADPRIVATE(irrigation_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: riverflow_mean !! Mean Outflow of the major rivers.
                                                                                             !! The flux will be located on the continental grid but this should be a coastal point (kg/dt)
!$OMP THREADPRIVATE(riverflow_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: coastalflow_mean !! Mean outflow on coastal points by small basins.
                                                                                             !! This is the water which flows in a disperse way into the ocean (kg/dt)
!$OMP THREADPRIVATE(coastalflow_mean)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: floodtemp !! Temperature to decide if floodplains work (K)
!$OMP THREADPRIVATE(floodtemp)
  INTEGER(i_std), SAVE :: floodtemp_lev !! Temperature level to decide if floodplains work (K)
!$OMP THREADPRIVATE(floodtemp_lev)
  !
  ! Diagnostic variables ... well sort of !
  !
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: irrig_netereq !! Irrigation requirement (water requirements by the crop for its optimal growth (kg/m^2/dt)
!$OMP THREADPRIVATE(irrig_netereq)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: hydrographs !! Hydrographs at the outflow of the grid box for major basins (kg/dt)
!$OMP THREADPRIVATE(hydrographs)
  !
  ! Diagnostics for the various reservoirs we use (Kg/m^2)
  !
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: fast_diag !! Diagnostic for the fast reservoir (kg/m^2)
!$OMP THREADPRIVATE(fast_diag)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: slow_diag !! Diagnostic for the slow reservoir (kg/m^2)
!$OMP THREADPRIVATE(slow_diag)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: stream_diag !! Diagnostic for the stream reservoir (kg/m^2)
!$OMP THREADPRIVATE(stream_diag)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: flood_diag !! Diagnostic for the floodplain reservoir (kg/m^2)
!$OMP THREADPRIVATE(flood_diag)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: pond_diag !! Diagnostic for the pond reservoir (kg/m^2)
!$OMP THREADPRIVATE(pond_diag)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: lake_diag !! Diagnostic for the lake reservoir (kg/m^2)
!$OMP THREADPRIVATE(lake_diag)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: mask_coast                   !! Mask with coastal gridcells on local grid(1/0)
!$OMP THREADPRIVATE(mask_coast)
  REAL(r_std), SAVE                            :: max_lake_reservoir           !! Maximum limit of water in lake_reservoir [kg/m2]
!$OMP THREADPRIVATE(max_lake_reservoir)
  INTEGER(i_std), SAVE                         :: nb_coast_gridcells           !! Number of gridcells which can receive coastalflow

  !
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: areaupbasin !!
!$OMP THREADPRIVATE(areaupbasin)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: areaupbasin_loc !!
!$OMP THREADPRIVATE(areaupbasin_loc)
  INTEGER(i_std), SAVE :: nb_small
!$OMP THREADPRIVATE(nb_small)
  CHARACTER(LEN=25), SAVE, ALLOCATABLE, DIMENSION(:) :: basin_names !! Names of the rivers (unitless)
!$OMP THREADPRIVATE(basin_names)
  CHARACTER(LEN=25), SAVE, ALLOCATABLE, DIMENSION(:,:) :: basin_allnames_glo !! Names of the rivers (unitless)
!$OMP THREADPRIVATE(basin_allnames_glo)
  CHARACTER(LEN=25), SAVE, ALLOCATABLE, DIMENSION(:,:) :: basin_allnames_loc !! Names of the rivers (unitless)
!$OMP THREADPRIVATE(basin_allnames_loc)

!
!!! Matched GRDC stations, Ranked by stations
! (to be written into output)
!
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:) :: idx_grdc_match_glo     !! Sorted (descend) GRDC index that matched with ORC
!$OMP THREADPRIVATE(idx_grdc_match_glo)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: og_grdc_match_glo   !! ig of ORC that matched with Sorted (descend) GRDC index that matched with ORC
!$OMP THREADPRIVATE(og_grdc_match_glo)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: ob_grdc_match_glo   !! ib of ORC grid that marched with Sorted (descend) GRDC index that matched with ORC
!$OMP THREADPRIVATE(ob_grdc_match_glo)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:) :: lat_grdc_match_glo     !! Lat of ORC subbasin
!!$OMP THREADPRIVATE(lat_grdc_match_glo)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:) :: lon_grdc_match_glo     !! Lon of ORC subbasin
!!$OMP THREADPRIVATE(lon_grdc_match_glo)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: area_grdc_match_glo    !! Area of ORC subbasin
!$OMP THREADPRIVATE(area_grdc_match_glo)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: idxsort_grdc_match_glo    !! Area of ORC subbasin
!$OMP THREADPRIVATE(idxsort_grdc_match_glo)
!  CHARACTER(LEN=25), SAVE, ALLOCATABLE, DIMENSION(:,:) :: basin_grdc_match_glo !! Names of the rivers (unitless)
!!$OMP THREADPRIVATE(basin_grdc_match_glo)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:) :: mergedhydro_grdc_match_glo   !! Hydrographs of ORC subbasin
!!$OMP THREADPRIVATE(mergedhydro_grdc_match_glo)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: idx_grdc_match_loc     !! Sorted (descend) GRDC index that matched with ORC
!$OMP THREADPRIVATE(idx_grdc_match_loc)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: lat_grdc_match_loc     !! Lat of ORC subbasin
!$OMP THREADPRIVATE(lat_grdc_match_loc)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: lon_grdc_match_loc     !! Lon of ORC subbasin
!$OMP THREADPRIVATE(lon_grdc_match_loc)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: area_grdc_match_loc    !! Area of ORC subbasin
!$OMP THREADPRIVATE(area_grdc_match_loc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: idxsort_grdc_match_loc    !! Area of ORC subbasin
!$OMP THREADPRIVATE(idxsort_grdc_match_loc)
  CHARACTER(LEN=25), SAVE, ALLOCATABLE, DIMENSION(:,:) :: basin_grdc_match_loc !! Names of the rivers (unitless)
!$OMP THREADPRIVATE(basin_grdc_match_loc)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)  :: mergedhydro_grdc_match_loc   !! Hydrographs of ORC subbasin
!$OMP THREADPRIVATE(mergedhydro_grdc_match_loc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: og_grdc_match_loc   !! ig of ORC that matched with Sorted (descend) GRDC index that matched with ORC
!$OMP THREADPRIVATE(og_grdc_match_loc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: ob_grdc_match_loc   !! ib of ORC grid that marched with Sorted (descend) GRDC index that matched with ORC
!$OMP THREADPRIVATE(ob_grdc_match_loc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: ogloc_grdc_match_loc   !! ig of ORC that matched with Sorted (descend) GRDC index that matched with ORC
!$OMP THREADPRIVATE(ogloc_grdc_match_loc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: obloc_grdc_match_loc   !! ib of ORC grid that marched with Sorted (descend) GRDC index that matched with ORC
!$OMP THREADPRIVATE(obloc_grdc_match_loc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: ox_grdc_match_loc   !! ix of ORC that matched with Sorted (descend) GRDC index that matched with ORC
!$OMP THREADPRIVATE(ox_grdc_match_loc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: oy_grdc_match_loc   !! iy of ORC grid that marched with Sorted (descend) GRDC index that matched with ORC
!$OMP THREADPRIVATE(oy_grdc_match_loc)

!
!!! Largest river basins
! (to be written into output)
!
!  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: outpt_coor_largest_glo !! Outflow point coordinate for each basin (unitless)
!!$OMP THREADPRIVATE(outpt_coor_largest_glo)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: totarea_largest_glo !! Total area of basin (m^2)
!$OMP THREADPRIVATE(totarea_largest_glo)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: idxsort_largest_glo    !! Area of ORC subbasin
!$OMP THREADPRIVATE(idxsort_largest_glo)
!  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: hydrographs_largest_glo !! Total area of basin (m^2)
!!$OMP THREADPRIVATE(hydroraphs_largest_glo)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: outpt_coor_largest_loc !! Outflow point coordinate for each basin (unitless)
!$OMP THREADPRIVATE(outpt_coor_largest_loc)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: totarea_largest_loc !! Total area of basin (m^2)
!$OMP THREADPRIVATE(totarea_largest_loc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: idxsort_largest_loc    !! Area of ORC subbasin
!$OMP THREADPRIVATE(idxsort_largest_loc)
  REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:) :: hydrographs_largest_loc !! Total area of basin (m^2)
!$OMP THREADPRIVATE(hydroraphs_largest_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:) :: og_largest_loc !! Total area of basin (m^2)
!$OMP THREADPRIVATE(og_largest_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:) :: ob_largest_loc !! Total area of basin (m^2)
!$OMP THREADPRIVATE(ob_largest_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:) :: ogloc_largest_loc !! Total area of basin (m^2)
!$OMP THREADPRIVATE(ogloc_largest_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:) :: obloc_largest_loc !! Total area of basin (m^2)
!$OMP THREADPRIVATE(obloc_largest_loc)


  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: ig_glo
!$OMP THREADPRIVATE(ig_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: ib_glo
!$OMP THREADPRIVATE(ib_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: ig_loc
!$OMP THREADPRIVATE(ig_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: ib_loc
!$OMP THREADPRIVATE(ib_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:) :: ix_glo
!$OMP THREADPRIVATE(ix_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:) :: iy_glo
!$OMP THREADPRIVATE(iy_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:) :: ix_loc
!$OMP THREADPRIVATE(ix_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:) :: iy_loc
!$OMP THREADPRIVATE(iy_loc)

!
!!!!! XIOS GRDC and 'largest' output parameters
!
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: nbsta    !! 
!$OMP THREADPRIVATE(nbsta)
  INTEGER(i_std),SAVE                             :: nbsta_mpi                         !! number of local stations in each mpi group
!$OMP THREADPRIVATE(nbsta_mpi)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: nbsta_sort, nbsta_tmp, nbsta_idx    !! Area of ORC subbasin
!$OMP THREADPRIVATE(nbsta_sort, nbsta_tmp, nbsta_idx)
  INTEGER,SAVE,ALLOCATABLE,DIMENSION(:)           :: data_ibegin_grdc
!$OMP THREADPRIVATE(data_ibegin_grdc)
  INTEGER,SAVE,ALLOCATABLE,DIMENSION(:)           :: data_iend_grdc
!$OMP THREADPRIVATE(data_iend_grdc)
  INTEGER(i_std), SAVE                            :: data_ibegin_grdc_tmp                       !! Number of GRDC stations match the ORC subbasin
!$OMP THREADPRIVATE(data_ibegin_grdc_tmp)
  INTEGER(i_std), SAVE                            :: ngrdc 
!$OMP THREADPRIVATE(ngrdc)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: lon_sta    !! 
!$OMP THREADPRIVATE(lon_sta)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: data_ibegin_grdc_gather
!$OMP THREADPRIVATE(data_ibegin_grdc_gather)

  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: nblargest    !! 
!$OMP THREADPRIVATE(nblargest)
  INTEGER(i_std),SAVE                             :: nblargest_mpi
!$OMP THREADPRIVATE(nblargest_mpi)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: nblargest_sort, nblargest_tmp, nblargest_idx    !! Area of ORC subbasin
!$OMP THREADPRIVATE(nblargest_sort, nblargest_tmp, nblargest_idx)
  INTEGER,SAVE,ALLOCATABLE,DIMENSION(:)           :: data_ibegin_largest
!$OMP THREADPRIVATE(data_ibegin_largest)
  INTEGER,SAVE,ALLOCATABLE,DIMENSION(:)           :: data_iend_largest
!$OMP THREADPRIVATE(data_iend_largest)
  INTEGER(i_std), SAVE                            :: data_ibegin_largest_tmp                       !! Number of GRDC stations match the ORC subbasin
!$OMP THREADPRIVATE(data_ibegin_largest_tmp)
  INTEGER(i_std), SAVE                            :: nlargest 
!$OMP THREADPRIVATE(nlargest)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)    :: lon_larg    !! 
!$OMP THREADPRIVATE(lon_larg)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: data_ibegin_largest_gather
!$OMP THREADPRIVATE(data_ibegin_largest_gather)

!
!!! No. of GRDC stations matching model subbasins
!
  INTEGER(i_std), SAVE :: nb_grdc_match_glo                       !! Number of GRDC stations match the ORC subbasin
!$OMP THREADPRIVATE(nb_grdc_match_glo)
  INTEGER(i_std), SAVE :: nb_grdc_match_loc                       !! Number of GRDC stations match the ORC subbasin
!$OMP THREADPRIVATE(nb_grdc_match_loc)
  INTEGER(i_std), SAVE :: nb_largest_loc
!$OMP THREADPRIVATE(nb_largest_loc)
!  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: nb_og_grdc !! No. of grids matched the same GRDC station
!!!$OMP THREADPRIVATE(nb_og_grdc)

!
!!! Variables of GRDC subbasin
!
  INTEGER(i_std), ALLOCATABLE, DIMENSION(:, :), SAVE :: g_stabsn, b_stabsn  !! 
!$OMP THREADPRIVATE(g_stabsn, b_stabsn)
  INTEGER(i_std), ALLOCATABLE, DIMENSION(:), SAVE    :: nb_subb !!
!$OMP THREADPRIVATE(nb_subb)
  INTEGER(i_std), ALLOCATABLE, DIMENSION(:, :), SAVE :: g_stabsn_loc, b_stabsn_loc  !! 
!$OMP THREADPRIVATE(g_stabsn_loc, b_stabsn_loc)
  INTEGER(i_std), ALLOCATABLE, DIMENSION(:), SAVE    :: nb_subb_loc !!
!$OMP THREADPRIVATE(nb_subb_loc)
  INTEGER(i_std), ALLOCATABLE, DIMENSION(:, :), SAVE :: gloc_stabsn_loc, bloc_stabsn_loc  !! 
!$OMP THREADPRIVATE(gloc_stabsn_loc, bloc_stabsn_loc)
  REAL(r_std), ALLOCATABLE, DIMENSION(:, :, :), SAVE :: map_stabsn_glo, map_stabsn_loc  !! 
!$OMP THREADPRIVATE(map_stabsn_glo, map_stabsn_loc)
  INTEGER(i_std), ALLOCATABLE, DIMENSION(:, :), SAVE :: nb_stabsn_glo, nb_stabsn_loc  !! 
!$OMP THREADPRIVATE(nb_stabsn_glo, nb_stabsn_loc)


!
! Comparing GRDC and Model values
!
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: area_grdc_glo    !! Area of ORC subbasin
!!$OMP THREADPRIVATE(area_grdc_glo)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: idx_grdc_glo    !! Index of ORC subbasin
!!$OMP THREADPRIVATE(idx_grdc_glo)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: lon_grdc_glo    !! Lon of ORC subbasin
!!$OMP THREADPRIVATE(lon_grdc_glo)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: lat_grdc_glo    !! Lat of ORC subbasin
!!$OMP THREADPRIVATE(lat_grdc_glo)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: area_grdc_loc    !! Area of ORC subbasin
!!$OMP THREADPRIVATE(area_grdc_loc)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: idx_grdc_loc    !! Index of ORC subbasin
!!$OMP THREADPRIVATE(idx_grdc_loc)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: lon_grdc_loc    !! Lon of ORC subbasin
!!$OMP THREADPRIVATE(lon_grdc_loc)
!  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: lat_grdc_loc    !! Lat of ORC subbasin
!!$OMP THREADPRIVATE(lat_grdc_loc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: idx_mod_match_loc !! Index of GRDC stations in each ORC model grid and each subbasin
!$OMP THREADPRIVATE(idx_mod_match_loc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: idx_mod_match_glo !! Index of GRDC stations in each ORC model grid and each subbasin
!$OMP THREADPRIVATE(idx_mod_match_glo)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: hydrodiag_grdc_glo    !! Lat of ORC subbasin
!$OMP THREADPRIVATE(hydrodiag_grdc_glo)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:,:) :: hydrodiag_grdc_loc    !! Lat of ORC subbasin
!$OMP THREADPRIVATE(hydrodiag_grdc_loc)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: hydrodiag_largest_glo !! Total area of basin (m^2)
!$OMP THREADPRIVATE(hydrodiag_largest_glo)
  INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: hydrodiag_largest_loc !! Total area of basin (m^2)
!$OMP THREADPRIVATE(hydrodiag_largest_loc)

!
!!! GRDC data in the predefined domain
!
    INTEGER(i_std), SAVE :: igrdc_reg
!$OMP THREADPRIVATE(igrdc_reg)
    INTEGER(i_std), SAVE :: tgrdc_cut
!$OMP THREADPRIVATE(tgrdc_cut)
    INTEGER(i_std), PARAMETER :: nstr = 60
!$OMP THREADPRIVATE(nstr)
    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)         :: lon_obs_reg    !! Longitude of GRDC (Global and user specified domain (_reg))
!$OMP THREADPRIVATE(lon_obs_reg)
    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)         :: lat_obs_reg    !! Latitude of GRDC 
!$OMP THREADPRIVATE(lat_obs_reg)
    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)         :: area_obs_reg  !! km^2 in GRDC, convert to m^2 to keep consistent with ORC
!$OMP THREADPRIVATE(area_obs_reg)
!    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:) :: hydrographs_obs_reg!! GRDC river discharge 
!!$OMP THREADPRIVATE(hydrographs_obs_reg)
    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:,:)       :: mergedhydro_obs_reg!! GRDC river discharge  
!$OMP THREADPRIVATE(mergedhydro_obs_reg)
    CHARACTER(LEN=nstr), SAVE, ALLOCATABLE, DIMENSION(:) :: sta_obs_reg !! Names for GRDC stations
!$OMP THREADPRIVATE(sta_obs_reg)
    CHARACTER(LEN=nstr), SAVE, ALLOCATABLE, DIMENSION(:) :: river_obs_reg !! Names for GRDC rivers
!$OMP THREADPRIVATE(river_obs_reg)
    CHARACTER(LEN=nstr), SAVE, ALLOCATABLE, DIMENSION(:) :: country_obs_reg  !! GRDC Country 
!$OMP THREADPRIVATE(country_obs_reg)
!    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)        :: number_obs_reg !! GRDC Index
!!$OMP THREADPRIVATE(number_obs_reg)
    INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:)      :: idx_obs_reg          !!     !Corresponding GRDC index for specified domain
!$OMP THREADPRIVATE(idx_obs_reg)
    INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:)      :: year_grdc_cut, month_grdc_cut, day_grdc_cut
!$OMP THREADPRIVATE(year_grdc_cut, month_grdc_cut, day_grdc_cut)
    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)         :: sec_grdc_cut, jds_grdc_cut
!$OMP THREADPRIVATE(sec_grdc_cut, jds_grdc_cut)
    INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:)      :: idx_obs_reg_sort     !! Sorted (descend) GRDC index in the domain
!$OMP THREADPRIVATE(idx_obs_reg_sort)
    REAL(r_std), SAVE, ALLOCATABLE, DIMENSION(:)         :: area_obs_reg_sort    !! Sorted (descend) GRDC by areas
!$OMP THREADPRIVATE(area_obs_reg_sort)

!
!!! Pre-defined parameters
!
!  LOGICAL, SAVE :: diag_grdc = .TRUE.
!!$OMP THREADPRIVATE(diag_grdc)
  CHARACTER(LEN=80) :: grdc_file !! Name of the GRDC netcdf file (unitless)
!$OMP THREADPRIVATE(grdc_file)
  REAL(r_std), SAVE :: thres_dlola      !! Threshold for difference in Lat. and Lon. between ORC and GRDC
!$OMP THREADPRIVATE(thres_dlola)
  REAL(r_std), SAVE :: thres_dist       !! Threshold for difference in distance between ORC and GRDC (unit: m)
!$OMP THREADPRIVATE(thres_dist)
  REAL(r_std), SAVE :: thres_diff_area  !! Threshold for difference in upstream area between ORC and GRDC (unit: m²) 
!$OMP THREADPRIVATE(thres_diff_area)
!    LOGICAL, SAVE     :: ok_grdc = .TRUE. !! To determine if we output the
!    outlet at GRDC station (GRDC) or at the point with the largest upstream area
!!$OMP THREADPRIVATE(ok_grdc)
!    LOGICAL, SAVE     :: ok_largest = .TRUE.
!!$OMP THREADPRIVATE(ok_largest)
    INTEGER(i_std), SAVE :: year0_grdc = 1807
!$OMP THREADPRIVATE(year0_grdc)
    INTEGER(i_std), SAVE :: month0_grdc = 1
!$OMP THREADPRIVATE(month0_grdc)
    INTEGER(i_std), SAVE :: day0_grdc = 15
!$OMP THREADPRIVATE(day0_grdc)
    REAL(r_std), SAVE  :: sec0_grdc = 0
!$OMP THREADPRIVATE(sec0_grdc)

CONTAINS

  !! =============================================================================================================================
  !! SUBROUTINE: routing_reg_initialize
  !!
  !>\BRIEF Initialize the routing module
  !!
  !! DESCRIPTION: Initialize the routing module. Read from restart file or read the routing.nc file to initialize the
  !! routing scheme.
  !!
  !! RECENT CHANGE(S)
  !!
  !! REFERENCE(S)
  !!
  !! FLOWCHART
  !! \n
  !_ ==============================================================================================================================

  SUBROUTINE routing_reg_initialize( kjit, nbpt, index, &
                                rest_id, hist_id, hist2_id, lalo, &
                                neighbours, resolution, contfrac, stempdiag, &
                                returnflow, reinfiltration, irrigation, riverflow, &
                                coastalflow, flood_frac, flood_res, minimalarg )

    IMPLICIT NONE

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in) :: kjit !! Time step number (unitless)
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    INTEGER(i_std), INTENT(in) :: index(nbpt) !! Indices of the points on the map (unitless)
    INTEGER(i_std),INTENT(in) :: rest_id !! Restart file identifier (unitless)
    INTEGER(i_std),INTENT(in) :: hist_id !! Access to history file (unitless)
    INTEGER(i_std),INTENT(in) :: hist2_id !! Access to history file 2 (unitless)
    REAL(r_std), INTENT(in) :: lalo(nbpt,2) !! Vector of latitude and longitudes (beware of the order !)

    INTEGER(i_std), INTENT(in) :: neighbours(nbpt,NbNeighb) !! Vector of neighbours for each grid point
                                                           !! (1=N, 2=NE, 3=E, 4=SE, 5=S, 6=SW, 7=W, 8=NW) (unitless)
    REAL(r_std), INTENT(in) :: resolution(nbpt,2) !! The size of each grid box in X and Y (m)
    REAL(r_std), INTENT(in) :: contfrac(nbpt) !! Fraction of land in each grid box (unitless;0-1)
    REAL(r_std), INTENT(in) :: stempdiag(nbpt,nbdl) !! Diagnostic soil temperature profile
    LOGICAL,     OPTIONAL   :: minimalarg !! Allows to do a minimal initilisation of routing. Only the flow matrix will be set.
    
    !! 0.2 Output variables
    REAL(r_std), INTENT(out) :: returnflow(nbpt) !! The water flow from lakes and swamps which returns to the grid box.
                                                           !! This water will go back into the hydrol or hydrolc module to allow re-evaporation (kg/m^2/dt)
    REAL(r_std), INTENT(out) :: reinfiltration(nbpt) !! Water flow from ponds and floodplains which returns to the grid box (kg/m^2/dt)
    REAL(r_std), INTENT(out) :: irrigation(nbpt) !! Irrigation flux. This is the water taken from the reservoirs and beeing put into the upper layers of the soil (kg/m^2/dt)
    REAL(r_std), INTENT(out) :: riverflow(nbpt) !! Outflow of the major rivers. The flux will be located on the continental grid but this should be a coastal point (kg/dt)

    REAL(r_std), INTENT(out) :: coastalflow(nbpt) !! Outflow on coastal points by small basins. This is the water which flows in a disperse way into the ocean (kg/dt)
    REAL(r_std), INTENT(out) :: flood_frac(nbpt) !! Flooded fraction of the grid box (unitless;0-1)
    REAL(r_std), INTENT(out) :: flood_res(nbpt) !! Diagnostic of water amount in the floodplains reservoir (kg)

    !! 0.3 Local variables
    REAL(r_std), DIMENSION(nbp_glo):: mask_coast_glo       !! Mask with coastal gridcells on global grid (1/0)
    LOGICAL                        :: init_irrig !! Logical to initialize the irrigation (true/false)
    LOGICAL                        :: init_flood !! Logical to initialize the floodplains (true/false)
    LOGICAL                        :: init_swamp !! Logical to initialize the swamps (true/false)
    INTEGER                        :: ig, ib, rtg, rtb     !! Index
    INTEGER                        :: ier                  !! Error handeling
    LOGICAL                        :: minimal=.FALSE.
    !

!_ ================================================================================================================================
    !
    IF ( PRESENT(minimalarg)) THEN
       minimal=minimalarg
    ELSE
       minimal=.FALSE.
    ENDIF
    !
    !Config Key = ROUTING_NBASMAX
    !Config Desc = Nbasmax of the routing scheme
    !Config Def = 200
    !Config Help = This values gives the maximum number of basins we wish to have per grid box
    !Config Units = [-]
    !
    nbasmax = 200
    CALL getin_p('ROUTING_NBASMAX', nbasmax)
    !
    ! Here we will allocate the memory and get the fixed fields from the restart file.
    ! If the info is not found then we will compute the routing map.
    !
    IF ( .NOT. ALLOCATED(routing_area_glo) ) THEN
       CALL routing_reg_init (kjit, nbpt, index, returnflow, reinfiltration, irrigation, &
            riverflow, coastalflow, flood_frac, flood_res, stempdiag, rest_id, minimal)

       routing_area => routing_area_loc  
       topo_resid => topo_resid_loc
       route_togrid => route_togrid_loc
       route_tobasin => route_tobasin_loc
       global_basinid => global_basinid_loc
       hydrodiag => hydrodiag_loc
    ENDIF
    !
    ! This routine computes the routing map if the route_togrid_glo is undefined. This means that the
    ! map has not been initialized during the restart process..
    !
    !! Reads in the map of the basins and flow directions to construct the catchments of each grid box
    !
    IF ( COUNT(route_togrid_glo .GE. undef_int) .GT. 0 ) THEN
       CALL routing_reg_basins(nbpt, lalo, neighbours, resolution, contfrac)
    ENDIF
    !
    !! Create a mask containing all possible coastal gridcells and count total number of coastal gridcells
    IF (is_root_prc) THEN
       mask_coast_glo(:)=0
       DO ib=1,nbasmax
          DO ig=1,nbp_glo
             rtg = route_togrid_glo(ig,ib)
             rtb = route_tobasin_glo(ig,ib)
             ! Coastal gridcells are stored in nbasmax+2
             IF (rtb == nbasmax+2) THEN
                mask_coast_glo(rtg) = 1
             END IF
          END DO
       END DO
       nb_coast_gridcells=SUM(mask_coast_glo)
       IF (printlev>=3) WRITE(numout,*) 'Number of coastal gridcells = ', nb_coast_gridcells
    ENDIF
    CALL bcast(nb_coast_gridcells)
    !
    ! We have built the flow tree and can thus diagnose it.
    !
    ! This routine gives a diagnostic of the basins used and the rivers to be diagnosed.
    CALL routing_reg_diagnostic_p(kjit, nbpt, index, lalo, resolution, contfrac, hist_id, hist2_id, minimal)
    !
    IF ( .NOT. minimal ) THEN
       !
       ALLOCATE(mask_coast(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_inititalize','Pb in allocate for mask_coast','','')
       CALL scatter(mask_coast_glo, mask_coast)
       CALL xios_orchidee_send_field("mask_coast",mask_coast)
       !
       ! Do we have what we need if we want to do irrigation
       !! Initialisation of flags for irrigated land, flood plains and swamps
       !
       init_irrig = .FALSE.
       IF ( do_irrigation ) THEN
          IF (COUNT(irrigated .GE. undef_sechiba-1) > 0) init_irrig = .TRUE.
       END IF

       init_flood = .FALSE.
       IF ( do_floodplains ) THEN
          IF (COUNT(floodplains .GE. undef_sechiba-1) > 0) init_flood = .TRUE.
       END IF

       init_swamp = .FALSE.
       IF ( doswamps ) THEN
          IF (COUNT(swamp .GE. undef_sechiba-1) > 0 ) init_swamp = .TRUE.
       END IF

       !! If we have irrigated land, flood plains or swamps then we need to interpolate the 0.5 degree
       !! base data set to the resolution of the model.

       IF ( init_irrig .OR. init_flood .OR. init_swamp ) THEN
          CALL routing_reg_irrigmap(nbpt, index, lalo, neighbours, resolution, &
               contfrac, init_irrig, irrigated, init_flood, floodplains, init_swamp, swamp, hist_id, hist2_id)
       ENDIF

       IF ( do_irrigation ) THEN
          CALL xios_orchidee_send_field("irrigmap",irrigated)

          WRITE(numout,*) 'Verification : range of irrigated : ', MINVAL(irrigated), MAXVAL(irrigated)
          IF ( .NOT. almaoutput ) THEN
             CALL histwrite_p(hist_id, 'irrigmap', 1, irrigated, nbpt, index)
          ELSE
             CALL histwrite_p(hist_id, 'IrrigationMap', 1, irrigated, nbpt, index)
          ENDIF
          IF ( hist2_id > 0 ) THEN
             IF ( .NOT. almaoutput ) THEN
                CALL histwrite_p(hist2_id, 'irrigmap', 1, irrigated, nbpt, index)
             ELSE
                CALL histwrite_p(hist2_id, 'IrrigationMap', 1, irrigated, nbpt, index)
             ENDIF
          ENDIF
       ENDIF

       IF ( do_floodplains ) THEN
          CALL xios_orchidee_send_field("floodmap",floodplains)

          WRITE(numout,*) 'Verification : range of floodplains : ', MINVAL(floodplains), MAXVAL(floodplains)
          IF ( .NOT. almaoutput ) THEN
             CALL histwrite_p(hist_id, 'floodmap', 1, floodplains, nbpt, index)
          ELSE
             CALL histwrite_p(hist_id, 'FloodplainsMap', 1, floodplains, nbpt, index)
          ENDIF
          IF ( hist2_id > 0 ) THEN
             IF ( .NOT. almaoutput ) THEN
                CALL histwrite_p(hist2_id, 'floodmap', 1, floodplains, nbpt, index)
             ELSE
                CALL histwrite_p(hist2_id, 'FloodplainsMap', 1, floodplains, nbpt, index)
             ENDIF
          ENDIF
       ENDIF

       IF ( doswamps ) THEN
          CALL xios_orchidee_send_field("swampmap",swamp)

          WRITE(numout,*) 'Verification : range of swamp : ', MINVAL(swamp), MAXVAL(swamp)
          IF ( .NOT. almaoutput ) THEN
             CALL histwrite_p(hist_id, 'swampmap', 1, swamp, nbpt, index)
          ELSE
             CALL histwrite_p(hist_id, 'SwampMap', 1, swamp, nbpt, index)
          ENDIF
          IF ( hist2_id > 0 ) THEN
             IF ( .NOT. almaoutput ) THEN
                CALL histwrite_p(hist2_id, 'swampmap', 1, swamp, nbpt, index)
             ELSE
                CALL histwrite_p(hist2_id, 'SwampMap', 1, swamp, nbpt, index)
             ENDIF
          ENDIF
       ENDIF
       !
    ENDIF
    !
  END SUBROUTINE routing_reg_initialize

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_main
!!
!>\BRIEF This module routes the water over the continents (runoff and
!! drainage produced by the hydrolc or hydrol module) into the oceans.
!!
!! DESCRIPTION (definitions, functional, design, flags):
!! The routing scheme (Polcher, 2003) carries the water from the runoff and drainage simulated by SECHIBA
!! to the ocean through reservoirs, with some delay. The routing scheme is based on
!! a parametrization of the water flow on a global scale (Miller et al., 1994; Hagemann
!! and Dumenil, 1998). Given the global map of the main watersheds (Oki et al., 1999;
!! Fekete et al., 1999; Vorosmarty et al., 2000) which delineates the boundaries of subbasins
!! and gives the eight possible directions of water flow within the pixel, the surface
!! runoff and the deep drainage are routed to the ocean. The time-step of the routing is one hour.
!! The scheme also diagnoses how much water is retained in the foodplains and thus return to soil
!! moisture or is taken out of the rivers for irrigation. \n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!! The result of the routing are 3 fluxes :
!! - riverflow : The water which flows out from the major rivers. The flux will be located
!! on the continental grid but this should be a coastal point.
!! - coastalflow : This is the water which flows in a disperse way into the ocean. Essentially these
!! are the outflows from all of the small rivers.
!! - returnflow : This is the water which flows into a land-point - typically rivers which end in
!! the desert. This water will go back into the hydrol module to allow re-evaporation.
!! - irrigation : This is water taken from the reservoir and is being put into the upper
!! layers of the soil.
!! The two first fluxes are in kg/dt and the last two fluxes are in kg/(m^2dt).\n
!!
!! REFERENCE(S) :
!! - Miller JR, Russell GL, Caliri G (1994)
!! Continental-scale river flow in climate models.
!! J. Clim., 7:914-928
!! - Hagemann S and Dumenil L. (1998)
!! A parametrization of the lateral waterflow for the global scale.
!! Clim. Dyn., 14:17-31
!! - Oki, T., T. Nishimura, and P. Dirmeyer (1999)
!! Assessment of annual runoff from land surface models using total runoff integrating pathways (TRIP)
!! J. Meteorol. Soc. Jpn., 77, 235-255
!! - Fekete BM, Charles V, Grabs W (2000)
!! Global, composite runoff fields based on observed river discharge and simulated water balances.
!! Technical report, UNH/GRDC, Global Runoff Data Centre, Koblenz
!! - Vorosmarty, C., B. Fekete, B. Meybeck, and R. Lammers (2000)
!! Global system of rivers: Its role in organizing continental land mass and defining land-to-ocean linkages
!! Global Biogeochem. Cycles, 14, 599-621
!! - Vivant, A-C. (?? 2002)
!! Développement du schéma de routage et des plaines d'inondation, MSc Thesis, Paris VI University
!! - J. Polcher (2003)
!! Les processus de surface a l'echelle globale et leurs interactions avec l'atmosphere
!! Habilitation a diriger les recherches, Paris VI University, 67pp.
!!
!! FLOWCHART :
!! \latexonly
!! \includegraphics[scale=0.75]{routing_main_flowchart.png}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_main(kjit, nbpt, index, &
       & lalo, neighbours, resolution, contfrac, totfrac_nobio, veget_max, floodout, runoff, &
       & drainage, transpot, precip_rain, humrel, k_litt, flood_frac, flood_res, &
       & stempdiag, reinf_slope, returnflow, reinfiltration, irrigation, riverflow, coastalflow, rest_id, hist_id, hist2_id)

    IMPLICIT NONE

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in) :: kjit !! Time step number (unitless)
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    INTEGER(i_std),INTENT(in) :: rest_id !! Restart file identifier (unitless)
    INTEGER(i_std),INTENT(in) :: hist_id !! Access to history file (unitless)
    INTEGER(i_std),INTENT(in) :: hist2_id !! Access to history file 2 (unitless)
    INTEGER(i_std), INTENT(in) :: index(nbpt) !! Indices of the points on the map (unitless)
    REAL(r_std), INTENT(in) :: lalo(nbpt,2) !! Vector of latitude and longitudes (beware of the order !)
    INTEGER(i_std), INTENT(in) :: neighbours(nbpt,NbNeighb) !! Vector of neighbours for each grid point (1=N, 2=NE, 3=E, 4=SE, 5=S, 6=SW, 7=W, 8=NW) (unitless)
    REAL(r_std), INTENT(in) :: resolution(nbpt,2) !! The size of each grid box in X and Y (m)
    REAL(r_std), INTENT(in) :: contfrac(nbpt) !! Fraction of land in each grid box (unitless;0-1)
    REAL(r_std), INTENT(in) :: totfrac_nobio(nbpt) !! Total fraction of no-vegetation (continental ice, lakes ...) (unitless;0-1)
    REAL(r_std), INTENT(in) :: veget_max(nbpt,nvm) !! Maximal fraction of vegetation (unitless;0-1)
    REAL(r_std), INTENT(in) :: floodout(nbpt) !! Grid-point flow out of floodplains (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: runoff(nbpt) !! Grid-point runoff (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: drainage(nbpt) !! Grid-point drainage (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: transpot(nbpt,nvm) !! Potential transpiration of the vegetation (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: precip_rain(nbpt) !! Rainfall (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: k_litt(nbpt) !! Averaged conductivity for saturated infiltration in the 'litter' layer (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: humrel(nbpt,nvm) !! Soil moisture stress, root extraction potential (unitless)
    REAL(r_std), INTENT(in) :: stempdiag(nbpt,nbdl) !! Diagnostic soil temperature profile
    REAL(r_std), INTENT(in) :: reinf_slope(nbpt) !! Coefficient which determines the reinfiltration ratio in the grid box due to flat areas (unitless;0-1)

    !! 0.2 Output variables
    REAL(r_std), INTENT(out) :: returnflow(nbpt) !! The water flow from lakes and swamps which returns to the grid box.
                                                           !! This water will go back into the hydrol or hydrolc module to allow re-evaporation (kg/m^2/dt)
    REAL(r_std), INTENT(out) :: reinfiltration(nbpt) !! Water flow from ponds and floodplains which returns to the grid box (kg/m^2/dt)
    REAL(r_std), INTENT(out) :: irrigation(nbpt) !! Irrigation flux. This is the water taken from the reservoirs and beeing put into the upper layers of the soil (kg/m^2/dt)
    REAL(r_std), INTENT(out) :: riverflow(nbpt) !! Outflow of the major rivers. The flux will be located on the continental grid but this should be a coastal point (kg/dt)
    REAL(r_std), INTENT(out) :: coastalflow(nbpt) !! Outflow on coastal points by small basins. This is the water which flows in a disperse way into the ocean (kg/dt)
    REAL(r_std), INTENT(out) :: flood_frac(nbpt) !! Flooded fraction of the grid box (unitless;0-1)
    REAL(r_std), INTENT(out) :: flood_res(nbpt) !! Diagnostic of water amount in the floodplains reservoir (kg)

    !! 0.3 Local variables
    CHARACTER(LEN=30) :: var_name !! To store variables names for I/O (unitless)
    REAL(r_std), DIMENSION(1) :: tmp_day !!
    REAL(r_std), DIMENSION(nbpt) :: return_lakes !! Water from lakes flowing back into soil moisture (kg/m^2/dt)
    INTEGER(i_std) :: ig, jv !! Indices (unitless)
    REAL(r_std), DIMENSION(nbpt) :: tot_vegfrac_nowoody !! Total fraction occupied by grass (0-1,unitless)

    REAL(r_std), DIMENSION(nbpt)   :: fast_diag_old        !! Reservoir in the beginning of the time step
    REAL(r_std), DIMENSION(nbpt)   :: slow_diag_old        !! Reservoir in the beginning of the time step
    REAL(r_std), DIMENSION(nbpt)   :: stream_diag_old      !! Reservoir in the beginning of the time step
    REAL(r_std), DIMENSION(nbpt)   :: lake_diag_old        !! Reservoir in the beginning of the time step
    REAL(r_std), DIMENSION(nbpt)   :: pond_diag_old        !! Reservoir in the beginning of the time step
    REAL(r_std), DIMENSION(nbpt)   :: flood_diag_old       !! Reservoir in the beginning of the time step

    !! For water budget check in the three routing reservoirs (positive if input > output)
    !! Net fluxes averaged over each grid cell in kg/m^2/dt
    REAL(r_std), DIMENSION(nbpt)   :: netflow_stream_diag  !! Input - Output flow to stream reservoir
    REAL(r_std), DIMENSION(nbpt)   :: netflow_fast_diag    !! Input - Output flow to fast reservoir
    REAL(r_std), DIMENSION(nbpt)   :: netflow_slow_diag    !! Input - Output flow to slow reservoir
    
    INTEGER(i_std) :: i, igrdc, itmp
    INTEGER(i_std) :: ALLOC_ERR
        
!_ ================================================================================================================================
    !
    ! Save reservoirs in beginning of time step to calculate the water budget
    fast_diag_old   = fast_diag
    slow_diag_old   = slow_diag
    stream_diag_old = stream_diag
    lake_diag_old   = lake_diag
    pond_diag_old   = pond_diag
    flood_diag_old  = flood_diag
    !
    !! Computes the variables averaged between routing time steps and which will be used in subsequent calculations
    !
    floodout_mean(:) = floodout_mean(:) + floodout(:)
    runoff_mean(:) = runoff_mean(:) + runoff(:)
    drainage_mean(:) = drainage_mean(:) + drainage(:)
    floodtemp(:) = stempdiag(:,floodtemp_lev)
    precip_mean(:) = precip_mean(:) + precip_rain(:)
    !
    !! Computes the total fraction occupied by the grasses and the crops for each grid cell
    tot_vegfrac_nowoody(:) = zero
    DO jv = 1, nvm
       IF ( (jv /= ibare_sechiba) .AND. .NOT.(is_tree(jv)) ) THEN
          tot_vegfrac_nowoody(:) = tot_vegfrac_nowoody(:) + veget_max(:,jv)
       END IF
    END DO

    DO ig = 1, nbpt
       IF ( tot_vegfrac_nowoody(ig) .GT. min_sechiba ) THEN
          DO jv = 1,nvm
             IF ( (jv /= ibare_sechiba) .AND. .NOT.(is_tree(jv)) ) THEN
                transpot_mean(ig) = transpot_mean(ig) + transpot(ig,jv) * veget_max(ig,jv)/tot_vegfrac_nowoody(ig)
             END IF
          END DO
       ELSE
          IF (MAXVAL(veget_max(ig,2:nvm)) .GT. min_sechiba) THEN
             DO jv = 2, nvm
                transpot_mean(ig) = transpot_mean(ig) + transpot(ig,jv) * veget_max(ig,jv)/ SUM(veget_max(ig,2:nvm))
             ENDDO
          ENDIF
       ENDIF
    ENDDO

    !
    ! Averaged variables (i.e. *dt_sechiba/dt_routing). This accounts for the difference between the shorter
    ! timestep dt_sechiba of other parts of the model and the long dt_routing timestep (set to one hour at present)
    !
    totnobio_mean(:) = totnobio_mean(:) + totfrac_nobio(:)*dt_sechiba/dt_routing
    k_litt_mean(:) = k_litt_mean(:) + k_litt(:)*dt_sechiba/dt_routing
    !
    ! Only potentially vegetated surfaces are taken into account. At the start of
    ! the growing seasons we will give more weight to these areas.
    !
    DO jv=2,nvm
       DO ig=1,nbpt
          humrel_mean(ig) = humrel_mean(ig) + humrel(ig,jv)*veget_max(ig,jv)*dt_sechiba/dt_routing
          vegtot_mean(ig) = vegtot_mean(ig) + veget_max(ig,jv)*dt_sechiba/dt_routing
       ENDDO
    ENDDO
    !
    time_counter = time_counter + dt_sechiba
    !
    ! If the time has come we do the routing.
    !
    IF ( NINT(time_counter) .GE. NINT(dt_routing) ) THEN
       !
       ! Check the water balance if needed
       !
       IF ( check_waterbal ) THEN
          CALL routing_reg_waterbal(nbpt, .TRUE., floodout_mean, runoff_mean, drainage_mean, returnflow_mean, &
               & reinfiltration_mean, irrigation_mean, riverflow_mean, coastalflow_mean)
       ENDIF
       !
       ! Make sure we do not flood north of 49N as there freezing processes start to play a role and they
       ! are not yet well treated in ORCHIDEE.
       !
       DO ig=1,nbpt
          IF ( lalo(ig,1) > 49.0 ) THEN
             floodtemp(ig) = tp_00 - un
          ENDIF
       ENDDO
       !
       !! Computes the transport of water in the various reservoirs
       !
       CALL routing_reg_flow(nbpt, dt_routing, lalo, floodout_mean, runoff_mean, drainage_mean, &
            & vegtot_mean, totnobio_mean, transpot_mean, precip_mean, humrel_mean, k_litt_mean, floodtemp, reinf_slope, &
            & lakeinflow_mean, returnflow_mean, reinfiltration_mean, irrigation_mean, riverflow_mean, &
            & coastalflow_mean, hydrographs, slowflow_diag, flood_frac, flood_res, &
            & netflow_stream_diag, netflow_fast_diag, netflow_slow_diag)
       !
       !! Responsible for storing the water in lakes
       !
       CALL routing_reg_lake(nbpt, dt_routing, lakeinflow_mean, humrel_mean, return_lakes)
       !
       returnflow_mean(:) = returnflow_mean(:) + return_lakes(:)
       !
       !! Check the water balance in the routing scheme
       !
       IF ( check_waterbal ) THEN
          CALL routing_reg_waterbal(nbpt, .FALSE., floodout_mean, runoff_mean, drainage_mean, returnflow_mean, &
               & reinfiltration_mean, irrigation_mean, riverflow_mean, coastalflow_mean)
       ENDIF
       !
       time_counter = zero
       !
       floodout_mean(:) = zero
       runoff_mean(:) = zero
       drainage_mean(:) = zero
       transpot_mean(:) = zero
       precip_mean(:) = zero
       !
       humrel_mean(:) = zero
       totnobio_mean(:) = zero
       k_litt_mean(:) = zero
       vegtot_mean(:) = zero
       !
       ! Change the units of the routing fluxes from kg/dt_routing into kg/dt_sechiba
       hydrographs(:) = hydrographs(:)/dt_routing*dt_sechiba
       slowflow_diag(:) = slowflow_diag(:)/dt_routing*dt_sechiba
       outflow_loc(:) = outflow_loc(:)/dt_routing*dt_sechiba
       !
       IF(ok_grdc) THEN
           mergedhydro_grdc_match_loc(:,1) = mergedhydro_grdc_match_loc(:,1) &
                                     & / dt_routing*dt_sechiba/mille
           hydrographs_largest_loc(:) = hydrographs_largest_loc(:) &
                                     & / dt_routing*dt_sechiba/mille

           mergedhydro_grdc_match_loc(:,2) = -999.9
           DO i = 1, size(year_grdc_cut)
               IF(year_grdc_cut(i) == year .AND. month_grdc_cut(i) == month) THEN
               !WRITE(numout,*) 'year_grdc_cut, month_grdc_cut,day_grdc_cut, sec_grdc_cut=', &
               !    & year_grdc_cut(i), month_grdc_cut(i), day_grdc_cut(i), sec_grdc_cut(i)
                   IF(nb_grdc_match_loc .GT. 0) THEN
                       DO igrdc = 1, nb_grdc_match_loc
                       !mergedhydro_grdc_match_loc(igrdc,2) = mergedhydro_obs_reg(idx_grdc_match_loc(igrdc),i) 
                           IF(ogloc_grdc_match_loc(igrdc) .LE. nbpt .AND.  &
                             & ogloc_grdc_match_loc(igrdc) .GT. 0 .AND. &
                             & obloc_grdc_match_loc(igrdc) .LE. nbasmax .AND. &
                             & obloc_grdc_match_loc(igrdc) .GT. 0) THEN
                               IF(idx_mod_match_loc(ogloc_grdc_match_loc(igrdc),obloc_grdc_match_loc(igrdc)) .GT. 0 .AND. &
                                 & idx_mod_match_loc(ogloc_grdc_match_loc(igrdc),obloc_grdc_match_loc(igrdc)) .LE. igrdc_reg) THEN
                                   mergedhydro_grdc_match_loc(igrdc,2) = & 
                                     & mergedhydro_obs_reg(idx_mod_match_loc(ogloc_grdc_match_loc(igrdc),obloc_grdc_match_loc(igrdc)), i)
                               ENDIF
                           ENDIF
                           IF(IsNAN(mergedhydro_grdc_match_loc(igrdc,2))) mergedhydro_grdc_match_loc(igrdc,2) = -999.9
                       ENDDO ! DO igrdc = 1, nb_grdc_match_loc
                   ENDIF ! IF(nb_grdc_match_loc .GT. 0) THEN
               ENDIF
           ENDDO !DO i = 1, size(year_grdc_cut)

       ENDIF !ok_grdc

       !
       ! Change the units of the routing fluxes from kg/dt_routing into kg/dt_sechiba
       ! and from m^3/dt_routing into m^3/dt_sechiba
       !
       returnflow_mean(:) = returnflow_mean(:)/dt_routing*dt_sechiba
       reinfiltration_mean(:) = reinfiltration_mean(:)/dt_routing*dt_sechiba
       irrigation_mean(:) = irrigation_mean(:)/dt_routing*dt_sechiba
       irrig_netereq(:) = irrig_netereq(:)/dt_routing*dt_sechiba
       !
       ! Change units as above but at the same time transform the kg/dt_routing to m^3/dt_sechiba
       !
       riverflow_mean(:) = riverflow_mean(:)/dt_routing*dt_sechiba/mille
       coastalflow_mean(:) = coastalflow_mean(:)/dt_routing*dt_sechiba/mille
       !
       ! Water budget residu of the three routing reservoirs (in kg/m^2/s)
       ! Note that these diagnostics are done using local variables only calculated 
       ! during the time steps when the routing is calculated
       CALL xios_orchidee_send_field("wbr_stream",(stream_diag - stream_diag_old - netflow_stream_diag)/dt_routing)
       CALL xios_orchidee_send_field("wbr_fast",  (fast_diag   - fast_diag_old - netflow_fast_diag)/dt_routing)
       CALL xios_orchidee_send_field("wbr_slow",  (slow_diag   - slow_diag_old - netflow_slow_diag)/dt_routing)
       CALL xios_orchidee_send_field("wbr_lake",  (lake_diag   - lake_diag_old - &
                                                   lakeinflow_mean + return_lakes)/dt_routing)
       !
    ENDIF
    !
    ! Return the fraction of routed water for this time step.
    !
    returnflow(:) = returnflow_mean(:)
    reinfiltration(:) = reinfiltration_mean(:)
    irrigation(:) = irrigation_mean(:)
    riverflow(:) = riverflow_mean(:)
    coastalflow(:) = coastalflow_mean(:)

    !
    ! Write diagnostics
    !

    ! Water storage in reservoirs [kg/m^2]
    CALL xios_orchidee_send_field("fastr",fast_diag)
    CALL xios_orchidee_send_field("slowr",slow_diag)
    CALL xios_orchidee_send_field("streamr",stream_diag)
    CALL xios_orchidee_send_field("laker",lake_diag)
    CALL xios_orchidee_send_field("pondr",pond_diag)
    CALL xios_orchidee_send_field("floodr",flood_diag)
    CALL xios_orchidee_send_field("floodh",flood_height)

    ! Difference between the end and the beginning of the routing time step [kg/m^2]
    CALL xios_orchidee_send_field("delfastr",   fast_diag   - fast_diag_old)
    CALL xios_orchidee_send_field("delslowr",   slow_diag   - slow_diag_old)
    CALL xios_orchidee_send_field("delstreamr", stream_diag - stream_diag_old)
    CALL xios_orchidee_send_field("dellaker",   lake_diag   - lake_diag_old)
    CALL xios_orchidee_send_field("delpondr",   pond_diag   - pond_diag_old)
    CALL xios_orchidee_send_field("delfloodr",  flood_diag  - flood_diag_old)

    ! Water fluxes converted from kg/m^2/dt_sechiba into kg/m^2/s 
    CALL xios_orchidee_send_field("irrigation",irrigation/dt_sechiba)
    CALL xios_orchidee_send_field("netirrig",irrig_netereq/dt_sechiba)
    CALL xios_orchidee_send_field("riversret",returnflow/dt_sechiba)
    CALL xios_orchidee_send_field("reinfiltration",reinfiltration/dt_sechiba)

    ! Transform from kg/dt_sechiba into m^3/s
    CALL xios_orchidee_send_field("hydrographs",hydrographs/mille/dt_sechiba)
    CALL xios_orchidee_send_field("slowflow",slowflow_diag/mille/dt_sechiba) ! previous id name: Qb
    CALL xios_orchidee_send_field("coastalflow",coastalflow/dt_sechiba)
    CALL xios_orchidee_send_field("riverflow",riverflow/dt_sechiba)

    IF(ok_grdc .AND. xios_orchidee_ok) THEN
      !WRITE(numout, *) "mergedhydro_grdc_match_loc=", mergedhydro_grdc_match_loc/dt_sechiba
      !WRITE(numout, *) "hydrographs_largest_loc=", hydrographs_largest_loc/dt_sechiba
      CALL xios_send_field("Dis_Stn_GRDC",mergedhydro_grdc_match_loc(:,2))
      CALL xios_send_field("Dis_Stn_Model",mergedhydro_grdc_match_loc(:,1)/dt_sechiba)
      CALL xios_send_field("Dis_Outflow_Model",hydrographs_largest_loc/dt_sechiba)
    ENDIF
    IF ( .NOT. almaoutput ) THEN
       !
       CALL histwrite_p(hist_id, 'riversret', kjit, returnflow, nbpt, index)
       IF (do_floodplains .OR. doponds) THEN
          CALL histwrite_p(hist_id, 'reinfiltration', kjit, reinfiltration, nbpt, index)
       ENDIF
       CALL histwrite_p(hist_id, 'hydrographs', kjit, hydrographs/mille, nbpt, index)
       !
       CALL histwrite_p(hist_id, 'fastr', kjit, fast_diag, nbpt, index)
       CALL histwrite_p(hist_id, 'slowr', kjit, slow_diag, nbpt, index)
       CALL histwrite_p(hist_id, 'streamr', kjit, stream_diag, nbpt, index)
       IF ( do_floodplains ) THEN
          CALL histwrite_p(hist_id, 'floodr', kjit, flood_diag, nbpt, index)
          CALL histwrite_p(hist_id, 'floodh', kjit, flood_height, nbpt, index)
       ENDIF
       CALL histwrite_p(hist_id, 'pondr', kjit, pond_diag, nbpt, index)
       CALL histwrite_p(hist_id, 'lakevol', kjit, lake_diag, nbpt, index)
       !
       IF ( do_irrigation ) THEN
          CALL histwrite_p(hist_id, 'irrigation', kjit, irrigation, nbpt, index)
          CALL histwrite_p(hist_id, 'returnflow', kjit, returnflow, nbpt, index)
          CALL histwrite_p(hist_id, 'netirrig', kjit, irrig_netereq, nbpt, index)
       ENDIF
       !
    ELSE
       !
       CALL histwrite_p(hist_id, 'SurfStor', kjit, flood_diag+pond_diag+lake_diag, nbpt, index)
       CALL histwrite_p(hist_id, 'Dis', kjit, hydrographs/mille, nbpt, index)
       IF ( do_irrigation ) THEN
          CALL histwrite_p(hist_id, 'Qirrig', kjit, irrigation, nbpt, index)
          CALL histwrite_p(hist_id, 'Qirrig_req', kjit, irrig_netereq, nbpt, index)
       ENDIF
       !
    ENDIF
    IF ( hist2_id > 0 ) THEN
       IF ( .NOT. almaoutput ) THEN
          !
          CALL histwrite_p(hist2_id, 'riversret', kjit, returnflow, nbpt, index)
          IF (do_floodplains .OR. doponds) THEN
             CALL histwrite_p(hist2_id, 'reinfiltration', kjit, reinfiltration, nbpt, index)
          ENDIF
          CALL histwrite_p(hist2_id, 'hydrographs', kjit, hydrographs/mille, nbpt, index)
          !
          CALL histwrite_p(hist2_id, 'fastr', kjit, fast_diag, nbpt, index)
          CALL histwrite_p(hist2_id, 'slowr', kjit, slow_diag, nbpt, index)
          IF ( do_floodplains ) THEN
             CALL histwrite_p(hist2_id, 'floodr', kjit, flood_diag, nbpt, index)
             CALL histwrite_p(hist2_id, 'floodh', kjit, flood_height, nbpt, index)
          ENDIF
          CALL histwrite_p(hist2_id, 'pondr', kjit, pond_diag, nbpt, index)
          CALL histwrite_p(hist2_id, 'streamr', kjit, stream_diag, nbpt, index)
          CALL histwrite_p(hist2_id, 'lakevol', kjit, lake_diag, nbpt, index)
          !
          IF ( do_irrigation ) THEN
             CALL histwrite_p(hist2_id, 'irrigation', kjit, irrigation, nbpt, index)
             CALL histwrite_p(hist2_id, 'returnflow', kjit, returnflow, nbpt, index)
             CALL histwrite_p(hist2_id, 'netirrig', kjit, irrig_netereq, nbpt, index)
          ENDIF
          !
       ELSE
          !
          CALL histwrite_p(hist2_id, 'SurfStor', kjit, flood_diag+pond_diag+lake_diag, nbpt, index)
          CALL histwrite_p(hist2_id, 'Dis', kjit, hydrographs/mille, nbpt, index)
          !
       ENDIF
    ENDIF
    !
    !
  END SUBROUTINE routing_reg_main

  !! =============================================================================================================================
  !! SUBROUTINE: routing_reg_finalize
  !!
  !>\BRIEF Write to restart file
  !!
  !! DESCRIPTION: Write module variables to restart file
  !!
  !! RECENT CHANGE(S)
  !!
  !! REFERENCE(S)
  !!
  !! FLOWCHART
  !! \n
  !_ ==============================================================================================================================

  SUBROUTINE routing_reg_finalize( kjit, nbpt, rest_id, flood_frac, flood_res )

    IMPLICIT NONE

    !! 0.1 Input variables
    INTEGER(i_std), INTENT(in) :: kjit !! Time step number (unitless)
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    INTEGER(i_std),INTENT(in) :: rest_id !! Restart file identifier (unitless)
    REAL(r_std), INTENT(in) :: flood_frac(nbpt) !! Flooded fraction of the grid box (unitless;0-1)
    REAL(r_std), INTENT(in) :: flood_res(nbpt) !! Diagnostic of water amount in the floodplains reservoir (kg)

    !! 0.2 Local variables
    REAL(r_std), DIMENSION(1) :: tmp_day

!_ ================================================================================================================================

    !
    ! Write restart variables
    !
    tmp_day(1) = time_counter
    IF (is_root_prc) CALL restput (rest_id, 'routingcounter', 1, 1, 1, kjit, tmp_day)

    CALL restput_p (rest_id, 'routingarea', nbp_glo, nbasmax, 1, kjit, routing_area_loc, 'scatter', nbp_glo, index_g)
    !
    CALL restput_p (rest_id, 'routetogrid', nbp_glo, nbasmax, 1, kjit, REAL(route_togrid_loc,r_std), 'scatter', &
         nbp_glo, index_g)
    CALL restput_p (rest_id, 'routetobasin', nbp_glo, nbasmax, 1, kjit, REAL(route_tobasin_loc,r_std), 'scatter', &
         nbp_glo, index_g)
    CALL restput_p (rest_id, 'basinid', nbp_glo, nbasmax, 1, kjit, REAL(global_basinid_loc,r_std), 'scatter', &
         nbp_glo, index_g)
    !
    CALL restput_p (rest_id, 'routenbintobas', nbp_glo, 1, 1, kjit, REAL(route_nbintobas_loc,r_std), 'scatter', &
         nbp_glo, index_g)
    CALL restput_p (rest_id, 'originnbintobas', nbp_glo, 1, 1, kjit, REAL(origin_nbintobas_loc,r_std), 'scatter', &
         nbp_glo, index_g)
    CALL restput_p (rest_id, 'outletlat', nbp_glo, nbasmax, 1, kjit, REAL(route_outlet_loc(:,:,1),r_std), 'scatter', &
         nbp_glo, index_g)
    CALL restput_p (rest_id, 'outletlon', nbp_glo, nbasmax, 1, kjit, REAL(route_outlet_loc(:,:,2),r_std), 'scatter', &
         nbp_glo, index_g)
    CALL restput_p (rest_id, 'outlettype', nbp_glo, nbasmax, 1, kjit, REAL(route_type_loc(:,:),r_std), 'scatter', &
         nbp_glo, index_g)
    !
    CALL restput_p (rest_id, 'topoindex', nbp_glo, nbasmax, 1, kjit, topo_resid_loc, 'scatter', nbp_glo, index_g)
    !
    CALL restput_p (rest_id, 'fastres', nbp_glo, nbasmax, 1, kjit, fast_reservoir, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'slowres', nbp_glo, nbasmax, 1, kjit, slow_reservoir, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'streamres', nbp_glo, nbasmax, 1, kjit, stream_reservoir, 'scatter',nbp_glo,index_g)
    CALL restput_p (rest_id, 'floodres', nbp_glo, nbasmax, 1, kjit, flood_reservoir, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'floodh', nbp_glo, 1, 1, kjit, flood_height, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'flood_frac_bas', nbp_glo, nbasmax, 1, kjit, flood_frac_bas, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'pond_frac', nbp_glo, 1, 1, kjit, pond_frac, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'flood_frac', nbp_glo, 1, 1, kjit, flood_frac, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'flood_res', nbp_glo, 1, 1, kjit, flood_res, 'scatter', nbp_glo, index_g)

    CALL restput_p (rest_id, 'lakeres', nbp_glo, 1, 1, kjit, lake_reservoir, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'pondres', nbp_glo, 1, 1, kjit, pond_reservoir, 'scatter', nbp_glo, index_g)

    CALL restput_p (rest_id, 'lakeinflow', nbp_glo, 1, 1, kjit, lakeinflow_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'returnflow', nbp_glo, 1, 1, kjit, returnflow_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'reinfiltration', nbp_glo, 1, 1, kjit, reinfiltration_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'riverflow', nbp_glo, 1, 1, kjit, riverflow_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'coastalflow', nbp_glo, 1, 1, kjit, coastalflow_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'hydrographs', nbp_glo, 1, 1, kjit, hydrographs, 'scatter', nbp_glo, index_g)  
    !
    ! Keep track of the accumulated variables
    !
    CALL restput_p (rest_id, 'floodout_route', nbp_glo, 1, 1, kjit, floodout_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'runoff_route', nbp_glo, 1, 1, kjit, runoff_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'drainage_route', nbp_glo, 1, 1, kjit, drainage_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'transpot_route', nbp_glo, 1, 1, kjit, transpot_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'precip_route', nbp_glo, 1, 1, kjit, precip_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'humrel_route', nbp_glo, 1, 1, kjit, humrel_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'totnobio_route', nbp_glo, 1, 1, kjit, totnobio_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'k_litt_route', nbp_glo, 1, 1, kjit, k_litt_mean, 'scatter', nbp_glo, index_g)
    CALL restput_p (rest_id, 'vegtot_route', nbp_glo, 1, 1, kjit, vegtot_mean, 'scatter', nbp_glo, index_g)

    IF ( do_irrigation ) THEN
       CALL restput_p (rest_id, 'irrigated', nbp_glo, 1, 1, kjit, irrigated, 'scatter', nbp_glo, index_g)
       CALL restput_p (rest_id, 'irrigation', nbp_glo, 1, 1, kjit, irrigation_mean, 'scatter', nbp_glo, index_g)
    ENDIF

    IF ( do_floodplains ) THEN
       CALL restput_p (rest_id, 'floodplains', nbp_glo, 1, 1, kjit, floodplains, 'scatter', nbp_glo, index_g)
    ENDIF
    IF ( doswamps ) THEN
       CALL restput_p (rest_id, 'swamp', nbp_glo, 1, 1, kjit, swamp, 'scatter', nbp_glo, index_g)
    ENDIF

    !IF ( ok_grdc ) THEN
    !   CALL restput_p (rest_id, 'idx_mod_match_glo', nbp_glo, nbasmax, 1, kjit, REAL(idx_mod_match_glo), 'scatter', nbp_glo, index_g)
    !   CALL restput_p (rest_id, 'hydrodiag_grdc_glo', nbp_glo, nbasmax, 1, kjit, REAL(hydrodiag_grdc_glo), 'scatter', nbp_glo, index_g)
    !   CALL restput_p (rest_id, 'hydrodiag_largest_glo', nbp_glo, nbasmax, 1, kjit, REAL(hydrodiag_largest_glo), 'scatter', nbp_glo, index_g)
    !ENDIF

  END SUBROUTINE routing_reg_finalize

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_init
!!
!>\BRIEF This subroutine allocates the memory and get the fixed fields from the restart file.
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_init(kjit, nbpt, index, returnflow, reinfiltration, irrigation, &
       & riverflow, coastalflow, flood_frac, flood_res, stempdiag, rest_id, minimal)
    !
    IMPLICIT NONE
    !
    ! interface description
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: kjit !! Time step number (unitless)
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    INTEGER(i_std), DIMENSION (nbpt), INTENT(in) :: index !! Indices of the points on the map (unitless)
    REAL(r_std), DIMENSION(nbpt,nbdl),INTENT(in) :: stempdiag !! Temperature profile in soil
    INTEGER(i_std), INTENT(in) :: rest_id !! Restart file identifier (unitless)
    LOGICAL, INTENT(in) :: minimal
    !
!! OUTPUT VARIABLES
    REAL(r_std), DIMENSION (nbpt),INTENT(out) :: returnflow !! The water flow from lakes and swamps which returns into the grid box.
                                                                   !! This water will go back into the hydrol or hydrolc module to allow re-evaporation (kg/m^2/dt)
    REAL(r_std), DIMENSION (nbpt),INTENT(out) :: reinfiltration !! Water flow from ponds and floodplains which returns to the grid box (kg/m^2/dt)
    REAL(r_std), DIMENSION (nbpt),INTENT(out) :: irrigation !! Irrigation flux. This is the water taken from the reservoirs and beeing put into the upper layers of the soil.(kg/m^2/dt)
    REAL(r_std), DIMENSION (nbpt),INTENT(out) :: riverflow !! Outflow of the major rivers. The flux will be located on the continental grid but this should be a coastal point (kg/dt)
    REAL(r_std), DIMENSION (nbpt),INTENT(out) :: coastalflow !! Outflow on coastal points by small basins. This is the water which flows in a disperse way into the ocean (kg/dt)
    REAL(r_std), DIMENSION (nbpt),INTENT(out) :: flood_frac !! Flooded fraction of the grid box (unitless;0-1)
    REAL(r_std), DIMENSION (nbpt),INTENT(out) :: flood_res !! Diagnostic of water amount in the floodplains reservoir (kg)
    !
!! LOCAL VARIABLES
    CHARACTER(LEN=80) :: var_name !! To store variables names for I/O (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: tmp_real_g !! A temporary real array for the integers
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: tmp_real_g_1d !! A temporary real array for the integers with 1 dimension
    REAL(r_std), DIMENSION(1) :: tmp_day !!
    REAL(r_std) :: ratio !! Diagnostic ratio to check that dt_routing is a multiple of dt_sechiba (unitless)
    REAL(r_std) :: totarea !! Total area of basin (m^2)
    INTEGER(i_std) :: ier, ig, in, ib, ipn(1) !! Indices (unitless)

!_ ================================================================================================================================
    !
    !
    ! These variables will require the configuration infrastructure
    !
    !Config Key = DT_ROUTING
    !Config If = RIVER_ROUTING
    !Config Desc = Time step of the routing scheme
    !Config Def = 3600.
    !Config Help = This values gives the time step in seconds of the routing scheme.
    !Config It should be multiple of the main time step of ORCHIDEE. One hour
    !Config is a good value.
    !Config Units = [seconds]
    !
    dt_routing = 3600.
    CALL getin_p('DT_ROUTING', dt_routing)
    !
    !Config Key = ROUTING_RIVERS
    !Config If = RIVER_ROUTING
    !Config Desc = Number of rivers
    !Config Def = 50
    !Config Help = This parameter chooses the number of largest river basins
    !Config which should be treated as independently as rivers and not
    !Config flow into the oceans as diffusion coastal flow.
    !Config Units = [-]
    num_largest = 50
    CALL getin_p('ROUTING_RIVERS', num_largest)
    !
    !Config Key = DO_FLOODINFILT
    !Config Desc = Should floodplains reinfiltrate into the soil
    !Config If = RIVER_ROUTING
    !Config Def = n
    !Config Help = This parameters allows the user to ask the model
    !Config to take into account the flood plains reinfiltration
    !Config into the soil moisture. It then can go
    !Config back to the slow and fast reservoirs
    !Config Units = [FLAG]
    !
    dofloodinfilt = .FALSE.
    CALL getin_p('DO_FLOODINFILT', dofloodinfilt)
    !
    !Config Key = DO_SWAMPS
    !Config Desc = Should we include swamp parameterization
    !Config If = RIVER_ROUTING
    !Config Def = n
    !Config Help = This parameters allows the user to ask the model
    !Config to take into account the swamps and return
    !Config the water into the bottom of the soil. It then can go
    !Config back to the atmopshere. This tried to simulate
    !Config internal deltas of rivers.
    !Config Units = [FLAG]
    !
    doswamps = .FALSE.
    CALL getin_p('DO_SWAMPS', doswamps)
    !
    !Config Key = DO_PONDS
    !Config Desc = Should we include ponds
    !Config If = RIVER_ROUTING
    !Config Def = n
    !Config Help = This parameters allows the user to ask the model
    !Config to take into account the ponds and return
    !Config the water into the soil moisture. It then can go
    !Config back to the atmopshere. This tried to simulate
    !Config little ponds especially in West Africa.
    !Config Units = [FLAG]
    !
    doponds = .FALSE.
    CALL getin_p('DO_PONDS', doponds)
    !
    ! Fix the time constants according to hydrol_cwrr flag
    !
    !
    !Config Key = SLOW_TCST
    !Config Desc = Time constant for the slow reservoir
    !Config If = RIVER_ROUTING
    !Config Def = n
    !Config Help = This parameters allows the user to fix the
    !Config time constant (in days) of the slow reservoir
    !Config in order to get better river flows for
    !Config particular regions.
    !Config Units = [days]
    !
!> A value for property of each reservoir (in day/m) is given to compute a time constant (in day)
!> for each reservoir (product of tcst and topo_resid).
!> The value of tcst has been calibrated for the three reservoirs over the Senegal river basin only,
!> during the 1 degree NCEP Corrected by Cru (NCC) resolution simulations (Ngo-Duc et al., 2005, Ngo-Duc et al., 2006) and
!> generalized for all the basins of the world. The "slow reservoir" and the "fast reservoir"
!> have the highest value in order to simulate the groundwater.
!> The "stream reservoir", which represents all the water of the stream, has the lowest value.
!> Those figures are the same for all the basins of the world.
!> The value of slow_tcst is equal to fast_tcst when CWRR is activated.
!> This assumption should be re-discussed.
    !
    IF ( hydrol_cwrr ) THEN
       slow_tcst = slow_tcst_cwrr
    ELSE
       slow_tcst = slow_tcst_chois
    ENDIF
    CALL getin_p('SLOW_TCST', slow_tcst)
    !
    !Config Key = FAST_TCST
    !Config Desc = Time constant for the fast reservoir
    !Config If = RIVER_ROUTING
    !Config Def = fast_tcst_cwrr or fast_tcst_chois depending on flag HYDROL_CWRR
    !Config Help = This parameters allows the user to fix the
    !Config time constant (in days) of the fast reservoir
    !Config in order to get better river flows for
    !Config particular regions.
    !Config Units = [days]
    !
    IF ( hydrol_cwrr ) THEN
       fast_tcst = fast_tcst_cwrr
    ELSE
       fast_tcst = fast_tcst_chois
    ENDIF
    CALL getin_p('FAST_TCST', fast_tcst)
    !
    !Config Key = STREAM_TCST
    !Config Desc = Time constant for the stream reservoir
    !Config If = RIVER_ROUTING
    !Config Def = stream_tcst_cwrr or stream_tcst_chois depending on flag HYDROL_CWRR
    !Config Help = This parameters allows the user to fix the
    !Config time constant (in days) of the stream reservoir
    !Config in order to get better river flows for
    !Config particular regions.
    !Config Units = [days]
    !
    IF ( hydrol_cwrr ) THEN
       stream_tcst = stream_tcst_cwrr
    ELSE
       stream_tcst = stream_tcst_chois
    ENDIF
    CALL getin_p('STREAM_TCST', stream_tcst)
    !
    !Config Key = FLOOD_TCST
    !Config Desc = Time constant for the flood reservoir
    !Config If = RIVER_ROUTING
    !Config Def = 4.0
    !Config Help = This parameters allows the user to fix the
    !Config time constant (in days) of the flood reservoir
    !Config in order to get better river flows for
    !Config particular regions.
    !Config Units = [days]
    !
    IF ( hydrol_cwrr ) THEN
       flood_tcst = flood_tcst_cwrr
    ELSE
       flood_tcst = flood_tcst_chois
    ENDIF
    CALL getin_p('FLOOD_TCST', flood_tcst)
    !
    !Config Key = SWAMP_CST
    !Config Desc = Fraction of the river that flows back to swamps
    !Config If = RIVER_ROUTING
    !Config Def = 0.2
    !Config Help = This parameters allows the user to fix the
    !Config fraction of the river transport
    !Config that flows to swamps
    !Config Units = [-]
    !
    IF ( hydrol_cwrr ) THEN
       swamp_cst = swamp_cst_cwrr
    ELSE
       swamp_cst = swamp_cst_chois
    ENDIF
    CALL getin_p('SWAMP_CST', swamp_cst)
    !
    !Config Key = FLOOD_BETA
    !Config Desc = Parameter to fix the shape of the floodplain
    !Config If = RIVER_ROUTING
    !Config Def = 2.0
    !Config Help = Parameter to fix the shape of the floodplain
    !Config (>1 for convex edges, <1 for concave edges)
    !Config Units = [-]
    CALL getin_p("FLOOD_BETA", beta)
    !
    !Config Key = POND_BETAP
    !Config Desc = Ratio of the basin surface intercepted by ponds and the maximum surface of ponds
    !Config If = RIVER_ROUTING
    !Config Def = 0.5
    !Config Help =
    !Config Units = [-]
    CALL getin_p("POND_BETAP", betap)
    !
    !Config Key = FLOOD_CRI
    !Config Desc = Potential height for which all the basin is flooded
    !Config If = DO_FLOODPLAINS or DO_PONDS
    !Config Def = 2000.
    !Config Help =
    !Config Units = [mm]
    CALL getin_p("FLOOD_CRI", floodcri)
    !
    !Config Key = POND_CRI
    !Config Desc = Potential height for which all the basin is a pond
    !Config If = DO_FLOODPLAINS or DO_PONDS
    !Config Def = 2000.
    !Config Help =
    !Config Units = [mm]
    CALL getin_p("POND_CRI", pondcri)
    !
    !Config Key   = MAX_LAKE_RESERVOIR
    !Config Desc  = Maximum limit of water in lake_reservoir
    !Config If    = RIVER_ROUTING
    !Config Def   = 7000
    !Config Help  = 
    !Config Units = [kg/m2(routing area)] 
    max_lake_reservoir = 7000
    CALL getin_p("MAX_LAKE_RESERVOIR", max_lake_reservoir)
    !
    !
    ! In order to simplify the time cascade check that dt_routing
    ! is a multiple of dt_sechiba
    !
    ratio = dt_routing/dt_sechiba
    IF ( ABS(NINT(ratio) - ratio) .GT. 10*EPSILON(ratio)) THEN
       WRITE(numout,*) 'WARNING -- WARNING -- WARNING -- WARNING'
       WRITE(numout,*) "The chosen time step for the routing is not a multiple of the"
       WRITE(numout,*) "main time step of the model. We will change dt_routing so that"
       WRITE(numout,*) "this condition os fulfilled"
       dt_routing = NINT(ratio) * dt_sechiba
       WRITE(numout,*) 'THE NEW DT_ROUTING IS : ', dt_routing
    ENDIF
    !
    IF ( dt_routing .LT. dt_sechiba) THEN
       WRITE(numout,*) 'WARNING -- WARNING -- WARNING -- WARNING'
       WRITE(numout,*) 'The routing timestep can not be smaller than the one'
       WRITE(numout,*) 'of the model. We reset its value to the model''s timestep.'
       WRITE(numout,*) 'The old DT_ROUTING is : ', dt_routing
       dt_routing = dt_sechiba
       WRITE(numout,*) 'THE NEW DT_ROUTING IS : ', dt_routing
    ENDIF
    !
    var_name ="routingcounter"
    IF (is_root_prc) THEN
       CALL ioconf_setatt('UNITS', 's')
       CALL ioconf_setatt('LONG_NAME','Time counter for the routing scheme')
       CALL restget (rest_id, var_name, 1, 1, 1, kjit, .TRUE., tmp_day)
       IF (tmp_day(1) == val_exp) THEN
          time_counter = zero
       ELSE
          time_counter = tmp_day(1)
       ENDIF
       CALL setvar (time_counter, val_exp, 'NO_KEYWORD', zero)
    ENDIF
    CALL bcast(time_counter)
!!$ CALL setvar_p (time_counter, val_exp, 'NO_KEYWORD', zero)


    ALLOCATE (routing_area_loc(nbpt,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for routing_area_loc','','')
    ALLOCATE (routing_area_glo(nbp_glo,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for routing_area_glo','','')

    var_name = 'routingarea'
    IF (is_root_prc) THEN
       CALL ioconf_setatt('UNITS', 'm^2')
       CALL ioconf_setatt('LONG_NAME','Area of basin')
       CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., routing_area_glo, "gather", nbp_glo, index_g)
    ENDIF
    CALL scatter(routing_area_glo,routing_area_loc)
    routing_area=>routing_area_loc
    !
    ALLOCATE (tmp_real_g(nbp_glo,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for tmp_real_g','','')
    ALLOCATE (tmp_real_g_1d(nbp_glo), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for tmp_real_g_1d','','')

    ALLOCATE (route_togrid_loc(nbpt,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for route_togrid_loc','','')
    ALLOCATE (route_togrid_glo(nbp_glo,nbasmax), stat=ier) ! used in global in routing_reg_flow
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for route_togrid_glo','','')

    IF (is_root_prc) THEN
       var_name = 'routetogrid'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','Grid into which the basin flows')
       CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., tmp_real_g, "gather", nbp_glo, index_g)
       route_togrid_glo(:,:) = undef_int
       WHERE ( tmp_real_g .LT. val_exp )
          route_togrid_glo = NINT(tmp_real_g)
    ENDWHERE
    ENDIF
    CALL bcast(route_togrid_glo) ! used in global in routing_flow
    CALL scatter(route_togrid_glo,route_togrid_loc)
    route_togrid=>route_togrid_loc
    !
    ALLOCATE (route_tobasin_loc(nbpt,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for route_tobasin_loc','','')

    ALLOCATE (route_tobasin_glo(nbp_glo,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for route_tobasin_glo','','')

    IF (is_root_prc) THEN
       var_name = 'routetobasin'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','Basin in to which the water goes')
       CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., tmp_real_g, "gather", nbp_glo, index_g)
       route_tobasin_glo = undef_int
       WHERE ( tmp_real_g .LT. val_exp )
         route_tobasin_glo = NINT(tmp_real_g)
      ENDWHERE
    ENDIF
    CALL scatter(route_tobasin_glo,route_tobasin_loc)
    route_tobasin=>route_tobasin_loc
    !
    ! nbintobasin
    !
    ALLOCATE (route_nbintobas_loc(nbpt), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for route_nbintobas_loc','','')
    ALLOCATE (route_nbintobas_glo(nbp_glo), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for route_nbintobas_glo','','')

    IF (is_root_prc) THEN
       var_name = 'routenbintobas'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','Number of basin into current one')
       CALL restget (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., tmp_real_g_1d, "gather", nbp_glo, index_g)
       route_nbintobas_glo = undef_int
       WHERE ( tmp_real_g_1d .LT. val_exp )
         route_nbintobas_glo = NINT(tmp_real_g_1d)
      ENDWHERE
    ENDIF
    CALL scatter(route_nbintobas_glo,route_nbintobas_loc)
    route_nbintobas=>route_nbintobas_loc
    !
    ALLOCATE (origin_nbintobas_loc(nbpt), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for origin_nbintobas_loc','','')
    ALLOCATE (origin_nbintobas_glo(nbp_glo), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for origin_nbintobas_glo','','')

    IF (is_root_prc) THEN
       var_name = 'originnbintobas'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','Number of subgrid basin into current one before truncation')
       CALL restget (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., tmp_real_g_1d, "gather", nbp_glo, index_g)
       origin_nbintobas_glo = undef_int
       WHERE ( tmp_real_g_1d .LT. val_exp )
         origin_nbintobas_glo = NINT(tmp_real_g_1d)
      ENDWHERE
    ENDIF
    CALL scatter(origin_nbintobas_glo,origin_nbintobas_loc)
    !
    ALLOCATE (global_basinid_loc(nbpt,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for global_basinid_loc','','')
    ALLOCATE (global_basinid_glo(nbp_glo,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for global_basinid_glo','','')

    IF (is_root_prc) THEN
       var_name = 'basinid'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','ID of basin')
       CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., tmp_real_g, "gather", nbp_glo, index_g)
       global_basinid_glo = undef_int
       WHERE ( tmp_real_g .LT. val_exp )
          global_basinid_glo = NINT(tmp_real_g)
       ENDWHERE
    ENDIF
    CALL scatter(global_basinid_glo,global_basinid_loc)
    global_basinid=>global_basinid_loc
    !
    ALLOCATE (route_outlet_loc(nbpt,nbasmax,2), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for route_outlet_loc','','')
    ALLOCATE (route_outlet_glo(nbp_glo,nbasmax,2), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for route_outlet_glo','','')

    IF (is_root_prc) THEN
       var_name = 'outletlat'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','Latitude of outlet')
       CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., tmp_real_g, "gather", nbp_glo, index_g)
       route_outlet_glo(:,:,1) = undef_int
       WHERE ( tmp_real_g .LT. val_exp )
          route_outlet_glo(:,:,1) = tmp_real_g
       ENDWHERE
       !
       var_name = 'outletlon'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','Longitude of outlet')
       CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., tmp_real_g, "gather", nbp_glo, index_g)
       route_outlet_glo(:,:,2) = undef_int
       WHERE ( tmp_real_g .LT. val_exp )
          route_outlet_glo(:,:,2) = tmp_real_g
       ENDWHERE
    ENDIF
    CALL scatter(route_outlet_glo,route_outlet_loc)
    !
    ALLOCATE (route_type_loc(nbpt,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for route_type_loc','','')
    ALLOCATE (route_type_glo(nbp_glo,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for route_type_glo','','')
 
   IF (is_root_prc) THEN
       var_name = 'outlettype'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','Type of outlet')
       CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., tmp_real_g, "gather", nbp_glo, index_g)
       route_type_glo(:,:) = undef_int
       WHERE ( tmp_real_g .LT. val_exp )
          route_type_glo(:,:) = tmp_real_g
       ENDWHERE
       !
    ENDIF
    CALL scatter(route_type_glo,route_type_loc)
    !
    ALLOCATE (topo_resid_loc(nbpt,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for topo_resid_loc','','')
    ALLOCATE (topo_resid_glo(nbp_glo,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for topo_resid_glo','','')

    IF (is_root_prc) THEN
       var_name = 'topoindex'
       CALL ioconf_setatt('UNITS', 'm')
       CALL ioconf_setatt('LONG_NAME','Topographic index of the residence time')
       CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., topo_resid_glo, "gather", nbp_glo, index_g)
    ENDIF
    CALL scatter(topo_resid_glo,topo_resid_loc)
    topo_resid=>topo_resid_loc
    !
    IF ( .NOT. minimal ) THEN
       !
       ALLOCATE (fast_reservoir(nbpt,nbasmax), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for fast_reservoir','','')
       var_name = 'fastres'
       CALL ioconf_setatt_p('UNITS', 'Kg')
       CALL ioconf_setatt_p('LONG_NAME','Water in the fast reservoir')
       CALL restget_p (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., fast_reservoir, "gather", nbp_glo, index_g)
       CALL setvar_p (fast_reservoir, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE (slow_reservoir(nbpt,nbasmax), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for slow_reservoir','','')
       var_name = 'slowres'
       CALL ioconf_setatt_p('UNITS', 'Kg')
       CALL ioconf_setatt_p('LONG_NAME','Water in the slow reservoir')
       CALL restget_p (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., slow_reservoir, "gather", nbp_glo, index_g)
       CALL setvar_p (slow_reservoir, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE (stream_reservoir(nbpt,nbasmax), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for stream_reservoir','','')
       var_name = 'streamres'
       CALL ioconf_setatt_p('UNITS', 'Kg')
       CALL ioconf_setatt_p('LONG_NAME','Water in the stream reservoir')
       CALL restget_p (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., stream_reservoir, "gather", nbp_glo, index_g)
       CALL setvar_p (stream_reservoir, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE (flood_reservoir(nbpt,nbasmax), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for flood_reservoir','','')
       var_name = 'floodres'
       CALL ioconf_setatt_p('UNITS', 'Kg')
       CALL ioconf_setatt_p('LONG_NAME','Water in the flood reservoir')
       CALL restget_p (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., flood_reservoir, "gather", nbp_glo, index_g)
       CALL setvar_p (flood_reservoir, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE (flood_frac_bas(nbpt,nbasmax), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for flood_frac_bas','','')
       var_name = 'flood_frac_bas'
       CALL ioconf_setatt_p('UNITS', '-')
       CALL ioconf_setatt_p('LONG_NAME','Flooded fraction per basin')
       CALL restget_p (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., flood_frac_bas, "gather", nbp_glo, index_g)
       CALL setvar_p (flood_frac_bas, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE (flood_height(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for flood_height','','')
       var_name = 'floodh'
       CALL ioconf_setatt_p('UNITS', '-')
       CALL ioconf_setatt_p('LONG_NAME','')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., flood_height, "gather", nbp_glo, index_g)
       CALL setvar_p (flood_height, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE (pond_frac(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for pond_frac','','')
       var_name = 'pond_frac'
       CALL ioconf_setatt_p('UNITS', '-')
       CALL ioconf_setatt_p('LONG_NAME','Pond fraction per grid box')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., pond_frac, "gather", nbp_glo, index_g)
       CALL setvar_p (pond_frac, val_exp, 'NO_KEYWORD', zero)

       var_name = 'flood_frac'
       CALL ioconf_setatt_p('UNITS', '-')
       CALL ioconf_setatt_p('LONG_NAME','Flooded fraction per grid box')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., flood_frac, "gather", nbp_glo, index_g)
       CALL setvar_p (flood_frac, val_exp, 'NO_KEYWORD', zero)

       var_name = 'flood_res'
       CALL ioconf_setatt_p('UNITS','mm')
       CALL ioconf_setatt_p('LONG_NAME','Flooded quantity (estimation)')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., flood_res, "gather", nbp_glo, index_g)
       CALL setvar_p (flood_res, val_exp, 'NO_KEYWORD', zero)
       ! flood_res = zero

       ALLOCATE (lake_reservoir(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for lake_reservoir','','')
       var_name = 'lakeres'
       CALL ioconf_setatt_p('UNITS', 'Kg')
       CALL ioconf_setatt_p('LONG_NAME','Water in the lake reservoir')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., lake_reservoir, "gather", nbp_glo, index_g)
       CALL setvar_p (lake_reservoir, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE (pond_reservoir(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for pond_reservoir','','')
       var_name = 'pondres'
       CALL ioconf_setatt_p('UNITS', 'Kg')
       CALL ioconf_setatt_p('LONG_NAME','Water in the pond reservoir')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., pond_reservoir, "gather", nbp_glo, index_g)
       CALL setvar_p (pond_reservoir, val_exp, 'NO_KEYWORD', zero)
       !
       ! Map of irrigated areas
       !
       IF ( do_irrigation ) THEN
          ALLOCATE (irrigated(nbpt), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for irrigated','','')
          var_name = 'irrigated'
          CALL ioconf_setatt_p('UNITS', 'm^2')
          CALL ioconf_setatt_p('LONG_NAME','Surface of irrigated area')
          CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., irrigated, "gather", nbp_glo, index_g)
          CALL setvar_p (irrigated, val_exp, 'NO_KEYWORD', undef_sechiba)
       ENDIF

       IF ( do_floodplains ) THEN
          ALLOCATE (floodplains(nbpt), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for floodplains','','')
          var_name = 'floodplains'
          CALL ioconf_setatt_p('UNITS', 'm^2')
          CALL ioconf_setatt_p('LONG_NAME','Surface which can be flooded')
          CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., floodplains, "gather", nbp_glo, index_g)
          CALL setvar_p (floodplains, val_exp, 'NO_KEYWORD', undef_sechiba)
       ENDIF
       IF ( doswamps ) THEN
          ALLOCATE (swamp(nbpt), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for swamp','','')
          var_name = 'swamp'
          CALL ioconf_setatt_p('UNITS', 'm^2')
          CALL ioconf_setatt_p('LONG_NAME','Surface which can become swamp')
          CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., swamp, "gather", nbp_glo, index_g)
          CALL setvar_p (swamp, val_exp, 'NO_KEYWORD', undef_sechiba)
       ENDIF
       !
       ! Put into the restart file the fluxes so that they can be regenerated at restart.
       !
       ALLOCATE (lakeinflow_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for lakeinflow_mean','','')
       var_name = 'lakeinflow'
       CALL ioconf_setatt_p('UNITS', 'Kg/dt')
       CALL ioconf_setatt_p('LONG_NAME','Lake inflow')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., lakeinflow_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (lakeinflow_mean, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE (returnflow_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for returnflow_mean','','')
       var_name = 'returnflow'
       CALL ioconf_setatt_p('UNITS', 'Kg/m^2/dt')
       CALL ioconf_setatt_p('LONG_NAME','Deep return flux')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., returnflow_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (returnflow_mean, val_exp, 'NO_KEYWORD', zero)
       returnflow(:) = returnflow_mean(:)

       ALLOCATE (reinfiltration_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for reinfiltration_mean','','')
       var_name = 'reinfiltration'
       CALL ioconf_setatt_p('UNITS', 'Kg/m^2/dt')
       CALL ioconf_setatt_p('LONG_NAME','Top return flux')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., reinfiltration_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (reinfiltration_mean, val_exp, 'NO_KEYWORD', zero)
       reinfiltration(:) = reinfiltration_mean(:)

       ALLOCATE (irrigation_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for irrigation_mean','','')
       ALLOCATE (irrig_netereq(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for irrig_netereq','','')
       irrig_netereq(:) = zero

       IF ( do_irrigation ) THEN
          var_name = 'irrigation'
          CALL ioconf_setatt_p('UNITS', 'Kg/dt')
          CALL ioconf_setatt_p('LONG_NAME','Artificial irrigation flux')
          CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., irrigation_mean, "gather", nbp_glo, index_g)
          CALL setvar_p (irrigation_mean, val_exp, 'NO_KEYWORD', zero)
       ELSE
          irrigation_mean(:) = zero
       ENDIF
       irrigation(:) = irrigation_mean(:)

       ALLOCATE (riverflow_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for riverflow_mean','','')
       var_name = 'riverflow'
       CALL ioconf_setatt_p('UNITS', 'Kg/dt')
       CALL ioconf_setatt_p('LONG_NAME','River flux into the sea')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., riverflow_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (riverflow_mean, val_exp, 'NO_KEYWORD', zero)
       riverflow(:) = riverflow_mean(:)

       ALLOCATE (coastalflow_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for coastalflow_mean','','')
       var_name = 'coastalflow'
       CALL ioconf_setatt_p('UNITS', 'Kg/dt')
       CALL ioconf_setatt_p('LONG_NAME','Diffuse flux into the sea')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., coastalflow_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (coastalflow_mean, val_exp, 'NO_KEYWORD', zero)
       coastalflow(:) = coastalflow_mean(:)

       ! Locate it at the 2m level
       ipn = MINLOC(ABS(diaglev-2))
       floodtemp_lev = ipn(1)
       ALLOCATE (floodtemp(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for floodtemp','','')
       floodtemp(:) = stempdiag(:,floodtemp_lev)

       ALLOCATE(hydrographs(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for hydrographs','','')
       var_name = 'hydrographs'
       CALL ioconf_setatt_p('UNITS', 'kg/dt_sechiba')
       CALL ioconf_setatt_p('LONG_NAME','Hydrograph at outlow of grid')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., hydrographs, "gather", nbp_glo, index_g)
       CALL setvar_p (hydrographs, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE(slowflow_diag(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_init','Pb in allocate for slowflow_diag','','')
       var_name = 'slowflow_diag'
       CALL ioconf_setatt_p('UNITS', 'kg/dt_sechiba')
       CALL ioconf_setatt_p('LONG_NAME','Slowflow hydrograph at outlow of grid')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE.,slowflow_diag, "gather", nbp_glo, index_g)
       CALL setvar_p (slowflow_diag, val_exp, 'NO_KEYWORD', zero)
       !
       ! The diagnostic variables, they are initialized from the above restart variables.
       !
       ALLOCATE(fast_diag(nbpt), slow_diag(nbpt), stream_diag(nbpt), flood_diag(nbpt), &
            & pond_diag(nbpt), lake_diag(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for fast_diag,..','','')

       fast_diag(:) = zero
       slow_diag(:) = zero
       stream_diag(:) = zero
       flood_diag(:) = zero
       pond_diag(:) = zero
       lake_diag(:) = zero

       DO ig=1,nbpt
          totarea = zero
          DO ib=1,nbasmax
             totarea = totarea + routing_area_loc(ig,ib)
             fast_diag(ig) = fast_diag(ig) + fast_reservoir(ig,ib)
             slow_diag(ig) = slow_diag(ig) + slow_reservoir(ig,ib)
             stream_diag(ig) = stream_diag(ig) + stream_reservoir(ig,ib)
             flood_diag(ig) = flood_diag(ig) + flood_reservoir(ig,ib)
          ENDDO
          !
          fast_diag(ig) = fast_diag(ig)/totarea
          slow_diag(ig) = slow_diag(ig)/totarea
          stream_diag(ig) = stream_diag(ig)/totarea
          flood_diag(ig) = flood_diag(ig)/totarea
          !
          ! This is the volume of the lake scaled to the entire grid.
          ! It would be better to scale it to the size of the lake
          ! but this information is not yet available.
          !
          lake_diag(ig) = lake_reservoir(ig)/totarea
          !
       ENDDO
       !
       ! Get from the restart the fluxes we accumulated.
       !
       ALLOCATE (floodout_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for floodout_mean','','')
       var_name = 'floodout_route'
       CALL ioconf_setatt_p('UNITS', 'Kg')
       CALL ioconf_setatt_p('LONG_NAME','Accumulated flow out of floodplains for routing')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., floodout_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (floodout_mean, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE (runoff_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for runoff_mean','','')
       var_name = 'runoff_route'
       CALL ioconf_setatt_p('UNITS', 'Kg')
       CALL ioconf_setatt_p('LONG_NAME','Accumulated runoff for routing')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., runoff_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (runoff_mean, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE(drainage_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for drainage_mean','','')
       var_name = 'drainage_route'
       CALL ioconf_setatt_p('UNITS', 'Kg')
       CALL ioconf_setatt_p('LONG_NAME','Accumulated drainage for routing')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., drainage_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (drainage_mean, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE(transpot_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for transpot_mean','','')
       var_name = 'transpot_route'
       CALL ioconf_setatt_p('UNITS', 'Kg/m^2')
       CALL ioconf_setatt_p('LONG_NAME','Accumulated potential transpiration for routing/irrigation')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., transpot_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (transpot_mean, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE(precip_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for precip_mean','','')
       var_name = 'precip_route'
       CALL ioconf_setatt_p('UNITS', 'Kg/m^2')
       CALL ioconf_setatt_p('LONG_NAME','Accumulated rain precipitation for irrigation')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., precip_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (precip_mean, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE(humrel_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for humrel_mean','','')
       var_name = 'humrel_route'
       CALL ioconf_setatt_p('UNITS', '-')
       CALL ioconf_setatt_p('LONG_NAME','Mean humrel for irrigation')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., humrel_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (humrel_mean, val_exp, 'NO_KEYWORD', un)

       ALLOCATE(k_litt_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for k_litt_mean','','')
       var_name = 'k_litt_route'
       CALL ioconf_setatt_p('UNITS', '-')
       CALL ioconf_setatt_p('LONG_NAME','Mean cond. for litter')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., k_litt_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (k_litt_mean, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE(totnobio_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for totnobio_mean','','')
       var_name = 'totnobio_route'
       CALL ioconf_setatt_p('UNITS', '-')
       CALL ioconf_setatt_p('LONG_NAME','Last Total fraction of no bio for irrigation')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., totnobio_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (totnobio_mean, val_exp, 'NO_KEYWORD', zero)

       ALLOCATE(vegtot_mean(nbpt), stat=ier)
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for vegtot_mean','','')
       var_name = 'vegtot_route'
       CALL ioconf_setatt_p('UNITS', '-')
       CALL ioconf_setatt_p('LONG_NAME','Last Total fraction of vegetation')
       CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., vegtot_mean, "gather", nbp_glo, index_g)
       CALL setvar_p (vegtot_mean, val_exp, 'NO_KEYWORD', un)
       !
    ENDIF

    IF (ok_grdc) THEN

        ALLOCATE (idx_mod_match_loc(nbpt,nbasmax), stat=ier)
        IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for idx_mod_match_loc','','')
        ALLOCATE (idx_mod_match_glo(nbp_glo,nbasmax), stat=ier)
        IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for idx_mod_match_glo','','')
        !IF (is_root_prc) THEN
        !    var_name = 'idx_mod_match_glo'
        !    CALL ioconf_setatt('UNITS', '-')
        !    CALL ioconf_setatt('LONG_NAME','idx_mod_match_glo')
        !    CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., tmp_real_g, "gather", nbp_glo, index_g)
        !    idx_mod_match_glo = undef_int
        !    WHERE ( tmp_real_g .LT. val_exp )
        !        idx_mod_match_glo = NINT(tmp_real_g)
        !    ENDWHERE
        !ENDIF
        !CALL scatter(idx_mod_match_glo,idx_mod_match_loc)

        ALLOCATE (hydrodiag_grdc_loc(nbpt,nbasmax), stat=ier)
        IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for hydrodiag_grdc_loc','','')
        ALLOCATE (hydrodiag_grdc_glo(nbp_glo,nbasmax), stat=ier)
        IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for hydrodiag_grdc_glo','','')

        !IF (is_root_prc) THEN
        !    var_name = 'hydrodiag_grdc_glo'
        !    CALL ioconf_setatt('UNITS', '-')
        !    CALL ioconf_setatt('LONG_NAME','hydrodiag_grdc_glo')
        !    CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., tmp_real_g, "gather", nbp_glo, index_g)
        !    hydrodiag_grdc_glo = undef_int
        !    WHERE ( tmp_real_g .LT. val_exp )
        !        hydrodiag_grdc_glo = NINT(tmp_real_g)
        !    ENDWHERE
        !ENDIF
        !CALL scatter(hydrodiag_grdc_glo,hydrodiag_grdc_loc)

        ALLOCATE (hydrodiag_largest_loc(nbpt,nbasmax), stat=ier)
        IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for hydrodiag_largest_loc','','')
        ALLOCATE (hydrodiag_largest_glo(nbp_glo,nbasmax), stat=ier)
        IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for hydrodiag_largest_glo','','')

        !IF (is_root_prc) THEN
        !    var_name = 'hydrodiag_largest_glo'
        !    CALL ioconf_setatt('UNITS', '-')
        !    CALL ioconf_setatt('LONG_NAME','hydrodiag_largest_glo')
        !    CALL restget (rest_id, var_name, nbp_glo, nbasmax, 1, kjit, .TRUE., tmp_real_g, "gather", nbp_glo, index_g)
        !    hydrodiag_largest_glo = undef_int
        !    WHERE ( tmp_real_g .LT. val_exp )
        !        hydrodiag_largest_glo = NINT(tmp_real_g)
        !    ENDWHERE
        !ENDIF
        !CALL scatter(hydrodiag_largest_glo,hydrodiag_largest_loc)

    ENDIF
    !
    DEALLOCATE(tmp_real_g)
    !
    ! Allocate diagnostic variables
    !
    ALLOCATE(hydrodiag_loc(nbpt,nbasmax),hydrodiag_glo(nbp_glo,nbasmax),stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for hydrodiag_glo','','')
    hydrodiag=>hydrodiag_loc

    ALLOCATE(hydroupbasin_loc(nbpt),hydroupbasin_glo(nbp_glo), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_init','Pb in allocate for hydroupbasin_glo','','')
    hydroupbasin=>hydroupbasin_loc

  END SUBROUTINE routing_reg_init
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_clear
!!
!>\BRIEF : This subroutine deallocates the block memory previously allocated.
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_clear()
    !
    IF (ALLOCATED(routing_area_loc)) DEALLOCATE(routing_area_loc)
    IF (ALLOCATED(route_togrid_loc)) DEALLOCATE(route_togrid_loc)
    IF (ALLOCATED(route_tobasin_loc)) DEALLOCATE(route_tobasin_loc)
    IF (ALLOCATED(route_nbintobas_loc)) DEALLOCATE(route_nbintobas_loc)
    IF (ALLOCATED(global_basinid_loc)) DEALLOCATE(global_basinid_loc)
    IF (ALLOCATED(topo_resid_loc)) DEALLOCATE(topo_resid_loc)
    IF (ALLOCATED(routing_area_glo)) DEALLOCATE(routing_area_glo)
    IF (ALLOCATED(route_togrid_glo)) DEALLOCATE(route_togrid_glo)
    IF (ALLOCATED(route_tobasin_glo)) DEALLOCATE(route_tobasin_glo)
    IF (ALLOCATED(route_nbintobas_glo)) DEALLOCATE(route_nbintobas_glo)
    IF (ALLOCATED(global_basinid_glo)) DEALLOCATE(global_basinid_glo)
    !
    IF (ALLOCATED(route_outlet_glo)) DEALLOCATE(route_outlet_glo)
    IF (ALLOCATED(route_outlet_loc)) DEALLOCATE(route_outlet_loc)
    IF (ALLOCATED(route_type_glo)) DEALLOCATE(route_type_glo)
    IF (ALLOCATED(route_type_loc)) DEALLOCATE(route_type_loc)
    IF (ALLOCATED(origin_nbintobas_loc)) DEALLOCATE(origin_nbintobas_loc)
    IF (ALLOCATED(origin_nbintobas_glo)) DEALLOCATE(origin_nbintobas_glo)
    !
    IF (ALLOCATED(topo_resid_glo)) DEALLOCATE(topo_resid_glo)
    IF (ALLOCATED(fast_reservoir)) DEALLOCATE(fast_reservoir)
    IF (ALLOCATED(slow_reservoir)) DEALLOCATE(slow_reservoir)
    IF (ALLOCATED(stream_reservoir)) DEALLOCATE(stream_reservoir)
    IF (ALLOCATED(flood_reservoir)) DEALLOCATE(flood_reservoir)
    IF (ALLOCATED(flood_frac_bas)) DEALLOCATE(flood_frac_bas)
    IF (ALLOCATED(flood_height)) DEALLOCATE(flood_height)
    IF (ALLOCATED(pond_frac)) DEALLOCATE(pond_frac)
    IF (ALLOCATED(lake_reservoir)) DEALLOCATE(lake_reservoir)
    IF (ALLOCATED(pond_reservoir)) DEALLOCATE(pond_reservoir)
    IF (ALLOCATED(returnflow_mean)) DEALLOCATE(returnflow_mean)
    IF (ALLOCATED(reinfiltration_mean)) DEALLOCATE(reinfiltration_mean)
    IF (ALLOCATED(riverflow_mean)) DEALLOCATE(riverflow_mean)
    IF (ALLOCATED(coastalflow_mean)) DEALLOCATE(coastalflow_mean)
    IF (ALLOCATED(lakeinflow_mean)) DEALLOCATE(lakeinflow_mean)
    IF (ALLOCATED(runoff_mean)) DEALLOCATE(runoff_mean)
    IF (ALLOCATED(floodout_mean)) DEALLOCATE(floodout_mean)
    IF (ALLOCATED(drainage_mean)) DEALLOCATE(drainage_mean)
    IF (ALLOCATED(transpot_mean)) DEALLOCATE(transpot_mean)
    IF (ALLOCATED(precip_mean)) DEALLOCATE(precip_mean)
    IF (ALLOCATED(humrel_mean)) DEALLOCATE(humrel_mean)
    IF (ALLOCATED(k_litt_mean)) DEALLOCATE(k_litt_mean)
    IF (ALLOCATED(totnobio_mean)) DEALLOCATE(totnobio_mean)
    IF (ALLOCATED(vegtot_mean)) DEALLOCATE(vegtot_mean)
    IF (ALLOCATED(floodtemp)) DEALLOCATE(floodtemp)
    IF (ALLOCATED(hydrodiag_loc)) DEALLOCATE(hydrodiag_loc)
    IF (ALLOCATED(hydrodiag_glo)) DEALLOCATE(hydrodiag_glo)
    IF (ALLOCATED(hydroupbasin_loc)) DEALLOCATE(hydroupbasin_loc)
    IF (ALLOCATED(hydroupbasin_glo)) DEALLOCATE(hydroupbasin_glo)
    IF (ALLOCATED(hydrographs)) DEALLOCATE(hydrographs)
    IF (ALLOCATED(irrigation_mean)) DEALLOCATE(irrigation_mean)
    IF (ALLOCATED(irrigated)) DEALLOCATE(irrigated)
    IF (ALLOCATED(floodplains)) DEALLOCATE(floodplains)
    IF (ALLOCATED(swamp)) DEALLOCATE(swamp)
    IF (ALLOCATED(fast_diag)) DEALLOCATE(fast_diag)
    IF (ALLOCATED(slow_diag)) DEALLOCATE(slow_diag)
    IF (ALLOCATED(stream_diag)) DEALLOCATE(stream_diag)
    IF (ALLOCATED(flood_diag)) DEALLOCATE(flood_diag)
    IF (ALLOCATED(pond_diag)) DEALLOCATE(pond_diag)
    IF (ALLOCATED(lake_diag)) DEALLOCATE(lake_diag)
    !    
    IF (ALLOCATED(outflow_pt_loc)) DEALLOCATE(outflow_pt_loc)
    IF (ALLOCATED(outflow_lalo_loc)) DEALLOCATE(outflow_lalo_loc)
    IF (ALLOCATED(outflow_type_loc)) DEALLOCATE(outflow_type_loc)
    IF (ALLOCATED(outflow_loc)) DEALLOCATE(outflow_loc)

    IF (ALLOCATED(basin_names))          DEALLOCATE(basin_names)
    IF (ALLOCATED(basin_allnames_glo))   DEALLOCATE(basin_allnames_glo)
    IF (ALLOCATED(basin_allnames_loc))   DEALLOCATE(basin_allnames_loc)
    IF (ALLOCATED(areaupbasin))          DEALLOCATE(areaupbasin)
    IF (ALLOCATED(areaupbasin_loc))      DEALLOCATE(areaupbasin_loc)

    IF(ok_grdc) THEN
! write into output
    IF (ALLOCATED(idx_grdc_match_glo))    DEALLOCATE(idx_grdc_match_glo)
    IF (ALLOCATED(og_grdc_match_glo))     DEALLOCATE(og_grdc_match_glo)
    IF (ALLOCATED(ob_grdc_match_glo))     DEALLOCATE(ob_grdc_match_glo)
!    IF (ALLOCATED(lat_grdc_match_glo))    DEALLOCATE(lat_grdc_match_glo)
!    IF (ALLOCATED(lon_grdc_match_glo))    DEALLOCATE(lon_grdc_match_glo)
    IF (ALLOCATED(area_grdc_match_glo))   DEALLOCATE(area_grdc_match_glo)
!    IF (ALLOCATED(basin_grdc_match_glo))   DEALLOCATE(basin_grdc_match_glo)
!    IF (ALLOCATED(mergedhydro_grdc_match_glo)) DEALLOCATE(mergedhydro_grdc_match_glo)
    IF (ALLOCATED(idx_grdc_match_loc))         DEALLOCATE(idx_grdc_match_loc)
    IF (ALLOCATED(og_grdc_match_loc))          DEALLOCATE(og_grdc_match_loc)
    IF (ALLOCATED(ob_grdc_match_loc))          DEALLOCATE(ob_grdc_match_loc)
    IF (ALLOCATED(ox_grdc_match_loc))          DEALLOCATE(ox_grdc_match_loc)
    IF (ALLOCATED(oy_grdc_match_loc))          DEALLOCATE(oy_grdc_match_loc)
    IF (ALLOCATED(ogloc_grdc_match_loc))       DEALLOCATE(ogloc_grdc_match_loc)
    IF (ALLOCATED(obloc_grdc_match_loc))       DEALLOCATE(obloc_grdc_match_loc)
    IF (ALLOCATED(lat_grdc_match_loc))         DEALLOCATE(lat_grdc_match_loc)
    IF (ALLOCATED(lon_grdc_match_loc))         DEALLOCATE(lon_grdc_match_loc)
    IF (ALLOCATED(area_grdc_match_loc))        DEALLOCATE(area_grdc_match_loc)
    IF (ALLOCATED(basin_grdc_match_loc))       DEALLOCATE(basin_grdc_match_loc)
    IF (ALLOCATED(mergedhydro_grdc_match_loc)) DEALLOCATE(mergedhydro_grdc_match_loc)

!    IF (ALLOCATED(outpt_coor_largest_glo))  DEALLOCATE(outpt_coor_largest_glo)
    IF (ALLOCATED(totarea_largest_glo))        DEALLOCATE(totarea_largest_glo)
!    IF (ALLOCATED(hydrographs_largest_glo)) DEALLOCATE(hydrographs_largest_glo)
    IF (ALLOCATED(outpt_coor_largest_loc))     DEALLOCATE(outpt_coor_largest_loc)
    IF (ALLOCATED(totarea_largest_loc))        DEALLOCATE(totarea_largest_loc)
    IF (ALLOCATED(hydrographs_largest_loc))    DEALLOCATE(hydrographs_largest_loc)
    IF (ALLOCATED(og_largest_loc))             DEALLOCATE(og_largest_loc)
    IF (ALLOCATED(ob_largest_loc))             DEALLOCATE(ob_largest_loc)
    IF (ALLOCATED(ogloc_largest_loc))          DEALLOCATE(ogloc_largest_loc)
    IF (ALLOCATED(obloc_largest_loc))          DEALLOCATE(obloc_largest_loc)

    IF (ALLOCATED(ig_glo))                     DEALLOCATE(ig_glo)
    IF (ALLOCATED(ib_glo))                     DEALLOCATE(ib_glo)
    IF (ALLOCATED(ig_loc))                     DEALLOCATE(ig_loc)
    IF (ALLOCATED(ib_loc))                     DEALLOCATE(ib_loc)
    IF (ALLOCATED(ix_glo))                     DEALLOCATE(ix_glo)
    IF (ALLOCATED(iy_glo))                     DEALLOCATE(iy_glo)
    IF (ALLOCATED(ix_loc))                     DEALLOCATE(ix_loc)
    IF (ALLOCATED(iy_loc))                     DEALLOCATE(iy_loc)

! Matching GRDC and MODEL
!    IF (ALLOCATED(idx_grdc_glo))          DEALLOCATE(idx_grdc_glo)
!    IF (ALLOCATED(area_grdc_glo))         DEALLOCATE(area_grdc_glo)
!    IF (ALLOCATED(lon_grdc_glo))          DEALLOCATE(lon_grdc_glo)
!    IF (ALLOCATED(lat_grdc_glo))          DEALLOCATE(lat_grdc_glo)
!    IF (ALLOCATED(idx_grdc_loc))          DEALLOCATE(idx_grdc_loc)
!    IF (ALLOCATED(area_grdc_loc))         DEALLOCATE(area_grdc_loc)
!    IF (ALLOCATED(lon_grdc_loc))          DEALLOCATE(lon_grdc_loc)
!    IF (ALLOCATED(lat_grdc_loc))          DEALLOCATE(lat_grdc_loc)
    IF (ALLOCATED(hydrodiag_grdc_glo))      DEALLOCATE(hydrodiag_grdc_glo)
    IF (ALLOCATED(hydrodiag_grdc_loc))      DEALLOCATE(hydrodiag_grdc_loc)
    IF (ALLOCATED(hydrodiag_largest_glo))   DEALLOCATE(hydrodiag_largest_glo)
    IF (ALLOCATED(hydrodiag_largest_loc))   DEALLOCATE(hydrodiag_largest_loc)
    IF (ALLOCATED(idx_mod_match_loc))       DEALLOCATE(idx_mod_match_loc)
    IF (ALLOCATED(idx_mod_match_glo))       DEALLOCATE(idx_mod_match_glo)

! GRDC data in the predefined domain
    IF (ALLOCATED(area_obs_reg))            DEALLOCATE(area_obs_reg)
    IF (ALLOCATED(lat_obs_reg))             DEALLOCATE(lat_obs_reg)
    IF (ALLOCATED(lon_obs_reg))             DEALLOCATE(lon_obs_reg)
!    IF (ALLOCATED(hydrographs_obs_reg))    DEALLOCATE(hydrographs_obs_reg)
    IF (ALLOCATED(mergedhydro_obs_reg))     DEALLOCATE(mergedhydro_obs_reg)
    IF (ALLOCATED(river_obs_reg))           DEALLOCATE(river_obs_reg)
    IF (ALLOCATED(sta_obs_reg))             DEALLOCATE(sta_obs_reg)
    IF (ALLOCATED(country_obs_reg))         DEALLOCATE(country_obs_reg)
    !IF (ALLOCATED(number_obs_reg))         DEALLOCATE(number_obs_reg)
    IF (ALLOCATED(idx_obs_reg))             DEALLOCATE(idx_obs_reg)
    IF (ALLOCATED(idx_obs_reg_sort))        DEALLOCATE(idx_obs_reg_sort)
    IF (ALLOCATED(area_obs_reg_sort))       DEALLOCATE(area_obs_reg_sort)
    IF (ALLOCATED(year_grdc_cut))           DEALLOCATE(year_grdc_cut)
    IF (ALLOCATED(month_grdc_cut))          DEALLOCATE(month_grdc_cut)
    IF (ALLOCATED(day_grdc_cut))            DEALLOCATE(day_grdc_cut)
    IF (ALLOCATED(sec_grdc_cut))            DEALLOCATE(sec_grdc_cut)
    IF (ALLOCATED(jds_grdc_cut))            DEALLOCATE(jds_grdc_cut)

!! XIOS for GRDC
    IF (ALLOCATED(nbsta))                   DEALLOCATE(nbsta)
    IF (ALLOCATED(nbsta_sort))              DEALLOCATE(nbsta_sort)
    IF (ALLOCATED(nbsta_tmp))               DEALLOCATE(nbsta_tmp)
    IF (ALLOCATED(nbsta_idx))               DEALLOCATE(nbsta_idx)
    IF (ALLOCATED(data_ibegin_grdc))        DEALLOCATE(data_ibegin_grdc)
    IF (ALLOCATED(data_iend_grdc))          DEALLOCATE(data_iend_grdc)
    IF (ALLOCATED(lon_sta))                 DEALLOCATE(lon_sta)
    IF (ALLOCATED(data_ibegin_grdc_gather)) DEALLOCATE(data_ibegin_grdc_gather)
    IF (ALLOCATED(idxsort_grdc_match_loc))  DEALLOCATE(idxsort_grdc_match_loc)
    IF (ALLOCATED(idxsort_grdc_match_glo))  DEALLOCATE(idxsort_grdc_match_glo)

!! XIOS for Largest
    IF (ALLOCATED(nblargest))               DEALLOCATE(nblargest)
    IF (ALLOCATED(nblargest_sort))          DEALLOCATE(nblargest_sort)
    IF (ALLOCATED(nblargest_tmp))           DEALLOCATE(nblargest_tmp)
    IF (ALLOCATED(nblargest_idx))           DEALLOCATE(nblargest_idx)
    IF (ALLOCATED(data_ibegin_largest))     DEALLOCATE(data_ibegin_largest)
    IF (ALLOCATED(data_iend_largest))       DEALLOCATE(data_iend_largest)
    IF (ALLOCATED(lon_larg))                DEALLOCATE(lon_larg)
    IF (ALLOCATED(data_ibegin_largest_gather)) DEALLOCATE(data_ibegin_largest_gather)
    IF (ALLOCATED(idxsort_largest_loc))        DEALLOCATE(idxsort_largest_loc)
    IF (ALLOCATED(idxsort_largest_glo))        DEALLOCATE(idxsort_largest_glo)

!! GRDC subbasins
    IF (ALLOCATED(g_stabsn))                DEALLOCATE(g_stabsn)
    IF (ALLOCATED(b_stabsn))                DEALLOCATE(b_stabsn)
    IF (ALLOCATED(nb_subb))                 DEALLOCATE(nb_subb)
    IF (ALLOCATED(g_stabsn_loc))            DEALLOCATE(g_stabsn_loc)
    IF (ALLOCATED(b_stabsn_loc))            DEALLOCATE(b_stabsn_loc)
    IF (ALLOCATED(nb_subb_loc))             DEALLOCATE(nb_subb_loc)
    IF (ALLOCATED(gloc_stabsn_loc))         DEALLOCATE(gloc_stabsn_loc)
    IF (ALLOCATED(bloc_stabsn_loc))         DEALLOCATE(bloc_stabsn_loc)
    IF (ALLOCATED(map_stabsn_glo))          DEALLOCATE(map_stabsn_glo)
    IF (ALLOCATED(map_stabsn_loc))          DEALLOCATE(map_stabsn_loc)
    IF (ALLOCATED(nb_stabsn_glo))           DEALLOCATE(nb_stabsn_glo)
    IF (ALLOCATED(nb_stabsn_loc))           DEALLOCATE(nb_stabsn_loc)

    ENDIF

    !
  END SUBROUTINE routing_reg_clear
  !

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_flow
!!
!>\BRIEF This subroutine computes the transport of water in the various reservoirs
!! (including ponds and floodplains) and the water withdrawals from the reservoirs for irrigation.
!!
!! DESCRIPTION (definitions, functional, design, flags) :
!! This will first compute the amount of water which flows out of each of the 3 reservoirs using the assumption of an
!! exponential decrease of water in the reservoir (see Hagemann S and Dumenil L. (1998)). Then we compute the fluxes
!! for floodplains and ponds. All this will then be used in order to update each of the basins : taking water out of
!! the up-stream basin and adding it to the down-stream one.
!! As this step happens globaly we have to stop the parallel processing in order to exchange the information. Once
!! all reservoirs are updated we deal with irrigation. The final step is to compute diagnostic fluxes. Among them
!! the hydrographs of the largest rivers we have chosen to monitor.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): lakeinflow, returnflow, reinfiltration, irrigation, riverflow, coastalflow, hydrographs, flood_frac, flood_res
!!
!! REFERENCES :
!! - Ngo-Duc, T., K. Laval, G. Ramillien, J. Polcher, and A. Cazenave (2007)
!! Validation of the land water storage simulated by Organising Carbon and Hydrology in Dynamic Ecosystems (ORCHIDEE) with Gravity Recovery and Climate Experiment (GRACE) data.
!! Water Resour. Res., 43, W04427, doi:10.1029/2006WR004941.
!! * Irrigation:
!! - de Rosnay, P., J. Polcher, K. Laval, and M. Sabre (2003)
!! Integrated parameterization of irrigation in the land surface model ORCHIDEE. Validation over Indian Peninsula.
!! Geophys. Res. Lett., 30(19), 1986, doi:10.1029/2003GL018024.
!! - A.C. Vivant (2003)
!! Les plaines d'inondations et l'irrigation dans ORCHIDEE, impacts de leur prise en compte.
!! , , 51pp.
!! - N. Culson (2004)
!! Impact de l'irrigation sur le cycle de l'eau
!! Master thesis, Paris VI University, 55pp.
!! - X.-T. Nguyen-Vinh (2005)
!! Analyse de l'impact de l'irrigation en Amerique du Nord - plaine du Mississippi - sur la climatologie regionale
!! Master thesis, Paris VI University, 33pp.
!! - M. Guimberteau (2006)
!! Analyse et modifications proposees de la modelisation de l'irrigation dans un modele de surface.
!! Master thesis, Paris VI University, 46pp.
!! - Guimberteau M. (2010)
!! Modelisation de l'hydrologie continentale et influences de l'irrigation sur le cycle de l'eau.
!! Ph.D. thesis, Paris VI University, 195pp.
!! - Guimberteau M., Laval K., Perrier A. and Polcher J. (2011).
!! Global effect of irrigation and its impact on the onset of the Indian summer monsoon.
!! In press, Climate Dynamics, doi: 10.1007/s00382-011-1252-5.
!! * Floodplains:
!! - A.C. Vivant (2002)
!! L'ecoulement lateral de l'eau sur les surfaces continentales. Prise en compte des plaines d'inondations dans ORCHIDEE.
!! Master thesis, Paris VI University, 46pp.
!! - A.C. Vivant (2003)
!! Les plaines d'inondations et l'irrigation dans ORCHIDEE, impacts de leur prise en compte.
!! , , 51pp.
!! - T. d'Orgeval (2006)
!! Impact du changement climatique sur le cycle de l'eau en Afrique de l'Ouest: modelisation et incertitudes.
!! Ph.D. thesis, Paris VI University, 188pp.
!! - T. d'Orgeval, J. Polcher, and P. de Rosnay (2008)
!! Sensitivity of the West African hydrological cycle in ORCHIDEE to infiltration processes.
!! Hydrol. Earth Syst. Sci., 12, 1387-1401
!! - M. Guimberteau, G. Drapeau, J. Ronchail, B. Sultan, J. Polcher, J.-M. Martinez, C. Prigent, J.-L. Guyot, G. Cochonneau,
!! J. C. Espinoza, N. Filizola, P. Fraizy, W. Lavado, E. De Oliveira, R. Pombosa, L. Noriega, and P. Vauchel (2011)
!! Discharge simulation in the sub-basins of the Amazon using ORCHIDEE forced by new datasets.
!! Hydrol. Earth Syst. Sci. Discuss., 8, 11171-11232, doi:10.5194/hessd-8-11171-2011
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_flow(nbpt, dt_routing, lalo, floodout, runoff, drainage, &
       &                      vegtot, totnobio, transpot_mean, precip, humrel, k_litt, floodtemp, reinf_slope, &
       &                      lakeinflow, returnflow, reinfiltration, irrigation, riverflow, &
       &                      coastalflow, hydrographs, slowflow_diag, flood_frac, flood_res, &
       &                      netflow_stream_diag, netflow_fast_diag, netflow_slow_diag)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    REAL(r_std), INTENT (in) :: dt_routing !! Routing time step (s)
    REAL(r_std), INTENT(in) :: lalo(nbpt,2) !! Vector of latitude and longitudes
    REAL(r_std), INTENT(in) :: runoff(nbpt) !! Grid-point runoff (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: floodout(nbpt) !! Grid-point flow out of floodplains (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: drainage(nbpt) !! Grid-point drainage (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: vegtot(nbpt) !! Potentially vegetated fraction (unitless;0-1)
    REAL(r_std), INTENT(in) :: totnobio(nbpt) !! Other areas which can not have vegetation
    REAL(r_std), INTENT(in) :: transpot_mean(nbpt) !! Mean potential transpiration of the vegetation (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: precip(nbpt) !! Rainfall (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: humrel(nbpt) !! Soil moisture stress, root extraction potential (unitless)
    REAL(r_std), INTENT(in) :: k_litt(nbpt) !! Averaged conductivity for saturated infiltration in the 'litter' layer (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: floodtemp(nbpt) !! Temperature to decide if floodplains work (K)
    REAL(r_std), INTENT(in) :: reinf_slope(nbpt) !! Coefficient which determines the reinfiltration ratio in the grid box due to flat areas (unitless;0-1)
    !
!! OUTPUT VARIABLES
    REAL(r_std), INTENT(out) :: lakeinflow(nbpt) !! Water inflow to the lakes (kg/dt)
    REAL(r_std), INTENT(out) :: returnflow(nbpt) !! The water flow from lakes and swamps which returns into the grid box.
                                                                              !! This water will go back into the hydrol or hydrolc module to allow re-evaporation (kg/m^2/dt_routing)
    REAL(r_std), INTENT(out) :: reinfiltration(nbpt) !! Water flow from ponds and floodplains which returns to the grid box (kg/m^2/dt)
    REAL(r_std), INTENT(out) :: irrigation(nbpt) !! Irrigation flux. This is the water taken from the reservoirs and beeing put into the upper layers of the soil (kg/m^2/dt_routing)
    REAL(r_std), INTENT(out) :: riverflow(nbpt) !! Outflow of the major rivers. The flux will be located on the continental grid but this should be a coastal point (kg/dt_routing)
    REAL(r_std), INTENT(out) :: coastalflow(nbpt) !! Outflow on coastal points by small basins. This is the water which flows in a disperse way into the ocean (kg/dt_routing)
    REAL(r_std), INTENT(out) :: hydrographs(nbpt) !! Hydrographs at the outflow of the grid box for major basins (kg/dt)
    REAL(r_std), INTENT(out) :: slowflow_diag(nbpt)       !! Hydrographs of slow_flow = routed slow_flow for major basins (kg/dt)
    REAL(r_std), INTENT(out) :: flood_frac(nbpt) !! Flooded fraction of the grid box (unitless;0-1)
    REAL(r_std), INTENT(out) :: flood_res(nbpt) !! Diagnostic of water amount in the floodplains reservoir (kg)

    REAL(r_std), INTENT(out) :: netflow_stream_diag(nbpt) !! Input - Output flow to stream reservoir
    REAL(r_std), INTENT(out) :: netflow_fast_diag(nbpt)   !! Input - Output flow to fast reservoir
    REAL(r_std), INTENT(out) :: netflow_slow_diag(nbpt)   !! Input - Output flow to slow reservoir
    !
    !
!! LOCAL VARIABLES
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: fast_flow !! Outflow from the fast reservoir (kg/dt)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: slow_flow !! Outflow from the slow reservoir (kg/dt)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: stream_flow !! Outflow from the stream reservoir (kg/dt)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: flood_flow !! Outflow from the floodplain reservoir (kg/dt)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: pond_inflow !! Inflow to the pond reservoir (kg/dt)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: pond_drainage !! Drainage from pond (kg/m^2/dt)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: flood_drainage !! Drainage from floodplains (kg/m^2/dt)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: return_swamp !! Inflow to the swamp (kg/dt)
    !
    ! Irrigation per basin
    !
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: irrig_needs !! Total irrigation requirement (water requirements by the crop for its optimal growth) (kg)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: irrig_actual !! Possible irrigation according to the water availability in the reservoirs (kg)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: irrig_deficit !! Amount of water missing for irrigation (kg)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: irrig_adduct !! Amount of water carried over from other basins for irrigation (kg)
    !
    REAL(r_std), DIMENSION(nbpt, 0:nbasmax+3) :: transport !! Water transport between basins (kg/dt)
    REAL(r_std), DIMENSION(nbp_glo, 0:nbasmax+3) :: transport_glo !! Water transport between basins (kg/dt)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: floods !! Water flow in to the floodplains (kg/dt)
    REAL(r_std), DIMENSION(nbpt, nbasmax) :: potflood !! Potential inflow to the swamps (kg/dt)
    REAL(r_std), DIMENSION(nbpt) :: tobeflooded !! Maximal surface which can be inundated in each grid box (m^2)
    REAL(r_std), DIMENSION(nbpt) :: totarea !! Total area of basin (m^2)
    REAL(r_std), DIMENSION(nbpt) :: totflood !! Total amount of water in the floodplains reservoir (kg)
    REAL(r_std), DIMENSION(nbasmax) :: pond_excessflow !!
    REAL(r_std) :: flow !! Outflow computation for the reservoirs (kg/dt)
    REAL(r_std) :: floodindex !! Fraction of grid box area inundated (unitless;0-1)
    REAL(r_std) :: pondex !!
    REAL(r_std) :: flood_frac_pot !! Total fraction of the grid box which is flooded at optimum repartition (unitless;0-1)
    REAL(r_std) :: stream_tot !! Total water amount in the stream reservoirs (kg)
    REAL(r_std) :: adduction !! Importation of water from a stream reservoir of a neighboring grid box (kg)

    REAL(r_std), DIMENSION(nbp_glo)              :: lake_overflow_g           !! Removed water from lake reservoir on global grid (kg/gridcell/dt_routing)
    REAL(r_std), DIMENSION(nbpt)                 :: lake_overflow             !! Removed water from lake reservoir on local grid (kg/gridcell/dt_routing)
    REAL(r_std), DIMENSION(nbpt)                 :: lake_overflow_coast       !! lake_overflow distributed on coast gridcells, only diag(kg/gridcell/dt_routing)
    REAL(r_std)                                  :: total_lake_overflow       !! Sum of lake_overflow over full grid (kg)
    
    REAL(r_std), DIMENSION(8,nbasmax) :: streams_around !! Stream reservoirs of the neighboring grid boxes (kg)
    INTEGER(i_std), DIMENSION(8) :: igrd !!
    INTEGER(i_std), DIMENSION(2) :: ff !!
    INTEGER(i_std), DIMENSION(1) :: fi !!
    INTEGER(i_std) :: ig, ib, ib2, ig2 !! Indices (unitless)
    INTEGER(i_std) :: rtg, rtb, in !! Indices (unitless)
    INTEGER(i_std) :: ier !! Error handling
    INTEGER(i_std) :: i, j
    INTEGER(i_std) :: igrdc, ilargest, igtmp, ibtmp
    !
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: fast_flow_g !! Outflow from the fast reservoir (kg/dt)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: slow_flow_g !! Outflow from the slow reservoir (kg/dt)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: stream_flow_g !! Outflow from the stream reservoir (kg/dt)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: irrig_deficit_glo !! Amount of water missing for irrigation (kg)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: stream_reservoir_glo !! Water amount in the stream reservoir (kg)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: irrig_adduct_glo !! Amount of water carried over from other basins for irrigation (kg)
    !

    REAL(r_std), DIMENSION(nbpt, nbasmax)        :: netflow_stream            !! Input - Output flow to stream reservoir
    REAL(r_std), DIMENSION(nbpt, nbasmax)        :: netflow_fast              !! Input - Output flow to fast reservoir
    REAL(r_std), DIMENSION(nbpt, nbasmax)        :: netflow_slow              !! Input - Output flow to slow reservoir
    
    !! PARAMETERS
    LOGICAL, PARAMETER :: check_reservoir = .FALSE. !! Logical to choose if we write informations when a negative amount of water is occurring in a reservoir (true/false)
    !
    LOGICAL, PARAMETER :: debug = .FALSE. !! New logical variable to debug
    LOGICAL :: use_exp = .FALSE. !! New logical variable to calculate outflow fluxes using the exponential funtion (advised by Agnes DUCHARNE) 

!_ ================================================================================================================================

    !
    ! Read option for use_exp from run.def
    !
    CALL getin_p('USING_EXP', use_exp)
    !
    ! Trung: In routing_reg_flow:
    ! Trung: when run with IFORT compiler, the variable one_day didn't get right value.
    ! Trung: So I attribute its value here.
    !
    one_day = 86400.
    !
    transport(:,:) = zero
    transport_glo(:,:) = zero
    irrig_netereq(:) = zero
    irrig_needs(:,:) = zero
    irrig_actual(:,:) = zero
    irrig_deficit(:,:) = zero
    irrig_adduct(:,:) = zero
    totarea(:) = zero
    totflood(:) = zero
    !
    ! Compute all the fluxes
    !
    DO ib=1,nbasmax
       DO ig=1,nbpt
          !
          totarea(ig) = totarea(ig) + routing_area_loc(ig,ib)
          totflood(ig) = totflood(ig) + flood_reservoir(ig,ib)
       ENDDO
    ENDDO
          !
!> The outflow fluxes from the three reservoirs are computed.
!> The outflow of volume of water Vi into the reservoir i is assumed to be linearly related to its volume.
!> The water travel simulated by the routing scheme is dependent on the water retention index topo_resid
!> given by a 0.01 degree resolution map for each pixel performed from a simplification of Manning's formula
!> (Dingman, 1994; Ducharne et al., 2003).
!> The resulting product of tcst (in day/m) and topo_resid (in m) represents the time constant (day)
!> which is an e-folding time, the time necessary for the water amount
!> in the stream reservoir to decrease by a factor e. Hence, it gives an order of
!> magnitude of the travel time through this reservoir between
!> the sub-basin considered and its downstream neighbor.

    DO ib=1,nbasmax
       DO ig=1,nbpt
          IF ( route_tobasin_loc(ig,ib) .GT. 0 ) THEN
             !
             ! Each of the fluxes is limited by the water in the reservoir and a small margin
             ! (min_reservoir) to avoid rounding errors.
             !
             ! Trung: because of unexpected small error in calculating accuracy,
             ! Trung: some points (!?) got topo_resid(ig,ib) = 0.
             ! Trung: I'll soon delete the IF statement below after
             ! Trung: correcting the negative value of topo_resid_loc
             !
             IF (topo_resid_loc(ig,ib) .GT. 0 ) THEN
                !
                IF ( use_exp ) THEN
                    !
                    ! Trung: Testing the formula which suggested by Agnes DUCHARNE
                    !
                    flow = MIN(fast_reservoir(ig,ib)*(1 - EXP((-1*dt_routing)/(one_day*(topo_resid_loc(ig,ib)/1000.)*fast_tcst))),&
                         & fast_reservoir(ig,ib)-min_sechiba)
                    fast_flow(ig,ib) = MAX(flow, zero)
                    !
                    flow = MIN(slow_reservoir(ig,ib)*(1 - EXP((-1*dt_routing)/(one_day*(topo_resid_loc(ig,ib)/1000.)*slow_tcst))),&
                         & slow_reservoir(ig,ib)-min_sechiba)
                    slow_flow(ig,ib) = MAX(flow, zero)
                    !
                    flow = MIN(stream_reservoir(ig,ib)*(1 - EXP((-1*dt_routing)/(one_day*(topo_resid_loc(ig,ib)/1000.)*stream_tcst* &
                         & MAX(un-SQRT(flood_frac_bas(ig,ib)),min_sechiba)))),&
                         & stream_reservoir(ig,ib)-min_sechiba)
                    stream_flow(ig,ib) = MAX(flow, zero)
                ELSE
                    !
                    flow = MIN(fast_reservoir(ig,ib)/((topo_resid_loc(ig,ib)/1000.)*fast_tcst*one_day/dt_routing),&
                         & fast_reservoir(ig,ib)-min_sechiba)
                    fast_flow(ig,ib) = MAX(flow, zero)
                    !
                    flow = MIN(slow_reservoir(ig,ib)/((topo_resid_loc(ig,ib)/1000.)*slow_tcst*one_day/dt_routing),&
                         & slow_reservoir(ig,ib)-min_sechiba)
                    slow_flow(ig,ib) = MAX(flow, zero)
                    !
                    flow = MIN(stream_reservoir(ig,ib)/((topo_resid_loc(ig,ib)/1000.)*stream_tcst* &
                         & MAX(un-SQRT(flood_frac_bas(ig,ib)),min_sechiba)*one_day/dt_routing),&
                         & stream_reservoir(ig,ib)-min_sechiba)
                    stream_flow(ig,ib) = MAX(flow, zero)
                ENDIF
             ELSE
                flow = fast_reservoir(ig,ib)-min_sechiba
                fast_flow(ig,ib) = MAX(flow, zero)
                !
                flow = slow_reservoir(ig,ib)-min_sechiba
                slow_flow(ig,ib) = MAX(flow, zero)
                !
                flow = stream_reservoir(ig,ib)-min_sechiba
                stream_flow(ig,ib) = MAX(flow, zero)
             ENDIF
          ELSE
             fast_flow(ig,ib) = zero
             slow_flow(ig,ib) = zero
             stream_flow(ig,ib) = zero
          ENDIF
       ENDDO
    ENDDO
    !-
    !- Compute the fluxes out of the floodplains and ponds if they exist.
    !-
    IF (do_floodplains .OR. doponds) THEN
       DO ig=1,nbpt
          IF (flood_frac(ig) .GT. min_sechiba) THEN
             !
             flow = MIN(floodout(ig)*totarea(ig)*pond_frac(ig)/flood_frac(ig), pond_reservoir(ig)+totflood(ig))
             pondex = MAX(flow - pond_reservoir(ig), zero)
             pond_reservoir(ig) = pond_reservoir(ig) - (flow - pondex)
             !
             ! If demand was over reservoir size, we will take it out from floodplains
             !
             pond_excessflow(:) = zero
             DO ib=1,nbasmax
                pond_excessflow(ib) = MIN(pondex*flood_frac_bas(ig,ib)/(flood_frac(ig)-pond_frac(ig)),&
                     & flood_reservoir(ig,ib))
                pondex = pondex - pond_excessflow(ib)
             ENDDO
             !
             IF ( pondex .GT. min_sechiba) THEN
                WRITE(numout,*) "Unable to redistribute the excess pond outflow over the water available in the floodplain."
                WRITE(numout,*) "Pondex = ", pondex
                WRITE(numout,*) "pond_excessflow(:) = ", pond_excessflow(:)
             ENDIF
             !
             DO ib=1,nbasmax
                !
                flow = floodout(ig)*routing_area_loc(ig,ib)*flood_frac_bas(ig,ib)/flood_frac(ig) + pond_excessflow(ib)
                !
                flood_reservoir(ig,ib) = flood_reservoir(ig,ib) - flow
                !
                !
                IF (flood_reservoir(ig,ib) .LT. min_sechiba) THEN
                   flood_reservoir(ig,ib) = zero
                ENDIF
                IF (pond_reservoir(ig) .LT. min_sechiba) THEN
                   pond_reservoir(ig) = zero
                ENDIF
             ENDDO
          ENDIF
       ENDDO
    ENDIF

    !-
    !- Computing the drainage and outflow from floodplains
!> Drainage from floodplains is depending on a averaged conductivity (k_litt)
!> for saturated infiltration in the 'litter' layer. Flood_drainage will be
!> a component of the total reinfiltration that leaves the routing scheme.
    !-
    IF (do_floodplains) THEN
       IF (dofloodinfilt) THEN
          DO ib=1,nbasmax
             DO ig=1,nbpt
                flood_drainage(ig,ib) = MAX(zero, MIN(flood_reservoir(ig,ib), &
                     & flood_frac(ig)*routing_area_loc(ig,ib)*k_litt(ig)*dt_routing/one_day))
                flood_reservoir(ig,ib) = flood_reservoir(ig,ib) - flood_drainage(ig,ib)
             ENDDO
          ENDDO
       ELSE
          DO ib=1,nbasmax
             DO ig=1,nbpt
                flood_drainage(ig,ib) = zero
             ENDDO
          ENDDO
       ENDIF
!> Outflow from floodplains is computed depending a delay. This delay is characterized by a time constant
!> function of the surface of the floodplains and the product of topo_resid and flood_tcst. flood_tcst
!> has been calibrated through observations in the Niger Inner Delta (D'Orgeval, 2006).
!
       DO ib=1,nbasmax
          DO ig=1,nbpt
             IF ( route_tobasin_loc(ig,ib) .GT. 0 ) THEN
                IF (flood_frac_bas(ig,ib) .GT. min_sechiba) THEN
                   flow = MIN(flood_reservoir(ig,ib) &
                        & /((topo_resid_loc(ig,ib)/1000.)*flood_tcst* &
                        & flood_frac_bas(ig,ib)*one_day/dt_routing),&
                        & flood_reservoir(ig,ib))
                ELSE
                   flow = zero
                ENDIF
                flood_flow(ig,ib) = flow
             ELSE
                flood_flow(ig,ib) = zero
             ENDIF
          ENDDO
       ENDDO
    ELSE
       DO ib=1,nbasmax
          DO ig=1,nbpt
             flood_drainage(ig,ib) = zero
             flood_flow(ig,ib) = zero
             flood_reservoir(ig,ib) = zero
          ENDDO
       ENDDO
    ENDIF

    !-
    !- Computing drainage and inflow for ponds
!> Drainage from ponds is computed in the same way than for floodplains.
!> Reinfiltrated fraction from the runoff (i.e. the outflow from the fast reservoir)
!> is the inflow of the pond reservoir.
    !-
    IF (doponds) THEN
       ! If used, the slope coef is not used in hydrol for water2infilt
       DO ib=1,nbasmax
          DO ig=1,nbpt
             pond_inflow(ig,ib) = fast_flow(ig,ib) * reinf_slope(ig)
             pond_drainage(ig,ib) = MIN(pond_reservoir(ig)*routing_area_loc(ig,ib)/totarea(ig), &
                  & pond_frac(ig)*routing_area_loc(ig,ib)*k_litt(ig)*dt_routing/one_day)
             fast_flow(ig,ib) = fast_flow(ig,ib) - pond_inflow(ig,ib)
          ENDDO
       ENDDO
    ELSE
       DO ib=1,nbasmax
          DO ig=1,nbpt
             pond_inflow(ig,ib) = zero
             pond_drainage(ig,ib) = zero
             pond_reservoir(ig) = zero
          ENDDO
       ENDDO
    ENDIF

!ym cette methode conserve les erreurs d'arrondie
!ym mais n'est pas la plus efficace

    !-
    !- Compute the transport from one basin to another
    !-

    IF (is_root_prc) THEN
       ALLOCATE( fast_flow_g(nbp_glo, nbasmax), slow_flow_g(nbp_glo, nbasmax), &
            stream_flow_g(nbp_glo, nbasmax), stat=ier)
    ELSE
       ALLOCATE( fast_flow_g(1,1), slow_flow_g(1,1), &
            stream_flow_g(1, 1), stat=ier)
    ENDIF
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_flow','Pb in allocate for fast_flow_g','','')

    CALL gather(fast_flow,fast_flow_g)
    CALL gather(slow_flow,slow_flow_g)
    CALL gather(stream_flow,stream_flow_g)

    IF (is_root_prc) THEN
       DO ib=1,nbasmax
          DO ig=1,nbp_glo
             !
             rtg = route_togrid_glo(ig,ib)
             rtb = route_tobasin_glo(ig,ib)
             transport_glo(rtg,rtb) = transport_glo(rtg,rtb) + fast_flow_g(ig,ib) + slow_flow_g(ig,ib) + &
                  & stream_flow_g(ig,ib)
             !
          ENDDO
       ENDDO
    ENDIF

    DEALLOCATE( fast_flow_g, slow_flow_g, stream_flow_g )

    CALL scatter(transport_glo,transport)

    !-
    !- Do the floodings - First initialize
    !-
    return_swamp(:,:)=zero
    floods(:,:)=zero
    !-
!> Over swamp areas, a fraction of water (return_swamp) is withdrawn from the river depending on the
!> parameter swamp_cst.
!> It will be transferred into soil moisture and thus does not return directly to the river.
    !
    !- 1. Swamps: Take out water from the river to put it to the swamps
    !-
    !
    IF ( doswamps ) THEN
       tobeflooded(:) = swamp(:)
       DO ib=1,nbasmax
          DO ig=1,nbpt
             potflood(ig,ib) = transport(ig,ib)
             !
             IF ( tobeflooded(ig) > 0. .AND. potflood(ig,ib) > 0. .AND. floodtemp(ig) > tp_00 ) THEN
                !
                IF (routing_area_loc(ig,ib) > tobeflooded(ig)) THEN
                   floodindex = tobeflooded(ig) / routing_area_loc(ig,ib)
                ELSE
                   floodindex = 1.0
                ENDIF
                return_swamp(ig,ib) = swamp_cst * potflood(ig,ib) * floodindex
                !
                tobeflooded(ig) = tobeflooded(ig) - routing_area_loc(ig,ib)
                !
             ENDIF
          ENDDO
       ENDDO
    ENDIF
    !-
    !- 2. Floodplains: Update the reservoir with the flux computed above.
    !-
    IF ( do_floodplains ) THEN
       DO ig=1,nbpt
          IF (floodplains(ig) .GT. min_sechiba .AND. floodtemp(ig) .GT. tp_00) THEN
             DO ib=1,nbasmax
                floods(ig,ib) = transport(ig,ib) - return_swamp(ig,ib)
             ENDDO
          ENDIF
       ENDDO
    ENDIF
    !
    ! Update all reservoirs
!> The slow and deep reservoir (slow_reservoir) collect the deep drainage whereas the
!> fast_reservoir collects the computed surface runoff. Both discharge into a third reservoir
!> (stream_reservoir) of the next sub-basin downstream.
!> Water from the floodplains reservoir (flood_reservoir) flows also into the stream_reservoir of the next sub-basin downstream.
!> Water that flows into the pond_reservoir is withdrawn from the fast_reservoir.
    !

    DO ig=1,nbpt
       DO ib=1,nbasmax
          fast_reservoir(ig,ib) = fast_reservoir(ig,ib) + runoff(ig)*routing_area_loc(ig,ib) - &
               & fast_flow(ig,ib) - pond_inflow(ig,ib)
          
          slow_reservoir(ig,ib) = slow_reservoir(ig,ib) + drainage(ig)*routing_area_loc(ig,ib) - &
               & slow_flow(ig,ib)
          
          stream_reservoir(ig,ib) = stream_reservoir(ig,ib) + flood_flow(ig,ib) + transport(ig,ib) - &
               & stream_flow(ig,ib) - return_swamp(ig,ib) - floods(ig,ib)
          !
          flood_reservoir(ig,ib) = flood_reservoir(ig,ib) + floods(ig,ib) - &
               & flood_flow(ig,ib)
          !
          pond_reservoir(ig) = pond_reservoir(ig) + pond_inflow(ig,ib) - pond_drainage(ig,ib)
          !
          IF ( flood_reservoir(ig,ib) .LT. zero ) THEN
             IF ( check_reservoir ) THEN
                WRITE(numout,*) "WARNING : negative flood reservoir at :", ig, ib, ". Problem is being corrected."
                WRITE(numout,*) "flood_reservoir, floods, flood_flow : ", flood_reservoir(ig,ib), floods(ig,ib), &
                     & flood_flow(ig,ib)
             ENDIF
             stream_reservoir(ig,ib) = stream_reservoir(ig,ib) + flood_reservoir(ig,ib)
             flood_reservoir(ig,ib) = zero
          ENDIF
          !
          IF ( stream_reservoir(ig,ib) .LT. zero ) THEN
             IF ( check_reservoir ) THEN
                WRITE(numout,*) "WARNING : negative stream reservoir at :", ig, ib, ". Problem is being corrected."
                WRITE(numout,*) "stream_reservoir, flood_flow, transport : ", stream_reservoir(ig,ib), flood_flow(ig,ib), &
                     & transport(ig,ib)
                WRITE(numout,*) "stream_flow, return_swamp, floods :", stream_flow(ig,ib), return_swamp(ig,ib), floods(ig,ib)
             ENDIF
             fast_reservoir(ig,ib) = fast_reservoir(ig,ib) + stream_reservoir(ig,ib)
             stream_reservoir(ig,ib) = zero
          ENDIF
          !
          IF ( fast_reservoir(ig,ib) .LT. zero ) THEN
             IF ( check_reservoir ) THEN
                WRITE(numout,*) "WARNING : negative fast reservoir at :", ig, ib, ". Problem is being corrected."
                WRITE(numout,*) "fast_reservoir, runoff, fast_flow, ponf_inflow  : ", fast_reservoir(ig,ib), &
                     &runoff(ig), fast_flow(ig,ib), pond_inflow(ig,ib)
             ENDIF
             slow_reservoir(ig,ib) = slow_reservoir(ig,ib) + fast_reservoir(ig,ib)
             fast_reservoir(ig,ib) = zero
          ENDIF
          !
          ! CWRR produces some very small negative drainage values which tend to accumulate in slow_reservoir
          ! and thus lead to some negative values in ver dry areas. 
          !
          ! First try to fill the slow_reservoir with water from the fast and stream reservoirs.
          ! If that is not enough then we print a WARNING.
          IF ( slow_reservoir(ig,ib) .LT. - min_sechiba ) THEN
             !
             flow = MIN( -slow_reservoir(ig,ib), fast_reservoir(ig,ib))
             slow_reservoir(ig,ib) = slow_reservoir(ig,ib) + flow
             fast_reservoir(ig,ib) = fast_reservoir(ig,ib) - flow
          ENDIF
          !
          ! Second attemps use water from the stream reservoir.
          !
          IF ( slow_reservoir(ig,ib) .LT. - min_sechiba ) THEN
             !
             flow = MIN( -slow_reservoir(ig,ib), stream_reservoir(ig,ib))
             slow_reservoir(ig,ib) = slow_reservoir(ig,ib) + flow
             stream_reservoir(ig,ib) = stream_reservoir(ig,ib) - flow
          ENDIF
          !
          ! The problem is not solved so we print a WARNING when the matter gets
          ! out of hand.
          !
          IF ( slow_reservoir(ig,ib) .LT. low_reservoir_warning ) THEN
             !
             WRITE(numout,*) 'WARNING : There is a negative reservoir at :', ig, ib,lalo(ig,:)
             WRITE(numout,*) 'WARNING : slowr, drainage :', &
                  & slow_reservoir(ig,ib), drainage(ig)
             WRITE(numout,*) 'WARNING : stream_reservoir, fast_reservoir : ', &
                  & stream_reservoir(ig,ib),  fast_reservoir(ig,ib)
             CALL ipslerr_p(2, 'routing_reg_flow', 'WARNING negative slow_reservoir.','','')
          ENDIF

       ENDDO
    ENDDO
 
    totflood(:) = zero
    DO ig=1,nbpt
       DO ib=1,nbasmax
          totflood(ig) = totflood(ig) + flood_reservoir(ig,ib)
       ENDDO
    ENDDO

    !-
    !- Computes the fraction of floodplains and ponds according to their volume
    !-
    IF (do_floodplains .OR. doponds) THEN
       flood_frac(:) = zero
       flood_height(:) = zero
       flood_frac_bas(:,:) = zero
       DO ig=1, nbpt
          IF (totflood(ig) .GT. min_sechiba) THEN
             ! We first compute the total fraction of the grid box which is flooded at optimum repartition
             flood_frac_pot = (totflood(ig) / (totarea(ig)*floodcri/(beta+un)))**(beta/(beta+un))
             flood_frac(ig) = MIN(floodplains(ig) / totarea(ig), flood_frac_pot)
             ! Then we diagnose the fraction for each basin with the size of its flood_reservoir
             ! (flood_frac_bas may be > 1)
             DO ib=1,nbasmax
                IF (routing_area_loc(ig,ib) .GT. min_sechiba) THEN
                   flood_frac_bas(ig,ib) = flood_frac(ig) * &
                        & (flood_reservoir(ig,ib) / totflood(ig)) / (routing_area_loc(ig,ib) / totarea(ig))
                ENDIF
             ENDDO
             ! We diagnose the maximum height of floodplain
             flood_height(ig) = (beta/(beta+1))*floodcri*(flood_frac(ig))**(un/beta) + totflood(ig)/(totarea(ig)*flood_frac(ig))
             ! And finally add the pond surface
             pond_frac(ig) = MIN(un-flood_frac(ig), ((betap+1)*pond_reservoir(ig) / (pondcri*totarea(ig)))**(betap/(betap+1)) )
             flood_frac(ig) = flood_frac(ig) + pond_frac(ig)
             !
          ENDIF
       ENDDO
    ELSE
       flood_frac(:) = zero
       flood_height(:) = zero
       flood_frac_bas(:,:) = zero
    ENDIF

    !-
    !- Compute the total reinfiltration and returnflow to the grid box
!> A term of returnflow is computed including the water from the swamps that does not return directly to the river
!> but will be put into soil moisture (see hydrol module).
!> A term of reinfiltration is computed including the water that reinfiltrated from the ponds and floodplains areas.
!> It will be put into soil moisture (see hydrol module).
    !-
    IF (do_floodplains .OR. doswamps .OR. doponds) THEN
       returnflow(:) = zero
       reinfiltration(:) = zero
       !
       DO ib=1,nbasmax
          DO ig=1,nbpt
             returnflow(ig) = returnflow(ig) + return_swamp(ig,ib)
             reinfiltration(ig) = reinfiltration(ig) + pond_drainage(ig,ib) + flood_drainage(ig,ib)
          ENDDO
       ENDDO
       !
       DO ig=1,nbpt
          returnflow(ig) = returnflow(ig)/totarea(ig)
          reinfiltration(ig) = reinfiltration(ig)/totarea(ig)
       ENDDO
    ELSE
       returnflow(:) = zero
       reinfiltration(:) = zero
    ENDIF

    !
    ! Compute the net irrigation requirement from Univ of Kassel
    !
    ! This is a very low priority process and thus only applies if
    ! there is some water left in the reservoirs after all other things.
    !
!> The computation of the irrigation is performed here.
!> * First step
!> In a first time, the water requirements (irrig_netereq) by the crops for their optimal growth are calculated
!> over each irrigated fraction (irrigated(ig)/totarea(ig)). It is the difference
!> between the maximal water loss by the crops (transpot_mean) and the net water amount kept by the soil
!> (precipitation and reinfiltration). Transpot_mean is computed in the routines enerbil and diffuco. It
!> is derived from the effective transpiration parametrization under stress-free conditions, called potential transpiration.
!> Crop_coef was used by a previous parametrization of irrigation in the code. Here, its value is equal to one.
!> The crop coefficient was constant in space and time to represent a mean resistance of the vegetation to the potential evaporation.
!> Now, the term crop_coef*Epot is substituted by transpot_mean (see Guimberteau et al., 2011).
!> * Second step
!> We compute irrigation needs in order to supply Irrig_netereq. Water for irrigation (irrig_actual) is withdrawn
!> from the reservoirs. The amount of water is withdrawn in priority from the stream reservoir.
!> If the irrigation requirement is higher than the water availability of the reservoir, water is withdrawn
!> from the fast reservoir or, in the extreme case, from the slow reservoir.
!> * Third step
!> We compute a deficit in water for irrigation. If it is positive, irrigation (depending on water availibility in the reservoirs)
!> has not supplied the crops requirements.
!
    IF ( do_irrigation ) THEN
       DO ig=1,nbpt
          !
          IF ((vegtot(ig) .GT. min_sechiba) .AND. (humrel(ig) .LT. un-min_sechiba) .AND. &
               & (runoff(ig) .LT. min_sechiba) ) THEN

             irrig_netereq(ig) = (irrigated(ig) / totarea(ig) ) * MAX(zero, transpot_mean(ig) - &
                  & (precip(ig)+reinfiltration(ig)) )

          ENDIF
          !
          DO ib=1,nbasmax
             IF ( routing_area_loc(ig,ib) .GT. 0 ) THEN

                irrig_needs(ig,ib) = irrig_netereq(ig) * routing_area_loc(ig,ib)

                irrig_actual(ig,ib) = MIN(irrig_needs(ig,ib),&
                     & stream_reservoir(ig,ib) + fast_reservoir(ig,ib) + slow_reservoir(ig,ib) )

                slow_reservoir(ig,ib) = MAX(zero, slow_reservoir(ig,ib) + &
                     & MIN(zero, fast_reservoir(ig,ib) + MIN(zero, stream_reservoir(ig,ib)-irrig_actual(ig,ib))))

                fast_reservoir(ig,ib) = MAX( zero, &
                     & fast_reservoir(ig,ib) + MIN(zero, stream_reservoir(ig,ib)-irrig_actual(ig,ib)))

                stream_reservoir(ig,ib) = MAX(zero, stream_reservoir(ig,ib)-irrig_actual(ig,ib) )

                irrig_deficit(ig,ib) = irrig_needs(ig,ib)-irrig_actual(ig,ib)

             ENDIF
          ENDDO
          !
          ! Check if we cannot find the missing water in another basin of the same grid (stream reservoir only).
          ! If we find that then we create some adduction from that subbasin to the one where we need it for
          ! irrigation.
          !
!> If crops water requirements have not been supplied (irrig_deficit>0), we check if we cannot find the missing water
!> in another basin of the same grid. If there is water in the stream reservoir of this subbasin, we create some adduction
!> from that subbasin to the one where we need it for irrigation.
!>
          DO ib=1,nbasmax

             stream_tot = SUM(stream_reservoir(ig,:))

             DO WHILE ( irrig_deficit(ig,ib) > min_sechiba .AND. stream_tot > min_sechiba)

                fi = MAXLOC(stream_reservoir(ig,:))
                ib2 = fi(1)

                irrig_adduct(ig,ib) = MIN(irrig_deficit(ig,ib), stream_reservoir(ig,ib2))
                stream_reservoir(ig,ib2) = stream_reservoir(ig,ib2)-irrig_adduct(ig,ib)
                irrig_deficit(ig,ib) = irrig_deficit(ig,ib)-irrig_adduct(ig,ib)

                stream_tot = SUM(stream_reservoir(ig,:))

             ENDDO

          ENDDO
          !
       ENDDO
       !
       ! If we are at higher resolution we might need to look at neighboring grid boxes to find the streams
       ! which can feed irrigation
!
!> At higher resolution (grid box smaller than 100x100km), we can import water from neighboring grid boxes
!> to the one where we need it for irrigation.
       !
       IF (is_root_prc) THEN
          ALLOCATE(irrig_deficit_glo(nbp_glo, nbasmax), stream_reservoir_glo(nbp_glo, nbasmax), &
               & irrig_adduct_glo(nbp_glo, nbasmax), stat=ier)
       ELSE
          ALLOCATE(irrig_deficit_glo(0, 0), stream_reservoir_glo(0, 0), &
               & irrig_adduct_glo(0, 0), stat=ier)
       ENDIF
       IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_flow','Pb in allocate for irrig_deficit_glo, stream_reservoir_glo,...','','')

       CALL gather(irrig_deficit, irrig_deficit_glo)
       CALL gather(stream_reservoir, stream_reservoir_glo)
       CALL gather(irrig_adduct, irrig_adduct_glo)

       IF (is_root_prc) THEN
          !
          DO ig=1,nbp_glo
             ! Only work if the grid box is smaller than 100x100km. Else the piplines we build
             ! here would be too long to be reasonable.
             IF ( resolution_g(ig,1) < 100000. .AND. resolution_g(ig,2) < 100000. ) THEN
                DO ib=1,nbasmax
                   !
                   IF ( irrig_deficit_glo(ig,ib) > min_sechiba ) THEN
                      !
                      streams_around(:,:) = zero
                      !
                      DO in=1,NbNeighb
                         ig2 = routing_nextgrid_g(ig,in)
                         IF (ig2 .GT. 0 ) THEN
                            streams_around(in,:) = stream_reservoir_glo(ig2,:)
                            igrd(in) = ig2
                         ENDIF
                      ENDDO
                      !
                      IF ( MAXVAL(streams_around) .GT. zero ) THEN
                         !
                         ff=MAXLOC(streams_around)
                         ig2=igrd(ff(1))
                         ib2=ff(2)
                         !
                         IF ( routing_area_glo(ig2,ib2) .GT. 0 .AND. stream_reservoir_glo(ig2,ib2) > zero ) THEN
                            adduction = MIN(irrig_deficit_glo(ig,ib), stream_reservoir_glo(ig2,ib2))
                            stream_reservoir_glo(ig2,ib2) = stream_reservoir_glo(ig2,ib2) - adduction
                            irrig_deficit_glo(ig,ib) = irrig_deficit_glo(ig,ib) - adduction
                            irrig_adduct_glo(ig,ib) = irrig_adduct_glo(ig,ib) + adduction
                         ENDIF
                         !
                      ENDIF
                      !
                   ENDIF
                   !
                ENDDO
             ENDIF
          ENDDO
          !
       ENDIF
       !

       CALL scatter(irrig_deficit_glo, irrig_deficit)
       CALL scatter(stream_reservoir_glo, stream_reservoir)
       CALL scatter(irrig_adduct_glo, irrig_adduct)

       DEALLOCATE(irrig_deficit_glo, stream_reservoir_glo, irrig_adduct_glo)

    ENDIF

    !! Calculate the net water flow to each routing reservoir (in kg/dt)
    !! to further diagnose the corresponding water budget residu
    !! in routing_main

    netflow_fast_diag(:) = zero
    netflow_slow_diag(:) = zero
    netflow_stream_diag(:) = zero

    DO ib=1,nbasmax
       DO ig=1,nbpt
          netflow_fast_diag(ig) = netflow_fast_diag(ig) + runoff(ig)*routing_area(ig,ib) &
               - fast_flow(ig,ib) - pond_inflow(ig,ib)
          netflow_slow_diag(ig) = netflow_slow_diag(ig) + drainage(ig)*routing_area(ig,ib) &
               - slow_flow(ig,ib)
          netflow_stream_diag(ig) = netflow_stream_diag(ig) + flood_flow(ig,ib) + transport(ig,ib) &
               - stream_flow(ig,ib) - return_swamp(ig,ib) - floods(ig,ib)
       ENDDO
    ENDDO

    !! Grid cell averaging
    DO ig=1,nbpt
       netflow_fast_diag(ig) = netflow_fast_diag(ig)/totarea(ig)
       netflow_slow_diag(ig) = netflow_slow_diag(ig)/totarea(ig)
       netflow_stream_diag(ig) = netflow_stream_diag(ig)/totarea(ig)
    ENDDO
    
    !
    ! Compute the fluxes which leave the routing scheme
    !
    ! Lakeinflow is in Kg/dt
    ! returnflow is in Kg/m^2/dt
    !
    hydrographs(:) = zero
    slowflow_diag(:) = zero
    fast_diag(:) = zero
    slow_diag(:) = zero
    stream_diag(:) = zero
    flood_diag(:) = zero
    pond_diag(:) = zero
    irrigation(:) = zero
    IF(ok_grdc) mergedhydro_grdc_match_loc(:,:) = zero
    IF(ok_grdc) hydrographs_largest_loc(:) = zero
    !
    !
    !DO ib=1,nbasmax
    !   !
    !   DO ig=1,nbpt
    !      IF (hydrodiag(ig,ib) > 0 ) THEN
    !         hydrographs(ig) = hydrographs(ig) + fast_flow(ig,ib) + slow_flow(ig,ib) + &
    !              & stream_flow(ig,ib)
    !      ENDIF
    !      fast_diag(ig) = fast_diag(ig) + fast_reservoir(ig,ib)
    !      slow_diag(ig) = slow_diag(ig) + slow_reservoir(ig,ib)
    !      stream_diag(ig) = stream_diag(ig) + stream_reservoir(ig,ib)
    !      flood_diag(ig) = flood_diag(ig) + flood_reservoir(ig,ib)
    !      irrigation (ig) = irrigation (ig) + irrig_actual(ig,ib) + irrig_adduct(ig,ib)
    !   ENDDO
    !ENDDO
    i = 0
    j = 0
    DO ig=1,nbpt
       DO ib=1,nbasmax
          IF (hydrodiag_loc(ig,ib) > 0 ) THEN
             hydrographs(ig) = hydrographs(ig) + fast_flow(ig,ib) + slow_flow(ig,ib) + &
                  & stream_flow(ig,ib)
             !WRITE(numout,*) 'i, hydrographs', i, hydrographs(ig)
          ENDIF
          IF(ok_grdc) THEN
            IF (hydrodiag_grdc_loc(ig,ib) > 0) THEN
               DO igrdc = 1, nb_grdc_match_loc
                 IF(ogloc_grdc_match_loc(igrdc) .EQ. ig .AND.  &
                     & obloc_grdc_match_loc(igrdc) .EQ. ib) THEN
                    i = i + 1
                    mergedhydro_grdc_match_loc(igrdc,1) = mergedhydro_grdc_match_loc(igrdc,1) + &
                      & fast_flow(ig,ib) + slow_flow(ig,ib) + stream_flow(ig,ib)
                    !WRITE(numout,*) 'i, mergedhydro_grdc_match_loc(i,1)', i, mergedhydro_grdc_match_loc(i,1)
                 ENDIF
               ENDDO ! DO igrdc = 1, nb_grdc_match_loc
            ENDIF

            IF (hydrodiag_largest_loc(ig,ib) > 0) THEN
               DO ilargest = 1, nb_largest_loc
                 IF(ogloc_largest_loc(ilargest) .EQ. ig .AND.  &
                     & obloc_largest_loc(ilargest) .EQ. ib) THEN
                     j = j + 1
                     hydrographs_largest_loc(ilargest) = hydrographs_largest_loc(ilargest) + &
                       & fast_flow(ig,ib) + slow_flow(ig,ib) + stream_flow(ig,ib)
                     !WRITE(numout,*) 'j, hydrograohs_largest_loc(j)', j, hydrographs_largest_loc(j)
                 ENDIF
               ENDDO ! DO ilarge = 1, nb_largest_loc
            ENDIF

!            IF (hydrodiag_grdc_loc(ig,ib) > 0) THEN
!              i = i + 1
!              mergedhydro_grdc_match_loc(i,1) = mergedhydro_grdc_match_loc(i,1) + &
!                  & fast_flow(ig,ib) + slow_flow(ig,ib) + stream_flow(ig,ib)
!              !WRITE(numout,*) 'i, mergedhydro_grdc_match_loc(i,1)', i, mergedhydro_grdc_match_loc(i,1)
!            ENDIF
!
!            IF (hydrodiag_largest_loc(ig,ib) > 0) THEN
!              j = j + 1
!              hydrographs_largest_loc(j) = hydrographs_largest_loc(j) + &
!                  & fast_flow(ig,ib) + slow_flow(ig,ib) + stream_flow(ig,ib)
!              !WRITE(numout,*) 'j, hydrograohs_largest_loc(j)', j, hydrographs_largest_loc(j)
!            ENDIF
          ENDIF

          fast_diag(ig) = fast_diag(ig) + fast_reservoir(ig,ib)
          slow_diag(ig) = slow_diag(ig) + slow_reservoir(ig,ib)
          stream_diag(ig) = stream_diag(ig) + stream_reservoir(ig,ib)
          flood_diag(ig) = flood_diag(ig) + flood_reservoir(ig,ib)
          irrigation (ig) = irrigation (ig) + irrig_actual(ig,ib) + irrig_adduct(ig,ib)
       ENDDO
    ENDDO
    !
    ! Transfer the outflow of the largest rivers.
    !
    outflow_loc(:) = zero
    DO in=1,outflow_nb_loc
       ig = outflow_pt_loc(in,1)
       ib = outflow_pt_loc(in,2)
       outflow_loc(in) = fast_flow(ig,ib) + slow_flow(ig,ib) + stream_flow(ig,ib)
    ENDDO
    !
    DO ig=1,nbpt
       fast_diag(ig) = fast_diag(ig)/totarea(ig)
       slow_diag(ig) = slow_diag(ig)/totarea(ig)
       stream_diag(ig) = stream_diag(ig)/totarea(ig)
       flood_diag(ig) = flood_diag(ig)/totarea(ig)
       pond_diag(ig) = pond_reservoir(ig)/totarea(ig)
       !
       irrigation(ig) = irrigation(ig)/totarea(ig)
       !
       ! The three output types for the routing : endoheric basins,, rivers and
       ! diffuse coastal flow.
       !
       lakeinflow(ig) = transport(ig,nbasmax+1)
       coastalflow(ig) = transport(ig,nbasmax+2)
       riverflow(ig) = transport(ig,nbasmax+3)
       !
    ENDDO
    !
    flood_res = flood_diag + pond_diag
    !

    !! Remove water from lake reservoir if it exceeds the maximum limit and distribute it 
    !! uniformly over all possible the coastflow gridcells
    
    ! Calculate lake_overflow and remove it from lake_reservoir
    DO ig=1,nbpt
       lake_overflow(ig) = MAX(0., lake_reservoir(ig) - max_lake_reservoir*totarea(ig))
       lake_reservoir(ig) = lake_reservoir(ig) - lake_overflow(ig)
    END DO
    ! Transform lake_overflow from kg/grid-cell/dt_routing into kg/m^2/s
    CALL xios_orchidee_send_field("lake_overflow",lake_overflow(:)/totarea(:)/dt_routing)

    ! Calculate the sum of the lake_overflow and distribute it uniformly over all gridboxes
    CALL gather(lake_overflow,lake_overflow_g)
    IF (is_root_prc) THEN
       total_lake_overflow=SUM(lake_overflow_g)
    END IF
    CALL bcast(total_lake_overflow)

    ! Distribute the lake_overflow uniformly over all coastal gridcells
    ! lake_overflow_coast is only calculated to be used as diagnostics if needed
    DO ig=1,nbpt
       coastalflow(ig) = coastalflow(ig) + total_lake_overflow/nb_coast_gridcells * mask_coast(ig)
       lake_overflow_coast(ig) = total_lake_overflow/nb_coast_gridcells * mask_coast(ig)
    END DO
    ! Transform from kg/grid-cell/dt_routing into m^3/grid-cell/s to match output unit of coastalflow
    CALL xios_orchidee_send_field("lake_overflow_coast",lake_overflow_coast/mille/dt_routing)
    
  END SUBROUTINE routing_reg_flow
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_lake
!!
!>\BRIEF : This subroutine stores water in lakes so that it does not cycle through the runoff.
!! For the moment it only works for endoheric lakes but I can be extended in the future.
!!
!! DESCRIPTION (definitions, functional, design, flags): The return flow to the soil moisture reservoir
!! is based on a maximum lake evaporation rate (maxevap_lake). \n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_lake(nbpt, dt_routing, lakeinflow, humrel, return_lakes)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    REAL(r_std), INTENT (in)   :: dt_routing !! Routing time step (s)
    REAL(r_std), INTENT(out)   :: lakeinflow(nbpt) !! Water inflow to the lakes (kg/dt)
    REAL(r_std), INTENT(in)    :: humrel(nbpt) !! Soil moisture stress, root extraction potential (unitless)
    !
!! OUTPUT VARIABLES
    REAL(r_std), INTENT(out) :: return_lakes(nbpt) !! Water from lakes flowing back into soil moisture (kg/m^2/dt)
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: ig !! Indices (unitless)
    REAL(r_std) :: refill !!
    REAL(r_std) :: total_area !! Sum of all the surfaces of the basins (m^2)

!_ ================================================================================================================================
    !
    !
    DO ig=1,nbpt
       !
       total_area = SUM(routing_area_loc(ig,:))
       !
       lake_reservoir(ig) = lake_reservoir(ig) + lakeinflow(ig)
       !
       IF ( doswamps ) THEN
          !uptake in Kg/dt
          refill = MAX(zero, maxevap_lake * (un - humrel(ig)) * dt_routing * total_area)
          return_lakes(ig) = MIN(refill, lake_reservoir(ig))
          lake_reservoir(ig) = lake_reservoir(ig) - return_lakes(ig)
          !Return in Kg/m^2/dt  
          return_lakes(ig) = return_lakes(ig)/total_area
       ELSE
          return_lakes(ig) = zero
       ENDIF
       !
       ! This is the volume of the lake scaled to the entire grid.
       ! It would be batter to scale it to the size of the lake
       ! but this information is not yet available.
       lake_diag(ig) = lake_reservoir(ig)/total_area
       !
       lakeinflow(ig) = lakeinflow(ig)/total_area
       !
    ENDDO
    !
  END SUBROUTINE routing_reg_lake
  !

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_readgrdc
!!
!>\BRIEF : This subroutine read GRDC observation dataset (nc file).
!!
!! DESCRIPTION (definitions, functional, design, flags): 
!!   (1) Read GRDC; 
!!   (2) Cut the GRDC data to pre-defined domain
!! . \n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_readgrdc !(kjit, nbpt) !, lakeinflow, humrel, return_lakes)
    !
    USE netcdf
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    !INTEGER(i_std), INTENT(in) :: kjit !! Time step number (unitless)
    !INTEGER(i_std), INTENT(in) :: nbpt                   !! Domain size (unitless)
    !
!! OUTPUT VARIABLES
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: ig, ib, i, j !! Indices (unitless)
    INTEGER(i_std) :: igrdc, jgrdc, lgrdc, tgrdc, fid , ttgrdc_cut !! Indices (unitless)
    REAL(r_std) :: lev(1), date, dt
    INTEGER(i_std) :: itau(1) !!
    INTEGER(i_std), PARAMETER :: nsta_obs = 5723                   !! Total number of GRDC stations
    INTEGER(i_std), PARAMETER :: nt_obs = 2496                     !! GRDC time series 
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: lon_obs              !! Longitude of GRDC (Global domain)
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: lat_obs              !! Latitude of GRDC 
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: area_obs             !! km^2 in GRDC, convert to m^2 to keep consistent with ORC
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: mergedhydro_obs   !! GRDC river discharge 
!    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: hydrographs_obs   !! GRDC river discharge 
    !REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: hydrographs_obs_cut, mergedhydro_obs_cut !! GRDC river discharge 
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: time_obs             !! Time of GRDC  
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: year_grdc, month_grdc, day_grdc   !! Year, Month, Day of GRDC
    REAL(r_std), ALLOCATABLE, DIMENSION(:)  :: sec_grdc, jds_grdc                  !! Second, julian day of GRDC
    REAL(r_std) :: jds0_grdc, js0_grdc

!    INTEGER(i_std), PARAMETER :: nstr = 60                        !! Length of GRDC river (station, country) names
    CHARACTER(LEN=nstr), ALLOCATABLE, DIMENSION(:) :: sta_obs      !! Names for GRDC stations
    CHARACTER(LEN=nstr), ALLOCATABLE, DIMENSION(:) :: river_obs    !! Names for GRDC rivers
    CHARACTER(LEN=nstr), ALLOCATABLE, DIMENSION(:) :: country_obs  !! GRDC Country 
!    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: number_obs           !! GRDC No.
!    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: area_obs_reg_sort    !! Sorted (descend) GRDC by areas
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: area_obs_reg_tmp
    !INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: idx_obs_reg_sort     !! Sorted (descend) GRDC index in the domain
    !INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: idx_obs_reg          !! Corresponding GRDC index for specified domain

    INTEGER(i_std) :: iigrdc, iigrdc_reg       !!
    REAL(r_std) :: min_lat_orc, max_lat_orc                        !! Latitude extent of GRDC
    REAL(r_std) :: min_lon_orc, max_lon_orc                        !! Longitude extend of GRDC
    LOGICAL, ALLOCATABLE, DIMENSION(:) :: is_grdc_in_reg           !! Determine if the GRDC station is within the study domain

    ! Finding the correponding GRDC stations
    INTEGER(i_std) :: ALLOC_ERR

    ! NETCDF variables
    INTEGER :: ncid
    INTEGER :: DimID_stations, DimID_time, DimID_str
    INTEGER :: varid_lat, varid_lon, varid_area
    INTEGER :: varid_name, varid_river, varid_country
    INTEGER :: varid_number
    INTEGER :: varid_time
    INTEGER :: varid_hydrographs, varid_mergedhydro

  ! We will learn about the data file and store results in these
  ! program variables.
    INTEGER :: ndims_in, nvars_in, ngatts_in, unlimdimid_in
    LOGICAL :: use_ioipsl = .FALSE.
!    INCLUDE 'mpif.h'
!_ ================================================================================================================================
    !
    !
    ! 1. Read all the GRDC data (root_prc) 

    IF(is_root_prc) THEN
        !IF(use_ioipsl) THEN
        !   CALL flininfo(grdc_file, igrdc, jgrdc, lgrdc, tgrdc, fid)
        !ELSE

        ! Open the file. 
        !! This is part of the netCDF package.
        !! Copyright 2006 University Corporation for Atmospheric Research/Unidata.
        !! See COPYRIGHT file for conditions of use.

        !! This is an example which reads some surface pressure and
        !! temperatures. The data file read by this program is produced
        !! comapnion program sfc_pres_temp_wr.f90. It is intended to illustrate
        !! the use of the netCDF fortran 90 API.

        !! This program is part of the netCDF tutorial:
        !! http://www.unidata.ucar.edu/software/netcdf/docs/netcdf-tutorial

        !! Full documentation of the netCDF Fortran 90 API can be found at:
        !! http://www.unidata.ucar.edu/software/netcdf/docs/netcdf-f90
        WRITE(numout,*) 'grdc_file:', grdc_file
        ALLOC_ERR = nf90_open(grdc_file, nf90_nowrite, ncid) 
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
        WRITE(numout,*) 'grdc_file:', grdc_file
       
        ALLOC_ERR = nf90_inquire(ncid, ndims_in, nvars_in, ngatts_in, unlimdimid_in) 
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
       
        ALLOC_ERR = nf90_inq_dimid(ncid, "stations", DimID_stations)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
          
        ALLOC_ERR = nf90_inquire_dimension(ncid, DimID_stations, len = igrdc)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
          
        ALLOC_ERR = nf90_inq_dimid(ncid, "time", DimID_time)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
          
        ALLOC_ERR = nf90_inquire_dimension(ncid, DimID_time, len = tgrdc)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
          
        !ALLOC_ERR = nf90_inq_dimid(ncid, "str", DimID_str)
        !ALLOC_ERR = nf90_inquire_dimension(ncid, DimID_str, len = nstr)
        !ENDIF ! IF(use_ioipsl)
       
        WRITE(numout, *) "igrdc, jgrdc, lgrdc, tgrdc, fid:", igrdc, jgrdc, lgrdc, tgrdc, fid
        !WRITE(numout, *) "grdc_file:", grdc_file

        !WRITE(numout, *) "ALLOCATE grdc variables"
        ALLOCATE(lon_obs(igrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for lon_obs','','')

        ALLOCATE(lat_obs(igrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for lat_obs','','')

        ALLOCATE (area_obs(igrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for area_obs','','')

!        ALLOCATE (hydrographs_obs(igrdc,tgrdc), STAT=ALLOC_ERR)
!        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for hydrographs_obs','','')

        ALLOCATE (mergedhydro_obs(igrdc,tgrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for mergedhydro_obs','','')

        ALLOCATE(sta_obs(igrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for sta_obs','','')

        ALLOCATE(river_obs(igrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for river_obs','','')

        ALLOCATE(country_obs(igrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for country_obs','','')

!        ALLOCATE(number_obs(igrdc), STAT=ALLOC_ERR)
!        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for number_obs','','')

        ALLOCATE(is_grdc_in_reg(igrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for is_grdc_in_reg','','')

        ALLOCATE(time_obs(tgrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for time_obs','','')

        ALLOCATE(year_grdc(tgrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for year_grdc','','')

        ALLOCATE(month_grdc(tgrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for month_grdc','','')

        ALLOCATE(day_grdc(tgrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for day_grdc','','')

        ALLOCATE(sec_grdc(tgrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for sec_grdc','','')

        ALLOCATE(jds_grdc(tgrdc), STAT=ALLOC_ERR)
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for jds_grdc','','')

        !WRITE(numout, *) size(hydrographs_obs)

        !IF(use_ioipsl) THEN
        !  !CALL flinopen(grdc_file, .FALSE., igrdc, jgrdc, lgrdc, lon_obs, lat_obs, lev, tgrdc, itau, date, dt, fid)
        !  CALL flinget(fid, 'hydrographs', igrdc, tgrdc, 0, 0, 1, 0, hydrographs_obs)
        !  !WRITE(numout, *) "read hydrographs:", hydrographs_obs(1:10,1:10)
        !  CALL flinget(fid, 'mergedhydro', igrdc, tgrdc, 0, 0, 1, 0, mergedhydro_obs)
        !  !WRITE(numout, *) "read mergedhydro:", mergedhydro_obs(1:10,1:10)
        !  CALL flinget(fid, 'area', igrdc, 0, 0, 0, 1, 0, area_obs)
        !  area_obs = area_obs * mille * mille  !convert Km2 (in GRDC) to m2 (in ORC)
        !  !WRITE(numout, *) "read area_obs:", area_obs(1:10)
        !  CALL flinget(fid, 'lat', igrdc, 0, 0, 0, 1, 0, lat_obs)
        !  !WRITE(numout, *) "lat:", lat_obs(1:10)
        !  CALL flinget(fid, 'lon', igrdc, 0, 0, 0, 1, 0, lon_obs)
        !  !WRITE(numout, *) "lon:", lon_obs(1:10)
        !  CALL flinget(fid, 'name', igrdc, nstr, 0, 0, 1, 0, sta_obs)
        !  !WRITE(numout, *) "read name:", size(sta_obs)
        !  CALL flinget(fid, 'river', igrdc, nstr, 0, 0, 1, 0, river_obs)
        !  !WRITE(numout, *) "read river:", size(river_obs)
        !  CALL flinget(fid, 'country', igrdc, nstr, 0, 0, 1, 0, country_obs)
        !  !WRITE(numout, *) "read country:", size(country_obs)
        !  CALL flinget(fid, 'number', igrdc, 0, 0, 0, 1, 0, number_obs)
        !  !WRITE(numout, *) "read number:", number_obs(1:10)
        !  CALL flinget(fid, 'time', tgrdc, 0, 0, 0, 1, 0, time_obs)
        !  !WRITE(numout, *) "read time:", time_obs(1:10)
        !  CALL flinclo(fid)
        !ELSE

        ! Get the varids of the variables.
        ALLOC_ERR = nf90_inq_varid(ncid, 'lat', varid_lat) 
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
       
        ALLOC_ERR = nf90_inq_varid(ncid, 'lon', varid_lon)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
     
        ALLOC_ERR = nf90_inq_varid(ncid, 'area', varid_area) 
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
       
        ALLOC_ERR = nf90_inq_varid(ncid, 'name', varid_name) 
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 

        ALLOC_ERR = nf90_inq_varid(ncid, 'river', varid_river) 
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 

        ALLOC_ERR = nf90_inq_varid(ncid, 'country', varid_country)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 

        ALLOC_ERR = nf90_inq_varid(ncid, 'number', varid_number) 
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 

        ALLOC_ERR = nf90_inq_varid(ncid, 'time', varid_time)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
  
        ALLOC_ERR = nf90_inq_varid(ncid, 'hydrographs', varid_hydrographs) 
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 

        ALLOC_ERR = nf90_inq_varid(ncid, 'mergedhydro', varid_mergedhydro) 
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 

        ! Read the data.
        ALLOC_ERR = nf90_get_var(ncid, varid_lat, lat_obs) 
        !WRITE(numout, *) "read lat_obs:", lat_obs(1:10)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
          
        ALLOC_ERR = nf90_get_var(ncid, varid_lon, lon_obs) 
        !WRITE(numout, *) "read lon_obs:", lon_obs(1:10)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
          
        ALLOC_ERR = nf90_get_var(ncid, varid_area, area_obs) 
        area_obs = area_obs * mille * mille  !convert Km2 (in GRDC) to m2 (in ORC)
        !WRITE(numout, *) "read area_obs:", area_obs(1:10)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
     
        ALLOC_ERR = nf90_get_var(ncid, varid_name, sta_obs) 
        !WRITE(numout, *) "read name:", size(sta_obs)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
       
        ALLOC_ERR = nf90_get_var(ncid, varid_river, river_obs) 
        !WRITE(numout, *) "read river:", size(river_obs)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
          
        ALLOC_ERR = nf90_get_var(ncid, varid_country, country_obs) 
        !WRITE(numout, *) "read country:", size(country_obs)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
       
!        ALLOC_ERR = nf90_get_var(ncid, varid_number, number_obs) 
!        !WRITE(numout, *) "read number:", number_obs(1:10)
!        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 
       
        ALLOC_ERR = nf90_get_var(ncid, varid_time, time_obs) 
        !WRITE(numout, *) "read time:", time_obs(1:10)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 

!        ALLOC_ERR = nf90_get_var(ncid, varid_hydrographs, hydrographs_obs) 
!        !WRITE(numout, *) "read hydrographs:", hydrographs_obs(1:10,1:10)
!        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 

        ALLOC_ERR = nf90_get_var(ncid, varid_mergedhydro, mergedhydro_obs) 
        !WRITE(numout, *) "read mergedhydro:", mergedhydro_obs(1:10,1:10)
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 

        ! Close the file. 
        ALLOC_ERR = nf90_close(ncid) 
        IF(ALLOC_ERR /= nf90_noerr) CALL handle_err(ALLOC_ERR) 

        !ENDIF ! IF(use_ioipsl)
        ! If we got this far, everything worked as expected. Yipee! 
        WRITE(numout, *) "*** SUCCESS reading GRDC file!"

      ! 2. Cut the full GRDC data based on the model time range and spatial domain (root_prc)

        CALL ymds2ju (year0_grdc, month0_grdc, day0_grdc, sec0_grdc, jds0_grdc)
        !CALL ymds2ju (year, month, day, sec, jds)

        js0_grdc = jds0_grdc * one_day
        !js = jds * one_day
        !djs0 = js0_grdc - jds0_mod * one_day
        !WRITE(numout,*) 'year0_grdc,month0_grdc,day0_grdc,sec0_grdc:', year0_grdc,month0_grdc,day0_grdc,sec0_grdc
        !WRITE(numout,*) 'year,month,day,sec:', year,month,day,sec
        !WRITE(numout,*) 'jds,js:', jds, js
        !WRITE(numout,*) 'jds0_grdc,js0_grdc, jds0_mod,js0_mod:', jds0_grdc, js0_grdc, jds0_mod, jds0_mod * 86400.0
        !WRITE(numout,*) 'djs0:',djs0
        !WRITE(numout,*) 'js+djs0', js+djs0
        !!js = REAL(INT(js + 0.5)) * 1.0
        !!WRITE(numout,*) 'jds,js:', jds, js
        year_grdc(:) = 0
        month_grdc(:) = 0
        day_grdc(:) = 0
        sec_grdc(:) = 0

        tgrdc_cut = 0
        DO i = 1, tgrdc 
           jds_grdc(i) = (time_obs(i) + js0_grdc) / one_day
           CALL ju2ymds (jds_grdc(i), year_grdc(i), month_grdc(i), day_grdc(i), sec_grdc(i))
           !WRITE(numout,*) "year, year_grdc(i),i",year,year_grdc(i),i
           IF(year_grdc(i) >= year) THEN
              tgrdc_cut = tgrdc_cut + 1
           ENDIF
           !WRITE(numout,*) 'year_grdc, month_grdc, day_grdc, sec_grdc=', &
           !    & year_grdc(i), month_grdc(i), day_grdc(i), sec_grdc(i)
           !WRITE(numout,*) 'jds_grdc(i), time_obs(i), i=', jds_grdc(i), time_obs(i), i
        ENDDO
        !WRITE(numout, *) "tgrdc_cut = ", tgrdc_cut

        ! selecting the GRDC stations within the predefined domain (to save some computing time)
        is_grdc_in_reg(:) = .FALSE.
        min_lat_orc = MINVAL(route_outlet_glo(:,:,1),mask=route_outlet_glo(:,:,1).GE.-90.)
        max_lat_orc = MAXVAL(route_outlet_glo(:,:,1),mask=route_outlet_glo(:,:,1).LE.90.)
        min_lon_orc = MINVAL(route_outlet_glo(:,:,2),mask=route_outlet_glo(:,:,2).GE.-180.)
        max_lon_orc = MAXVAL(route_outlet_glo(:,:,2),mask=route_outlet_glo(:,:,2).LE.180.)

        WRITE(numout, *) ' Selecting the GRDC stations within the predefined domain: '
        WRITE(numout, '(" minlat:", F12.6, "; minlon:", F12.6)')  min_lat_orc, min_lon_orc
        WRITE(numout, '(" maxlat:", F12.6, "; maxlon:", F12.6)')  max_lat_orc, max_lon_orc
        WRITE(numout, *) '==> Matching the GRDC stations and ORC outlets:'

        DO iigrdc = 1, igrdc
            IF(lat_obs(iigrdc) .GE. min_lat_orc  - thres_dlola &
                & .AND. lat_obs(iigrdc) .LE. max_lat_orc + thres_dlola &
                & .AND. lon_obs(iigrdc) .GE. min_lon_orc - thres_dlola &
                & .AND. lon_obs(iigrdc) .LE. max_lon_orc + thres_dlola) THEN
            !WRITE(numout, '(A20, A20, F14.4, F14.6, F14.6, I10)') &
            !   & river_obs(iigrdc), sta_obs(iigrdc), area_obs(iigrdc)/mille/mille, & 
            !   & lat_obs(iigrdc), lon_obs(iigrdc), iigrdc
            is_grdc_in_reg(iigrdc) = .TRUE.
            ENDIF
        ENDDO
        igrdc_reg = COUNT(is_grdc_in_reg.EQ..TRUE.)
    ELSE 
        WRITE(numout, *) "reading GRDC on root_proc..."
    ENDIF !(is_root_prc)
    WRITE(numout, *) "Finishe reading GRDC data."

   ! 3. Put the GRDC variables into igrdc_reg and tgrdc_cut domain (do it on all processors)
   ! 
    CALL bcast(igrdc_reg)
    CALL bcast(tgrdc_cut)
    WRITE(numout, *) "The full GRDC data will be cut by predefined domain size: "
    WRITE(numout, *) "tgrdc_cut, igrdc_reg = ", tgrdc_cut, igrdc_reg

    !! ALLOCATE  
    ALLOCATE(year_grdc_cut(tgrdc_cut), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for year_grdc_cut','','')

    ALLOCATE(month_grdc_cut(tgrdc_cut), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for month_grdc_cut','','')

    ALLOCATE(day_grdc_cut(tgrdc_cut), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for day_grdc_cut','','')

    ALLOCATE(sec_grdc_cut(tgrdc_cut), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for sec_grdc_cut','','')

    ALLOCATE(jds_grdc_cut(tgrdc_cut), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for jds_grdc_cut','','')

    !ALLOCATE (mergedhydro_obs_cut(igrdc, tgrdc_cut), STAT=ALLOC_ERR)
    !IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for mergedhydro_obs_cut','','')

    !ALLOCATE (hydrographs_obs_cut(igrdc, tgrdc_cut), STAT=ALLOC_ERR)
    !IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for hydrographs_obs_cut','','')

    !WRITE(*,*) "mpi_rank:", mpi_rank
    ALLOCATE(lon_obs_reg(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for lon_obs_reg','','')

    ALLOCATE(lat_obs_reg(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for lat_obs_reg','','')

    ALLOCATE (area_obs_reg(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for area_obs_reg','','')

    ALLOCATE (sta_obs_reg(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for sta_obs_reg','','')

    ALLOCATE (river_obs_reg(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for river_obs_reg','','')

    ALLOCATE (country_obs_reg(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for country_obs_reg','','')

!    ALLOCATE (number_obs_reg(igrdc_reg), STAT=ALLOC_ERR)
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for number_obs_reg','','')

    ALLOCATE (mergedhydro_obs_reg(igrdc_reg, tgrdc_cut), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for mergedhydro_obs_reg','','')

!      ALLOCATE (hydrographs_obs_reg(igrdc_reg, tgrdc_cut), STAT=ALLOC_ERR)
!      IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for hydrographs_obs_reg','','')

    ALLOCATE (idx_obs_reg(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for idx_obs_reg','','')

    ALLOCATE (idx_obs_reg_sort(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for idx_obs_reg_sort','','')

    ALLOCATE (area_obs_reg_sort(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for area_obs_reg_sort','','')

    ALLOCATE (area_obs_reg_tmp(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_readgrdc','Pb in allocate for area_obs_reg_tmp','','')

    ! 4. Cut GRDC (on root_prc) 
    IF(is_root_prc) THEN
        ttgrdc_cut = 0
        DO i = 1, tgrdc 
           IF(year_grdc(i) >= year) THEN
              ttgrdc_cut = ttgrdc_cut + 1
              jds_grdc_cut(ttgrdc_cut) = jds_grdc(i)
              year_grdc_cut(ttgrdc_cut) = year_grdc(i)
              month_grdc_cut(ttgrdc_cut) = month_grdc(i)
              day_grdc_cut(ttgrdc_cut) = day_grdc(i)
              sec_grdc_cut(ttgrdc_cut) = sec_grdc(i)
              !mergedhydro_obs_cut(:,ttgrdc_cut) = mergedhydro_obs(:,i)
              !hydrographs_obs_cut(:,ttgrdc_cut) = hydrographs_obs(:,i)
           ENDIF
        ENDDO

        ! Pick up the GRDC stations inside the domain
        iigrdc_reg = 0
        idx_obs_reg(:) = 0
        idx_obs_reg_sort(:) = 0
        DO iigrdc = 1, igrdc
            IF(is_grdc_in_reg(iigrdc)) THEN
              iigrdc_reg = iigrdc_reg + 1
              idx_obs_reg(iigrdc_reg) = iigrdc
              lat_obs_reg(iigrdc_reg) = lat_obs(iigrdc)
              lon_obs_reg(iigrdc_reg) = lon_obs(iigrdc)
              area_obs_reg(iigrdc_reg) = area_obs(iigrdc)
              sta_obs_reg(iigrdc_reg) = sta_obs(iigrdc)
              river_obs_reg(iigrdc_reg) = river_obs(iigrdc)
              country_obs_reg(iigrdc_reg) = country_obs(iigrdc)
!              number_obs_reg(iigrdc_reg) = number_obs(iigrdc)
              ttgrdc_cut = 0
              DO i = 1, tgrdc
                 IF(year_grdc(i) .GE.  year) THEN
                    ttgrdc_cut = ttgrdc_cut + 1
                    mergedhydro_obs_reg(iigrdc_reg,ttgrdc_cut) = mergedhydro_obs(iigrdc,i)
!                    hydrographs_obs_reg(iigrdc_reg,ttgrdc_cut) = hydrographs_obs(iigrdc,i)
                 ENDIF
              ENDDO
            ENDIF
        ENDDO
        ! Sorting the secelted GRDC stations according to the upstream area
        WRITE(numout,'(A7, A18, A20, A14, A14, A14, A10)') &
             & "No.", "GRDC River      ", "GRDC Station        ", "Area[km^2]", "Lat", "Lon", "GRDC Idx"
        area_obs_reg_tmp = area_obs_reg
        DO iigrdc_reg = 1, igrdc_reg
           area_obs_reg_sort(iigrdc_reg) = MAXVAL(area_obs_reg_tmp, 1)
           idx_obs_reg_sort(iigrdc_reg) = MAXLOC(area_obs_reg_tmp, 1)
           area_obs_reg_tmp(idx_obs_reg_sort(iigrdc_reg)) = MINVAL(area_obs_reg, 1) - 1.0
           WRITE(numout, '(I6, A2, A20, A20, F14.4, F14.6, F14.6, I10)') &
                & iigrdc_reg, ': ', &
                & river_obs_reg(idx_obs_reg_sort(iigrdc_reg)),&
                & sta_obs_reg(idx_obs_reg_sort(iigrdc_reg)), &
                & area_obs_reg(idx_obs_reg_sort(iigrdc_reg))/mille/mille, &
                & lat_obs_reg(idx_obs_reg_sort(iigrdc_reg)), &
                & lon_obs_reg(idx_obs_reg_sort(iigrdc_reg)), &
                & idx_obs_reg(idx_obs_reg_sort(iigrdc_reg))
        ENDDO
        WRITE(numout, '(" ==> Total No. of GRDC stations in this domain:", I6)') igrdc_reg
    ELSE
      WRITE(numout, *) "Cut GRDC on root_prc"
    ENDIF !IF(is_root_prc)

    !WRITE(numout,*) "size(lon_obs_reg)=", size(lon_obs_reg)
    !WRITE(numout,*) "size(basin_names)=", size(basin_names)
    !WRITE(numout,*) "nb_small=", nb_small
    !WRITE(numout,*) "size(river_obs_reg)=", size(river_obs_reg)

    ! 5. Bcast (on all processors)
    !CALL bcast(diag_grdc)
    !!WRITE(numout,*) "1diag_grdc=", diag_grdc
    CALL bcast(idx_obs_reg)
    !WRITE(numout,*) "idx_obs_reg=", idx_obs_reg
    CALL bcast(lat_obs_reg)
    !WRITE(numout,*) "lat_obs_reg=", lat_obs_reg
    CALL bcast(lon_obs_reg)
    !WRITE(numout,*) "lon_obs_reg=", lon_obs_reg
    CALL bcast(area_obs_reg)
    !WRITE(numout,*) "area_obs_reg=", area_obs_reg
    CALL bcast(sta_obs_reg)
    !WRITE(numout,*) "sta_obs_reg=", sta_obs_reg
    CALL bcast(river_obs_reg)
    !WRITE(numout,*) "river_obs_reg=", river_obs_reg
    CALL bcast(country_obs_reg)
    !WRITE(numout,*) "country_obs_reg=", country_obs_reg
!   CALL bcast(number_obs_reg)
!   !WRITE(numout,*) "number_obs_reg=", number_obs_reg(1:10)
    CALL bcast(mergedhydro_obs_reg)
    !WRITE(numout,*) "mergedhydro_obs_reg=", mergedhydro_obs_reg(1:10,1:10)
!   CALL bcast(hydrographs_obs_reg)
    !WRITE(numout,*) "hydrographs_obs_reg=", hydrographs_obs_reg(1:10,1:10)
    !CALL bcast(time_obs)
    !WRITE(numout,*) "time_obs=", time_obs(1:10)
    CALL bcast(year_grdc_cut)
    !WRITE(numout,*) "year_grdc_cut=", year_grdc_cut(1:10)
    CALL bcast(month_grdc_cut)
    !WRITE(numout,*) "month_grdc_cut=", month_grdc_cut(1:10)
    CALL bcast(day_grdc_cut)
    !WRITE(numout,*) "day_grdc_cut=", day_grdc_cut(1:10)
    CALL bcast(sec_grdc_cut)
    !WRITE(numout,*) "sec_grdc_cut=", sec_grdc_cut(1:10)
    CALL bcast(jds_grdc_cut)
    !WRITE(numout,*) "jds_grdc_cut=", jds_grdc_cut(1:10)
    CALL bcast(area_obs_reg_sort)
    !WRITE(numout,*) "area_obs_reg_sort:", area_obs_reg_sort
    CALL bcast(idx_obs_reg_sort)
    !WRITE(numout,*) "size(area_obs_reg_sort):", size(area_obs_reg_sort)

    ! 6. DEALLOCATE (on all processors)
    IF (ALLOCATED(area_obs))         DEALLOCATE(area_obs, stat=ALLOC_ERR)
    IF (ALLOCATED(lat_obs))          DEALLOCATE(lat_obs, stat=ALLOC_ERR)
    IF (ALLOCATED(lon_obs))          DEALLOCATE(lon_obs, stat=ALLOC_ERR)
    !!IF(ier /= 0) STOP 'area_obs deallocation ERROR'
    !IF (ALLOCATED(hydrographs_obs))  DEALLOCATE(hydrographs_obs,stat=ALLOC_ERR)
    IF (ALLOCATED(mergedhydro_obs))  DEALLOCATE(mergedhydro_obs,stat=ALLOC_ERR)
    !IF (ALLOCATED(hydrographs_obs_cut))  DEALLOCATE(hydrographs_obs_cut,stat=ALLOC_ERR)
    !IF (ALLOCATED(mergedhydro_obs_cut))  DEALLOCATE(mergedhydro_obs_cut,stat=ALLOC_ERR)
    IF (ALLOCATED(river_obs))        DEALLOCATE(river_obs, stat=ALLOC_ERR)
    IF (ALLOCATED(sta_obs))          DEALLOCATE(sta_obs, stat=ALLOC_ERR)
    IF (ALLOCATED(country_obs))      DEALLOCATE(country_obs, stat=ALLOC_ERR)
    !IF (ALLOCATED(number_obs))       DEALLOCATE(number_obs, stat=ALLOC_ERR)
    IF (ALLOCATED(time_obs))         DEALLOCATE(time_obs, stat=ALLOC_ERR)
    IF (ALLOCATED(year_grdc))        DEALLOCATE(year_grdc, stat=ALLOC_ERR)
    IF (ALLOCATED(month_grdc))       DEALLOCATE(month_grdc, stat=ALLOC_ERR)
    IF (ALLOCATED(day_grdc))         DEALLOCATE(day_grdc, stat=ALLOC_ERR)
    IF (ALLOCATED(sec_grdc))         DEALLOCATE(sec_grdc, stat=ALLOC_ERR)
    IF (ALLOCATED(jds_grdc))         DEALLOCATE(jds_grdc, stat=ALLOC_ERR)
    IF (ALLOCATED(is_grdc_in_reg))   DEALLOCATE(is_grdc_in_reg, stat=ALLOC_ERR)
    IF (ALLOCATED(area_obs_reg_tmp))  DEALLOCATE(area_obs_reg_tmp,stat=ALLOC_ERR)

  END SUBROUTINE routing_reg_readgrdc

 
  SUBROUTINE handle_err(status)
     INTEGER, INTENT (IN) :: status
     IF(status /= nf90_noerr) THEN
          WRITE(*,*) trim(nf90_strerror(status))
          STOP "Stopped"
     ENDIF
  END SUBROUTINE handle_err
 

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_xiosgrdc
!!
!>\BRIEF : This subroutine initialize XIOS for GRDC and outflow.
!!         The data in output are ranked by descending order of upstream basin area  
!!
!! DESCRIPTION (definitions, functional, design, flags): 
!! . \n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_xiosgrdc(kjit, nbpt) 
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: kjit                   !! Time step number (unitless)
    INTEGER(i_std), INTENT(in) :: nbpt                   !! Domain size (unitless)
    !
!! OUTPUT VARIABLES
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: ig, ib, i, j
    INTEGER(i_std) :: igrdc, jgrdc, ilargest, jlargest                    !! Indices (unitless)
    REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)::  area_grdc_match_tmp
    REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)::  totarea_largest_tmp
    INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: idxsort_grdc_match_tmp    !! Sorted index for GRDC (by upstream area)
    INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION(:) :: idxsort_largest_tmp       !! Sorted index for Outflow (by upstream area)
    INTEGER(i_std) :: ALLOC_ERR

    INCLUDE 'mpif.h'
!_ ================================================================================================================================
    !
    !
    ! 1. Initialization variables (get nbsta, nblargest)

    ALLOCATE(nbsta(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for nbsta','','')
    ALLOCATE(nbsta_sort(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for nbsta_sort','','')
    ALLOCATE(nbsta_tmp(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for nbsta_tmp','','')
    ALLOCATE(nbsta_idx(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for nbsta_idx','','')
    ALLOCATE(data_ibegin_grdc(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for data_ibegin_grdc','','')
    ALLOCATE(data_iend_grdc(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for data_iend_grdc','','')
    ALLOCATE( lon_sta(nbsta_mpi), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for lon_sta','','')
    ALLOCATE(data_ibegin_grdc_gather(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for data_ibegin_grdc_gather','','')

    ALLOCATE(nblargest(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for nblargest','','')
    ALLOCATE(nblargest_sort(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for nblargest_sort','','')
    ALLOCATE(nblargest_tmp(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for nblargest_tmp','','')
    ALLOCATE(nblargest_idx(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for nblargest_idx','','')
    ALLOCATE(data_ibegin_largest(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for data_ibegin_largest','','')
    ALLOCATE(data_iend_largest(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for data_iend_largest','','')
    ALLOCATE(lon_larg(nblargest_mpi), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for lon_larg','','')
    ALLOCATE(data_ibegin_largest_gather(mpi_size), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for data_ibegin_largest_gather','','')


    CALL MPI_Gather(nb_grdc_match_loc, 1, MPI_INT_ORCH, nbsta, 1, MPI_INT_ORCH, 0, MPI_COMM_ORCH, ALLOC_ERR)
    CALL MPI_Gather(nb_largest_loc, 1, MPI_INT_ORCH, nblargest, 1, MPI_INT_ORCH, 0, MPI_COMM_ORCH, ALLOC_ERR)

    WRITE(numout, *) "gather nbsta=", nbsta
    WRITE(numout, *) "gather nblargest=", nblargest
    CALL bcast(nbsta)
    CALL bcast(nblargest)

    ! 2. Initialize XIOS for GRDC and outflow. Note that data in XIOS begins from 0
 
    IF(is_root_prc) THEN
        nbsta_sort(:) = -1
        nbsta_tmp(:) = nbsta(:)
        nbsta_idx(:) = -1
        data_ibegin_grdc(:) = 0
        data_iend_grdc(:) = 0

        nblargest_sort(:) = -1
        nblargest_tmp(:) = nblargest(:)
        nblargest_idx(:) = -1
        data_ibegin_largest(:) = 0
        data_iend_largest(:) = 0

        nbsta_sort(:) = -1
        nbsta_tmp(:) = nbsta(:)
        nbsta_idx(:) = -1

        DO i = 1, mpi_size
          nbsta_idx(i) = MAXLOC(nbsta_tmp, 1)
          nbsta_sort(i) = MAXVAL(nbsta_tmp, 1)
          nblargest_idx(i) = MAXLOC(nblargest_tmp, 1)
          nblargest_sort(i) = MAXVAL(nblargest_tmp, 1)
          IF(i == 1) THEN
             data_ibegin_grdc(i) = 0
             data_ibegin_largest(i) = 0
          ELSE
             data_ibegin_grdc(i) = data_iend_grdc(i-1) + 1 !data_ibegin_grdc(i-1) + MAX(nbsta_sort(i), 1)
             data_ibegin_largest(i) = data_iend_largest(i-1) + 1 
          ENDIF
          data_iend_grdc(i) = data_ibegin_grdc(i) + MAX(nbsta_sort(i), 0) - 1
          nbsta_tmp(MAXLOC(nbsta_tmp, 1)) = -1
          data_iend_largest(i) = data_ibegin_largest(i) + MAX(nblargest_sort(i), 0) - 1
          nblargest_tmp(MAXLOC(nblargest_tmp, 1)) = -1
        ENDDO
        !WRITE(numout, *) "gather nbsta_sort=",nbsta_sort
        !WRITE(numout, *) "gather nbsta_idx=",nbsta_idx
        !WRITE(numout, *) "gather data_ibegin_grdc=",data_ibegin_grdc
        !WRITE(numout, *) "gather data_iend_grdc=",data_iend_grdc

        !WRITE(numout, *) "gather nblargest_sort=",nblargest_sort
        !WRITE(numout, *) "gather nblargest_idx=",nblargest_idx
        !WRITE(numout, *) "gather data_ibegin_largest=",data_ibegin_largest
        !WRITE(numout, *) "gather data_iend_largest=",data_iend_largest
    ENDIF       

    CALL bcast(nbsta_sort)
    CALL bcast(nbsta_idx)
    CALL bcast(data_ibegin_grdc)
    CALL bcast(data_iend_grdc)
    CALL bcast(nblargest_sort)
    CALL bcast(nblargest_idx)
    CALL bcast(data_ibegin_largest)
    CALL bcast(data_iend_largest)

 
    WRITE(numout, *) " XIOS parameters for GRDC:"
    WRITE(numout, *) " nbsta=",nbsta
    WRITE(numout, *) " nbsta_sort=",nbsta_sort
    WRITE(numout, *) " nbsta_idx=",nbsta_idx
    WRITE(numout, *) " data_ibegin_grdc=",data_ibegin_grdc
    WRITE(numout, *) " data_iend_grdc=",data_iend_grdc

    WRITE(numout, *) " XIOS parameters for Outflow:"
    WRITE(numout, *) " nblargest=",nblargest
    WRITE(numout, *) " nblargest_sort=",nblargest_sort
    WRITE(numout, *) " nblargest_idx=",nblargest_idx
    WRITE(numout, *) " data_ibegin_largest=",data_ibegin_largest
    WRITE(numout, *) " data_iend_largest=",data_iend_largest

    DO i = 1, mpi_size
       IF(nbsta_idx(i) == mpi_rank+1) THEN
          data_ibegin_grdc_tmp = i - 1
          data_ibegin_grdc_tmp = data_ibegin_grdc(i) 
       ENDIF
       IF(nblargest_idx(i) == mpi_rank+1) THEN
          data_ibegin_largest_tmp = data_ibegin_largest(i)
       ENDIF
    ENDDO
    WRITE(numout, *) "data_ibegin_grdc_tmp=", data_ibegin_grdc_tmp
    WRITE(numout, *) "data_ibegin_largest_tmp=", data_ibegin_largest_tmp

    CALL MPI_Gather(data_ibegin_grdc_tmp, 1, MPI_INT_ORCH, data_ibegin_grdc_gather, 1, MPI_INT_ORCH, 0, MPI_COMM_ORCH, ALLOC_ERR)
    CALL bcast(data_ibegin_grdc_gather)   
    CALL MPI_Gather(data_ibegin_largest_tmp, 1, MPI_INT_ORCH, data_ibegin_largest_gather, 1, MPI_INT_ORCH, 0, MPI_COMM_ORCH, ALLOC_ERR)
    CALL bcast(data_ibegin_largest_gather)   

    DO j = 1, nbsta_mpi
          lon_sta(j) = REAL(j)
    ENDDO
    DO j = 1, nblargest_mpi
        lon_larg(j) = REAL(j)
    ENDDO

!
    CALL MPI_REDUCE(nbsta_mpi, ngrdc, 1, MPI_INT_ORCH, MPI_SUM, 0, MPI_COMM_ORCH, ALLOC_ERR) 
    CALL bcast(ngrdc)
    CALL MPI_REDUCE(nblargest_mpi, nlargest, 1, MPI_INT_ORCH, MPI_SUM, 0, MPI_COMM_ORCH, ALLOC_ERR) 
    CALL bcast(nlargest)
    WRITE(numout, *) "ngrdc, nlargest=", ngrdc, nlargest

    ! 3. Assign local values to global values

    ALLOCATE( idx_grdc_match_glo(ngrdc), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for idx_grdc_match_glo','','')
    ALLOCATE( og_grdc_match_glo(ngrdc), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for og_grdc_match_glo','','')
    ALLOCATE( ob_grdc_match_glo(ngrdc), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for ob_grdc_match_glo','','')
    ALLOCATE( area_grdc_match_glo(ngrdc, 2), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for area_grdc_match_glo','','')
    ALLOCATE( idxsort_grdc_match_glo(ngrdc), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for idxsort_grdc_match_glo','','')
    ALLOCATE( idxsort_grdc_match_loc(nbsta_mpi), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for idxsort_grdc_match_loc','','')
    ALLOCATE(area_grdc_match_tmp(ngrdc), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for area_grdc_match_tmp','','')
    ALLOCATE(idxsort_grdc_match_tmp(ngrdc), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for idxsort_grdc_match_tmp','','')

    ALLOCATE(totarea_largest_glo(nlargest), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for totarea_largest_glo','','')
    ALLOCATE( idxsort_largest_glo(nlargest), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for idxsort_largest_glo','','')
    ALLOCATE( idxsort_largest_loc(nblargest_mpi), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for idxsort_largest_loc','','')
    ALLOCATE(totarea_largest_tmp(nlargest), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for totarea_largest_tmp','','')
    ALLOCATE(idxsort_largest_tmp(nlargest), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_xiosgrdc','Pb in allocate for idxsort_largest_tmp','','')

    idx_grdc_match_glo = -1
    og_grdc_match_glo = -1
    ob_grdc_match_glo = -1
    area_grdc_match_glo = -1
    totarea_largest_glo = -1

    CALL MPI_Gatherv(idx_grdc_match_loc, nbsta(mpi_rank+1), MPI_REAL_ORCH, idx_grdc_match_glo, nbsta, data_ibegin_grdc_gather, MPI_REAL_ORCH, 0, MPI_COMM_ORCH, ALLOC_ERR)
    CALL MPI_Gatherv(og_grdc_match_loc,  nbsta(mpi_rank+1), MPI_INT_ORCH,  og_grdc_match_glo,  nbsta, data_ibegin_grdc_gather, MPI_INT_ORCH,  0, MPI_COMM_ORCH, ALLOC_ERR)
    CALL MPI_Gatherv(ob_grdc_match_loc,  nbsta(mpi_rank+1), MPI_INT_ORCH,  ob_grdc_match_glo,  nbsta, data_ibegin_grdc_gather, MPI_INT_ORCH,  0, MPI_COMM_ORCH, ALLOC_ERR)
    CALL MPI_Gatherv(area_grdc_match_loc(:,2),nbsta(mpi_rank+1), MPI_REAL_ORCH, area_grdc_match_glo(:,2),nbsta, data_ibegin_grdc_gather, MPI_REAL_ORCH, 0, MPI_COMM_ORCH, ALLOC_ERR)
    CALL MPI_Gatherv(totarea_largest_loc(:),nblargest(mpi_rank+1), MPI_REAL_ORCH, totarea_largest_glo(:),nblargest, data_ibegin_largest_gather, MPI_REAL_ORCH, 0, MPI_COMM_ORCH, ALLOC_ERR)

    WRITE(numout,*) 'MPI_Gatherv og_grdc_match_glo ob_grdc_match_glo done!'

    CALL bcast(idx_grdc_match_glo)
    CALL bcast(og_grdc_match_glo)
    CALL bcast(ob_grdc_match_glo)
    CALL bcast(area_grdc_match_glo)
    CALL bcast(totarea_largest_glo)

    WRITE(numout,*) 'size of og_grdc_match_glo:', SIZE(og_grdc_match_glo)
    WRITE(numout,*) 'idx_grdc_match_glo=:',  idx_grdc_match_glo
    WRITE(numout,*) 'og_grdc_match_glo=: ',  og_grdc_match_glo
    WRITE(numout,*) 'ob_grdc_match_glo=: ',  ob_grdc_match_glo
    WRITE(numout,*) 'area_grdc_match_glo=:', area_grdc_match_glo
    WRITE(numout,*) 'totarea_largest_glo=:', totarea_largest_glo

    ! 4. Sort GRDC/Outflow by upstream area (descending)
    ! So that the data in output can keep the same order even though the number of CPUs varies

    ! 4.1 Sort the GRDC stations by the upstream area on 'glo'
    area_grdc_match_tmp = area_grdc_match_glo(:,2)
    DO igrdc = 1, ngrdc
        idxsort_grdc_match_tmp(igrdc) = MAXLOC(area_grdc_match_tmp, 1) 
        area_grdc_match_tmp(idxsort_grdc_match_tmp(igrdc)) = MINVAL(area_grdc_match_glo(:,2), 1) - 1.0
    ENDDO
    DO igrdc = 1, ngrdc
        DO jgrdc = 1, ngrdc
            IF (idxsort_grdc_match_tmp(jgrdc) == igrdc) THEN
                idxsort_grdc_match_glo(igrdc) =  jgrdc
            ENDIF
        ENDDO
    ENDDO
    DO igrdc = 1, ngrdc
        WRITE(numout,*) 'ranked area_grdc_match_glo=:', area_grdc_match_glo(idxsort_grdc_match_tmp(igrdc),2)
    ENDDO

    CALL MPI_Scatterv(idxsort_grdc_match_glo, nbsta, data_ibegin_grdc_gather, MPI_INT_ORCH, idxsort_grdc_match_loc,  nbsta(mpi_rank+1), MPI_INT_ORCH, 0, MPI_COMM_ORCH, ALLOC_ERR)
    WRITE(numout,*) 'idxsort_grdc_match_glo=:',  idxsort_grdc_match_glo
    WRITE(numout,*) 'idxsort_grdc_match_loc=:',  idxsort_grdc_match_loc

    idxsort_grdc_match_loc = idxsort_grdc_match_loc-1

    ! 4.2 Sort the 'Largest' by the upstream area on 'glo'
    totarea_largest_tmp = totarea_largest_glo(:)
    DO ilargest = 1, nlargest
        idxsort_largest_tmp(ilargest) = MAXLOC(totarea_largest_tmp, 1) 
        totarea_largest_tmp(idxsort_largest_tmp(ilargest)) = MINVAL(totarea_largest_glo(:), 1) - 1.0
    ENDDO
    DO ilargest = 1, nlargest
        DO jlargest = 1, nlargest
            IF (idxsort_largest_tmp(jlargest) == ilargest) THEN
                idxsort_largest_glo(ilargest) =  jlargest
            ENDIF
        ENDDO
    ENDDO
    DO ilargest = 1, nlargest
        WRITE(numout,*) 'ranked totarea_largest_glo=:', totarea_largest_glo(idxsort_largest_tmp(ilargest))
    ENDDO

    CALL MPI_Scatterv(idxsort_largest_glo, nblargest, data_ibegin_largest_gather, MPI_INT_ORCH, idxsort_largest_loc,  nblargest(mpi_rank+1), MPI_INT_ORCH, 0, MPI_COMM_ORCH, ALLOC_ERR)
    WRITE(numout,*) 'idxsort_largest_glo=:',  idxsort_largest_glo
    WRITE(numout,*) 'idxsort_largest_loc=:',  idxsort_largest_loc

    idxsort_largest_loc = idxsort_largest_loc-1


    ! 5. Define 'domain_grdc', 'domain_largest' and 'domain_landpt'
    IF(.NOT.ok_calendar) THEN

        ! domain_grdc
        CALL xios_set_domain_attr("domain_grdc",ni_glo=ngrdc,type='unstructured')  
        CALL xios_set_domain_attr("domain_grdc",ibegin=mpi_rank, ni=nbsta_mpi) 
        CALL xios_set_domain_attr("domain_grdc",data_dim=1, data_ibegin=0, data_ni=nbsta_mpi, i_index=idxsort_grdc_match_loc)

        ! domain_largest
        CALL xios_set_domain_attr("domain_largest",ni_glo=nlargest,type='unstructured')
        CALL xios_set_domain_attr("domain_largest",ibegin=mpi_rank,ni=nblargest_mpi)
        CALL xios_set_domain_attr("domain_largest",data_dim=1, data_ibegin=0, data_ni=nblargest_mpi, i_index=idxsort_largest_loc)

        ! domain_landpt
        CALL xios_set_domain_attr("domain_landpt",ni_glo=nbp_glo,type='unstructured')  !ngrdc
        CALL xios_set_domain_attr("domain_landpt",ibegin=nbp_mpi_begin-1,ni=nbpt) !ngrdc
        CALL xios_set_domain_attr("domain_landpt",data_dim=1, data_ibegin=0, data_ni=nbpt)

        WRITE(*,'(" -> mpi_rank: ", I4, "  ngrdc: ", I5, "  nb_grdc_match_glo: ", I5, "  nbsta_mpi: ", I4, &
               & "  num_largest:", I5, "  nblargest_mpi: ", I5)') & 
               & mpi_rank, ngrdc, nb_grdc_match_glo, nbsta_mpi, &
               & num_largest, nblargest_mpi
        !
        ! close xios
        ! 
        CALL xios_close_context_definition()
        CALL xios_orchidee_update_calendar(kjit)
        WRITE(numout, *) "calendar updated!"
        !CALL xios_context_finalize()
        !!CALL xios_finalize()
    ENDIF

    ! Deallocate variables
    IF (ALLOCATED(area_grdc_match_tmp))     DEALLOCATE(area_grdc_match_tmp)
    IF (ALLOCATED(idxsort_grdc_match_tmp))  DEALLOCATE(idxsort_grdc_match_tmp)
    IF (ALLOCATED(totarea_largest_tmp))     DEALLOCATE(totarea_largest_tmp)
    IF (ALLOCATED(idxsort_largest_tmp))     DEALLOCATE(idxsort_largest_tmp)

  END SUBROUTINE routing_reg_xiosgrdc


!! ================================================================================================================================
!! SUBROUTINE : routing_reg_unique
!!
!>\BRIEF : This subroutine return only the unique values in a vector ...
!!
!! DESCRIPTION (definitions, functional, design, flags): 
!!   Count the number of occurrences of this element:
!!   Modified from an example in http://degenerateconic.com/unique/
!!   Return only the unique values from vec.
!! . \n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_unique(vec, val_mask, nb_unique, mask, vec_unique) 
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), DIMENSION(:), INTENT(IN) :: vec                   !! Domain size (unitless)
    INTEGER(i_std), INTENT(IN) :: val_mask
    !
!! OUTPUT VARIABLES
    INTEGER(i_std), INTENT(OUT) :: nb_unique
    LOGICAL, DIMENSION(SIZE(vec)), INTENT(OUT) :: mask
    INTEGER(i_std), DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: vec_unique
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: i, j, num !! Indices (unitless)
    INTEGER(i_std) :: ALLOC_ERR

!_ ================================================================================================================================

    mask = .FALSE.
    num = -1
 
    DO i=1,SIZE(vec)
        !count the number of occurrences of this element:
        num = COUNT( vec(i)==vec .AND. vec(i).NE.val_mask)
        IF (num == 0) THEN
            mask(i) = .FALSE.
        ELSEIF (num == 1) THEN
            mask(i) = .TRUE. !! there is only one, flag it:
        ELSE
          !flag this value only if it hasn't already been flagged:
            if (.NOT. ANY(vec(i)==vec .AND. mask .AND. vec(i).NE.val_mask) )  &
               & mask(i) = .TRUE.
        ENDIF
    ENDDO
    nb_unique = COUNT(mask) 

    !return only flagged elements:
    IF(nb_unique .GT. 0) THEN
        ALLOCATE( vec_unique(nb_unique), stat = ALLOC_ERR )
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_unique',' Pb in allocate for vec_unique','','')
        vec_unique = PACK( vec, mask )
    ENDIF

  END SUBROUTINE routing_reg_unique

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_findgrdc
!!
!>\BRIEF : This subroutine find the GRDC stations.
!!
!! DESCRIPTION (definitions, functional, design, flags): 
!!   Locating ORC outlet point by comparing with GRDC stations. This is done by: 
!!   Step (1): remove the basins with upstream area out of the range of GRDC area.
!!   Step (2): find the index of the closest upstream area (idx_find) between ORC and GRDC
!!   Step (3): compare the lon/lat at 'idx_find' between ORC outlet and GRDC stations
!! . \n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_findgrdc(nbpt, index) 
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbpt                   !! Domain size (unitless)
    INTEGER(i_std), INTENT(in) :: index(nbpt)            !! Indices of the points on the map (unitless)
    !
!! OUTPUT VARIABLES
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: ig, ib, i, j !! Indices (unitless)

    ! Finding the correponding GRDC stations
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: darea_tmp          !! Difference in Area between ORC and GRDC (m²)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: idx_grdc_unique !! Index of unique GRDC stations in each ORC model grid
!    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: og_grdc_unique !! Unique Grid match the GRDC stations
!    REAL(r_std), DIMENSION(nbpt,nbasmax) :: dlat, dlon   !! Difference in Lat. and Lon. between ORC and GRDC
!    REAL(r_std), DIMENSION(nbpt,nbasmax) :: min_darea    !! Minimum upstream area difference between ORC and GRDC (m²)
!    REAL(r_std), DIMENSION(nbpt,nbasmax) :: darea_rel    !! Area difference between ORC and GRDC (m²)
!    REAL(r_std), DIMENSION(nbpt,nbasmax) :: dist         !! Distance between ORC and GRDC station; unit:m
    LOGICAL, DIMENSION(nbpt,nbasmax) :: mask_nbgrdc      !! Mask of GRDC stations found in each grid 
!    LOGICAL, ALLOCATABLE, DIMENSION(:) :: mask_og_grdc      !! Mask of grid matching GRDC stations  
!    INTEGER(i_std), DIMENSION(nbp_glo, nbasmax) :: ig_glo
!    INTEGER(i_std), DIMENSION(nbp_glo, nbasmax) :: ib_glo
!    INTEGER(i_std), DIMENSION(nbpt, nbasmax) :: ig_loc
!    INTEGER(i_std), DIMENSION(nbpt, nbasmax) :: ib_loc

    LOGICAL, ALLOCATABLE, DIMENSION(:) :: is_grdc_match      !! Determine if the GRDC station matches the ORC subbasin
    LOGICAL, ALLOCATABLE, DIMENSION(:) :: is_grdc_match_loc  !! Determine if the GRDC station matches the ORC subbasin
    LOGICAL, ALLOCATABLE, DIMENSION(:) :: is_grdc_match_glo  !! Determine if the GRDC station matches the ORC subbasin
!    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: ig_glo
!    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: ib_glo
!    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: ig_loc
!    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: ib_loc
!    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: ix_glo
!    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: iy_glo
!    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: ix_loc
!    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: iy_loc
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: og_grdc_mindarea
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: ob_grdc_mindarea
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: dlat, dlon   !! difference in lat. and lon. between orc and grdc
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: darea    !! Minimum upstream area difference between ORC and GRDC (m²)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: darea_rel    !! Area difference between ORC and GRDC (m²)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: dist         !! Distance between ORC and GRDC station; unit:m
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: dlat_glo, dlon_glo   !! Difference in Lat. and Lon. between ORC and GRDC
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: darea_glo    !! Minimum upstream area difference between ORC and GRDC (m²)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: darea_rel_glo    !! Area difference between ORC and GRDC (m²)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: dist_glo         !! Distance between ORC and GRDC station; unit:m

    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: ngrdc_grid_loc !! No. of GRDC stations in each model grid (local)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: ngrdc_grid_glo !! No. of GRDC stations in each model grid (global)


    REAL(r_std) ::  min_darea_grdc
    INTEGER(i_std) :: idx_find
    INTEGER(i_std) :: og_obs, ob_obs, og_tmp, ob_tmp
    INTEGER(i_std) :: og_loc, ob_loc, ogloc_loc, obloc_loc
    INTEGER(i_std) :: ox_loc, oy_loc     !!  
    LOGICAL :: err_find_grdc                             !! To determine if there are errors in finding GRDC data. (true/false)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: og_grdc !! The correponding grid (g) of each GRDC station
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: ob_grdc !! The correponding subbasin (b) of each GRDC station
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: nb_subbsn         !! No. of subbasins that match each GRDC stations
    INTEGER(i_std) :: nloop                              !! No. of loops for finding GRDC stations 
    INTEGER(i_std) :: nmatch, nmatch_glo                             !! NO. of matched ORC grid and GRDC stations
    REAL(r_std) :: ratio_minarea, ratio_maxarea          !! Ratio between ORC area and Min. (Max.) GRDC area
    REAL(r_std) :: tmp_a, tmp_c
    !INTEGER(i_std), DIMENSION(nbpt,nbasmax) :: idx_grdc !! Index of GRDC stations in each ORC model grid and each subbasin
    INTEGER(i_std), DIMENSION(nbpt)     :: min_darea_loc_grid      !! The subbasin with the lowest difference in upstream area between ORC and GRDC (in each grid)
    INTEGER(i_std), DIMENSION(nbpt, 15) :: min_darea_loc_grdc  !! The subbasin with the lowest difference in upstream area between ORC and different GRDCs (in each grid)
    INTEGER(i_std) :: iigrdc, iigrdc_reg       !!
    INTEGER(i_std) :: ib_grdc
    INTEGER(i_std) :: ALLOC_ERR
    REAL(r_std), PARAMETER :: thres_ratio_minarea = 0.5     !! Max. threshold of ratio between ORC area and GRDC area
    REAL(r_std), PARAMETER :: thres_ratio_maxarea = 2.0     !! Max. threshold of ratio between ORC area and GRDC area
    INTEGER(i_std), PARAMETER :: max_iter = 250             !! Max. number of iterations
    REAL(r_std) :: min_darea_og_grdc
    INTEGER(i_std) :: min_og_grdc

    INCLUDE 'mpif.h'
!_ ================================================================================================================================
    !
    !
    ! 1. Initialization

    ! Allocate local variables
    !WRITE(numout,*) "ALLOCATE GRDC variables"

    ALLOCATE( ig_glo(nbp_glo, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ig_glo','','')
    ALLOCATE( ib_glo(nbp_glo, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ib_glo','','')
    ALLOCATE( ig_loc(nbpt, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ig_loc','','')
    ALLOCATE( ib_loc(nbpt, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ib_loc','','')
    !ig_glo(:,:) = -1
    !ib_glo(:,:) = -1
    !ig_loc(:,:) = -1
    !ib_loc(:,:) = -1

    ALLOCATE( ix_glo(nbp_glo), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ix_glo','','')
    ALLOCATE( iy_glo(nbp_glo), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for iy_glo','','')
    ALLOCATE( ix_loc(nbpt), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ix_loc','','')
    ALLOCATE( iy_loc(nbpt), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for iy_loc','','')


    ALLOCATE( dlat(nbpt, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for dlat','','')
    ALLOCATE( dlon(nbpt, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for dlon','','')
    ALLOCATE( darea(nbpt, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for darea','','')
    ALLOCATE( darea_rel(nbpt, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for darea_rel','','')
    ALLOCATE( dist(nbpt, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for dist','','')


    ALLOCATE( ngrdc_grid_loc(nbpt), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ngrdc_grid_loc','','')
    ALLOCATE( ngrdc_grid_glo(nbp_glo), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ngrdc_grid_glo','','')
!    ALLOCATE( idx_grdc_loc(nbpt,10), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for idx_grdc_loc','','')
!    ALLOCATE( area_grdc_loc(nbpt,10), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for area_grdc_loc','','')
!    ALLOCATE( lon_grdc_loc(nbpt,10), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for lon_grdc_loc','','')
!    ALLOCATE( lat_grdc_loc(nbpt,10), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for lat_grdc_loc','','')
!    ALLOCATE( hydrodiag_grdc_loc(nbpt,nbasmax), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for hydrodiag_grdc_loc','','')
!    idx_grdc_loc(:,:) = -1.0
!    lat_grdc_loc(:,:) = -1.0
!    lon_grdc_loc(:,:) = -1.0
!    area_grdc_loc(:,:) = -1.0
    hydrodiag_grdc_loc(:,:) = -1
    ngrdc_grid_loc(:) = -1
    !ngrdc_grid_glo(:) = -1

    !! Allocae glo variables

    ALLOCATE( dlat_glo(nbp_glo, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for dlat_glo','','')
    ALLOCATE( dlon_glo(nbp_glo, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for dlon_glo','','')
    ALLOCATE( darea_glo(nbp_glo, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for darea_glo','','')
    ALLOCATE( darea_rel_glo(nbp_glo, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for darea_rel_glo','','')
    ALLOCATE( dist_glo(nbp_glo, nbasmax), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for dist_glo','','')

!    ALLOCATE( idx_grdc_glo(nbp_glo,10), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for idx_grdc_glo','','')
!    ALLOCATE( area_grdc_glo(nbp_glo,10), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for area_grdc_glo','','')
!    ALLOCATE( lon_grdc_glo(nbp_glo,10), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for lon_grdc_glo','','')
!    ALLOCATE( lat_grdc_glo(nbp_glo,10), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for lat_grdc_glo','','')
!    ALLOCATE( hydrodiag_grdc_glo(nbp_glo,nbasmax), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for hydrodiag_grdc_glo','','')
!    idx_grdc_glo = -1.0
!    area_grdc_glo = -1.0
!    lon_grdc_glo = -1.0
!    lat_grdc_glo = -1.0
!!    hydrodiag_grdc_glo = -1

!    ALLOCATE( idx_mod_match_loc(nbpt,nbasmax), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for idx_mod_match_loc','','')
!    ALLOCATE( idx_mod_match_glo(nbp_glo,nbasmax), STAT=ALLOC_ERR )
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for idx_mod_match_glo','','')

    nmatch = 0
    idx_mod_match_loc(:,:) = -1
    !idx_mod_match_glo(:,:) = -1
    darea_rel(:,:) = 99999.0
    dlat(:,:) = 999.0
    dlon(:,:) = 999.0
    dist(:,:) = 999999.0
    darea(:,:) = 99999.0

    !darea_rel_glo(:,:) = 99999.0
    !dlat_glo(:,:) = 999.0
    !dlon_glo(:,:) = 999.0
    !dist_glo(:,:) = 999999.0
    !darea_glo(:,:) = 99999.0
    min_darea_loc_grid(:) = -1 !nbasmax+1
    min_darea_loc_grdc(:,:) = -1

    ALLOCATE(nb_subbsn(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for nb_subbsn','','')

    ALLOCATE(og_grdc(igrdc_reg, 100), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for og_grdc','','')

    ALLOCATE(ob_grdc(igrdc_reg, 100), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ob_grdc','','')

    ALLOCATE (darea_tmp(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for darea_tmp','','')

    ALLOCATE(is_grdc_match(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for is_grdc_match','','')

    ALLOCATE(is_grdc_match_loc(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for is_grdc_match_loc','','')

    ALLOCATE(is_grdc_match_glo(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for is_grdc_match_glo','','')

!    ALLOCATE(nb_og_grdc(igrdc_reg), STAT=ALLOC_ERR)
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for nb_og_grdc','','')

!    ALLOCATE(mask_og_grdc(100), STAT=ALLOC_ERR)
!    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for mask_og_grdc','','')

    ALLOCATE(og_grdc_mindarea(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for og_grdc_mindarea','','')

    ALLOCATE(ob_grdc_mindarea(igrdc_reg), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ob_grdc_mindarea','','')

    is_grdc_match(:) = .FALSE.
    is_grdc_match_loc(:) = .FALSE.
    idx_find = -1
    !og_grdc(:,:) = -1
    !ob_grdc(:,:) = -1

    WRITE(numout, *) "The model domain for the current CPU:"
    WRITE(numout, '("Model minlat:", F14.3, "; minlon:", F14.3)')  &
          & MINVAL(MINVAL(route_outlet_loc(:,:,1), DIM=1, MASK=route_outlet_loc(:,:,1).LE.360.), DIM=1), &
          & MINVAL(MINVAL(route_outlet_loc(:,:,2), DIM=1, MASK=route_outlet_loc(:,:,2).LE.360.), DIM=1)
    WRITE(numout, '("Model maxlat:", F14.3, "; maxlon:", F14.3)')  &
          & MAXVAL(MAXVAL(route_outlet_loc(:,:,1), DIM=1, MASK=route_outlet_loc(:,:,1).LE.360.), DIM=1), &
          & MAXVAL(MAXVAL(route_outlet_loc(:,:,2), DIM=1, MASK=route_outlet_loc(:,:,2).LE.360.), DIM=1)
    !
    ! 2. Matching the GRDC stations and ORC outlets 
    !
    DO ig=1,nbpt
         DO ib=1,nbasmax
            !WRITE(numout, *) "ig, ib:", ig, ib
            err_find_grdc = .FALSE.
            !IF ( routing_area_loc(ig,ib) .GT. zero ) THEN
            ! (1) Determine if the  reaupbasin(ig,ib) is within the range of area_obs
            ratio_minarea = areaupbasin_loc(ig,ib) / MINVAL(area_obs_reg, 1)
            ratio_maxarea = areaupbasin_loc(ig,ib) / MAXVAL(area_obs_reg, 1)

            IF(ratio_minarea < thres_ratio_minarea .OR. &
               & ratio_maxarea > thres_ratio_maxarea) THEN  ! or igrdc? 
               err_find_grdc = .TRUE.
               !WRITE(numout, *) "areaupbasin_loc,ratio_minarea,ratio_maxarea,ig,ib:", &
               !      & areaupbasin_loc(ig,ib), ratio_minarea, ratio_maxarea, ig,ib
               CYCLE
            ENDIF
            ! (2) and (3) compare the areaupbasin_loc(ig,ib) and area_obs(1:igrdc),
            ! compare the route_outlet_loc and lon_obs/lat_obs
            darea_tmp = ABS(areaupbasin_loc(ig,ib) - area_obs_reg(1:igrdc_reg)) !/ areaupbasin(ig,ib)
            nloop = 0
            og_obs = -1
            ob_obs = -1

            !DO WHILE ((ABS(dlat) .GT. thres_dlola .OR. ABS(dlon) .GT. thres_dlola &
            !    & .OR. (ABS(dist(ig,ib)) .GT. thres_dist)) &
            !DO WHILE (((ABS(dist(ig,ib)) .GT. thres_dist)) &
            DO WHILE ((ABS(dlat(ig,ib)) .GT. thres_dlola .OR. ABS(dlon(ig,ib)) .GT. thres_dlola) &
                & .AND. (nloop .LE. MIN(max_iter, igrdc_reg)))
                nloop = nloop + 1
                darea(ig,ib) = MINVAL(darea_tmp(1:igrdc_reg), 1)
                idx_find = MINLOC(darea_tmp(1:igrdc_reg), 1)
                ! To compare the distance between ORC point and GRDC station
                !http://www.cs.nyu.edu/visual/home/proj/tiger/gisfaq.html
                !dlon = lon2 - lon1
                !dlat = lat2 - lat1
                !a = (sin(dlat/2))^2 + cos(lat1) * cos(lat2) * (sin(dlon/2))^2
                !c = 2 * arcsin(min(1,sqrt(a)))
                !d = R * c
                dlat(ig,ib) = route_outlet_loc(ig,ib,1) - lat_obs_reg(idx_find)
                dlon(ig,ib) = route_outlet_loc(ig,ib,2) - lon_obs_reg(idx_find)
                tmp_a = (SIN(dlat(ig,ib)*(pi/180)/2.))**2 + &
                  & COS(route_outlet_loc(ig,ib,1)*(pi/180)) * COS(lat_obs_reg(idx_find)*(pi/180)) &
                  & * (SIN(dlon(ig,ib)*(pi/180)/2.))**2
                !tmp_c = 2.0 * ATAN2(SQRT(tmp_a), SQRT(1.0 - tmp_a))
                tmp_c = 2.0 * ASIN(MIN(1.0, SQRT(tmp_a)))
                dist(ig,ib) = R_Earth * tmp_c
                darea_tmp(idx_find) = MAXVAL(darea_tmp(1:igrdc_reg), 1)
            ENDDO ! DO WHILE


            og_obs = ig
            ob_obs = ib
            !darea_rel(ig,ib) = (areaupbasin_loc(ig,ib) - area_obs(idx_obs_reg(idx_find))) &
            !    & / areaupbasin_loc(ig,ib) * 100.
            darea_rel(ig,ib) = (areaupbasin_loc(ig,ib) - area_obs_reg(idx_find)) &
              & / areaupbasin_loc(ig,ib) * 100.
            !IF(ABS(dlat) .GT. thres_dlola .OR. ABS(dlon) .GT. thres_dlola .OR. &
            !    & ABS(darea_rel(ig,ib)) .GT. thres_diff_area)  err_find_grdc = .TRUE.
            !IF(ABS(dist(ig,ib)) .GT. thres_dist .OR. &
            !    & ABS(darea_rel(ig,ib)) .GT. thres_diff_area)  err_find_grdc = .TRUE.
            !IF(nloop.GE.max_iter+1)  err_find_grdc = .TRUE.
            IF(ABS(dlat(ig,ib)) .GT. thres_dlola .OR. ABS(dlon(ig,ib)) .GT. thres_dlola .OR. &
                & ABS(dist(ig,ib)) .GT. thres_dist .OR. &
                & ABS(darea_rel(ig,ib)) .GT. thres_diff_area)  err_find_grdc = .TRUE.
            !WRITE(numout, *) 'areaupbasin_loc(og_obs,ob_obs), & 
            !    & area_obs_reg(idx_find), &
            !    & darea_rel(ig,ib)', &
            !    & areaupbasin_loc(og_obs,ob_obs), area_obs_reg(idx_find), &
            !    & darea_rel(ig,ib) 
            !WRITE(numout, *) 'err_find_grdc, nloop:', err_find_grdc,nloop
            
            IF(.NOT.err_find_grdc) THEN
              idx_mod_match_loc(ig,ib) = idx_find !idx_obs_reg_sort(idx_find)
              nmatch = nmatch + 1
              is_grdc_match(idx_find) = .TRUE.
              WRITE(numout,'("===> Matched grids/subbasins: No.", I5, "  >>>")') nmatch
              WRITE(numout,'(" og_obs:", I4, "; ob_obs:", I4, "; nloop:", I4, "; idx_find:", I6)') &
                  & og_obs, ob_obs, nloop, idx_find
              !WRITE(numout,'(" GRDC River:", A15, "; GRDC Station:", A20, "; GRDC Country:", A5, & 
              !    & "; GRDC Idx:", I6)') &
              !    & river_obs(idx_obs_reg(idx_find)), &
              !    & sta_obs(idx_obs_reg(idx_find)),country_obs(idx_obs_reg(idx_find)), &
              !    & idx_obs_reg(idx_find) !,sorted_names(idx_find)
              !IF(global_basinid_glo(og_obs,ob_obs) .GT. 0 &
              !    & .AND. global_basinid_glo(og_obs,ob_obs) .LE. nb_small) THEN 
              !    WRITE(numout,'(" ORC Basin: ", A15, "; ORC Basin ID:",I7)'), &
              !    & basin_allnames_glo(og_obs,ob_obs), global_basinid_glo(og_obs,ob_obs)
              !ELSE
              !    WRITE(numout,'(" Basin ID:",I7)'), &
              !    & global_basinid_glo(og_obs,ob_obs)
              !ENDIF
              !WRITE(numout,'(" Lat ORC:      ", F12.5, "; Lat GRDC:      ", F12.5, "; &
              !    & dlat:      ", F12.5)') &
              !    & route_outlet_glo(og_obs,ob_obs,1),lat_obs(idx_obs_reg(idx_find)), dlat(ig,ib)
              !WRITE(numout,'(" Lon ORC:      ", F12.5, "; Lon GRDC:      ", F12.5, "; &
              !    & dlon:      ", F12.5, "; dist[Km]:    ", F8.3)') &
              !    & route_outlet_glo(og_obs,ob_obs,2),lon_obs(idx_obs_reg(idx_find)), dlon(ig,ib), dist(ig,ib)/mille
              !WRITE(numout,'(" Area ORC[Km2]:", F12.5, "; Area GRDC[Km2]:", F12.5, "; & 
              !    & dArea[Km2]:", F12.3, "; Area diff[%]:", F8.4)') &
              !    & areaupbasin_loc(og_obs,ob_obs)/mille/mille, area_obs(idx_obs_reg(idx_find))/mille/mille, & 
              !    & min_darea(ig,ib)/mille/mille, darea_rel(ig,ib)
            ELSE
               !WRITE(numout, *) "GRDC station NOT found, the outlet with the largest upstream area is chosen"
            ENDIF !IF(.NOT.err_find_grdc) 
           !
         ENDDO  ! DO ib=1,nbasmax

         ! To find the number of GRDC stations in each model grid: ngrdc_grid_loc(ig)

         CALL routing_reg_unique(idx_mod_match_loc(ig, :), -1, ngrdc_grid_loc(ig), mask_nbgrdc(ig,:), idx_grdc_unique) 


         ! If there are more than 1 subbasins in each grid corresponds to the same GRDC stations
         ! We keep the subbasins with the cloest upstream area to that of GRDC
         IF(ngrdc_grid_loc(ig) .GT. 1) THEN
             WRITE(numout,'(" >>> Attention: ", I3, " GRDC stations in grid: ", I6)') ngrdc_grid_loc(ig), ig
             WRITE(numout,'("  GRDC stations: ", 20I6)') idx_grdc_unique
             WRITE(numout,'(A4, A10, 3A18, 2A15, 8A11, 2A8)') &
                 & " ib", "GRDC Idx", "GRDC River   ", "GRDC Station    ", "ORC River ", &
                 & "ORC Area[km2]", "GRDC Area[km2]", "dArea[km2]", "dArea[%]", &
                 & "ORC Lat", "GRDC Lat", "ORC Lon", "GRDC Lon", "dLat", "dLon", "minloc" !, "minloc"
         ENDIF ! IF(ngrdc_grid_loc(ig) .GT. 1)

         min_darea_loc_grid(ig) = MINLOC(ABS(darea_rel(ig,1:nbasmax)), 1, MASK=idx_mod_match_loc(ig,:).NE.-1)

         IF(ngrdc_grid_loc(ig) .GT. 0) THEN
            DO iigrdc_reg = 1,ngrdc_grid_loc(ig)
              !min_darea_loc_grdc(ig, iigrdc_reg) = MINLOC(ABS(darea_rel(ig,:)), 1, &
              !   & MASK = (idx_grdc(ig,:) .EQ. idx_grdc_unique(iigrdc_reg) .AND.  &
              !   & idx_grdc_unique(iigrdc_reg) .NE. -1))
              min_darea_loc_grdc(ig, iigrdc_reg) = MINLOC(ABS(darea_rel(ig,:)), 1, &
                 & MASK = (idx_mod_match_loc(ig,:) .EQ. idx_grdc_unique(iigrdc_reg)))

              DO ib=1,nbasmax
                IF(idx_mod_match_loc(ig,ib) .EQ. idx_grdc_unique(iigrdc_reg)) THEN !& & .AND. idx_mod_match_loc(ig,ib) .NE. -1) THEN
                   IF(ngrdc_grid_loc(ig) .GT. 1) THEN
                     !IF(global_basinid_loc(ig,ib) .LE. 0 &
                     !   & .OR. global_basinid_loc(ig,ib) .GT. nb_small) THEN
                     !   basin_names(global_basinid_loc(ig,ib)) = "NOT DEFINED"
                     !ENDIF
                     WRITE(numout,'(I4, I8, "  ", 2A15, A20, " ", 2F15.4, 8F11.3, 2I8)') &
                        & ib, idx_mod_match_loc(ig,ib), &
                        & river_obs_reg(idx_mod_match_loc(ig,ib)), sta_obs_reg(idx_mod_match_loc(ig,ib)), &
                        !& "basin_names", & 
                        & basin_allnames_loc(ig, ib), &
                        & areaupbasin_loc(ig,ib)/mille/mille, &
                        & area_obs_reg(idx_mod_match_loc(ig,ib))/mille/mille, &
                        & darea(ig,ib)/mille/mille, darea_rel(ig,ib), &
                        & route_outlet_loc(ig,ib,1),lat_obs_reg(idx_mod_match_loc(ig,ib)), &
                        & route_outlet_loc(ig,ib,2),lon_obs_reg(idx_mod_match_loc(ig,ib)), &
                        & dlat(ig,ib), dlon(ig,ib), &  !! min_darea_loc_grid(ig), &
                        & min_darea_loc_grdc(ig, iigrdc_reg) !, idx_grdc_unique ! '-1' is excluded because it is not a GRDC Index 
                    ENDIF !IF(ngrdc_grid_loc(ig) .GT. 1)
                ENDIF ! IF(idx_mod_match_loc(ig,ib) .EQ. idx_grdc_unique(iigrdc_reg)) THEN
              ENDDO ! DO ib=1,nbasmax
            ENDDO ! DO iigrdc_reg = 1,ngrdc_grid_loc(ig)
            IF(ALLOCATED(idx_grdc_unique)) DEALLOCATE(idx_grdc_unique)
         ENDIF !IF(ngrdc_grid_loc(ig) .GT. 0)
    ENDDO  ! DO ig=1,nbpt


    ! 3. To count how many subbasins corresponds to one GRDC station
    !
    CALL gather(idx_mod_match_loc,idx_mod_match_glo)
    CALL gather(dlat,dlat_glo)
    CALL gather(dlon,dlon_glo)
    CALL gather(darea,darea_glo)
    CALL gather(darea_rel,darea_rel_glo)
    CALL gather(dist,dist_glo)
    CALL gather(ngrdc_grid_loc, ngrdc_grid_glo)
    WRITE(numout, *) "SIZE, MAX, MIN: idx_mod_match_loc", &
         & SIZE(idx_mod_match_loc), MAXVAL(idx_mod_match_loc), MINVAL(idx_mod_match_loc)
    IF(MAXVAL(idx_mod_match_loc).GT.igrdc_reg .OR. MINVAL(idx_mod_match_loc).LT.-1) THEN
        CALL ipslerr_p(3, 'routing_reg_findgrdc', &
             & 'The Max. and Min. of idx_mod_match_loc out of range.',&
             & 'Check idx_mod_match_loc','')
    ENDIF

    nb_grdc_match_loc = COUNT(is_grdc_match(:))
    is_grdc_match_loc(:) = is_grdc_match(:)
    CALL MPI_REDUCE(nmatch, nmatch_glo, 1, MPI_INT_ORCH, MPI_SUM, 0, MPI_COMM_ORCH, ALLOC_ERR) 
    CALL bcast(nmatch_glo)
    !WRITE(numout, *) "nb_grdc_match_loc:", nb_grdc_match_loc
    WRITE(numout, *) "nmatch, nmatch_glo:", nmatch, nmatch_glo

    IF(is_root_prc) THEN
      WRITE(numout, *) "SIZE, MAX, MIN: idx_mod_match_glo", &
         & SIZE(idx_mod_match_glo), MAXVAL(idx_mod_match_glo), MINVAL(idx_mod_match_glo)
      IF(MAXVAL(idx_mod_match_loc).GT.igrdc_reg .OR. MINVAL(idx_mod_match_loc).LT.-1) THEN
          CALL ipslerr_p(3, 'routing_reg_findgrdc', &
               & 'The Max. and Min. of idx_mod_match_glo out of range.',&
               & 'Check idx_mod_match_glo','')
      ENDIF
      nb_subbsn(:) = 0
      og_grdc(:,:) = -1
      ob_grdc(:,:) = -1
      og_grdc_mindarea(:) = -1
      ob_grdc_mindarea(:) = -1
      is_grdc_match_glo(:) = .FALSE.
      DO iigrdc_reg = 1,igrdc_reg
         !nb_subbsn(iigrdc_reg) = COUNT(idx_obs_reg(iigrdc_reg) .EQ. idx_moc_match_loc(:,:))      
         DO ig=1,nbp_glo
            DO ib=1,nbasmax
               IF(idx_mod_match_glo(ig,ib) .EQ. iigrdc_reg) THEN
                  !WRITE(numout, *) 'nb_subbsn(iigrdc_reg)', nb_subbsn(iigrdc_reg)
                  is_grdc_match_glo(iigrdc_reg) = .TRUE.
                  nb_subbsn(iigrdc_reg) = nb_subbsn(iigrdc_reg) + 1
                  og_grdc(iigrdc_reg, nb_subbsn(iigrdc_reg)) = ig
                  ob_grdc(iigrdc_reg, nb_subbsn(iigrdc_reg)) = ib
               ENDIF
            ENDDO ! DO ib=1,nbasmax
         ENDDO  ! DO ig=1,nbpt

         !! If the matched model subbasins are at different grids, 
         !! we select the one with the lowest difference in upstream area.
         !WRITE(numout, *) "og_grdc(iigrdc_reg, :)", og_grdc(iigrdc_reg, :)
         !WRITE(numout, *) "ob_grdc(iigrdc_reg, :)", ob_grdc(iigrdc_reg, :)

         IF(nb_subbsn(iigrdc_reg) .GT. 0) THEN
            min_darea_grdc = darea_glo( og_grdc(iigrdc_reg,1), ob_grdc(iigrdc_reg, 1) )
            og_grdc_mindarea(iigrdc_reg) = og_grdc(iigrdc_reg, 1)
            ob_grdc_mindarea(iigrdc_reg) = ob_grdc(iigrdc_reg, 1) 
            DO i = 1, nb_subbsn(iigrdc_reg)
               IF(darea_glo(og_grdc(iigrdc_reg,i),ob_grdc(iigrdc_reg,i)) .LT. min_darea_grdc) THEN
                   min_darea_grdc = darea_glo(og_grdc(iigrdc_reg,i),ob_grdc(iigrdc_reg,i))
                   og_grdc_mindarea(iigrdc_reg) = og_grdc(iigrdc_reg, i)
                   ob_grdc_mindarea(iigrdc_reg) = ob_grdc(iigrdc_reg, i) 
               ENDIF
            ENDDO
         ENDIF

         IF(MAXVAL(og_grdc_mindarea).GT.nbp_glo .OR. MINVAL(og_grdc_mindarea).LT.-1.OR. &
             & MAXVAL(ob_grdc_mindarea).GT.nbasmax .OR. MINVAL(ob_grdc_mindarea).LT.-1 ) THEN
             CALL ipslerr_p(3, 'routing_reg_findgrdc', &
                  & 'The Max. and Min. of og_grdc_mindarea and ob_grdc_mindarea out of range.',&
                  & 'Check og_grdc_mindarea and ob_grdc_mindarea','')
         ENDIF
 
         WRITE(numout,'(" iigrdc_reg: ", I4, " nb_subbsn: ", I3, & 
               & " og_grdc_mindarea: ", I5, " ob_grdc_mindarea: ", I5)') &
               & iigrdc_reg, nb_subbsn(iigrdc_reg), &
               & og_grdc_mindarea(iigrdc_reg), ob_grdc_mindarea(iigrdc_reg)

         !CALL routing_reg_unique(og_grdc(iigrdc_reg, :), -1, nb_og_grdc(iigrdc_reg), mask_og_grdc(:), og_grdc_unique) 
         !WRITE(numout,*) "nb_og_grdc(iigrdc_reg):", nb_og_grdc(iigrdc_reg)
         !IF(nb_og_grdc(iigrdc_reg).GT.0) WRITE(numout,*) "og_grdc_unique=", og_grdc_unique 

         !IF( (nb_og_grdc(iigrdc_reg) .GT. 1) ) THEN
           !WRITE(numout, *) "Deal with the situation with multi-subbasins match one GRDC station" 
           !min_darea_og_grdc = darea_glo(og_grdc(iigrdc_reg,1),ob_grdc(iigrdc_reg, 1))
           !WRITE(numout,*) "nb_og_grdc(iigrdc_reg), nb_subbsn(iigrdc_reg)", &
           !     & nb_og_grdc(iigrdc_reg),nb_subbsn(iigrdc_reg) 
           !DO i = 1, nb_subbsn(iigrdc_reg)
           !   IF(darea_glo(og_grdc(iigrdc_reg,i),ob_grdc(iigrdc_reg,i)) .LT. min_darea_og_grdc) THEN
           !      min_darea_og_grdc = darea_glo(og_grdc(iigrdc_reg,i),ob_grdc(iigrdc_reg,i))
           !      min_og_grdc = og_grdc(iigrdc_reg,i)
           !      WRITE(numout,*) "min_darea_og_grdc,min_og_grdc, i", &
           !             & min_darea_og_grdc,min_og_grdc,i
           !   ENDIF
           !ENDDO
           !DO i = 1, nb_og_grdc(iigrdc_reg)
           !   IF(og_grdc_unique(i).NE. min_og_grdc) THEN
           !      !ngrdc_grid_glo(og_grdc_unique(i)) = ngrdc_grid_glo(og_grdc_unique(i)) - 1
           !      WRITE(numout,*) "ngrdc_grid_glo(og_grdc_unique(i))",ngrdc_grid_glo(og_grdc_unique(i))
           !      WRITE(numout,*) "og_grdc_unique(i)", og_grdc_unique(i)
           !   ENDIF
           !ENDDO
         !ENDIF
         !IF(ALLOCATED(og_grdc_unique)) DEALLOCATE(og_grdc_unique)
      ENDDO !iigrdc_reg = 1,igrdc_reg


      WRITE(numout, *) 'SIZE, MAXVAL, MAXLOC, MINVAL MINLOC for global_basinid_loc, global_basinid_glo:', &
         & SIZE(global_basinid_loc),   SIZE(global_basinid_glo), & 
         & MAXVAL(MAXVAL(global_basinid_loc, DIM=1)), MAXVAL(MAXVAL(global_basinid_glo, DIM=1)), &
         & MAXLOC(MAXLOC(global_basinid_loc, DIM=1)), MAXLOC(MAXLOC(global_basinid_glo, DIM=1)), &
         & MINVAL(MINVAL(global_basinid_loc, DIM=1)), MINVAL(MINVAL(global_basinid_glo, DIM=1)), &
         & MINLOC(MINLOC(global_basinid_loc, DIM=1)), MINLOC(MINLOC(global_basinid_glo, DIM=1))

      !! Print the GRDC and model searching results
      WRITE(numout,'(" ==> Comparing GRDC station and model subbasins:")')
      WRITE(numout,'(A8, A15, A18, A7,     3A15, A10,  A10)') &
         & " GRDC No.", "River    ", "Station   ", "Country", &
         & "Area[km^2]", "Lat", "Lon", "GRDC Idx", "Data"
      WRITE(numout,'(A8, A15,  A7, A9, A9, 3A15, 5A13, 3A6, A10)') &
         & " ORC No. ", "River   ", "BasinID", "og", "ob", &
         & "Area[km^2]", "Lat", "Lon", "dLat", "dLon", &
         & "Dist[km]", "dArea[km2]", "dArea[%]", "Data", "minloc"


      DO iigrdc_reg = 1, igrdc_reg
          IF(idx_obs_reg_sort(iigrdc_reg).GT.igrdc_reg .OR. & 
             & idx_obs_reg_sort(iigrdc_reg) .LE. 0 ) THEN
             CALL ipslerr_p(3, 'routing_reg_findgrdc', &
                  & 'The Max. and Min. of idx_obs_reg_sort out of range.',&
                  & 'Check idx_obs_reg_sort(iigrdc_reg)','')
          ENDIF
          WRITE(numout, '(" ", I5, ": ", A15, A18, A7, 3F15.4, I10, A10)') &
              & iigrdc_reg, &
              & river_obs_reg(idx_obs_reg_sort(iigrdc_reg)), sta_obs_reg(idx_obs_reg_sort(iigrdc_reg)), &
              & country_obs_reg(idx_obs_reg_sort(iigrdc_reg)), &
              & area_obs_reg(idx_obs_reg_sort(iigrdc_reg))/mille/mille, &
              & lat_obs_reg(idx_obs_reg_sort(iigrdc_reg)), lon_obs_reg(idx_obs_reg_sort(iigrdc_reg)), &
              & idx_obs_reg(idx_obs_reg_sort(iigrdc_reg)), "GRDC"
          IF(nb_subbsn(idx_obs_reg_sort(iigrdc_reg).GT.0)) THEN
              DO i = 1, nb_subbsn(idx_obs_reg_sort(iigrdc_reg))
                   og_tmp = og_grdc(idx_obs_reg_sort(iigrdc_reg), i)
                   ob_tmp = ob_grdc(idx_obs_reg_sort(iigrdc_reg), i)
                   IF(og_tmp .LE. 0 .OR. og_tmp .GT. nbp_glo .OR. &
                     & ob_tmp .LE. 0 .OR. ob_tmp .GT. nbasmax ) THEN
                        CALL ipslerr_p(3, 'routing_reg_findgrdc', &
                          & 'The og_tmp and ob_tmp are out of range.', 'Check og_tmp and ob_tmp','')
                   ENDIF
                   !IF(global_basinid_glo(og_tmp,ob_tmp) .GT. 0 &
                   !     & .AND. global_basinid_glo(og_tmp,ob_tmp) .LE. nb_small) THEN
                       WRITE(numout,'("  (", I2, "): ", A15, I7, I9, I9, &
                           & 3F15.4, 5F13.4, A5, I10)') &
                           & i, &
                           !& "basin_names", & 
                           & basin_allnames_glo(og_tmp,ob_tmp), &
                           & global_basinid_glo(og_tmp,ob_tmp), og_tmp, ob_tmp, &
                           & areaupbasin(og_tmp,ob_tmp)/mille/mille, &
                           & route_outlet_glo(og_tmp,ob_tmp,1), route_outlet_glo(og_tmp,ob_tmp,2), &
                           & dlat_glo(og_tmp,ob_tmp), dlon_glo(og_tmp,ob_tmp), &
                           & dist_glo(og_tmp,ob_tmp)/mille, darea_glo(og_tmp,ob_tmp)/mille/mille, &
                           & darea_rel_glo(og_tmp,ob_tmp), "ORC" !, min_darea_loc_grid(og_tmp) !,min_darea_loc_grid .EQ. ob_tmp     
                   !ELSE
                       !CALL ipslerr_p(3, 'routing_reg_findgrdc', &
                       !    & 'The global_basinid_glo is out of range.', 'Check global_basinid_glo','')
                       !    basin_names(global_basinid_glo(og_tmp,ob_tmp)) = "NOT DEFINED"
   
                   !ENDIF
              ENDDO !DO i = 1, nb_subbsn(idx_obs_reg_sort(iigrdc_reg))
          ENDIF  !IF(nb_subbsn(idx_obs_reg_sort(iigrdc_reg).GT.0)) 
      ENDDO !DO iigrdc_reg = 1, igrdc_reg
      WRITE(numout, *) "Finish printing GRDC and Model comparisons!"

      DO ig = 1, nbp_glo
        ! j = ((index(ig)-1)/iim) + 1
        ! i = (index(ig) - (j-1)*iim)
          j = ((index_g(ig)-1)/iim_g) + 1
          i = (index_g(ig)-(j-1)*iim_g)

          ix_glo(ig) = i
          iy_glo(ig) = j
          DO ib = 1, nbasmax
             ig_glo(ig, ib) = ig
             ib_glo(ig, ib) = ib
          ENDDO
      ENDDO

      WRITE(numout, *) 'root_prc: SIZE, MAXVAL, MAXLOC, MINVAL MINLOC for og_grdc_mindarea and ob_grdc_mindarea:', &
         & SIZE(og_grdc_mindarea),   SIZE(ob_grdc_mindarea), & 
         & MAXVAL(og_grdc_mindarea), MAXVAL(ob_grdc_mindarea), &
         & MAXLOC(og_grdc_mindarea), MAXLOC(ob_grdc_mindarea), &
         & MINVAL(og_grdc_mindarea), MINVAL(ob_grdc_mindarea), &
         & MINLOC(og_grdc_mindarea), MINLOC(ob_grdc_mindarea)
    !
    ENDIF !IF(is_root_prc)

    !
!    CALL scatter(ngrdc_grid_glo, ngrdc_grid_loc)
!    WRITE(numout, *) "ngrdc_grid_loc", ngrdc_grid_loc
    CALL scatter(ig_glo, ig_loc)
    CALL scatter(ib_glo, ib_loc)
    CALL scatter(ix_glo, ix_loc)
    CALL scatter(iy_glo, iy_loc)

    WRITE(numout, *) "Size, Max, Min of ig_loc: ", SIZE(ig_loc), MAXVAL(ig_loc), MINVAL(ig_loc)
    WRITE(numout, *) "Size, Max, Min of ib_loc: ", SIZE(ib_loc), MAXVAL(ib_loc), MINVAL(ib_loc)
    WRITE(numout, *) "Size, Max, Min of ix_loc: ", SIZE(ix_loc), MAXVAL(ix_loc), MINVAL(ix_loc)
    WRITE(numout, *) "Size, Max, Min of iy_loc: ", SIZE(iy_loc), MAXVAL(iy_loc), MINVAL(iy_loc)

    CALL bcast(is_grdc_match_glo)
    WRITE(numout, *) 'Size of is_grdc_match_glo:', SIZE(is_grdc_match_glo)
!    DO iigrdc_reg = 1, igrdc_reg
!      IF(og_grdc_mindarea(iigrdc_reg).GT.nbp_glo.OR. &
!          & og_grdc_mindarea(iigrdc_reg).LT.-1 .OR. &
!          & ob_grdc_mindarea(iigrdc_reg).GT.nbasmax .OR.&
!          & ob_grdc_mindarea(iigrdc_reg).LT.-1) THEN
!          WRITE(numout, *) '1:og_grdc_mindarea, ob_grdc_mindarea, iigrdc_reg:', &
!               & og_grdc_mindarea(iigrdc_reg), ob_grdc_mindarea(iigrdc_reg), iigrdc_reg
!      ENDIF
!    ENDDO

    WRITE(numout, *) 'Before bcast: SIZE, MAXVAL, MAXLOC, MINVAL, MINLOC for og_grdc_mindarea and ob_grdc_mindarea:', &
         & SIZE(og_grdc_mindarea),   SIZE(ob_grdc_mindarea), & 
         & MAXVAL(og_grdc_mindarea), MAXVAL(ob_grdc_mindarea), &
         & MAXLOC(og_grdc_mindarea), MAXLOC(ob_grdc_mindarea), &
         & MINVAL(og_grdc_mindarea), MINVAL(ob_grdc_mindarea), &
         & MINLOC(og_grdc_mindarea), MINLOC(ob_grdc_mindarea)
    CALL bcast(og_grdc_mindarea)
    CALL bcast(ob_grdc_mindarea)
    !CALL MPI_BCAST(og_grdc_mindarea,igrdc_reg,MPI_INT_ORCH,0,MPI_COMM_ORCH,ALLOC_ERR)
    !CALL MPI_BCAST(ob_grdc_mindarea,igrdc_reg,MPI_INT_ORCH,0,MPI_COMM_ORCH,ALLOC_ERR)

    WRITE(numout, *) 'After bcast: SIZE, MAXVAL, MAXLOC, MINVAL MINLOC for og_grdc_mindarea and ob_grdc_mindarea:', &
         & SIZE(og_grdc_mindarea),   SIZE(ob_grdc_mindarea), & 
         & MAXVAL(og_grdc_mindarea), MAXVAL(ob_grdc_mindarea), &
         & MAXLOC(og_grdc_mindarea), MAXLOC(ob_grdc_mindarea), &
         & MINVAL(og_grdc_mindarea), MINVAL(ob_grdc_mindarea), &
         & MINLOC(og_grdc_mindarea), MINLOC(ob_grdc_mindarea)
    IF(MAXVAL(og_grdc_mindarea).GT.nbp_glo .OR. MINVAL(og_grdc_mindarea).LT.-1.OR. &
          MAXVAL(ob_grdc_mindarea).GT.nbasmax .OR. MINVAL(ob_grdc_mindarea).LT.-1 ) THEN
        CALL ipslerr_p(3, 'routing_reg_findgrdc', &
             & 'The Max. and Min. of og_grdc_mindarea and ob_grdc_mindarea out of range.',&
             & 'Check og_grdc_mindarea and ob_grdc_mindarea','')
    ENDIF

    nb_grdc_match_glo = COUNT(is_grdc_match_glo(:))
    !IF(is_root_prc) nb_grdc_match_glo = SUM(ngrdc_grid_glo, MASK=(ngrdc_grid_glo.GT.0)) !COUNT(idx_grdc_glo.GT.0)
    !CALL MPI_REDUCE(nb_grdc_match_loc, nb_grdc_match_glo, 1, MPI_INT_ORCH, MPI_SUM, 0, MPI_COMM_ORCH, ALLOC_ERR) 
    CALL bcast(nb_grdc_match_glo)
    WRITE(numout, *) "nb_grdc_match_loc, COUNT(is_grdc_match_loc):", nb_grdc_match_loc, COUNT(is_grdc_match_loc(:))
    WRITE(numout, *) "nb_grdc_match_glo, COUNT(is_grdc_match_glo):", nb_grdc_match_glo, COUNT(is_grdc_match_glo(:))
    
    !
    ! 4. Deal with the matched GRDC stations for each processor
    !
    i = 0
    nbsta_mpi = MAX(nb_grdc_match_loc, 0) 
    !nbsta_mpi = MAX(nb_grdc_match_loc, 1) 
    WRITE(numout,*) 'nbsta_mpi=', nbsta_mpi

    ALLOCATE( idx_grdc_match_loc(nbsta_mpi), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for idx_grdc_match_loc','','')
    !idx_grdc_match = PACK(idx_grdc_match_tmp, idx_grdc_match_tmp .NE. -1)
    ALLOCATE( og_grdc_match_loc(nbsta_mpi), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for og_grdc_match_loc','','')
    ALLOCATE( ob_grdc_match_loc(nbsta_mpi), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ob_grdc_match_loc','','')
    ALLOCATE( ogloc_grdc_match_loc(nbsta_mpi), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ogloc_grdc_match_loc','','')
    ALLOCATE( obloc_grdc_match_loc(nbsta_mpi), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for obloc_grdc_match_loc','','')
    ALLOCATE( ox_grdc_match_loc(nbsta_mpi), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for ox_grdc_match_loc','','')
    ALLOCATE( oy_grdc_match_loc(nbsta_mpi), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for oy_grdc_match_loc','','')
    ALLOCATE( lat_grdc_match_loc(nbsta_mpi, 2), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for lat_grdc_match_loc','','')
    ALLOCATE( lon_grdc_match_loc(nbsta_mpi, 2), STAT=ALLOC_ERR ) 
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for lon_grdc_match_loc','','')
    ALLOCATE( area_grdc_match_loc(nbsta_mpi, 2), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for area_grdc_match_loc','','')
    ALLOCATE( basin_grdc_match_loc(nbsta_mpi, 2), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for basin_grdc_match_loc','','')
    ALLOCATE( mergedhydro_grdc_match_loc(nbsta_mpi,2), STAT=ALLOC_ERR )
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findgrdc','Pb in allocate for mergedhydro_grdc_match_loc','','')
    idx_grdc_match_loc(:) = -1.0
    og_grdc_match_loc(:) = -1
    ob_grdc_match_loc(:) = -1
    ogloc_grdc_match_loc(:) = -1
    obloc_grdc_match_loc(:) = -1
    ox_grdc_match_loc(:) = -1
    oy_grdc_match_loc(:) = -1
    lat_grdc_match_loc(:,:) = -999.9
    lon_grdc_match_loc(:,:) = -999.9
    area_grdc_match_loc(:,:) = -999.9
    basin_grdc_match_loc(:,:) = "NOTDEFINED"
    mergedhydro_grdc_match_loc(:,:) = 0.0


    !  DO iigrdc_reg = 1,igrdc_reg
    !     !nb_subbsn(iigrdc_reg) = COUNT(idx_obs_reg(iigrdc_reg) .EQ. idx_moc_match_loc(:,:))      
    !     DO ig=1,nbp_glo
    !        DO ib=1,nbasmax
    !           IF(idx_mod_match_glo(ig,ib) .EQ. iigrdc_reg) THEN
    !              !WRITE(numout, *) 'nb_subbsn(iigrdc_reg)', nb_subbsn(iigrdc_reg)
    !              is_grdc_match_glo(iigrdc_reg) = .TRUE.
    !              nb_subbsn(iigrdc_reg) = nb_subbsn(iigrdc_reg) + 1
    !              og_grdc(iigrdc_reg, nb_subbsn(iigrdc_reg)) = ig
    !              ob_grdc(iigrdc_reg, nb_subbsn(iigrdc_reg)) = ib
    !           ENDIF
    !        ENDDO ! DO ib=1,nbasmax
    !     ENDDO  ! DO ig=1,nbpt
!
    WRITE(numout, *) "COUNT(is_grdc_match), COUNT(is_grdc_match_loc):", &
        & COUNT(is_grdc_match(:)), COUNT(is_grdc_match_loc(:))
    j = 0
    DO iigrdc_reg = 1, igrdc_reg
       og_loc = -1
       ob_loc = -1
       ogloc_loc = -1
       obloc_loc = -1
       ox_loc = -1
       oy_loc = -1
       IF(is_grdc_match(iigrdc_reg)) THEN
          !WRITE(numout, *) "is_grdc_match, is_grdc_match_loc, iigrdc_reg:", & 
          !    & is_grdc_match(iigrdc_reg), is_grdc_match_loc(iigrdc_reg), iigrdc_reg
          j = j + 1  
          !idx_grdc_match_loc(j) = REAL(iigrdc_reg) 
          
          DO ig = 1, nbpt
             DO ib = 1, nbasmax
                IF((og_grdc_mindarea(iigrdc_reg) == ig_loc(ig, ib)).AND.&
                  & (ob_grdc_mindarea(iigrdc_reg) == ib_loc(ig, ib)) &
                  & .AND. og_grdc_mindarea(iigrdc_reg).GT.0 .AND. ob_grdc_mindarea(iigrdc_reg).GT.0) THEN
                    og_loc = ig_loc(ig, ib)
                    ob_loc = ib_loc(ig, ib)
                    ogloc_loc = ig
                    obloc_loc = ib
                    ox_loc = ix_loc(ig)
                    oy_loc = iy_loc(ig)
                ENDIF
             ENDDO
          ENDDO


          og_grdc_match_loc(j) = og_loc
          ob_grdc_match_loc(j) = ob_loc
          ogloc_grdc_match_loc(j) = ogloc_loc
          obloc_grdc_match_loc(j) = obloc_loc
          ox_grdc_match_loc(j) = ox_loc
          oy_grdc_match_loc(j) = oy_loc

          WRITE(numout, '(" No. ", I3, " of  ", I4, " GRDC stations. The GRDC index: ", I4)') &
               &  j, nb_grdc_match_loc, iigrdc_reg
          WRITE(numout, '(" The corresponding og_glo: ", I6, " ob_glo: ", I4, &
               & " og_loc: ", I6, " ob_loc: ", I4, &
               & " ogloc_loc: ", I6, " obloc_loc: ", I4 )') &
               & og_grdc_mindarea(iigrdc_reg), ob_grdc_mindarea(iigrdc_reg), &
               & og_grdc_match_loc(j), ob_grdc_match_loc(j), &
               & ogloc_grdc_match_loc(j), obloc_grdc_match_loc(j)
           
          !IF(ogloc_loc .NE. -1 .AND. obloc_loc .NE. -1) THEN
          IF(ogloc_loc .GT. 0 .AND. ogloc_loc .LE. nbpt .AND. &
             obloc_loc .GT. 0 .AND. obloc_loc .LE. nbasmax ) THEN

             !IF(global_basinid_loc(ogloc_loc,obloc_loc) .LT. 0 &
             !   & .OR. global_basinid_loc(ogloc_loc,obloc_loc) .GT. nb_small) THEN
             !   basin_names(global_basinid_loc(ogloc_loc,obloc_loc)) = "NOT DEFINED"
             !ENDIF

             lat_grdc_match_loc(j,1) = route_outlet_loc(ogloc_loc,obloc_loc, 1)
             lon_grdc_match_loc(j,1) = route_outlet_loc(ogloc_loc,obloc_loc, 2)
             WRITE(numout, '(" lat_grdc_match_loc(mod): ",F10.3, "; lon_grdc_match_loc(mod): ", F10.3)')  &
                  & lat_grdc_match_loc(j,1), lon_grdc_match_loc(j,1)
             area_grdc_match_loc(j,1) = areaupbasin_loc(ogloc_loc,obloc_loc)/mille/mille
             WRITE(numout, '(" area_grdc_match_loc(mod): ", F12.4)') area_grdc_match_loc(j,1)
             !basin_grdc_match_loc(j,1) = "basin_names" !basin_names(global_basinid_loc(ogloc_loc,obloc_loc))
             basin_grdc_match_loc(j,1) = basin_allnames_loc(ogloc_loc,obloc_loc)
             WRITE(numout, '(" basin_grdc_match_loc(mod): ", A20)') basin_grdc_match_loc(j,1)


             IF(idx_mod_match_loc(ogloc_loc, obloc_loc).LT.0 .OR. & 
                 & idx_mod_match_loc(ogloc_loc, obloc_loc).GT.igrdc_reg) THEN
                 CALL ipslerr_p(3, 'routing_reg_findgrdc', &
                    & 'The idx_mod_match_loc is wrong.', 'Check ogloc_loc and obloc_loc','')
             ENDIF
             idx_grdc_match_loc(j) = REAL(idx_obs_reg(idx_mod_match_loc(ogloc_loc, obloc_loc)))
             WRITE(numout, '(" idx_grdc_match_loc(obs): ", F6.0)') idx_grdc_match_loc(j)
             lat_grdc_match_loc(j,2) = lat_obs_reg(idx_mod_match_loc(ogloc_loc, obloc_loc))
             lon_grdc_match_loc(j,2) = lon_obs_reg(idx_mod_match_loc(ogloc_loc, obloc_loc))
             WRITE(numout,  '(" lat_grdc_match_loc(obs): ", F10.3, " lon_grdc_match_loc(obs): ", F10.3)') &
                  & lat_grdc_match_loc(j,2), lon_grdc_match_loc(j,2)
             area_grdc_match_loc(j,2) = area_obs_reg(idx_mod_match_loc(ogloc_loc, obloc_loc))/mille/mille
             WRITE(numout, '(" area_grdc_match_loc(obs): ", F12.4)') area_grdc_match_loc(j,2)
             basin_grdc_match_loc(j,2) = river_obs_reg(idx_mod_match_loc(ogloc_loc, obloc_loc))
             WRITE(numout, '(" basin_grdc_match_loc(obs):", A20)') basin_grdc_match_loc(j,2)
             !mergedhydro_grdc_match(i,2) = mergedhydro_obs(idx_mod_match_loc(ogloc_grdc_match_loc(i), obloc_grdc_match_loc(i)))

             hydrodiag_grdc_loc(ogloc_grdc_match_loc(j), obloc_grdc_match_loc(j)) = 1
             !WRITE(numout, *) 'i,idx_grdc_match_loc(j):', i, idx_grdc_match_loc(j)
          ENDIF ! IF(ogloc_loc .NE. -1 .AND. obloc_loc .NE. -1) THEN 

       ENDIF
    ENDDO


    IF(j .NE. nb_grdc_match_loc)  THEN
        WRITE(numout,*) 'ATTENTION: Total matched GRDC not consistent, j, nb_grdc_match_loc:', &
             j, nb_grdc_match_loc
        CALL ipslerr_p(3, 'routing_reg_findgrdc', &
             & 'The total matched GRDC stations not consistent.',&
             & 'Check j and nb_grdc_match_loc','')
    ENDIF

    CALL gather(hydrodiag_grdc_loc,hydrodiag_grdc_glo)
    WRITE(numout, *) "SIZE, MAX, MIN: idx_mod_match_glo", &
         & SIZE(hydrodiag_grdc_glo), MAXVAL(hydrodiag_grdc_glo), MINVAL(hydrodiag_grdc_glo)
    IF(MAXVAL(hydrodiag_grdc_glo).GT.1 .OR. MINVAL(hydrodiag_grdc_glo).LT.-1) THEN
        CALL ipslerr_p(3, 'routing_reg_findgrdc', &
             & 'The Max. and Min. of hydrodiag_grdc_glo out of range.',&
             & 'Check hydrodiag_grdc_glo','')
    ENDIF

    ! 5. Post-Processing for matched GRDC stations 
    !
    ! Put all the matched grdc stations into a 1D array.

    IF(is_root_prc) THEN    
       WRITE(numout, *) "nb_grdc_match_glo, num_largest:", nb_grdc_match_glo, num_largest
       ! Print some diagnostics
       WRITE(numout, *) ' >>> Statistics for GRDC vs. model:'
       WRITE(numout, '(" nbp_glo:", I8, "; nbasmax:", I5, "; nbp_glo*nbasmax:", I14)') &
           & nbp_glo, nbasmax, nbp_glo * nbasmax
       WRITE(numout, '(" ==> Total GRDC stations in this domain:", I6)') igrdc_reg
       WRITE(numout, '(" No. of GRDC with area >= 1e6 km2:     ", I7, "; No. of matched:", I5)') &
           & COUNT(area_obs_reg(:)/mille/mille .GE. 1.0E6), &
           & COUNT(is_grdc_match_glo(:) .AND. area_obs_reg(:)/mille/mille .GE. 1.0E6)
       WRITE(numout, '(" No. of GRDC with area >=1e5, <1e6 km2:", I7, "; No. of matched:", I5)') &
           & COUNT(area_obs_reg(:)/mille/mille .GE. 1.0E5 .AND. area_obs_reg(:)/mille/mille .LT. 1.0E6),&
           & COUNT(is_grdc_match_glo(:) .AND. area_obs_reg(:)/mille/mille .GE. 1.0E5 .AND. area_obs_reg(:)/mille/mille .LT. 1.0E6)
       WRITE(numout, '(" No. of GRDC with area >=1e4, <1e5 km2:", I7, "; No. of matched:", I5)') &
           & COUNT(area_obs_reg(:)/mille/mille .GE. 1.0E4 .AND. area_obs_reg(:)/mille/mille .LT. 1.0E5), &
           & COUNT(is_grdc_match_glo(:) .AND. area_obs_reg(:)/mille/mille .GE. 1.0E4 .AND. area_obs_reg(:)/mille/mille .LT. 1.0E5)
       WRITE(numout, '(" No. of GRDC with area >=1e3, <1e4 km2:", I7, "; No. of matched:", I5)') &
           & COUNT(area_obs_reg(:)/mille/mille .GE. 1.0E3 .AND. area_obs_reg(:)/mille/mille .LT. 1.0E4), &
           & COUNT(is_grdc_match_glo(:) .AND. area_obs_reg(:)/mille/mille .GE. 1.0E3 .AND. area_obs_reg(:)/mille/mille .LT. 1.0E4)
       WRITE(numout, '(" No. of GRDC with area < 1e3 km2:      ", I7, "; No. of matched:", I5)') &
           & COUNT(area_obs_reg(:)/mille/mille .LT. 1.0E3), &
           & COUNT(is_grdc_match_glo(:) .AND. area_obs_reg(:)/mille/mille .LT. 1.0E3)

       WRITE(numout, '(" Total matched GRDC stations : ", I6, F10.3, "%")'), nb_grdc_match_glo, REAL(nb_grdc_match_glo)/REAL(igrdc_reg)*100.0
       WRITE(numout, '(" Total matched grids/subbasins:", I6, F10.3, "%")'), nmatch_glo, REAL(nmatch_glo)/(REAL(nbp_glo)*REAL(nbasmax))*100.0
       WRITE(numout, '(" Total grids with >2 GRDC stations: ", I6)') COUNT(ngrdc_grid_glo(:) .GT. 2, 1)
       WRITE(numout, '("                   2 GRDC stations: ", I6)') COUNT(ngrdc_grid_glo(:) .EQ. 2, 1)
       WRITE(numout, '("                   1 GRDC stations: ", I6)') COUNT(ngrdc_grid_glo(:) .EQ. 1, 1)
       WRITE(numout, '("                   0 GRDC stations: ", I6)') COUNT(ngrdc_grid_glo(:) .EQ. 0, 1)
       WRITE(numout, *) '========================================='
    ENDIF ! IF(is_root_prc)

    ! 5. Deallocate variables 
    !IF (ALLOCATED(idx_obs_reg))      DEALLOCATE(idx_obs_reg, stat=ALLOC_ERR)
    IF (ALLOCATED(darea_tmp))         DEALLOCATE(darea_tmp, stat=ALLOC_ERR)
    IF (ALLOCATED(nb_subbsn))         DEALLOCATE(nb_subbsn, stat=ALLOC_ERR)
    IF (ALLOCATED(og_grdc))           DEALLOCATE(og_grdc, stat=ALLOC_ERR)
    IF (ALLOCATED(ob_grdc))           DEALLOCATE(ob_grdc, stat=ALLOC_ERR)
!    IF (ALLOCATED(mask_og_grdc))      DEALLOCATE(mask_og_grdc, stat=ALLOC_ERR)

    IF (ALLOCATED(is_grdc_match))     DEALLOCATE(is_grdc_match, stat=ALLOC_ERR)
    IF (ALLOCATED(is_grdc_match_loc))     DEALLOCATE(is_grdc_match_loc, stat=ALLOC_ERR)
    IF (ALLOCATED(is_grdc_match_glo))     DEALLOCATE(is_grdc_match_glo, stat=ALLOC_ERR)
!    IF (ALLOCATED(ig_glo))                  DEALLOCATE(ig_glo)
!    IF (ALLOCATED(ib_glo))                  DEALLOCATE(ib_glo)
!    IF (ALLOCATED(ig_loc))                  DEALLOCATE(ig_loc)
!    IF (ALLOCATED(ib_loc))                  DEALLOCATE(ib_loc)
!    IF (ALLOCATED(ix_glo))                  DEALLOCATE(ix_glo)
!    IF (ALLOCATED(iy_glo))                  DEALLOCATE(iy_glo)
!    IF (ALLOCATED(ix_loc))                  DEALLOCATE(ix_loc)
!    IF (ALLOCATED(iy_loc))                  DEALLOCATE(iy_loc)
    IF (ALLOCATED(og_grdc_mindarea))  DEALLOCATE(og_grdc_mindarea, stat=ALLOC_ERR)
    IF (ALLOCATED(ob_grdc_mindarea))  DEALLOCATE(ob_grdc_mindarea, stat=ALLOC_ERR)
    IF (ALLOCATED(dlat))                  DEALLOCATE(dlat)
    IF (ALLOCATED(dlon))                  DEALLOCATE(dlon)
    IF (ALLOCATED(darea))                 DEALLOCATE(darea)
    IF (ALLOCATED(darea_rel))             DEALLOCATE(darea_rel)
    IF (ALLOCATED(dist))                  DEALLOCATE(dist)
    IF (ALLOCATED(dlat_glo))              DEALLOCATE(dlat_glo)
    IF (ALLOCATED(dlon_glo))              DEALLOCATE(dlon_glo)
    IF (ALLOCATED(darea_glo))             DEALLOCATE(darea_glo)
    IF (ALLOCATED(darea_rel_glo))         DEALLOCATE(darea_rel_glo)
    IF (ALLOCATED(dist_glo))              DEALLOCATE(dist_glo)

    IF (ALLOCATED(ngrdc_grid_glo))             DEALLOCATE(ngrdc_grid_glo)
    IF (ALLOCATED(ngrdc_grid_loc))             DEALLOCATE(ngrdc_grid_loc)

    WRITE(numout, *) 'routing_reg_findgrdc finished!'
   
  END SUBROUTINE routing_reg_findgrdc
  !


!! ================================================================================================================================
!! SUBROUTINE : routing_reg_findlargest
!!
!>\BRIEF : This subroutine find the largest ...
!!
!! DESCRIPTION (definitions, functional, design, flags): 
!! . \n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_findlargest(nbpt) 
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbpt                   !! Domain size (unitless)
    !
!! OUTPUT VARIABLES
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: ig, ib, i, j !! Indices (unitless)
    INTEGER(i_std) :: ALLOC_ERR

!_ ================================================================================================================================

    !ALLOCATE(hydrodiag_largest_loc(nbpt, nbasmax), stat=ALLOC_ERR)
    !IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findlargest','Pb in allocate for hydrodiag_largest_loc','','')
    !!hydrodiag_largest_loc(:,:) = 0

    CALL scatter(hydrodiag_largest_glo, hydrodiag_largest_loc)
    nb_largest_loc = COUNT(hydrodiag_largest_loc.GT.0)
    WRITE(numout, *) "nb_largest_loc, hydrodiag_largest_loc=", &
         & nb_largest_loc !, hydrodiag_largest_loc

    nblargest_mpi = MAX(nb_largest_loc, 0) 
    !WRITE(numout, *) "nblargest_mpi=", nblargest_mpi
   
    ALLOCATE(outpt_coor_largest_loc(nblargest_mpi,2), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findlargest','Pb in allocate for outpt_coor_largest_loc','','')

    ALLOCATE(totarea_largest_loc(nblargest_mpi), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findlargest','Pb in allocate for totarea_largest_loc','','')

    ALLOCATE(hydrographs_largest_loc(nblargest_mpi), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findlargest','Pb in allocate for hydrographs_largest_loc','','')

    ALLOCATE(og_largest_loc(nblargest_mpi), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findlargest','Pb in allocate for og_largest_loc','','')

    ALLOCATE(ob_largest_loc(nblargest_mpi), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findlargest','Pb in allocate for ob_largest_loc','','')

    ALLOCATE(ogloc_largest_loc(nblargest_mpi), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findlargest','Pb in allocate for ogloc_largest_loc','','')

    ALLOCATE(obloc_largest_loc(nblargest_mpi), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_findlargest','Pb in allocate for obloc_largest_loc','','')

    outpt_coor_largest_loc(:,:) = -999.9
    totarea_largest_loc(:) = -999.9
    hydrographs_largest_loc(:) = 0.0
    og_largest_loc(:) = -1
    ob_largest_loc(:) = -1
    ogloc_largest_loc(:) = -1
    obloc_largest_loc(:) = -1

    i = 0
    DO ig = 1, nbpt
      DO ib = 1, nbasmax
        IF(hydrodiag_largest_loc(ig, ib) > 0) THEN
          i = i + 1
          outpt_coor_largest_loc(i,:) = route_outlet_loc(ig, ib, :)
          totarea_largest_loc(i) = areaupbasin_loc(ig, ib)
          ogloc_largest_loc(i) = ig
          obloc_largest_loc(i) = ib
          og_largest_loc(i) = ig_loc(ig, ib)
          ob_largest_loc(i) = ib_loc(ig, ib)
        ENDIF
      ENDDO
    ENDDO
    WRITE(numout,*) "nblargest_mpi:", nblargest_mpi
    WRITE(numout,*) "outpt_coor_largest_loc:",outpt_coor_largest_loc
    WRITE(numout,*) "totarea_largest_loc:", totarea_largest_loc
    WRITE(numout,*) "og_largest_loc:", og_largest_loc
    WRITE(numout,*) "ob_largest_loc:", ob_largest_loc
    WRITE(numout,*) "ogloc_largest_loc:", ogloc_largest_loc
    WRITE(numout,*) "obloc_largest_loc:", obloc_largest_loc

  END SUBROUTINE routing_reg_findlargest

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_allnames
!!
!>\BRIEF This subroutine lists the name of all the rivers in ORCHIDEE.
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_allnames(nbpt, nbasmax, id, allnames)
    !
    IMPLICIT NONE
    !
    ! Arguments
    !
!! In
    INTEGER(i_std), INTENT(in) :: nbpt !!
    INTEGER(i_std), INTENT(in) :: nbasmax !!
    INTEGER(i_std), DIMENSION(nbpt, nbasmax), INTENT(in) :: id !!
    !
!! Out
    CHARACTER(LEN=*), DIMENSION(nbpt, nbasmax), INTENT(inout) :: allnames !! Name of the basins (unitless)
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: lenstr, i, ig, ib !!
    CHARACTER(LEN=60) :: tmp_str !!

!_ ================================================================================================================================
    !
    DO ig = 1, nbpt
       DO ib = 1, nbasmax
          IF(id(ig,ib) .GT. 0 .AND. &
            & id(ig,ib) .LE. nb_small) THEN
              allnames(ig, ib) = basin_names(id(ig,ib))
          ELSE
              WRITE (allnames(ig, ib),'(A5,I7)') 'GRDC_', id(ig,ib)
          ENDIF
       ENDDO
    ENDDO

    !CALL routing_reg_names(nb_small, basin_names)
    !lenstr = LEN(basin_names(1))
    !
    !DO i=1,numlar
    !   tmp_str = list_names(i)
    !   basin_names(i) = tmp_str(1:MIN(lenstr,LEN_TRIM(tmp_str)))
    !ENDDO
    !
  END SUBROUTINE routing_reg_allnames

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_grdcbasin
!!
!>\BRIEF : This subroutine write the basin of each GRDC station.
!!
!! DESCRIPTION (definitions, functional, design, flags): 
!! . \n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_grdcbasin 
    !
    USE netcdf
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    !
!! OUTPUT VARIABLES
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: ig, ib, i, j, ic, icc !! Indices (unitless)
    INTEGER(i_std) :: ij, ik, lcc, ierr_tot !! Indices (unitless)
    INTEGER(i_std) :: ign,ibn, og, ob, igrdc          !! Indices (unitless)
    INTEGER(i_std) :: igrdc_loc, igrdc_glo
    INTEGER(i_std) :: ig_tmp, ib_tmp

    ! Finding the correponding GRDC stations
    LOGICAL :: err_nbpt_grdc_basin                             !!  (true/false)
    CHARACTER(LEN=80) :: grdc_river_file_name !! Filename in which we write the description of the rivers (unitless)
    CHARACTER(LEN=45), ALLOCATABLE, DIMENSION(:) :: grdcbsn_names !!
    INTEGER(i_std) :: name_found, ierr_name
    CHARACTER(LEN=6) :: name_tmp
    CHARACTER(LEN=3) :: nn !!
    INTEGER(i_std) :: fid, iret
    INTEGER(i_std) :: dims(2) !!
    REAL(r_std) :: lon_min, lon_max, lat_min, lat_max
    CHARACTER(LEN=80) :: lon_name, lat_name, var_name, long_name, nc_name, att_str
    INTEGER(i_std) :: nlonid, nlatid, varid, varid2, varid3
    LOGICAL :: check=.TRUE. !! (true/false)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: stabasin2d !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: stabasin1d_glo !!
    !
!! PARAMETERS
    INTEGER(i_std),PARAMETER :: kind_r_diag=NF90_REAL8 !!
    INTEGER(i_std),PARAMETER :: kind_i_diag=NF90_INT !!

    INTEGER(i_std) :: iigrdc, iigrdc_reg       !!
    INTEGER(i_std) :: ib_grdc
    INTEGER(i_std) :: ALLOC_ERR

    INCLUDE 'mpif.h'

!_
!================================================================================================================================
    !
    !
    ! 1. Initialization
    !WRITE(numout,*) 'start allocate in reg_grdcbasin g_stabsn'
    WRITE(numout,*) 'nb_grdc_match_glo, nbp_glo, iim_g, jjm_g:', nb_grdc_match_glo, nbp_glo, iim_g, jjm_g

    ALLOCATE(g_stabsn(ngrdc, nbp_glo*16), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for g_stabsn','','')

    !WRITE(numout,*) 'start allocate in reg_grdcbasin b_stabsn'
    ALLOCATE(b_stabsn(ngrdc, nbp_glo*16), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for b_stabsn','','')

    !WRITE(numout,*) 'start allocate in reg_grdcbasin nb_subb'
    ALLOCATE(nb_subb(ngrdc), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for nb_subb','','')


    ALLOCATE(g_stabsn_loc(nbsta_mpi, nbp_glo*16), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for g_stabsn_loc','','')

    !WRITE(numout,*) 'start allocate in reg_grdcbasin b_stabsn'
    ALLOCATE(b_stabsn_loc(nbsta_mpi, nbp_glo*16), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for b_stabsn_loc','','')

    !WRITE(numout,*) 'start allocate in reg_grdcbasin nb_subb'
    ALLOCATE(nb_subb_loc(nbsta_mpi), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for nb_subb_loc','','')


    ALLOCATE(gloc_stabsn_loc(nbsta_mpi, nbp_glo*16), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for gloc_stabsn_loc','','')

    ALLOCATE(bloc_stabsn_loc(nbsta_mpi, nbp_glo*16), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for bloc_stabsn_loc','','')


    ALLOCATE(map_stabsn_glo(nbp_glo, nbasmax, 30), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for map_stabsn_glo','','')

    ALLOCATE(map_stabsn_loc(nbp_loc, nbasmax, 30), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for map_stabsn_loc','','')

    ALLOCATE(nb_stabsn_glo(nbp_glo, nbasmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for nb_stabsn_glo','','')

    ALLOCATE(nb_stabsn_loc(nbp_loc, nbasmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for nb_stabsn_loc','','')

    IF (is_root_prc) THEN
    !WRITE(numout,*) 'start allocate in reg_grdcbasin stabasin2d'
    ALLOCATE(stabasin2d(iim_g, jjm_g), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for stabasin2d','','')

    !WRITE(numout,*) 'start allocate in reg_grdcbasin stabasin1d_glo'
    ALLOCATE(stabasin1d_glo(nbp_glo), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for stabasin1d_glo','','')

    !WRITE(numout,*) 'start allocate in reg_grdcbasin grdcbsn_names'
    ALLOCATE(grdcbsn_names(ngrdc), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_grdcbasin','Pb in allocate for grdcbsn_names','','')

    nb_subb(:) = 0
    err_nbpt_grdc_basin = .FALSE.
    stabasin2d(:,:) = -1
    g_stabsn(:,:) = -1
    b_stabsn(:,:) = -1
    stabasin1d_glo(:) = -1 
    ierr_name = 0
    map_stabsn_glo(:,:,:) = -1

    DO i = 1, ngrdc
        WRITE (name_tmp,'(I6)') INT(idx_grdc_match_glo(i))
        WRITE (numout, *) 'name_tmp=', name_tmp, INT(idx_grdc_match_glo(i))
        
        IF(og_grdc_match_glo(i) > 0 .AND. ob_grdc_match_glo(i) > 0 &
           .AND. og_grdc_match_glo(i) < nbp_glo .AND. ob_grdc_match_glo(i) < nbasmax ) THEN
            grdcbsn_names(i) = &
              & TRIM(basin_allnames_glo(og_grdc_match_glo(i),ob_grdc_match_glo(i))) !//"_"//TRIM(name_tmp)
        ELSE
            ierr_name = ierr_name + 1
            WRITE (grdcbsn_names(i),'(A9,I4)') 'GRDC_ERR_', INT(ierr_name)
        ENDIF       
        WRITE (numout, *) 'grdcbsn_names=', grdcbsn_names(i)
         
    ENDDO

    ! Check for doubles and rename if needed
    !
    DO icc = 1, ngrdc
       name_found=0
       DO ic=1, ngrdc
          IF ( TRIM(grdcbsn_names(icc)) == TRIM(grdcbsn_names(ic)) ) THEN
             name_found = name_found + 1
          ENDIF
       ENDDO
       !
       IF ( name_found > 1 ) THEN
          DO ic=ngrdc,1,-1
             IF ( TRIM(grdcbsn_names(icc)) == TRIM(grdcbsn_names(ic)) ) THEN
                IF ( name_found > 1 ) THEN
                   WRITE(nn,'(I3)') name_found
                   grdcbsn_names(ic) = TRIM(grdcbsn_names(ic))//nn
                   name_found = name_found - 1
                ENDIF
             ENDIF
          ENDDO
       ENDIF
       !
    ENDDO


    WRITE(numout,*) 'start loopgrdcbasin'
    og = -1
    ob = -1
    IF (1== 0) THEN
    DO icc = 1,ngrdc
      !WRITE(numout,*) 'icc=', icc
      loopgrdcbasin : DO ig=1,nbp_glo
      !     
      !WRITE(numout,*) 'icc, ig=', icc, ig
        DO ib=1,nbasmax
            IF ( routing_area_glo(ig,ib) .GT. zero ) THEN
                ic = 0
                ign = ig
                ibn = ib
                ! Locate outflow point
                DO WHILE (ibn .GT. 0 .AND. ibn .LE. nbasmax .AND. ic .LT. nbasmax*nbp_glo &
                  & .AND. ign .GT. 0 .AND. ign .LE. nbp_glo &
                  & .AND. (ibn.NE.ob_grdc_match_glo(icc) .OR. ign.NE.og_grdc_match_glo(icc)))
                    ic = ic + 1
                    og = ign
                    ob = ibn
                    ign = route_togrid_glo(og, ob)
                    ibn = route_tobasin_glo(og, ob)
                    !areaupbasin(og, ob) = areaupbasin(og, ob) + routing_area_glo(ig,ib)
                    !nbasupbasin(og, ob) = nbasupbasin(og, ob) + 1
                    !
                ENDDO
                !WRITE(numout,*) 'og, ob, ign, ibn=', og, ob, ign, ibn
                !
!                longest_river = MAX(longest_river, ic)
                !
                ! Now that we have an outflow check if it is one of the num_largest rivers.
                ! In this case we keeps the location so we diagnose it.
                !
                !IF ( ibn .EQ. nbasmax + 3) THEN
                    !DO icc = 1,num_largest
                IF ( ign .EQ. og_grdc_match_glo(icc) .AND. ibn .EQ. ob_grdc_match_glo(icc) & 
                  & .AND. ign .NE. -1 .AND. ibn .NE. -1) THEN
                    !IF ( outpt(icc,1) .EQ. og .AND. outpt(icc,2) .EQ. ob ) THEN
                    !
                    ! We only keep this point for our map if it is large enough.
                    !
                    nb_subb(icc) = nb_subb(icc) + 1
                    !
                    ! Trung: using nbpt*64 here with the same reason which mentioned above
                    !
                    IF ( nb_subb(icc) > nbp_glo*16 ) THEN
                        err_nbpt_grdc_basin = .TRUE.
                        EXIT loopgrdcbasin
                    ENDIF
                    !
                    g_stabsn(icc, nb_subb(icc)) = ig
                    b_stabsn(icc, nb_subb(icc)) = ib
                    !WRITE(numout,*) 'nb_subb, ib, ig, ibn, ign=', nb_subb(icc), ib, ig, ibn, ign
                ENDIF
            ENDIF
        ENDDO
      ENDDO loopgrdcbasin
    ENDDO 
    ENDIF
    
    loopgrdcbsn : DO ig=1,nbp_glo
        IF(MOD(ig,100) == 0) WRITE(numout,*) 'ig=', ig, 'of', nbp_glo
        DO ib=1,nbasmax
            IF ( routing_area_glo(ig,ib) .GT. zero ) THEN
                ic = 0
                ign = ig
                ibn = ib
                ! Locate outflow point
                DO WHILE (ibn .GT. 0 .AND. ibn .LE. nbasmax .AND. ic .LT. nbasmax*nbp_glo*16 &
                  & .AND. ign .GT. 0 .AND. ign .LE. nbp_glo ) !& 
                !  !& .AND. (ibn.NE.ob_grdc_match_glo(icc) .OR. ign.NE.og_grdc_match_glo(icc)))
                    !WRITE(numout,*) 'icc=', icc
                    !WRITE(numout,*) 'ob_grdc_match_glo=', ob_grdc_match_glo(icc)
                    !WRITE(numout,*) 'og_grdc_match_glo=', og_grdc_match_glo(icc)
                    ic = ic + 1
                    og = ign
                    ob = ibn
                    !ign = route_togrid_glo(og, ob)
                    !ibn = route_tobasin_glo(og, ob)
                    DO icc = 1, ngrdc             
                        IF ( ign .EQ. og_grdc_match_glo(icc) .AND. ibn .EQ. ob_grdc_match_glo(icc) & 
                          & .AND. ign .NE. -1 .AND. ibn .NE. -1) THEN
                            !
                            ! We only keep this point for our map if it is large enough.
                            !
                            nb_subb(icc) = nb_subb(icc) + 1
                            !WRITE(numout,*) 'nb_subbi,icc:', nb_subb(icc),icc
                            !
                            ! Trung: using nbpt*64 here with the same reason which mentioned above
                            !
                            IF ( nb_subb(icc) > nbp_glo*16 ) THEN
                                err_nbpt_grdc_basin = .TRUE.
                                EXIT !loopgrdcbasin
                            ENDIF
                            !
                            g_stabsn(icc, nb_subb(icc)) = ig
                            b_stabsn(icc, nb_subb(icc)) = ib
                            !WRITE(numout,*) 'nb_subb, ib, ig, ibn, ign=', nb_subb(icc), ib, ig, ibn, ign
                        ENDIF
                    ENDDO
                    ign = route_togrid_glo(og, ob)
                    ibn = route_tobasin_glo(og, ob)
                    !areaupbasin(og, ob) = areaupbasin(og, ob) + routing_area_glo(ig,ib)
                ENDDO
            ENDIF
        ENDDO
    ENDDO loopgrdcbsn
    !WRITE(numout,*) "g_stabsn=", g_stabsn(3,1:nb_subb(3))
    !WRITE(numout,*) "b_stabsn=", b_stabsn(3,1:nb_subb(3))
    !WRITE(numout,*) "nb_subb=", nb_subb(3)
    !WRITE(numout,*) 'og_grdc_match_glo:', og_grdc_match_glo(3)
    !WRITE(numout,*) 'ob_grdc_match_glo:', ob_grdc_match_glo(3)
    !WRITE(numout,*) "g_stabsn=", g_stabsn(2,1:nb_subb(2))
    !WRITE(numout,*) "b_stabsn=", b_stabsn(2,1:nb_subb(2))
    !WRITE(numout,*) "nb_subb=", nb_subb(2)
    !WRITE(numout,*) 'og_grdc_match_glo:', og_grdc_match_glo(2)
    !WRITE(numout,*) 'ob_grdc_match_glo:', ob_grdc_match_glo(2)
    !
    !
    ! 1.0 Create the NETCDF file and store the coordinates.
    !
    !
    !
    !Config Key = GRDC_RIVER_DESC_FILE
    !Config Desc = Filename in which we write the description of the rivers. If suffix is ".nc" a netCDF file is created
    !Config If = GRDC_RIVER_DESC
    !Config Def = grdc_river_desc.txt
    !Config Help = File name where we will write the information. If the suffix is ".nc" a netCDF file is generated. Else
    !Config a simple text file will contain some information. The netCDF file is valuable for post-processing the
    ! data as it will contain the fraction of the large basins in each grid box.
    !Config Units = [FILE]
    !
    grdc_river_file_name="Output/grdc_river_desc.nc"
    CALL getin('GRDC_RIVER_DESC_FILE', grdc_river_file_name)
    !grdcbsn_file_name = "Output/grdcbsn.nc"

    iret = NF90_CREATE(TRIM(grdc_river_file_name), NF90_CLOBBER, fid)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not create file :', &
            & TRIM(grdc_river_file_name), '(Problem with disk place or filename ?)')
    ENDIF
    WRITE(numout,*) 'grdc_river_file created'
    !
    ! 1.1 Define dimensions
    !
    IF ( INDEX(GridType, "RegLonLat") == 1 ) THEN
       !
       ! 1.1.1 regular grid
       !
       iret = NF90_DEF_DIM(fid, 'lon', iim_g, dims(1))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Dimension "x" can not be defined for the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
       iret = NF90_DEF_DIM(fid, 'lat', jjm_g, dims(2))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Dimension "y" can not be defined for the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
    ELSE IF ( INDEX(GridType, "RegXY") == 1 ) THEN
       !
       ! 1.1.2 irregular grid
       !
       iret = NF90_DEF_DIM(fid, 'x', iim_g, dims(1))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Dimension "x" can not be defined for the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF

       iret = NF90_DEF_DIM(fid, 'y', jjm_g, dims(2))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Dimension "y" can not be defined for the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
    ELSE
       WRITE(numout,*) "Gridtype =", GridType, "not implemented yet"
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Cannot deal with this grid type yet.', &
               &         '', '')
    ENDIF
    WRITE(numout,*) 'grdc_river_file dimensions defined'
    !
    !
    ! 1.2 Define variables and attributes
    !
    IF ( INDEX(GridType, "RegLonLat") == 1 ) THEN
       !
       ! 1.2.1 regular grid
       !
       lon_name = 'lon'
       !
       iret = NF90_DEF_VAR(fid, lon_name, kind_r_diag, dims(1), nlonid)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Variable '//lon_name//' can not be defined for the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
       !
       lat_name = 'lat'
       iret = NF90_DEF_VAR(fid, lat_name, kind_r_diag, dims(2), nlatid)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Variable '//lat_name//' can not be defined for the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
       !
    ELSE IF ( INDEX(GridType, "RegXY") == 1 ) THEN
       !
       ! 1.2.2 irregular grid
       !
       lon_name = 'nav_lon'
       !
       iret = NF90_DEF_VAR(fid, lon_name, kind_r_diag, dims, nlonid)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Variable '//lon_name//' can not be defined for the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
       !
       lat_name = 'nav_lat'
       iret = NF90_DEF_VAR(fid, lat_name, kind_r_diag, dims, nlatid)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Variable '//lat_name//' can not be defined for the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
       !
    ELSE
       WRITE(numout,*) "Gridtype = ", GridType, " not implemented yet."
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Cannot deal with this grid type yet.', &
               &         '', '')
    ENDIF
    WRITE(numout,*) 'grdc_river_file variables defined'
    !
    ! 1.3 Add attributes to the coordinate variables
    !
    iret = NF90_PUT_ATT(fid, nlonid, 'units', "degrees_east")
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not add attribut to variable '//lon_name//' for the file :', &
            & TRIM(grdc_river_file_name),'(Solution ?)')
    ENDIF
    !
    lon_min = -180.
    lon_max = 180.
    !
    iret = NF90_PUT_ATT(fid, nlonid, 'valid_min', lon_min)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not add attribut to variable '//lon_name//' for the file :', &
            & TRIM(grdc_river_file_name),'(Solution ?)')
    ENDIF
    iret = NF90_PUT_ATT(fid, nlonid, 'valid_max', lon_max)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not add attribut to variable '//lon_name//' for the file :', &
            & TRIM(grdc_river_file_name),'(Solution ?)')
    ENDIF
    !
    iret = NF90_PUT_ATT(fid, nlonid, 'long_name', "Longitude")
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not add attribut to variable '//lon_name//' for the file :', &
            & TRIM(grdc_river_file_name),'(Solution ?)')
    ENDIF
    iret = NF90_PUT_ATT(fid, nlatid, 'units', "degrees_north")
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not add attribut to variable '//lat_name//' for the file :', &
            & TRIM(grdc_river_file_name),'(Solution ?)')
    ENDIF
    !
    lat_max = 90.
    lat_min = -90.
    !
    iret = NF90_PUT_ATT(fid, nlatid, 'valid_min', lat_min)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not add attribut to variable '//lat_name//' for the file :', &
            & TRIM(grdc_river_file_name),'(Solution ?)')
    ENDIF
    iret = NF90_PUT_ATT(fid, nlatid, 'valid_max', lat_max)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not add attribut to variable '//lat_name//' for the file :', &
            & TRIM(grdc_river_file_name),'(Solution ?)')
    ENDIF
    iret = NF90_PUT_ATT(fid, nlatid, 'long_name', "Latitude")
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not add attribut to variable '//lat_name//' for the file :', &
            & TRIM(grdc_river_file_name),'(Solution ?)')
    ENDIF
    !
    iret = NF90_ENDDEF(fid)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not end definitions in the file : ', &
            & TRIM(grdc_river_file_name),'(Solution ?)')
    ENDIF
    !
    WRITE(numout,*) 'grdc_river_file attributes added'
    ! 1.4 Write coordinates
    !
    IF ( INDEX(GridType, "RegLonLat") == 1 ) THEN
       !
       ! 1.4.1 regular grid
       !
       iret = NF90_PUT_VAR(fid, nlonid, lon_g(1:iim_g,1))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not put variable nav_lon  in the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
       !
       iret = NF90_PUT_VAR(fid, nlatid, lat_g(1,1:jjm_g))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not put variable nav_lat  in the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
    ELSE IF ( INDEX(GridType, "RegXY") == 1 ) THEN
       !
       ! 1.4.2 irregular grid
       !
       iret = NF90_PUT_VAR(fid, nlonid, lon_g)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not put variable nav_lon  in the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
       !
       iret = NF90_PUT_VAR(fid, nlatid, lat_g)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not put variable nav_lat  in the file : ', &
               & TRIM(grdc_river_file_name),'(Solution ?)')
       ENDIF
    ELSE
       !
       ! This case was already caught before !
       !
    ENDIF
    WRITE(numout,*) 'grdc_river_file coordinates written'
    !
    ! 2.0 Go through all basins and wirte the information into the netCDF file.
    !

    DO icc = 1, ngrdc
       !
       ! 2.1 Compute the fields to be saved in the file
       !
       ib=icc !sortedrivs(icc)
       !
       !
       IF ( nb_subb(ib) > 0 ) THEN
          !
          WRITE(numout,*) 'nb_subb(ib), ib=', nb_subb(ib), ib
          stabasin2d(:,:) = zero
          !
          ! To write number of cascade (you can delete it later)
          !
          !
          DO ij=1, nb_subb(ib)

             ik=g_stabsn(ib,ij)

             j = ((index_g(ik)-1)/iim_g) + 1
             i = (index_g(ik)-(j-1)*iim_g)

             stabasin2d(i,j) = 1
             !
          ENDDO
          !
          DO i=1,iim_g
             DO j=1,jjm_g
                IF ( stabasin2d(i,j) <= EPSILON(zero) ) THEN
                   stabasin2d(i,j) = undef_sechiba
                ELSE
                   stabasin2d(i,j) = MIN(stabasin2d(i,j), un)
                ENDIF
             ENDDO
          ENDDO
          
         
          ! 2.2 Define the variables in the netCDF file
          !
          iret = NF90_REDEF(fid)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_grdcbasin', &
                  & 'Could not restart definitions in the file : ', &
                  & TRIM(grdc_river_file_name),'(Solution ?)')
          ENDIF
          !
          ! Create a name more suitable for a variable in a netCDF file
          !
          nc_name = TRIM(grdcbsn_names(icc))
          ! Take out all character which could cause problems
          lcc=LEN_TRIM(nc_name)
          DO ij=1,lcc
             IF ( nc_name(ij:ij) == " " ) nc_name(ij:ij) = "_"
             IF ( nc_name(ij:ij) == "(" ) nc_name(ij:ij) = "_"
             IF ( nc_name(ij:ij) == ")" ) nc_name(ij:ij) = "_"
          ENDDO
          ! reduce redundant "__"
          DO ij=1,lcc
             IF ( nc_name(ij:ij+1) == "__" ) nc_name(ij+1:)=nc_name(ij+2:lcc)
          ENDDO
          lcc=LEN_TRIM(nc_name)
          IF ( nc_name(lcc:lcc) == "_" ) nc_name(lcc:lcc) = " "
          !
          !
          ! 2.3 Fraction variable
          !
          IF (check) WRITE(*,*) "Define basin variable and add attributes"
          !
          var_name = TRIM(nc_name)//"_bsn"
          !
          iret = NF90_DEF_VAR(fid, TRIM(var_name), kind_r_diag, dims, varid)
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) trim(NF90_STRERROR(iret))
             CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Variable '//TRIM(var_name)//' can not be defined for the file : ', &
                  & TRIM(grdc_river_file_name),'(Solution ?)')
          ENDIF
          !
          ierr_tot = 0
          ! Units
          iret = NF90_PUT_ATT(fid, varid, 'units', "-")
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Units', iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          ! Long name
          long_name = "Flag of basin "//TRIM(grdcbsn_names(icc))//" per grid box"
          iret = NF90_PUT_ATT(fid, varid, 'long_name', long_name)
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Long_Name', long_name, iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          ! Missing value
          iret = NF90_PUT_ATT(fid, varid, 'missing_value', undef_sechiba)
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Missing value', undef_sechiba, iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ib=icc !sortedrivs(icc)
          IF ( check ) WRITE(*,*) "Doing basin ", icc," corrsdponding to index = ", idx_grdc_match_glo(ib), "ngrdc : ", ngrdc
          !
          ! Nb of grid points in basin
          att_str='Nb_of_grid_points_in_basin'
          iret = NF90_PUT_ATT(fid, varid, att_str, nb_subb(ib))
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Nb of grid points in basin', nb_subb(ib), iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ! Index of GRDC station
          att_str='Index_of_GRDC_Station'
          ! Instead of longitude of ORCHIDEE grid cell:
          ! iret = NF90_PUT_ATT(fid, varid, att_str, lalo(outpt(ib,1),2))
          ! Put the longitude which got from 1 km resolution HydroSHEDS data
          iret = NF90_PUT_ATT(fid, varid, att_str, INT(idx_grdc_match_glo(ib)))
          IF (iret /= NF90_NOERR) THEN
          !   WRITE(*,*) 'Longitude of outflow point', lalo(outpt(ib,1),2), iret
             WRITE(*,*) 'Index of GRDC Station', INT(idx_grdc_match_glo(ib)), iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          !
!          !
          IF (ierr_tot > 0 ) THEN
             CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not add some attributes to variable '//var_name//' for the file :', &
                  & TRIM(grdc_river_file_name),'(Solution ?)')
          ENDIF
          !
          ! 2.6 ENDDEF of netCDF file
          !
          IF (check) WRITE(*,*) "END define"
          !
          iret = NF90_ENDDEF(fid)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_grdbasin', &
                  & 'Could not end definitions in the file : ', &
                  & TRIM(grdc_river_file_name),'(Solution ?)')
          ENDIF
          !
          ! 2.7 Write the data to the file
          !
          IF (check) WRITE(*,*) "Put stabasin2d"
          iret = NF90_PUT_VAR(fid, varid, stabasin2d)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_grdcbasin', 'Could not put variable stabasin2d in the file : ', &
                  & TRIM(grdc_river_file_name),'(Solution ?)')
          ENDIF
          !
       ENDIF
       !
    ENDDO
    !
    IF (check) WRITE(*,*) "Close file"
    !
    ! Close netCDF file and do some memory management.
    !
    iret = NF90_CLOSE(fid)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_grdcbasin', &
            & 'Could not end definitions in the file : ', &
            & TRIM(grdc_river_file_name),'(Solution ?)')
    ENDIF

    ! Map of GRDC subbasins
    nb_stabsn_glo(:,:) = 0
    IF(nb_grdc_match_glo .GT. 0) THEN
        DO igrdc = 1, ngrdc !nb_grdc_match_glo
            !WRITE(numout,*) 'igrdc=', igrdc
            DO i = 1, nb_subb(igrdc)
                ig_tmp = g_stabsn(igrdc, i)
                ib_tmp = b_stabsn(igrdc, i)
                IF(ig_tmp.GT.0 .AND. ig_tmp .LE. nbp_glo &
                  & .AND. ib_tmp.GT.0 .AND. ib_tmp.LE.nbasmax) THEN
                    nb_stabsn_glo(ig_tmp, ib_tmp) = nb_stabsn_glo(ig_tmp, ib_tmp) + 1
                    IF(nb_stabsn_glo(ig_tmp, ib_tmp).GT.30) THEN
                        nb_stabsn_glo(ig_tmp, ib_tmp) = 30
                        WRITE(numout,*) 'igrdc, nb_stabsn_glo > ', igrdc, nb_stabsn_glo(ig_tmp,ib_tmp)
                    ENDIF
                    map_stabsn_glo(ig_tmp, ib_tmp, nb_stabsn_glo(ig_tmp, ib_tmp)) = idx_grdc_match_glo(igrdc)
                ENDIF

            ENDDO
        ENDDO ! DO igrdc = 1, nb_grdc_match_loc
    ENDIF ! IF(nb_grdc_match_loc .GT. 0) THEN


    WRITE(numout, *) 'routing_reg_grdcbasin finished!'
   
    IF (ALLOCATED(stabasin2d))             DEALLOCATE(stabasin2d)
    IF (ALLOCATED(stabasin1d_glo))         DEALLOCATE(stabasin1d_glo)
    IF (ALLOCATED(grdcbsn_names))          DEALLOCATE(grdcbsn_names)

    ENDIF !is_root_proc


    WRITE(numout,*) ' map_stabsn_glo scatter start !'
    WRITE(numout,*) ' map_stabsn_glo: ', SIZE(map_stabsn_glo), MAXVAL(map_stabsn_glo), MINVAL(map_stabsn_glo)
    WRITE(numout,*) ' map_stabsn_loc: ', SIZE(map_stabsn_loc), MAXVAL(map_stabsn_loc), MINVAL(map_stabsn_loc)
    WRITE(numout,*) ' nb_stabsn_glo: ', SIZE(nb_stabsn_glo), MAXVAL(nb_stabsn_glo), MINVAL(nb_stabsn_glo)
    WRITE(numout,*) ' nb_stabsn_loc: ', SIZE(nb_stabsn_loc), MAXVAL(nb_stabsn_loc), MINVAL(nb_stabsn_glo)

    CALL scatter(nb_stabsn_glo,  nb_stabsn_loc)
    WRITE(numout,*) ' nb_stabsn_glo scatter done !'
    CALL scatter(map_stabsn_glo, map_stabsn_loc)
    WRITE(numout,*) ' map_stabsn_glo scatter done !'

    WRITE(numout,*) 'size max min of map_stabsn_loc:', SIZE(map_stabsn_loc), MAXVAL(map_stabsn_loc), MINVAL(map_stabsn_loc) 
    WRITE(numout,*) 'size max min of nb_stabsn_loc:',  SIZE(nb_stabsn_loc),  MAXVAL(nb_stabsn_loc),  MINVAL(nb_stabsn_loc) 
    CALL bcast(g_stabsn) 
    CALL bcast(b_stabsn) 

  END SUBROUTINE routing_reg_grdcbasin
  !

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_diagnostic_p
!!
!>\BRIEF This parallelized subroutine gives a diagnostic of the basins used
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_diagnostic_p(kjit, nbpt, index, lalo, resolution, contfrac, hist_id, hist2_id, minimal)
    !
    IMPLICIT NONE

!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: kjit !! Time step number (unitless)
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    INTEGER(i_std), INTENT(in) :: index(nbpt) !! Indices of the points on the map (unitless)
    REAL(r_std), INTENT(in) :: lalo(nbpt,2) !! Vector of latitude and longitudes (beware of the order !)
    REAL(r_std), INTENT(in) :: resolution(nbpt,2) !! The size of each grid box in X and Y (m)
    REAL(r_std), INTENT(in) :: contfrac(nbpt) !! Fraction of land in each grid box (unitless;0-1)
    INTEGER(i_std),INTENT (in) :: hist_id !! Access to history file (unitless)
    INTEGER(i_std),INTENT (in) :: hist2_id !! Access to history file 2 (unitless)
    LOGICAL, INTENT (in)       :: minimal
    !
!! LOCAL VARIABLES
    REAL(r_std), DIMENSION(nbpt) :: nbrivers !! Number of rivers in the grid (unitless)
    REAL(r_std), DIMENSION(nbpt) :: basinmap !! Map of basins (unitless)
    REAL(r_std), DIMENSION(nbp_glo) :: nbrivers_g !! Number of rivers in the grid (unitless)
    REAL(r_std), DIMENSION(nbp_glo) :: basinmap_g !! Map of basins (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: outpt !! Outflow point for each basin (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: outpt_coor !! Outflow point coordinate for each basin (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: outpt_type !!
    INTEGER(i_std), DIMENSION(nbpt) :: index_gloc
    INTEGER(i_std) :: ij, ijglo, is, ier
    !
    INTEGER(i_std) :: ALLOC_ERR
    INTEGER(i_std) :: i, j

!_ ================================================================================================================================

    !
    IF ( .NOT. ALLOCATED(outpt) ) ALLOCATE(outpt(num_largest, 2), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for outpt','','')

    IF ( .NOT. ALLOCATED(outpt_coor) ) ALLOCATE(outpt_coor(num_largest, 2), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for outpt_coor','','')
   
    IF ( .NOT. ALLOCATED(outpt_type) ) ALLOCATE(outpt_type(num_largest), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for outpt_type','','')
    
    IF ( .NOT. ALLOCATED(outflow_pt_loc) ) ALLOCATE(outflow_pt_loc(num_largest, 2), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for outflow_pt_loc','','')

    IF ( .NOT. ALLOCATED(outflow_lalo_loc) ) ALLOCATE(outflow_lalo_loc(num_largest, 2), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for outflow_lalo_loc','','')
   
    IF ( .NOT. ALLOCATED(outflow_type_loc) ) ALLOCATE(outflow_type_loc(num_largest), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for outflow_type_loc','','')

    IF ( .NOT. ALLOCATED(outflow_loc) ) ALLOCATE(outflow_loc(num_largest), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for outflow_loc','','')
    
    IF (is_root_prc) THEN
       WRITE(numout, *) 'routing_reg_diagnostic start on root_proc'
       CALL routing_reg_diagnostic(nbp_glo, index_g, lalo_g, resolution_g, contfrac_g, minimal, &
            &                      nbrivers_g, basinmap_g, outpt, outpt_coor, outpt_type)
       WRITE(numout, *) 'routing_reg_diagnostic done on root_proc'
    ENDIF
    !
    CALL scatter(routing_area_glo, routing_area_loc)
    CALL scatter(topo_resid_glo, topo_resid_loc)
    CALL scatter(route_togrid_glo, route_togrid_loc)
    CALL scatter(route_tobasin_glo, route_tobasin_loc)
    CALL scatter(route_nbintobas_glo, route_nbintobas_loc)
    CALL scatter(global_basinid_glo, global_basinid_loc)
    !
    CALL scatter(route_outlet_glo, route_outlet_loc)
    CALL scatter(route_type_glo, route_type_loc)
    CALL scatter(origin_nbintobas_glo, origin_nbintobas_loc)

    CALL scatter(nbrivers_g,nbrivers)
    CALL scatter(basinmap_g,basinmap)
    CALL scatter(hydrodiag_glo,hydrodiag_loc)
    CALL scatter(hydroupbasin_glo,hydroupbasin_loc)

    CALL bcast(outpt)
    CALL bcast(outpt_coor)
    CALL bcast(outpt_type)
    CALL scatter(index_g, index_gloc)
    !
    ! Build the list of river outflow points per processor.
    ! When the global index of the grid is a large river then
    ! keep the index of the local point.
    !
    outflow_nb_loc = 0
    outflow_loc(:) = 0
    DO ij = 1,nbpt
       DO is = 1,num_largest
          IF ( outpt(is,1) > 0 ) THEN
             IF ( index_gloc(ij) == index_g(outpt(is,1)) ) THEN
                outflow_nb_loc = outflow_nb_loc + 1
                IF ( outflow_nb_loc == 1 ) outflow_offset_loc = is-1
                outflow_pt_loc(outflow_nb_loc,1) = ij
                outflow_pt_loc(outflow_nb_loc,2) = outpt(is,2)
                outflow_lalo_loc(outflow_nb_loc,:) = outpt_coor(is,:)
                IF ( .NOT. Minimal ) THEN
                   outflow_loc(outflow_nb_loc) = hydrographs(ij)
                ENDIF
                ! Note as arealdy used
                outpt(is,1)=-1
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    !
    ! In .NOT. minimal then Write some information to the history files
    !
    IF ( .NOT. Minimal ) THEN
       IF ( .NOT. ok_calendar ) THEN
          CALL xios_orchidee_send_field("basinmap",basinmap)
          CALL xios_orchidee_send_field("nbrivers",nbrivers)
       ENDIF
       
       IF ( .NOT. almaoutput ) THEN
          CALL histwrite_p(hist_id, 'basinmap', 1, basinmap, nbpt, index)
          CALL histwrite_p(hist_id, 'nbrivers', 1, nbrivers, nbpt, index)
       ELSE
       ENDIF
       IF ( hist2_id > 0 ) THEN
          IF ( .NOT. almaoutput ) THEN
             CALL histwrite_p(hist2_id, 'basinmap', 1, basinmap, nbpt, index)
             CALL histwrite_p(hist2_id, 'nbrivers', 1, nbrivers, nbpt, index)
          ELSE
          ENDIF
       ENDIF
    ENDIF
    !
    ! Free the memory
    !
    DEALLOCATE(outpt)
    DEALLOCATE(outpt_coor)
    DEALLOCATE(outpt_type)     
    !
    ! 2. Write GRDC observations into output
    !
    IF(ok_grdc) THEN

        !
        ! The thres_dlola 
        !
        !Config Key = THRES_DLOLA
        !Config Desc = 
        !Config If = 
        !Config Def = 
        !Config Help = Threshold for the difference in longitude and latitude
        !between ORC and GRDC stations
        !Config Units = degree
        !
        thres_dlola = 0.25
        CALL getin('THRES_DLOLA',thres_dlola)
        WRITE(numout, '(" => THRES_DLOLA: ", F5.2, " Degree")') thres_dlola

        !
        ! The thres_dist 
        !
        !Config Key = THRES_DIST
        !Config Desc = 
        !Config If = 
        !Config Def = 
        !Config Help = Threshold for the distance between ORC and GRDC stations
        !Config Units = m
        !

        thres_dist = 25000.0
        CALL getin('THRES_DIST',thres_dist)
        WRITE(numout, '(" => THRES_DIST: ", F10.2, " m")') thres_dist

        !
        ! The thres_diff_area 
        !
        !Config Key = THRES_DIFF_AREA
        !Config Desc = 
        !Config If = 
        !Config Def = 
        !Config Help = Threshold for the difference in upstream areas
        !between ORC and GRDC 
        !Config Units = %
        !
        thres_diff_area = 10.0
        CALL getin('THRES_DIFF_AREA',thres_diff_area)
        WRITE(numout, '(" => THRES_DIFF_AREA: ", F7.3, " %")') thres_diff_area

        !
        ! Read GRDC data
        !
        !Config Key = GRDC_FILE
        !Config Desc = Global Runoff Data Centre
        !Config If = GRDC
        !Config Def = GRDC_Monthly_June14.nc
        !Config Help = To evaluate the river discharge
        !dimensions: time = 2496; stations = 5723
        !Config Units = [FILE]
        !Config Key = GRDC_FILE
        !Config Desc = Global Runoff Data Centre
        !
        grdc_file = 'GRDC_Monthly_June14.nc'
        CALL getin('GRDC_FILE',grdc_file)
        WRITE(numout, '(" => GRDC data: ", A100)') grdc_file

        !diag_grdc=.TRUE.
        !CALL getin('DIAG_GRDC', diag_grdc)
        !WRITE(numout, *) "diag_grdc:", diag_grdc


        ALLOCATE( basin_allnames_loc(nbpt,nbasmax), STAT=ALLOC_ERR )
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for basin_allnames_loc','','')

        ALLOCATE( basin_allnames_glo(nbp_glo,nbasmax), STAT=ALLOC_ERR )
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for basin_allnames_glo','','')

        ALLOCATE( areaupbasin_loc(nbpt,nbasmax), STAT=ALLOC_ERR )
        IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for areaupbasin_loc','','')

        !
        ! Basin names
        !
        CALL bcast(nb_small)
        !WRITE(numout,*) "nb_small=", nb_small
        IF(.NOT.is_root_prc) THEN
            ALLOCATE(basin_names(nb_small), stat=ALLOC_ERR)
            IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic_p','Pb in allocate for basins_names','','')
        ENDIF
        CALL bcast(basin_names)
        WRITE(numout,*) "size(basin_names), basin_names=", size(basin_names), basin_names(1:10)

        CALL routing_reg_allnames(nbpt,    nbasmax, global_basinid_loc, basin_allnames_loc)
        WRITE(numout, *) 'basin_allnames_loc=', basin_allnames_loc(1:10, 1:10)

        IF(is_root_prc) THEN
           CALL routing_reg_allnames(nbp_glo, nbasmax, global_basinid_glo, basin_allnames_glo)
           WRITE(numout, *) 'basin_allnames_glo=', basin_allnames_glo(1:10, 1:10)
        ENDIF
        !
        !
        CALL scatter(areaupbasin, areaupbasin_loc)
        ! WRITE(numout, *) "areaupbasin_loc=", areaupbasin_loc
        ! WRITE(numout, *) "routing_area_loc", routing_area_loc
        ! WRITE(numout, *) "route_togrid_loc", route_togrid_loc
        ! WRITE(numout, *) "route_tobasin_loc", route_tobasin_loc

        !
        !! Read GRDC data
        CALL routing_reg_readgrdc
        WRITE(numout, *) "CALL readgrdc done!"
        !WRITE(numout, *) "after readgrdc, igrdc_reg=", igrdc_reg

        !! Find GRDC
        CALL routing_reg_findgrdc(nbpt, index) !, idx_obs_reg_sort)
        WRITE(numout,*) "CALL routing_reg_findgrdc done!"
        !ENDIF

        !! Find largest river
        CALL routing_reg_findlargest(nbpt) !, idx_obs_reg_sort)
        WRITE(numout,*) "CALL routing_reg_findlargest done!"
        !ENDIF

        !! Define xios for GRDC
        IF(xios_orchidee_ok) THEN
        CALL routing_reg_xiosgrdc(kjit, nbpt)
        WRITE(numout, *) "CALL routing_reg_xiosgrdc done!"


        IF(.NOT.ok_calendar) THEN
            WRITE(numout, *) "xios_send_field GRDC start"
            ! send field
            !CALL xios_orchidee_send_field("area_grdc_match", area_grdc_match_loc)
            CALL xios_send_field("Index_Stn_GRDC", idx_grdc_match_loc)
            CALL xios_send_field("Og_Stn_Model", REAL(og_grdc_match_loc))
            CALL xios_send_field("Ob_Stn_Model", REAL(ob_grdc_match_loc))
            CALL xios_send_field("Ox_Stn_Model", REAL(ox_grdc_match_loc))
            CALL xios_send_field("Oy_Stn_Model", REAL(oy_grdc_match_loc))
            !CALL xios_send_field("Og_Stn_Model", (idx_grdc_match_loc))
            !CALL xios_send_field("Ob_Stn_Model", (idx_grdc_match_loc))
            WRITE(numout, *) "xios_send_field idx_grdc_match_loc"
            !CALL xios_send_field("River_Stn_GRDC", basin_grdc_match_loc(:,2))
            !CALL xios_send_field("River_Stn_Model", basin_grdc_match_loc(:,1))
            !CALL xios_send_field("River_Stn_GRDC", idx_grdc_match_loc)
            !CALL xios_send_field("River_Stn_Model", idx_grdc_match_loc)
            CALL xios_send_field("Lat_Stn_GRDC", lat_grdc_match_loc(:,2))
            CALL xios_send_field("Lat_Stn_Model", lat_grdc_match_loc(:,1))
            !WRITE(numout, *) "xios_send_field Lat_Stn"
            CALL xios_send_field("Lon_Stn_GRDC", lon_grdc_match_loc(:,2))
            CALL xios_send_field("Lon_Stn_Model", lon_grdc_match_loc(:,1))
            !WRITE(numout, *) "xios_send_field Lon_Stn"
            CALL xios_send_field("Area_Stn_GRDC", area_grdc_match_loc(:,2))
            CALL xios_send_field("Area_Stn_Model", area_grdc_match_loc(:,1))
            !WRITE(numout, *) "xios_send_field Area_Stn"
            CALL xios_send_field("Lat_Outflow_Model", outpt_coor_largest_loc(:,1))
            CALL xios_send_field("Lon_Outflow_Model", outpt_coor_largest_loc(:,2))
            CALL xios_send_field("Area_Outflow_Model", totarea_largest_loc)
            CALL xios_send_field("Og_Outflow_Model", REAL(og_largest_loc))
            CALL xios_send_field("Ob_Outflow_Model", REAL(ob_largest_loc))
            CALL xios_send_field("Ix_Model", REAL(ix_loc))
            CALL xios_send_field("Iy_Model", REAL(iy_loc))
            CALL xios_send_field("Index_Model", REAL(index+(jj_begin-1)*iim_g))
            !WRITE(numout, *) "xios_send_field Outflow"
        ENDIF
        ENDIF

        !! Write the basin of each GRDC station
        CALL routing_reg_grdcbasin !(nbp_glo)

    ENDIF ! IF(ok_grdc)

    !IF(ok_grdc) THEN
        !!IF(ier /= 0) STOP 'area_obs deallocation ERROR'
    !ENDIF
    WRITE(numout, *) 'routing_reg_diagnostic_p finished'

  END SUBROUTINE routing_reg_diagnostic_p

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_diagnostic
!!
!>\BRIEF This non-parallelized subroutine gives a diagnostic of the basins used. This produces some information
!! on the rivers which are being diagnosed.
!!
!! DESCRIPTION (definitions, functional, design, flags) : As not all rivers can be monitored in the model, we will only
!! archive num_largest rivers. In this routine we will diagnose the num_largest largest rivers and print to the standard
!! output the names of these basins and their area. The list of names of these largest rivers are taken from a list coded in the
!! routine routing_reg_names. As this standard output is not sufficient, we will also write it to a netCDF file with the routine
!! routing_reg_diagncfile. It is important to keep for diagnostic the fraction of the largest basins in each grid box and keep information
!! how they are linked one to the other.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): No output variables.
!!
!! REFERENCES : None
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_diagnostic(nbpt, l_index, lalo, resolution, contfrac, minimal, nbrivers, basinmap, &
       &                            outpt, outpt_coor, outpt_type)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    INTEGER(i_std), INTENT(in) :: l_index(nbpt) !! Indices of the points on the map (unitless)
    REAL(r_std), INTENT(in) :: lalo(nbpt,2) !! Vector of latitude and longitudes (beware of the order !)
    REAL(r_std), INTENT(in) :: resolution(nbpt,2) !! The size of each grid box in X and Y (m)
    REAL(r_std), INTENT(in) :: contfrac(nbpt) !! Fraction of land in each grid box (unitless;0-1)
    LOGICAL, INTENT(in)     :: minimal
    !
!! OUTPUT VARIABLES
    REAL(r_std), DIMENSION(nbpt), INTENT(out)             :: nbrivers !! Number of rivers in the grid (unitless)
    REAL(r_std), DIMENSION(nbpt), INTENT(out)             :: basinmap !! Map of basins (unitless)
    INTEGER(i_std), DIMENSION(num_largest,2), INTENT(out) :: outpt    !! Outflow point index for each of the largest basin (grid,sub-basin)
    REAL(r_std), DIMENSION(num_largest,2), INTENT(out)    :: outpt_coor !! Outflow point coordinate for each basin (lat,lon)
    REAL(r_std), DIMENSION(num_largest), INTENT(out)      :: outpt_type !!
    !
!! LOCAL VARIABLES
    INTEGER(i_std), DIMENSION(nbpt,nbasmax) :: outids !! IDs of river to which this basin contributes (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: pts !! List the points belonging to the basin (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: ptbas !! List the basin number for this point (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: nb_pts !! Number of points in the basin (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: totarea !! Total area of basin (m^2)
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: tmparea !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: topids !! The IDs of the first num_largest basins (unitless)
    !CHARACTER(LEN=25), ALLOCATABLE, DIMENSION(:) :: basin_names !! Names of the rivers (unitless)
    CHARACTER(LEN=25) :: name_str !!
    !
    LOGICAL :: river_file !! Choose to write a description of the rivers (true/false)
    CHARACTER(LEN=80) :: river_file_name !! Filename in which we write the description of the rivers (unitless)
    !
    CHARACTER(LEN=25), ALLOCATABLE, DIMENSION(:) :: sorted_names !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: streams_nb !! Number of streams in basin (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: streams_avehops !! Average number of hops in streams (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: streams_minhops !! Minimum number of hops in streams (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: streams_maxhops !! Minimum number of hops in streams (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: streams_resid !! Average residence time
    !
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lbasin_area !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lbasin_uparea !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: lrivercode !!
    !
    INTEGER(i_std) :: ig, ib, og, ob, ign, ibn, ff(1), ic, icc, idbas, slen, ii !! Indices (unitless)
    INTEGER(i_std) :: ier !! Error handling
    CHARACTER(LEN=3) :: nn !!
    INTEGER(i_std) :: name_found !!
    !
    REAL(r_std) :: averesid !!
    REAL(r_std), DIMENSION(nbasmax) :: tmpbas !!
    !REAL(r_std), DIMENSION(nbpt,nbasmax) :: areaupbasin !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: sortedrivs !!
    !
    ! Variables for the river coding
    !
    INTEGER(i_std) :: longest_river !!
    INTEGER(i_std) :: nbmax !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: allstreams !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: upstreamchange !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: tstreams, tslen, tpts, tptbas, tcode !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: tuparea !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: tupstreamchange !!
    !
    LOGICAL :: err_nbpt_grid_basin !! (true/false)
    LOGICAL :: err_basin_number !! (true/false)
    !
    ! To write number of cascade (from each sub-grid basin to the outlet)
    !
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lbasin_upnbas !!
    REAL(r_std), DIMENSION(nbpt,nbasmax) :: nbasupbasin !!
    !
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lbasin_nbas !!

    INTEGER :: ALLOC_ERR !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: idx_maxarea_bas !! 
!_ ================================================================================================================================
    !
    ! Verify first thing that we are opn the root processor.
    !
    IF ( .NOT. is_root_prc) THEN
       WRITE(numout,*) "routing_reg_diagnostic is not suitable for running in parallel"
       WRITE(numout,*) "We are here on a non root processor. is_root_prc = ", is_root_prc
       WRITE(numout,*) "STOP from routing_reg_diagnostic"
       CALL ipslerr_p(3,'routing_reg_diagnostic','This routine is not suitable for running in parallel','','')
    ENDIF
    !
    ! Allocate the variables needed for minimal diagnostics
    !
    IF ( .NOT. ALLOCATED(nb_pts) ) ALLOCATE(nb_pts(num_largest), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for nb_pts','','')

    IF ( .NOT. ALLOCATED(topids) ) ALLOCATE(topids(num_largest), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for topids','','')
 
    IF ( .NOT. ALLOCATED(totarea) ) ALLOCATE(totarea(num_largest), tmparea(num_largest), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for totarea','','')

    IF ( .NOT. ALLOCATED(sortedrivs) ) ALLOCATE(sortedrivs(num_largest), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for sortedrivs','','')
    
    IF ( .NOT. ALLOCATED(sorted_names) ) ALLOCATE(sorted_names(num_largest), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for sorted_names','','')

    ALLOCATE(areaupbasin(nbpt,nbasmax), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for areaupbasin','','')
    WRITE(numout, *) 'finish allocated on root_proc'
    !
    !
    ! First we get the list of all river outflow points
    ! We work under the assumption that we only have num_largest basins finishing with
    ! nbasmax+3. This is checked in routing_reg_truncate.
    !
    nb_small = 1
    outpt(:,:) = -1
    ic = 0
    DO ig=1,nbpt
       DO ib=1,nbasmax
          ign = route_togrid_glo(ig, ib)
          ibn = route_tobasin_glo(ig, ib)
          IF ( ibn .EQ. nbasmax+3) THEN
             ic = ic + 1
             outpt(ic,1) = ig
             outpt(ic,2) = ib
             outpt_coor(ic,1)=route_outlet_glo(ig,ib,1)
             outpt_coor(ic,2)=route_outlet_glo(ig,ib,2)
             !
             outpt_type(ic)=route_type_glo(ig,ib)
             !
             ! Get the largest id of the basins we call a river. This is
             ! to extract the names of all rivers.
             !
             IF ( global_basinid_glo(ig,ib) > nb_small ) THEN
                nb_small = global_basinid_glo(ig,ib)
             ENDIF
             IF(ok_largest) hydrodiag_largest_glo(ig, ib) = 1
          ENDIF
       ENDDO
    ENDDO
!    outpt_coor_largest_glo(:,:) = outpt_coor(:,:)
   
    WRITE(numout, *) 'obtain nb_small'
    !
    ! Trung: With GRDC station information, the maximum ID which can have a
    ! Trung: river name is 6529 (RIO DE ACAPONETA). Need to recheck river names.
    !
    nb_small = MIN(nb_small, 6530)

    ALLOCATE(basin_names(nb_small), stat=ier)
    IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for basins_names','','')

    CALL routing_reg_names(nb_small, basin_names)
    !
    ! Go through all points and basins to see if they outflow as a river and store the
    ! information needed in the various arrays.
    !
    nb_pts(:) = 0
    totarea(:) = zero
    hydrodiag_glo(:,:) = 0
    !
    ! To write number of cascade (from each sub-grid basin to the outlet)
    !
    areaupbasin(:,:) = zero
    nbasupbasin(:,:) = zero
    !
    outids(:,:) = -1
    ob = -1
    og = -1
    !
    !
    longest_river = 0
    !
    err_nbpt_grid_basin = .FALSE.
    DO ig=1,nbpt
       !
       DO ib=1,nbasmax
          IF ( routing_area_glo(ig,ib) .GT. zero ) THEN
             ic = 0
             ign = ig
             ibn = ib
             ! Locate outflow point
             DO WHILE (ibn .GT. 0 .AND. ibn .LE. nbasmax .AND. ic .LT. nbasmax*nbpt)
                ic = ic + 1
                og = ign
                ob = ibn
                ign = route_togrid_glo(og, ob)
                ibn = route_tobasin_glo(og, ob)
                areaupbasin(og, ob) = areaupbasin(og, ob) + routing_area_glo(ig,ib)
                nbasupbasin(og, ob) = nbasupbasin(og, ob) + 1
                !
             ENDDO
             !
             longest_river = MAX(longest_river, ic)
             !
             IF ( ibn .EQ. nbasmax + 3) THEN
                DO icc = 1,num_largest
                   IF ( outpt(icc,1) .EQ. og .AND. outpt(icc,2) .EQ. ob ) THEN
                      !
                      ! We only keep this point for our map if it is large enough.
                      !
                      nb_pts(icc) = nb_pts(icc) + 1
                      !
                      totarea(icc) = totarea(icc) + routing_area_glo(ig,ib)
                      !
                      ! ID of the river is taken from the last point before the outflow.
                      topids(icc) = global_basinid_glo(og,ob)
                      outids(ig,ib) = global_basinid_glo(og,ob)
                      !
                      ! On this gridbox and basin we will diagnose the hydrograph
                      !
                      hydrodiag_glo(ig, ib) = 1
                   ENDIF
                ENDDO
             ENDIF
          ENDIF
          !
       ENDDO
       !
    ENDDO
    !
    ALLOCATE(idx_maxarea_bas(num_largest), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for idx_maxarea_bas','','')
    !
    IF ( err_nbpt_grid_basin ) THEN
       WRITE(numout, *) "routing_reg_diagnostic : The number of grid points in basin ", icc
       WRITE(numout, *) "routing_reg_diagnostic : is larger than anticiped. "
       CALL ipslerr_p(3, 'routing_reg_diagnostic', 'We are heading for a out of bounds in arrays pts, ptsbas and lbasin_area.',&
                     & 'Increase the last dimension of these arrays.','')
    ENDIF
    !
    ! Now we decide which points we will keep from the largest basins
    !
    ! Temporary fix
    !route_nbintobas_glo(:) = 0
    !
    basinmap(:) = zero
    DO ig=1,nbpt
       !
       ! Look for the dominant basin in this grid. This information only affects some
       ! diagnostics : hydrographs and saved area upstream.
       !
       icc = 0
       idbas = -1
       !
       DO ib=1,nbasmax
          IF ( outids(ig,ib) > 0 ) THEN
             IF ( COUNT(outids(ig,:) == outids(ig,ib)) > icc ) THEN
                icc = COUNT(outids(ig,:) == outids(ig,ib))
                idbas = outids(ig,ib)
             ENDIF
          ENDIF
       ENDDO
       !
       ! If we have found a point from the large basins and decided which one
       ! takes over this grid then we note it on the map.
       ! Clean-up a little the hydrodiag array
       !
       IF ( idbas > 0 ) THEN
          basinmap(ig) = REAL(idbas, r_std)
       ENDIF
       !
       ! Now place the hydrograph diagnostic on the point closest to the
       ! ocean.
       !
       tmpbas(:) = zero
       DO ib=1,nbasmax
          IF ( outids(ig,ib) .EQ. idbas) THEN
             tmpbas(ib) = areaupbasin(ig,ib)
          ENDIF
       ENDDO
       hydrodiag_glo(ig,:) = 0
       ff=MAXLOC(tmpbas)
       hydrodiag_glo(ig,ff(1)) = 1
       hydroupbasin_glo(ig) = areaupbasin(ig,ff(1))
       !
    ENDDO
    !
    !
    !
    tmparea(:) = totarea(:)
    DO icc = 1, num_largest
       ff = MAXLOC(tmparea)
       sortedrivs(icc) = ff(1)
       tmparea(ff(1)) = 0.0
    ENDDO

    !
    ! Create the sorted list of names
    !
    WRITE(numout, *) 'sorted list of names start root_proc'
    err_basin_number = .FALSE.
    DO icc = 1, num_largest
       !
       ib=sortedrivs(icc)
       !
       IF ( topids(ib) .GT. nb_small ) THEN
          IF (topids(ib) <= 99 ) THEN
             WRITE(sorted_names(icc), '("Nb_",I2.2)') topids(ib)
          ELSE IF (topids(ib) <= 999 ) THEN
             WRITE(sorted_names(icc), '("Nb_",I3.3)') topids(ib)
          ELSE IF (topids(ib) <= 9999 ) THEN
             WRITE(sorted_names(icc), '("Nb_",I4.4)') topids(ib)
          ELSE IF (topids(ib) <= 99999 ) THEN
             WRITE(sorted_names(icc), '("Nb_",I5.5)') topids(ib)
          ELSE IF (topids(ib) <= 999999 ) THEN
             WRITE(sorted_names(icc), '("Nb_",I6.6)') topids(ib)
          ELSE
             err_basin_number = .TRUE.
             EXIT
          ENDIF
       !
       ELSE
          IF (topids(ib) <= -1 ) THEN
             WRITE(sorted_names(icc), '("Ne_",I2.2)') -1*topids(ib)
          ELSE
             WRITE(*,*) ">>> nb_small, ib, topids :", nb_small, ib, topids(ib)
             sorted_names(icc) = basin_names(topids(ib))
          ENDIF
       ENDIF
       !
    ENDDO
    !
    WRITE(numout, *) 'sorted list of names done on root_proc'
    IF ( err_basin_number ) THEN
       CALL ipslerr_p(3, 'routing_reg_diagnostic', 'We found a basin number larger than 999999.',&
            & 'This is impossible. Please verify your configuration.','')
    ENDIF
    !
    ! Check for doubles and rename if needed
    !
    DO icc = 1, num_largest
       name_found=0
       DO ic=1, num_largest
          IF ( TRIM(sorted_names(icc)) == TRIM(sorted_names(ic)) ) THEN
             name_found = name_found + 1
          ENDIF
       ENDDO
       !
       IF ( name_found > 1 ) THEN
          DO ic=num_largest,1,-1
             IF ( TRIM(sorted_names(icc)) == TRIM(sorted_names(ic)) ) THEN
                IF ( name_found > 1 ) THEN
                   WRITE(nn,'(I1)') name_found
                   sorted_names(ic) = TRIM(sorted_names(ic))//nn
                   name_found = name_found - 1
                ENDIF
             ENDIF
          ENDDO
       ENDIF
       !
    ENDDO
    WRITE(numout, *) 'doulles and rename'
    ! Print to stdout on ROOT_PROC the diagnostics for the largest basins we have found.
    !
    DO icc = 1, num_largest
       !
       IF ( nb_pts(sortedrivs(icc)) .GT. 2 ) THEN
          name_str = sorted_names(icc)
          WRITE(numout,'("Basin ID ", I5," ", A15, " Area [km^2] : ", F13.4, " Nb points : ", I6)')&
               & topids(sortedrivs(icc)), name_str(1:15), totarea(sortedrivs(icc))/1.e6, nb_pts(sortedrivs(icc))
          ENDIF
          !
    ENDDO
    !
    ! Print information if there is more than 1 river outlet for a grid
    !
    nbrivers(:) = zero
    DO ig=1,nbpt
       nbrivers(ig) = COUNT(route_tobasin_glo(ig,1:nbasmax) == nbasmax+3)
    ENDDO
    DO ig=1,nbpt
       IF ( nbrivers(ig) > 1 ) THEN
          WRITE(numout,*) 'Grid box ', ig, ' has ', NINT(nbrivers(ig)), ' outflow points.'
          WRITE(numout,*) 'The rivers which flow into the ocean at this point are :'
          DO icc=1,nbasmax
             IF ( route_tobasin_glo(ig,icc) == nbasmax+3) THEN
                IF ( global_basinid_glo(ig,icc) <= nb_small ) THEN
                   WRITE(numout,*) 'ID = ',global_basinid_glo(ig,icc), ' Name = ', basin_names(global_basinid_glo(ig,icc))
                ELSE
                   WRITE(numout,*) 'ID = ',global_basinid_glo(ig,icc), ' Problem ===== ID is larger than possible'
                ENDIF
             ENDIF
          ENDDO
       ENDIF
    ENDDO
    WRITE(numout, *) 'Print information if there is more than 1 river outlet for a grid'
    !
    !_ ================================================================================================================================    
    !
    !Config Key = RIVER_DESC
    !Config Desc = Writes out a description of the rivers
    !Config If = RIVER_ROUTING
    !Config Def = n
    !Config Help = This flag allows to write out a file containing the list of
    !Config rivers which are beeing simulated. It provides location of outflow
    !Config drainage area, name and ID.
    !Config Units = [FLAG]
    !
    river_file=.FALSE.
    CALL getin('RIVER_DESC', river_file)
    !
    !Config Key = RIVER_DESC_FILE
    !Config Desc = Filename in which we write the description of the rivers. If suffix is ".nc" a netCDF file is created
    !Config If = RIVER_DESC
    !Config Def = river_desc.txt
    !Config Help = File name where we will write the information. If the suffix is ".nc" a netCDF file is generated. Else
    !Config a simple text file will contain some information. The netCDF file is valuable for post-processing the
    !Config data as it will contain the fraction of the large basins in each grid box.
    !Config Units = [FILE]
    !
    river_file_name="river_desc.txt"
    CALL getin('RIVER_DESC_FILE', river_file_name)
    !
    !
    IF ( river_file .AND. (.NOT. minimal) ) THEN
       
       IF ( INDEX(river_file_name,".nc") < 0 ) THEN       

          OPEN(diagunit, FILE=river_file_name)
          WRITE(diagunit,'(A)') "Basin ID, Area [km^2], Nb points, Lon and Lat of outflow"
          WRITE(diagunit,'(A)') "Nb streams, total number of hops, min, ave and max number of hops per stream"
          !
          DO icc = 1, num_largest
             !
             IF ( nb_pts(sortedrivs(icc)) .GT. 2 ) THEN
                !
                name_str = sorted_names(icc)
                !
                WRITE(diagunit,'(I5,A25,F14.5,I5,2F9.2)') topids(sortedrivs(icc)), name_str(1:15), totarea(sortedrivs(icc))/1.e6, &
                     & nb_pts(sortedrivs(icc)), lalo(outpt(sortedrivs(icc),1),2), lalo(outpt(sortedrivs(icc),1),1)
                WRITE(diagunit,'(5I9,F16.4)') streams_nb(sortedrivs(icc)), &
                     & streams_avehops(sortedrivs(icc))*streams_nb(sortedrivs(icc)), &
                     & streams_minhops(sortedrivs(icc)), &
                     & streams_avehops(sortedrivs(icc)), &
                     & streams_maxhops(sortedrivs(icc)), streams_resid(sortedrivs(icc))
                !
             ENDIF
             !
          ENDDO
          !
          CLOSE(diagunit)

       ELSE
          !
          ! Trung: In routing_reg_diagnostic:
          ! Trung: please, notice that few arrays have the last dimension equals
          ! Trung: nbpt*64. This scale (64 times of nbpt) is temporarily attributed.
          ! Trung: Depend on the simulation domain, it might be changed.
          !
          ALLOCATE(pts(num_largest, nbpt*64), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for pts','','')

          ALLOCATE(ptbas(num_largest, nbpt*64), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for ptbas','','')

          ALLOCATE(streams_nb(num_largest), streams_avehops(num_largest), streams_minhops(num_largest), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for streams_nb','','')

          ALLOCATE(streams_maxhops(num_largest), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for streams_maxhops','','')

          ALLOCATE(streams_resid(num_largest), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for streams_resid','','')

          ALLOCATE(lbasin_nbas(num_largest,nbpt*64), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for lbasin_nbas','','')
          
          ALLOCATE(lbasin_area(num_largest,nbpt*64), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for lbasin_area','','')
          
          ALLOCATE(lbasin_uparea(num_largest,nbpt*64), lrivercode(num_largest,nbpt*64), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for lbasin_area','','')
          !
          ! To write number of cascade (from each sub-grid basin to the outlet)
          !
          ALLOCATE(lbasin_upnbas(num_largest,nbpt*64), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for lbasin_upnbas','','')
          !
          !
          ! Diagnose the complexity of the basins obtained and determine their code in the Pfafstetter system
          !
          nbmax=MAXVAL(nb_pts)
          ALLOCATE(allstreams(nbmax, longest_river), upstreamchange(nbmax, longest_river), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for allstreams','','')

          ALLOCATE(tstreams(longest_river), tupstreamchange(longest_river), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for tstreams','','')

          ALLOCATE(tslen(nbmax), tpts(nbmax), tptbas(nbmax), tuparea(nbmax), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for tslen','','')

          ALLOCATE(tcode(nbmax), stat=ier)
          IF (ier /= 0) CALL ipslerr_p(3,'routing_reg_diagnostic','Pb in allocate for tcode','','')
          !
          nbasupbasin(:,:) = zero
          areaupbasin(:,:) = zero
          !
          lbasin_uparea(:,:) = zero
          lbasin_upnbas(:,:) = zero
          lbasin_nbas(:,:) = zero
          lbasin_area(:,:) = zero
          longest_river = 0
          nb_pts(:) = 0
          !
          err_nbpt_grid_basin = .FALSE.
          loopgridbasin : DO ig=1,nbpt
             !
             DO ib=1,nbasmax
                IF ( routing_area_glo(ig,ib) .GT. zero ) THEN
                   ic = 0
                   ign = ig
                   ibn = ib
                   ! Locate outflow point
                   DO WHILE (ibn .GT. 0 .AND. ibn .LE. nbasmax .AND. ic .LT. nbasmax*nbpt)
                      ic = ic + 1
                      og = ign
                      ob = ibn
                      ign = route_togrid_glo(og, ob)
                      ibn = route_tobasin_glo(og, ob)
                      areaupbasin(og, ob) = areaupbasin(og, ob) + routing_area_glo(ig,ib)
                      nbasupbasin(og, ob) = nbasupbasin(og, ob) + 1
                      !
                   ENDDO
                   !
                   longest_river = MAX(longest_river, ic)
                   !
                   ! Now that we have an outflow check if it is one of the num_largest rivers.
                   ! In this case we keeps the location so we diagnose it.
                   !
                   IF ( ibn .EQ. nbasmax + 3) THEN
                      DO icc = 1,num_largest
                         IF ( outpt(icc,1) .EQ. og .AND. outpt(icc,2) .EQ. ob ) THEN
                            !
                            ! We only keep this point for our map if it is large enough.
                            !
                            nb_pts(icc) = nb_pts(icc) + 1
                            !
                            ! Trung: using nbpt*64 here with the same reason which mentioned above
                            !
                            IF ( nb_pts(icc) > nbpt*64 ) THEN
                               err_nbpt_grid_basin = .TRUE.
                               EXIT loopgridbasin
                            ENDIF
                            !
                            pts(icc, nb_pts(icc)) = ig
                            ptbas(icc, nb_pts(icc)) = ib
                            !
                            lbasin_area(icc,nb_pts(icc)) = routing_area_glo(ig,ib)
                            lbasin_nbas(icc,nb_pts(icc)) = route_nbintobas_glo(ig)
                            !
                         ENDIF
                      ENDDO
                   ENDIF
                ENDIF
                !
             ENDDO
             !
          ENDDO loopgridbasin


          DO icc = 1, num_largest
             !
             ! Work through the largest basins
             !
             idbas = sortedrivs(icc)
             !
             streams_nb(idbas) = 0
             streams_avehops(idbas) = 0
             streams_minhops(idbas) = undef_int
             streams_maxhops(idbas) = 0
             streams_resid(idbas) = zero
             tslen(:) = 0
             !
             allstreams(:,:) = 0
             upstreamchange(:,:) = zero
             !
             DO ii=1,nb_pts(idbas)
                !
                ig = pts(idbas, ii)
                ib = ptbas(idbas, ii)
                !
                lbasin_uparea(idbas,ii) = areaupbasin(ig,ib)
                !
                ! To write number of cascade (from each sub-grid basin to the outlet)
                !
                lbasin_upnbas(idbas,ii) = nbasupbasin(ig,ib)
                !
                !
                slen = 0
                ign = ig
                ibn = ib
                og = ig
                ob = ib
                !
                averesid = zero
                tupstreamchange(:) = zero
                ! go to outflow point to count the number of hops
                DO WHILE (ibn .GT. 0 .AND. ibn .LE. nbasmax)
                   ! Store data
                   slen = slen + 1
                   tstreams(slen) = ign
                   tupstreamchange(slen) = areaupbasin(ign,ibn)-areaupbasin(og,ob)
                   ! Move to next point
                   og = ign
                   ob = ibn
                   ign = route_togrid_glo(og, ob)
                   ibn = route_tobasin_glo(og, ob)
                   averesid = averesid + topo_resid_glo(og, ob)**2
                ENDDO
                !
                allstreams(ii,1:slen) = tstreams(slen:1:-1)
                upstreamchange(ii,1:slen) = tupstreamchange(slen:1:-1)
                tslen(ii) = slen
                !
                ! Save diagnostics
                !
                streams_nb(idbas) = streams_nb(idbas) + 1
                streams_avehops(idbas) = streams_avehops(idbas) + slen
                streams_resid(idbas) = streams_resid(idbas) + SQRT(averesid)
                IF ( slen < streams_minhops(idbas) ) THEN
                   streams_minhops(idbas) = slen
                ENDIF
                IF ( slen > streams_maxhops(idbas) ) THEN
                   streams_maxhops(idbas) = slen
                ENDIF
                !
             ENDDO
             ! build the average
             IF ( streams_nb(idbas) > 0 ) THEN
                streams_avehops(idbas) = streams_avehops(idbas)/streams_nb(idbas)
                streams_resid(idbas) = streams_resid(idbas)/REAL(streams_nb(idbas), r_std)
             ELSE
                ! River without streams ... very rare but happens
                streams_avehops(idbas) = zero
                streams_resid(idbas) = zero
                streams_maxhops(idbas) = zero
                streams_minhops(idbas) = zero
             ENDIF
             !
             !
             ii=nb_pts(idbas)
             tpts(:) = 0
             tpts(1:ii) = pts(idbas,1:ii)
             tptbas(:) = 0
             tptbas(1:ii) = ptbas(idbas,1:ii)
             tuparea(:) = 0
             tuparea(1:ii) = lbasin_uparea(idbas,1:ii)
             !
             CALL routing_reg_diagcode(ii, tpts, tptbas, tuparea, tslen, MAXVAL(tslen), allstreams, upstreamchange, tcode)
             !
             lrivercode(idbas,:) = 0
             lrivercode(idbas,1:ii) = tcode(1:ii)
             !
          ENDDO
          !
          ! Save some of the basin information into files.
          !
          !
          CALL routing_reg_diagncfile(river_file_name, nbpt, lalo, nb_pts, topids, sorted_names, sortedrivs, &
               & pts, lbasin_area, lbasin_uparea, lrivercode, outpt, outpt_coor, outpt_type, streams_nb, streams_avehops, &
               & streams_minhops, streams_maxhops, streams_resid, lbasin_upnbas, lbasin_nbas)
          !
          ! Trung: the Maximum value of topo_resid_glo need to recheck (= 999999.000)
          ! 
          WRITE(numout,*) 'Maximum topographic index :', MAXVAL(topo_resid_glo)
          ic = COUNT(topo_resid_glo .GT. 0.)
          WRITE(numout,*) 'Mean topographic index :', SUM(topo_resid_glo, MASK=topo_resid_glo .GT. zero)/ic
          WRITE(numout,*) 'Minimum topographic index :', MINVAL(topo_resid_glo, MASK=topo_resid_glo .GT. zero)
          !
          DEALLOCATE(pts)
          DEALLOCATE(ptbas)
          DEALLOCATE(nb_pts)
          DEALLOCATE(topids)
          DEALLOCATE(streams_nb, streams_avehops, streams_minhops, streams_maxhops)
          DEALLOCATE(streams_resid)
          DEALLOCATE(lbasin_nbas)
          DEALLOCATE(lbasin_area)
          DEALLOCATE(lbasin_uparea, lrivercode)
          DEALLOCATE(lbasin_upnbas)
          DEALLOCATE(allstreams, upstreamchange)
          DEALLOCATE(tstreams, tupstreamchange)
          DEALLOCATE(tslen, tpts, tptbas, tuparea)
          DEALLOCATE(tcode)
       ENDIF
    ENDIF
    !
    DEALLOCATE(totarea, tmparea)
    DEALLOCATE(sortedrivs)
    DEALLOCATE(sorted_names)
    !
    !
    IF(ok_grdc) THEN
        IF (ALLOCATED(idx_maxarea_bas))  DEALLOCATE(idx_maxarea_bas, stat=ier)
    ENDIF

    !
    !
  END SUBROUTINE routing_reg_diagnostic
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_diagcode
!!
!>\BRIEF This subroutine determines the code in the Pfafstetter system for all points
!! within the given catchment.
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): streamcode
!!
!! REFERENCES : None
!!
!! FLOWCHART :None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_diagcode(ip, tpts, tpbas, tuparea, tslen, ls, allstreams, upstreamchange, streamcode)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: ip !!
    INTEGER(i_std), INTENT(in) :: ls !!
    INTEGER(i_std), DIMENSION(ip), INTENT(in) :: tpts !!
    INTEGER(i_std), DIMENSION(ip), INTENT(in) :: tpbas !!
    REAL(r_std), DIMENSION(ip), INTENT(in) :: tuparea !!
    INTEGER(i_std), DIMENSION(ip), INTENT(in) :: tslen !!
    INTEGER(i_std), DIMENSION(ip,ls), INTENT(in) :: allstreams !!
    REAL(r_std), DIMENSION(ip,ls), INTENT(in) :: upstreamchange !!
    !
!! OUTPUT VARIABLES
    INTEGER(i_std), DIMENSION(ip), INTENT(out) :: streamcode !!
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: ilev, cntsubbas, ib, ic, i, it, ilevmax, imaxlen, nbzero !!
    INTEGER(i_std) :: tstreamcode(ip)!!
    INTEGER(i_std) :: indsubbas(ip) !!
    INTEGER(i_std) :: iw(ip) !!
    INTEGER(i_std) :: tdiff(ip) !!
    INTEGER(i_std) :: tmpjunc(4) !!
    INTEGER(i_std) :: junction(4) !!
    INTEGER(i_std) :: ff(1) !!
    INTEGER(i_std) :: ll !!
    REAL(r_std) :: chguparea(ip) !!
    REAL(r_std) :: largest !!

!_ ================================================================================================================================
    !
    streamcode(:) = 0
    !
    ! If we accept 4 grid boxes per coded basin then per level we need at least
    ! 4*9=36 boxes.
    !
    ilevmax = 0
    it = ip
    DO WHILE (it >= 36)
       ilevmax = ilevmax+1
       it = it/9
    ENDDO
    !
    DO ilev=1,ilevmax
       !
       ! Count number of sub-basins we already have
       !
       cntsubbas=0
       tstreamcode(:) = streamcode(:)
       DO WHILE ( COUNT(tstreamcode(:) >= 0) > 0 )
         cntsubbas=cntsubbas+1
         indsubbas(cntsubbas) = MAXVAL(tstreamcode(:))
         WHERE ( tstreamcode(:) == indsubbas(cntsubbas) ) tstreamcode = -1
       ENDDO
       !
       ! Go through all these basins in order to find the next Pfafstetter numbers
       !
       DO ib=1,cntsubbas
          !
          ! Get all the streams which have the current Pfadstetter number
          !
          it=0
          DO ic=1,ip
             IF ( streamcode(ic) == indsubbas(ib) ) THEN
                it =it+1
                iw(it)=ic
             ENDIF
          ENDDO
          !
          ! Which is the longest stream in this basin ?
          !
          ff=MAXLOC(tslen(iw(1:it)))
          imaxlen=iw(ff(1))
          chguparea(:) = zero
          chguparea(1:tslen(imaxlen)) = upstreamchange(imaxlen, 1:tslen(imaxlen))
          !
          IF ( COUNT(chguparea(1:tslen(imaxlen)) > 0) < 4 ) THEN
             !
             ! If this subbasin is too small we just set all points to zero
             !
             DO i=1,it
                streamcode(iw(i)) = streamcode(iw(i))*10
             ENDDO
          ELSE
             !
             ! Else do the Pfafstetter numbering
             !
             !
             ! Where do we have the 4 largest change in upstream area on this stream.
             ! This must be the confluence of 2 rivers and thus a junction point.
             !
             largest=pi*R_Earth*R_Earth
             DO i=1,4
                ff = MAXLOC(chguparea(1:tslen(imaxlen)), MASK = chguparea(1:tslen(imaxlen)) < largest)
                tmpjunc(i) = ff(1)
                largest=chguparea(tmpjunc(i))
             ENDDO
             ! sort junctions to go from the outflow up-stream
             ff(1)=0
             DO i=1,4
                junction(i) = MINVAL(tmpjunc, MASK=tmpjunc > ff(1))
                ff(1) = junction(i)
             ENDDO
             !
             ! Find all streams which are identical up to that junction and increase their code accordingly
             !
             DO i=1,it
                ll=MIN(tslen(imaxlen),tslen(iw(i)))
                tdiff(1:ll) = allstreams(imaxlen,1:ll)-allstreams(iw(i),1:ll)
                nbzero = COUNT(tdiff(1:ll) == 0)
                IF (nbzero < junction(1) ) THEN
                   ! Before first of the 4 largest basins
                   streamcode(iw(i)) = streamcode(iw(i))*10+1
                ELSE IF (nbzero == junction(1) ) THEN
                   ! Stream part of the first largest basin
                   streamcode(iw(i)) = streamcode(iw(i))*10+2
                ELSE IF (nbzero < junction(2) ) THEN
                   ! Between first and second stream
                   streamcode(iw(i)) = streamcode(iw(i))*10+3
                ELSE IF (nbzero == junction(2) ) THEN
                   ! Stream part of the second basin
                   streamcode(iw(i)) = streamcode(iw(i))*10+4
                ELSE IF (nbzero < junction(3) ) THEN
                   ! In between stream 2 and 3
                   streamcode(iw(i)) = streamcode(iw(i))*10+5
                ELSE IF (nbzero == junction(3) ) THEN
                   ! Part of 3rd basin
                   streamcode(iw(i)) = streamcode(iw(i))*10+6
                ELSE IF (nbzero < junction(4) ) THEN
                   ! In between 3 and 4th basins
                   streamcode(iw(i)) = streamcode(iw(i))*10+7
                ELSE IF (nbzero == junction(4) ) THEN
                   ! Final of the 4 largest basins
                   streamcode(iw(i)) = streamcode(iw(i))*10+8
                ELSE
                   ! The rest of the points and also the basin of the longest stream
                   streamcode(iw(i)) = streamcode(iw(i))*10+9
                ENDIF
             ENDDO
          ENDIF
       ENDDO
       !
    ENDDO
    !
    !
  END SUBROUTINE routing_reg_diagcode
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_diagncfile
!!
!>\BRIEF This subroutine creates a netCDF file containing all the informations
!! on the largest rivers which can be used for a refined analysis.
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================
!
  SUBROUTINE routing_reg_diagncfile(river_file_name, nbpt, lalo, nb_pts, topids, sorted_names, sortedrivs, &
       & lbasin_index, lbasin_area, lbasin_uparea, lrivercode, outpt, outpt_coor, outpt_type, streams_nb, streams_avehops, &
       & streams_minhops, streams_maxhops, streams_resid, lbasin_upnbas, lbasin_nbas)
    !
    USE netcdf
    !
    IMPLICIT NONE
    !
    !
!! INPUT VARIABLES
    REAL(r_std), INTENT(in) :: lalo(nbpt,2) !! Vector of latitude and longitudes (beware of the order !)

!! LOCAL VARIABLES
    CHARACTER(LEN=80) :: river_file_name !! Filename in which we write the description of the rivers (1)
    INTEGER(i_std) :: nbpt !! Domain size (unitless)
    INTEGER(i_std), DIMENSION(num_largest) :: nb_pts !! Number of points in the basin (unitless)
    INTEGER(i_std), DIMENSION(num_largest) :: topids !! The IDs of the first num_largest basins (unitless)
    CHARACTER(LEN=25), DIMENSION(num_largest) :: sorted_names !! Names of the basins to be put into the file (unitless)
    INTEGER(i_std), DIMENSION(num_largest) :: sortedrivs !!
    INTEGER(i_std), DIMENSION(num_largest,nbpt*64) :: lbasin_index !!
    REAL(r_std), DIMENSION(num_largest,nbpt*64) :: lbasin_area !!
    REAL(r_std), DIMENSION(num_largest,nbpt*64) :: lbasin_uparea !!
    INTEGER(i_std), DIMENSION(num_largest,nbpt*64) :: lrivercode !!
    !
    INTEGER(i_std), DIMENSION(num_largest,2) :: outpt !! Outflow point for each basin (unitless)
    REAL(r_std), DIMENSION(num_largest,2) :: outpt_coor !! Outflow point coordinate for each basin (unitless)
    REAL(r_std), DIMENSION(num_largest) :: outpt_type !! 
    INTEGER(i_std), DIMENSION(num_largest) :: streams_nb !! Number of streams in basin (unitless)
    INTEGER(i_std), DIMENSION(num_largest) :: streams_avehops !! Average number of hops in streams (unitless)
    INTEGER(i_std), DIMENSION(num_largest) :: streams_minhops !! Minimum number of hops in streams (unitless)
    INTEGER(i_std), DIMENSION(num_largest) :: streams_maxhops !! Minimum number of hops in streams (unitless)
    REAL(r_std), DIMENSION(num_largest) :: streams_resid !! Average residence time
    !
    INTEGER(i_std) :: icc, fid, iret, ierr_tot, ib, ij, ik, i, j, lcc !! Indices (unitless)
    INTEGER(i_std) :: nlonid, nlatid, varid, varid2, varid3
    INTEGER(i_std) :: dims(2) !!
    REAL(r_std) :: lon_min, lon_max, lat_min, lat_max
    CHARACTER(LEN=80) :: lon_name, lat_name, var_name, long_name, nc_name, att_str
    !
    REAL(r_std) :: basinfrac(iim_g,jjm_g) !!
    REAL(r_std) :: basinuparea(iim_g,jjm_g) !!
    INTEGER(i_std) :: basincode(iim_g,jjm_g) !!
    !
    LOGICAL :: check=.FALSE. !! (true/false)
    !
!! PARAMETERS
    INTEGER(i_std),PARAMETER :: kind_r_diag=NF90_REAL8 !!
    INTEGER(i_std),PARAMETER :: kind_i_diag=NF90_INT !!
    !
    ! To write number of cascade (from each sub-grid basin to the outlet)
    !
    REAL(r_std), DIMENSION(num_largest,nbpt*64) :: lbasin_upnbas !!
    REAL(r_std) :: basinupnbas(iim_g,jjm_g) !!
    INTEGER(i_std) :: varid4
    !
    REAL(r_std), DIMENSION(num_largest,nbpt*64) :: lbasin_nbas !!
    REAL(r_std) :: basinnbas(iim_g,jjm_g) !!
    INTEGER(i_std) :: varid5
    !

!_ ================================================================================================================================
    !
    !
    ! 1.0 Create the NETCDF file and store the coordinates.
    !
    !
    iret = NF90_CREATE(TRIM(river_file_name), NF90_CLOBBER, fid)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not create file :', &
            & TRIM(river_file_name), '(Problem with disk place or filename ?)')
    ENDIF
    !
    ! 1.1 Define dimensions
    !
    IF ( INDEX(GridType, "RegLonLat") == 1 ) THEN
       !
       ! 1.1.1 regular grid
       !
       iret = NF90_DEF_DIM(fid, 'lon', iim_g, dims(1))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Dimension "x" can not be defined for the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
       iret = NF90_DEF_DIM(fid, 'lat', jjm_g, dims(2))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Dimension "y" can not be defined for the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
    ELSE IF ( INDEX(GridType, "RegXY") == 1 ) THEN
       !
       ! 1.1.2 irregular grid
       !
       iret = NF90_DEF_DIM(fid, 'x', iim_g, dims(1))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Dimension "x" can not be defined for the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF

       iret = NF90_DEF_DIM(fid, 'y', jjm_g, dims(2))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Dimension "y" can not be defined for the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
    ELSE
       WRITE(numout,*) "Gridtype =", GridType, "not implemented yet"
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Cannot deal with this grid type yet.', &
               &         '', '')
    ENDIF
    !
    !
    ! 1.2 Define variables and attributes
    !
    IF ( INDEX(GridType, "RegLonLat") == 1 ) THEN
       !
       ! 1.2.1 regular grid
       !
       lon_name = 'lon'
       !
       iret = NF90_DEF_VAR(fid, lon_name, kind_r_diag, dims(1), nlonid)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Variable '//lon_name//' can not be defined for the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
       !
       lat_name = 'lat'
       iret = NF90_DEF_VAR(fid, lat_name, kind_r_diag, dims(2), nlatid)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Variable '//lat_name//' can not be defined for the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
       !
    ELSE IF ( INDEX(GridType, "RegXY") == 1 ) THEN
       !
       ! 1.2.2 irregular grid
       !
       lon_name = 'nav_lon'
       !
       iret = NF90_DEF_VAR(fid, lon_name, kind_r_diag, dims, nlonid)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Variable '//lon_name//' can not be defined for the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
       !
       lat_name = 'nav_lat'
       iret = NF90_DEF_VAR(fid, lat_name, kind_r_diag, dims, nlatid)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Variable '//lat_name//' can not be defined for the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
       !
    ELSE
       WRITE(numout,*) "Gridtype = ", GridType, " not implemented yet."
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Cannot deal with this grid type yet.', &
               &         '', '')
    ENDIF
    !
    ! 1.3 Add attributes to the coordinate variables
    !
    iret = NF90_PUT_ATT(fid, nlonid, 'units', "degrees_east")
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add attribut to variable '//lon_name//' for the file :', &
            & TRIM(river_file_name),'(Solution ?)')
    ENDIF
    !
    lon_min = -180.
    lon_max = 180.
    !
    iret = NF90_PUT_ATT(fid, nlonid, 'valid_min', lon_min)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add attribut to variable '//lon_name//' for the file :', &
            & TRIM(river_file_name),'(Solution ?)')
    ENDIF
    iret = NF90_PUT_ATT(fid, nlonid, 'valid_max', lon_max)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add attribut to variable '//lon_name//' for the file :', &
            & TRIM(river_file_name),'(Solution ?)')
    ENDIF
    !
    iret = NF90_PUT_ATT(fid, nlonid, 'long_name', "Longitude")
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add attribut to variable '//lon_name//' for the file :', &
            & TRIM(river_file_name),'(Solution ?)')
    ENDIF
    iret = NF90_PUT_ATT(fid, nlatid, 'units', "degrees_north")
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add attribut to variable '//lat_name//' for the file :', &
            & TRIM(river_file_name),'(Solution ?)')
    ENDIF
    !
    lat_max = 90.
    lat_min = -90.
    !
    iret = NF90_PUT_ATT(fid, nlatid, 'valid_min', lat_min)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add attribut to variable '//lat_name//' for the file :', &
            & TRIM(river_file_name),'(Solution ?)')
    ENDIF
    iret = NF90_PUT_ATT(fid, nlatid, 'valid_max', lat_max)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add attribut to variable '//lat_name//' for the file :', &
            & TRIM(river_file_name),'(Solution ?)')
    ENDIF
    iret = NF90_PUT_ATT(fid, nlatid, 'long_name', "Latitude")
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add attribut to variable '//lat_name//' for the file :', &
            & TRIM(river_file_name),'(Solution ?)')
    ENDIF
    !
    iret = NF90_ENDDEF(fid)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not end definitions in the file : ', &
 & TRIM(river_file_name),'(Solution ?)')
    ENDIF
    !
    ! 1.4 Write coordinates
    !
    IF ( INDEX(GridType, "RegLonLat") == 1 ) THEN
       !
       ! 1.4.1 regular grid
       !
       iret = NF90_PUT_VAR(fid, nlonid, lon_g(1:iim_g,1))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not put variable nav_lon  in the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
       !
       iret = NF90_PUT_VAR(fid, nlatid, lat_g(1,1:jjm_g))
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not put variable nav_lat  in the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
    ELSE IF ( INDEX(GridType, "RegXY") == 1 ) THEN
       !
       ! 1.4.2 irregular grid
       !
       iret = NF90_PUT_VAR(fid, nlonid, lon_g)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not put variable nav_lon  in the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
       !
       iret = NF90_PUT_VAR(fid, nlatid, lat_g)
       IF (iret /= NF90_NOERR) THEN
          CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not put variable nav_lat  in the file : ', &
               & TRIM(river_file_name),'(Solution ?)')
       ENDIF
    ELSE
       !
       ! This case was already caught before !
       !
    ENDIF
    !
    ! 2.0 Go through all basins and wirte the information into the netCDF file.
    !
    DO icc = 1, num_largest
       !
       ! 2.1 Compute the fields to be saved in the file
       !
       ib=sortedrivs(icc)
       !
       !
       IF ( nb_pts(ib) > 2 ) THEN
          !
          basinfrac(:,:) = zero
          basinuparea(:,:) = zero
          basincode(:,:) = zero
          !
          ! To write number of cascade (you can delete it later)
          !
          basinupnbas(:,:) = zero
          basinnbas(:,:) = zero
          !
          DO ij=1, nb_pts(ib)

             ik=lbasin_index(ib,ij)

             j = ((index_g(ik)-1)/iim_g) + 1
             i = (index_g(ik)-(j-1)*iim_g)

             basinfrac(i,j) = basinfrac(i,j) + lbasin_area(ib,ij)/(resolution_g(ik,1)*resolution_g(ik,2))
             basinuparea(i,j) = MAX(basinuparea(i,j), lbasin_uparea(ib,ij))
             !
             ! To write number of cascade (you can delete it later)
             !
             basinupnbas(i,j) = MAX(basinupnbas(i,j), lbasin_upnbas(ib,ij))
             basinnbas(i,j) = MAX(basinnbas(i,j), lbasin_nbas(ib,ij))
             !
             basincode(i,j) = lrivercode(ib,ij)

          ENDDO
          !
          DO i=1,iim_g
             DO j=1,jjm_g
                IF ( basinfrac(i,j) <= EPSILON(zero) ) THEN
                   basinfrac(i,j) = undef_sechiba
                   basinuparea(i,j) = undef_sechiba
                   !
                   ! To write number of cascade (you can delete it later)
                   !
                   basinupnbas(i,j) = undef_sechiba
                   basinnbas(i,j) = undef_sechiba
                   !
                   basincode(i,j) = undef_int
                ELSE
                   basinfrac(i,j) = MIN(basinfrac(i,j), un)
                ENDIF
             ENDDO
          ENDDO
          !
          !
          ! 2.2 Define the variables in the netCDF file
          !
          iret = NF90_REDEF(fid)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', &
                  & 'Could not restart definitions in the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ! Create a name more suitable for a variable in a netCDF file
          !
          nc_name = TRIM(sorted_names(icc))
          ! Take out all character which could cause problems
          lcc=LEN_TRIM(nc_name)
          DO ij=1,lcc
             IF ( nc_name(ij:ij) == " " ) nc_name(ij:ij) = "_"
             IF ( nc_name(ij:ij) == "(" ) nc_name(ij:ij) = "_"
             IF ( nc_name(ij:ij) == ")" ) nc_name(ij:ij) = "_"
          ENDDO
          ! reduce redundant "__"
          DO ij=1,lcc
             IF ( nc_name(ij:ij+1) == "__" ) nc_name(ij+1:)=nc_name(ij+2:lcc)
          ENDDO
          lcc=LEN_TRIM(nc_name)
          IF ( nc_name(lcc:lcc) == "_" ) nc_name(lcc:lcc) = " "
          !
          !
          ! 2.3 Fraction variable
          !
          IF (check) WRITE(*,*) "Define Fraction variable and add attributes"
          !
          var_name = TRIM(nc_name)//"_frac"
          !
          iret = NF90_DEF_VAR(fid, TRIM(var_name), kind_r_diag, dims, varid)
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) trim(NF90_STRERROR(iret))
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Variable '//TRIM(var_name)//' can not be defined for the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ierr_tot = 0
          ! Units
          iret = NF90_PUT_ATT(fid, varid, 'units', "-")
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Units', iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          ! Long name
          long_name = "Fraction of basin "//TRIM(sorted_names(icc))//" per grid box"
          iret = NF90_PUT_ATT(fid, varid, 'long_name', long_name)
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Long_Name', long_name, iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          ! Missing value
          iret = NF90_PUT_ATT(fid, varid, 'missing_value', undef_sechiba)
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Missing value', undef_sechiba, iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ib=sortedrivs(icc)
          IF ( check ) WRITE(*,*) "Doing basin ", icc," corrsdponding to index = ", ib, "num_largest : ", num_largest
          !
          ! Nb of grid points in basin
          att_str='Nb_of_grid_points_in_basin'
          iret = NF90_PUT_ATT(fid, varid, att_str, nb_pts(ib))
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Nb of grid points in basin', nb_pts(ib), iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ! Longitude of outflow point
          att_str='Longitude_of_outflow_point'
          ! Instead of longitude of ORCHIDEE grid cell:
          ! iret = NF90_PUT_ATT(fid, varid, att_str, lalo(outpt(ib,1),2))
          ! Put the longitude which got from 1 km resolution HydroSHEDS data
          iret = NF90_PUT_ATT(fid, varid, att_str, outpt_coor(ib,2))
          IF (iret /= NF90_NOERR) THEN
          !   WRITE(*,*) 'Longitude of outflow point', lalo(outpt(ib,1),2), iret
             WRITE(*,*) 'Longitude of outflow point', outpt_coor(ib,2), iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ! Latitide of outflow point
          att_str='Latitude_of_outflow_point'
          ! Same as above longitude
          !iret = NF90_PUT_ATT(fid, varid, att_str, lalo(outpt(ib,1),1))
          iret = NF90_PUT_ATT(fid, varid, att_str, outpt_coor(ib,1))
          IF (iret /= NF90_NOERR) THEN
          !   WRITE(*,*) 'Latitude of outflow point', lalo(outpt(ib,1),1), iret
             WRITE(*,*) 'Latitude of outflow point', outpt_coor(ib,1), iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ! Kind of outlet (to lake or ocean)
          att_str='Type_of_outlet'
          iret = NF90_PUT_ATT(fid, varid, att_str, outpt_type(ib))
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Type of outlet', outpt_type(ib), iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ! Number of streams
          att_str= 'Number_of_streams'
          iret = NF90_PUT_ATT(fid, varid, att_str, streams_nb(ib))
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Number of streams', streams_nb(ib), iret
             WRITE(*,*) TRIM(NF90_STRERROR(iret))
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ! Total number of hops to go to the oceans
          att_str='Total_number_of_hops_to_ocean'
          iret = NF90_PUT_ATT(fid, varid, att_str, streams_avehops(ib)*streams_nb(ib))
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Total number of hops to go to the oceans ', streams_avehops(ib)*streams_nb(ib), iret
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ! Minimum number of hops to go to the ocean for any stream
          att_str='Minimum_number_of_hops_to_ocean_for_any_stream'
          iret = NF90_PUT_ATT(fid, varid, att_str, streams_minhops(ib))
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Minimum number of hops to go tp the ocean for any stream', streams_minhops(ib), iret
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ! Average number of hops to go to the ocean for any stream
          att_str='Average_number_of_hops_to_ocean_for_any_stream'
          iret = NF90_PUT_ATT(fid, varid, att_str, streams_avehops(ib))
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Average number of hops to go tp the ocean for any stream', streams_avehops(ib), iret
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ! Maximum number of hops to go to the ocean for any stream
          att_str='Maximum_number_of_hops_to_ocean_for_any_stream'
          iret = NF90_PUT_ATT(fid, varid, att_str, streams_maxhops(ib))
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Maximum number of hops to go tp the ocean for any stream', streams_maxhops(ib), iret
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          ! Average residence time in the basin
          att_str='Average_residence_time_in_basin'
          iret = NF90_PUT_ATT(fid, varid, att_str, streams_resid(ib))
          IF (iret /= NF90_NOERR) THEN
             WRITE(*,*) 'Average residence time in the basin', streams_resid(ib), iret
             ierr_tot = ierr_tot + 1
          ENDIF
          !
          IF (ierr_tot > 0 ) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add some attributes to variable '//var_name//' for the file :', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ! 2.4 Upstream area variable variable
          !
          IF (check) WRITE(*,*) "Define Upstream variable and add attributes"
          !
          ! Create a name more suitable for a variable in a netCDF file
          !
          var_name = TRIM(nc_name)//"_upstream"
          DO ij=1,LEN_TRIM(var_name)
             IF ( var_name(ij:ij) == " " ) var_name(ij:ij) = "_"
          ENDDO
          !
          iret = NF90_DEF_VAR(fid, TRIM(var_name), kind_r_diag, dims, varid2)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Variable '//TRIM(var_name)//' can not be defined for the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ierr_tot = 0
          ! Units
          iret = NF90_PUT_ATT(fid, varid2, 'units', "m^2")
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          ! Long name
          long_name = "Upstream area of basin "//TRIM(sorted_names(icc))//" in the grid box"
          iret = NF90_PUT_ATT(fid, varid2, 'long_name', long_name)
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          ! Missing value
          iret = NF90_PUT_ATT(fid, varid2, 'missing_value', undef_sechiba)
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          !
          IF (ierr_tot > 0 ) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add some attributes to variable '//var_name//' for the file :', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ! To write number of cascade (you can delete it later)
          !
          var_name = TRIM(nc_name)//"_nbupstream"
          DO ij=1,LEN_TRIM(var_name)
             IF ( var_name(ij:ij) == " " ) var_name(ij:ij) = "_"
          ENDDO
          !
          iret = NF90_DEF_VAR(fid, TRIM(var_name), kind_r_diag, dims, varid4)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Variable'//TRIM(var_name)//' can not be defined for the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ierr_tot = 0
          ! Units
          iret = NF90_PUT_ATT(fid, varid4, 'units', "-")
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          ! Long name
          long_name = "Upstream sub-grid basins "//TRIM(sorted_names(icc))//" in the grid box"
          iret = NF90_PUT_ATT(fid, varid4, 'long_name', long_name)
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          ! Missing value
          iret = NF90_PUT_ATT(fid, varid4, 'missing_value', undef_sechiba)
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          !
          IF (ierr_tot > 0 ) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add some attributes to variable '//var_name//' for the file :', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ! To write number of sub-grid basins (you can delete it later)
          !
          var_name = TRIM(nc_name)//"_nbas"
          DO ij=1,LEN_TRIM(var_name)
             IF ( var_name(ij:ij) == " " ) var_name(ij:ij) = "_"
          ENDDO
          !
          iret = NF90_DEF_VAR(fid, TRIM(var_name), kind_r_diag, dims, varid5)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Variable'//TRIM(var_name)//' can not be defined for the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ierr_tot = 0
          ! Units
          iret = NF90_PUT_ATT(fid, varid5, 'units', "-")
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          ! Long name
          long_name = "Number of sub-grid basins "//TRIM(sorted_names(icc))//" in the grid box"
          iret = NF90_PUT_ATT(fid, varid5, 'long_name', long_name)
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          ! Missing value
          iret = NF90_PUT_ATT(fid, varid5, 'missing_value', undef_sechiba)
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          !
          IF (ierr_tot > 0 ) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add some attributes to variable '//var_name//' for the file :', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ! 2.5 Pfafstetter codes for basins
          !
          IF (check) WRITE(*,*) "Define Pfafstetter codes variable and add attributes"
          !
          var_name = TRIM(nc_name)//"_coding"
          DO ij=1,LEN_TRIM(var_name)
             IF ( var_name(ij:ij) == " " ) var_name(ij:ij) = "_"
          ENDDO
          !
          iret = NF90_DEF_VAR(fid, TRIM(var_name), kind_i_diag, dims, varid3)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Variable '//TRIM(var_name)//' can not be defined for the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ierr_tot = 0
          ! Units
          iret = NF90_PUT_ATT(fid, varid3, 'units', "-")
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          ! Long name
          long_name = "Pfafstetter codes of grid boxes in basin "//TRIM(sorted_names(icc))
          iret = NF90_PUT_ATT(fid, varid3, 'long_name', long_name)
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          ! Missing value
          iret = NF90_PUT_ATT(fid, varid3, 'missing_value', undef_int)
          IF (iret /= NF90_NOERR) ierr_tot = ierr_tot + 1
          !
          IF (ierr_tot > 0 ) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not add some attributes to variable '//var_name//' for the file :', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ! 2.6 ENDDEF of netCDF file
          !
          IF (check) WRITE(*,*) "END define"
          !
          iret = NF90_ENDDEF(fid)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', &
                  & 'Could not end definitions in the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ! 2.7 Write the data to the file
          !
          IF (check) WRITE(*,*) "Put basinfrac"
          iret = NF90_PUT_VAR(fid, varid, basinfrac)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not put variable basinfrac in the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF

          IF (check) WRITE(*,*) "Put basinuparea"
          iret = NF90_PUT_VAR(fid, varid2, basinuparea)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not put variable basinuparea in the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF

          !
          ! To write number of cascade (you can delete it later)
          !
          IF (check) WRITE(*,*) "Put basinupnbas"
          iret = NF90_PUT_VAR(fid, varid4, basinupnbas)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not put variable basinupnbas in the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
          ! To write number of sub-grid basins (you can delete it later)
          !
          IF (check) WRITE(*,*) "Put basinnbas"
          iret = NF90_PUT_VAR(fid, varid5, basinnbas)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagncfile', 'Could not put variable basinnbas in the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !

          IF (check) WRITE(*,*) "Put basincode"
          iret = NF90_PUT_VAR(fid, varid3, basincode)
          IF (iret /= NF90_NOERR) THEN
             CALL ipslerr_p (3,'routing_reg_diagfile', 'Could not put variable basincode in the file : ', &
                  & TRIM(river_file_name),'(Solution ?)')
          ENDIF
          !
       ENDIF
       !
    ENDDO
    !
    IF (check) WRITE(*,*) "Close file"
    !
    ! Close netCDF file and do some memory management.
    !
    iret = NF90_CLOSE(fid)
    IF (iret /= NF90_NOERR) THEN
       CALL ipslerr_p (3,'routing_reg_diagncfile', &
            & 'Could not end definitions in the file : ', &
            & TRIM(river_file_name),'(Solution ?)')
    ENDIF
    !
    !
  END SUBROUTINE routing_reg_diagncfile
  !

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_basins
!!
!>\BRIEF This partly-parallelized subroutine reads in the map of basins and flow direction to construct
!! the catchments of each grid box.
!!
!! DESCRIPTION (definitions, functional, design, flags) :
!! The work is done in a number of steps which are performed locally on the
!! GCM grid:
!! 1) First we find the grid-points of the high resolution routing grid which are
!! within the coarser grid of the GCM.
!! 2) When we have these grid points we decompose them into basins in the routine
!! routing_reg_findbasins. A number of simplifications are done if needed.
!! 3) In the routine routing_reg_globalize we put the basin information of this grid
!! into the global fields.
!! Then we work on the global grid to perform the following tasks :
!! 1) We link up the basins of the various grid points and check the global consistency.
!! 2) The area of each outflow point is computed.
!! 3) The final step is to reduce the number of basins in order to fit into the truncation.\n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): None, as the routine puts information into the global variables of the module.
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_basins(nbpt, lalo, neighbours, resolution, contfrac)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    REAL(r_std), INTENT(in) :: lalo(nbpt,2) !! Vector of latitude and longitudes (beware of the order !)
    INTEGER(i_std), INTENT(in) :: neighbours(nbpt,NbNeighb)!! Vector of neighbours for each grid point
                                                                           !! (1=North and then cloxkwise)
    REAL(r_std), INTENT(in) :: resolution(nbpt,2) !! The size of each grid box in X and Y (m)
    REAL(r_std), INTENT(in) :: contfrac(nbpt) !! Fraction of land in each grid box (unitless;0-1)
    !
!! LOCAL VARIABLES
    CHARACTER(LEN=80) :: filename !! Name of the netcdf file (unitless)
    INTEGER(i_std) :: iml, jml, lml, tml, fid, ib, ip, jp, fopt !! Indices (unitless)
    REAL(r_std) :: lev(1), date, dt, coslat
    INTEGER(i_std) :: itau(1) !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: trip !! The trip field (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basins !! The basin field (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: topoindex !! Topographic index of the residence time (m)
    !
    INTEGER(i_std) :: option !!
    !
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: hierarchy !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: fac !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lat_rel !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lon_rel !!
    !
    INTEGER(i_std) :: nbi, nbj !! Number of point in x and y within the grid (unitless)
    REAL(r_std) :: min_topoind !! The current minimum of topographic index (m)
    REAL(r_std) :: max_basins !!
    REAL(r_std) :: invented_basins !!
    !
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: area_bx    !! Area of each small box in the grid box (m^2)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: hierarchy_bx !! Level in the basin of the point
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: fac_bx     !! Flow accumulation 
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: lon_bx     !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: lat_bx     !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: lshead_bx  !! Large scale heading of the flow out of the grid box.
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: topoind_bx !! Topographic index of the residence time for each of the smaller boxes (m)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: trip_bx    !! The trip field for each of the smaller boxes (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: basin_bx   !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:)   :: coast_pts  !! The coastal flow points (unitless)
    !
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: basin_count !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: basin_notrun !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: basin_id !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:) :: basin_coor !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_type !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_area !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_hierarchy !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_fac !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_topoind !! Topographic index of the residence time for a basin (m)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: fetch_basin !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: basin_flowdir !! Water flow directions in the basin (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: basin_lshead !! Large scale heading for basin flowing out of the grid. 
                                                                !! (degrees from North).
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: outflow_grid !! Type of outflow on the grid box (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: outflow_basin !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: inflow_number !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:,:) :: inflow_basin !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:,:) :: inflow_grid !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: nbcoastal !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: coastal_basin !!
    !
    ! Interpolation help variables
    !
    INTEGER(i_std) :: nix, njx !!
    CHARACTER(LEN=30) :: callsign !!
    !
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: resol_lu !! Resolution
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: mask !! Mask to exclude some points (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: sub_area !! Area on the fine grid (m^2)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:,:) :: sub_index !! Indices of the points we need on the fine grid (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: sub_pts !! Number of high resolution points on this grid (unitless)
    INTEGER :: ALLOC_ERR !!
    LOGICAL :: ok_interpol = .FALSE. !! Flag for interpolation (true/false)
    !
    INTEGER(i_std) :: nb_basin !! Number of sub-basins (unitless)
    INTEGER(i_std) :: nwbas !!
    INTEGER(i_std) :: ii
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: basin_inbxid !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_outlet !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: basin_outtp !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: basin_sz !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: basin_bxout !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: basin_bbout !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:,:) :: basin_pts !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: lshead
    CHARACTER(LEN=7) :: fmt !!
    LOGICAL :: debug = .TRUE. !! (true/false)
    LOGICAL :: testdebug
    !
    REAL(r_std), DIMENSION(1,2) :: diaglalo !! Point (in Lat/Lon) where diagnostics will be printed.
    !
    ! Simple time checking (only use in the development step)
    ! You can also use stop_timer() & start_timer() as you wish.
    ! Using it in others version of this code, I found that it is unnecessary
    !
    LOGICAL, PARAMETER :: timesimple=.FALSE.
    INTEGER(i_std) :: timeint, time
    CHARACTER(LEN=30) :: stime, ctime
    !
    ! New LOCAL variables (when parallelize the older routing code)
    !
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: basin_count_loc !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: basin_notrun_loc !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: basin_id_loc !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:) :: basin_coor_loc !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_type_loc !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_area_loc !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_hierarchy_loc !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_fac_loc !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basin_topoind_loc !! Topographic index of the residence time for a basin (m)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: basin_flowdir_loc !! Water flow directions in the basin (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: outflow_grid_loc !! Type of outflow on the grid box (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: outflow_basin_loc !! Type of outflow on the grid box (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)    :: basin_lshead_loc !! Large scale heading for basin flowing out of the grid. 
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: nbcoastal_loc !!
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: coastal_basin_loc !!
    !
    ! To use scattermap 
    !
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: trip_loc !! Area on the fine grid (m^2)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: basins_loc !! The basin field (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: topoindex_loc !! Topographic index of the residence time (m)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: hierarchy_loc !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: fac_loc !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lat_rel_loc !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lon_rel_loc !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: sub_area_g !! Area on the fine grid (m^2)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:,:) :: sub_index_g !! Indices of the points we need on the fine grid (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:) :: sub_pts_g !! Number of high resolution points on this grid (unitless)

!_ ================================================================================================================================
    !
    ! The points on which diagnostics are to be printed in case debug=.TRUE.
    ! WARNING : The order of the coordinates is Latitude, Longitude !
    !
    diaglalo(1,:) = (/ 45.125, 29.125 /)
!!$    diaglalo(2,:) = (/ 44.875, 29.125 /)
!!$    diaglalo(2,:) = (/ 39.6446, 26.9151 /)
!!$    diaglalo(3,:) = (/ 39.4648, 26.8760 /)
    !
    ! Time checking, keep it on ROOT only here
    !
    IF ( timesimple ) THEN
       IF (is_root_prc) THEN
          timeint=time() ; stime=ctime(timeint)
          WRITE(*,'(I3,1X,A40,A26)') mpi_rank, '##> START routing_reg_basins at :', stime
       ENDIF
    ENDIF
    !
    ! Read information from routing.nc to aggregate
    !
    IF (is_root_prc) THEN
       !
       ! Needs to be a configurable variable
       !
       !Config Key = ROUTING_FILE
       !Config Desc = Name of file which contains the routing information
       !Config If = RIVER_ROUTING
       !Config Def = routing.nc
       !Config Help = The file provided here should alow the routing module to
       !Config read the high resolution grid of basins and the flow direction
       !Config from one mesh to the other.
       !Config Units = [FILE]
       !
       filename = 'routing.nc'
       CALL getin('ROUTING_FILE',filename)
       !
       CALL flininfo(filename, iml, jml, lml, tml, fid)
       CALL flinclo(fid)
       !
    ENDIF
    !
    ! Get value for "mask" which is neccesary for aggregate_p
    !
    IF (is_root_prc) THEN
       ALLOCATE(mask(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for mask','','')

       ALLOCATE(lat_rel(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for lat_rel','','')

       ALLOCATE(lon_rel(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for lon_rel','','')

       ALLOCATE (trip(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for trip','','')
    ELSE
       ALLOCATE(mask(1,1), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for mask','','')

       ALLOCATE(lat_rel(1,1), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for lat_rel','','')

       ALLOCATE(lon_rel(1,1), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for lon_rel','','')

       ALLOCATE (trip(1,1), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for trip','','')
    ENDIF
    !
    ! Read trip() for mask()
    !
    IF (is_root_prc) THEN
       !!
       !! Trip : Provides the flow direction following the convention :
       !! trip = 1 : flow = N
       !! trip = 2 : flow = NE
       !! trip = 3 : flow = E
       !! trip = 4 : flow = SE
       !! trip = 5 : flow = S
       !! trip = 6 : flow = SW
       !! trip = 7 : flow = W
       !! trip = 8 : flow = NW
       !! trip = 97 : return flow into the ground
       !! trip = 98 : coastal flow (diffuse flow into the oceans)
       !! trip = 99 : river flow into the oceans
       !!
       CALL flinopen(filename, .FALSE., iml, jml, lml, lon_rel, lat_rel, lev, tml, itau, date, dt, fid)
       CALL flinget(fid, 'trip', iml, jml, lml, tml, 1, 1, trip)
       CALL flinclo(fid)
    ENDIF
    !
    IF (is_root_prc) THEN
       !
       ! Consider all points a priori
       !
       mask(:,:) = 0
       !
       DO ip=1,iml
          DO jp=1,jml
             !
             ! Determine the land mask of the basin map read from the file ROUTING_FILE
             !
             IF ( trip(ip,jp) < 1.e10 ) THEN
                mask(ip,jp) = 1
             ENDIF
             !
          ENDDO
       ENDDO
       !
       ! The maximum number of points of the source map (basin description here) which can fit into
       ! any grid point of the ORCHIDEE grid is stimated here.
       ! Some margin is taken.
       !
       callsign = "routing_reg_basins"
       ok_interpol = .FALSE.
       !
       ! Compare with the older version, we changed the way we calculate resol_lu here.
       !
       ALLOCATE(resol_lu(2), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for resol_lu','','')
       !
       nix=INT(iml/2)
       njx=INT(jml/2)
       !
       coslat = MAX( COS( lat_rel(nix-1,njx) * pi/180. ), mincos )
       resol_lu(1) = ABS( lon_rel(nix+1,njx) - lon_rel(nix-1,njx) )/2. * pi/180. * R_Earth * coslat
       resol_lu(2) = ABS( lat_rel(nix,njx-1) - lat_rel(nix,njx+1) )/2. * pi/180. * R_Earth
       !
       nix=INT(MAXVAL(resolution_g(:,1))/resol_lu(1))+2
       njx=INT(MAXVAL(resolution_g(:,2))/resol_lu(2))+2
       !
       ! Trung: In routing_reg_basins (about nbvmax)
       ! Trung:            nbvmax = nix*njx
       ! Trung: Saving memory here with factor of 1.0 (*nix*njx)
       ! Trung: This scale is very sensitive to the memory problem,
       ! Trung: but it also can get problem with arregate_p (need to
       ! Trung: increase to 1.5 or 2.0, 3.0) 
       !
       nbvmax = nix*njx*2
       !
       ! Use nbxmax to save memory in routing_reg_getgrid
       !
       nbxmax = MAX(nix,njx)*2
       !
       IF (ALLOCATED(resol_lu)) DEALLOCATE(resol_lu)
       !
    ENDIF
    !
    ! Note: aggregate_p still only do aggregate_2d only in ROOT processor
    !
    CALL bcast(nbvmax)
    CALL bcast(nbxmax)
    !
    ALLOCATE (sub_area(nbp_loc,nbvmax), sub_index(nbp_loc,nbvmax,2), sub_pts(nbp_loc), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for sub_area, sub_index, sub_pts','','')
    !
    ALLOCATE (sub_area_g(nbp_glo,nbvmax), sub_index_g(nbp_glo,nbvmax,2), sub_pts_g(nbp_glo), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for sub_area_g, sub_index_g, sub_pts_g ','','')
    !
    ! To write the output of aggregate_2d for saving time of experiments
    !
    !Config Key = AGGREGATE
    !Config Desc = Options to write/read or calculate in aggregate step
    !Config Def = 1
    !Config Help = This flag allows to do aggregate with different options
    !Config Units = [-]
    !
    option = 1
    CALL getin('AGGREGATE', option)
    !
    IF ( timesimple ) THEN
       IF (is_root_prc) THEN
          timeint=time() ; stime=ctime(timeint)
          WRITE(*,'(I3,1X,A40,A26)') mpi_rank, '##> START aggregate_p at :', stime
       ENDIF
    ENDIF
    !
    !
    !
    IF ( option .EQ. 0 ) THEN
       ! Do everything as usual and write output for skipping aggregate next time
       CALL aggregate_p(nbp_loc, lalo, neighbours, resolution, contfrac, &
                  & iml, jml, lon_rel, lat_rel, mask, callsign, &
                  & nbvmax, sub_index, sub_area, ok_interpol)
       IF ( .NOT. ok_interpol ) THEN
          CALL ipslerr_p(3,'routing_reg_basins','Please increase nbvmax','','')
       ENDIF
       !
       CALL gather(sub_index, sub_index_g)
       CALL gather(sub_area, sub_area_g)
       IF (is_root_prc) CALL routing_reg_writeaggr(nbp_glo, nbvmax, sub_index_g, sub_area_g)
    ELSEIF ( option .EQ. 1 ) THEN
       ! Do everything as usual (no read/write)
       CALL aggregate_p(nbp_loc, lalo, neighbours, resolution, contfrac, &
                  & iml, jml, lon_rel, lat_rel, mask, callsign, &
                  & nbvmax, sub_index, sub_area, ok_interpol)
       IF ( .NOT. ok_interpol ) THEN
          CALL ipslerr_p(3,'routing_reg_basins','Please increase nbvmax','','')
       ENDIF
       CALL gather(sub_index, sub_index_g)
       CALL gather(sub_area, sub_area_g)
    ELSEIF ( option .EQ. 2 ) THEN
       ! Read previous input for skipping aggregate
       IF (is_root_prc) CALL routing_reg_readaggr(nbp_glo, nbvmax, sub_index_g, sub_area_g)
       ok_interpol = .TRUE.
       CALL BCAST(ok_interpol)
       CALL scatter(sub_index_g, sub_index)
       CALL scatter(sub_area_g, sub_area)
    ELSE
       WRITE(numout,*) "ERROR: Can not aggregate"
    ENDIF
    !
    IF (ALLOCATED(mask)) DEALLOCATE(mask)
    !
    IF ( timesimple ) THEN
       IF (is_root_prc) THEN
          timeint=time() ; stime=ctime(timeint)
          WRITE(*,'(I3,1X,A40,A26)') mpi_rank, '##> FINISH aggregate_p at :', stime
       ENDIF
    ENDIF
    !
    WHERE (sub_area < 0) sub_area=zero
    !
    ! Some verifications
    !
    DO ib=1,nbp_loc
       sub_pts(ib) = COUNT(sub_area(ib,:) > zero)
       DO fopt=1,sub_pts(ib)
          IF (sub_area(ib, fopt) == 0 ) THEN
             WRITE(numout,*) "Zero Area - Sub_area > 0 : ", ib, fopt
             WRITE(numout,*) "Zero Area - lon : ",lalo(ib,2)
             WRITE(numout,*) "Zero Area - lon_rel : ", lon_rel(sub_index(ib, fopt, 1),sub_index(ib, fopt, 2))
             WRITE(numout,*) "Zero Area - lat : ",lalo(ib,1)
             WRITE(numout,*) "Zero Area - lat_rel : ", lat_rel(sub_index(ib, fopt, 1),sub_index(ib, fopt, 2))
          ENDIF
       ENDDO
    ENDDO
    !
    ! Do some memory management.
    !
    CALL gather(sub_pts, sub_pts_g)
    ! Jan: debug on 26 Oct 2015
    !nwbas = MAX(MAXVAL(sub_pts), 9)
    IF (is_root_prc) THEN
       nwbas = MAXVAL(sub_pts_g)
       IF ( debug ) WRITE(*,*) "Routing : nwbas = ", nwbas
    ENDIF
    !
    CALL bcast(nwbas)
    !
    IF (ALLOCATED(sub_pts_g)) DEALLOCATE(sub_pts_g)
    !
    IF ( debug ) WRITE(*,*) mpi_rank,' Routing: nwbas :', nwbas
    !
    ! Try to use scattermap() here to release lon_rel & lat_rel
    !
    ALLOCATE (lon_rel_loc(nbp_loc,nbvmax), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for lon_rel_loc','','')
    CALL scattermap(lon_rel, sub_index_g, sub_area_g, lon_rel_loc)
    !
    ALLOCATE (lat_rel_loc(nbp_loc,nbvmax), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for lat_rel_loc','','')
    CALL scattermap(lat_rel, sub_index_g, sub_area_g, lat_rel_loc)
    !
    ! Start to read topoindex from file
    !
    IF (is_root_prc) THEN
       ALLOCATE (basins(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basins','','')

       ALLOCATE (topoindex(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for topoindex','','')

       ALLOCATE (hierarchy(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for hierarchy','','')

       ALLOCATE (fac(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for fac','','')
       !
    ELSE
       ALLOCATE (basins(1,1), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basins','','')

       ALLOCATE (topoindex(1,1), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for topoindex','','')

       ALLOCATE (hierarchy(1,1), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for hierarchy','','')

       ALLOCATE (fac(1,1), STAT=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for fac','','')
    ENDIF
    !
    ! Come back to ROOT
    !
    IF (is_root_prc) THEN
       !
       CALL flinopen(filename, .FALSE., iml, jml, lml, lon_rel, lat_rel, lev, tml, itau, date, dt, fid)
       !
       ! Now we can deallocate lon_rel & lat_rel
       !
       IF (ALLOCATED(lon_rel)) DEALLOCATE(lon_rel)
       IF (ALLOCATED(lat_rel)) DEALLOCATE(lat_rel)
       !!
       !! From the basin description data we will read the following variables :
       !!
       !! Basins : Provides a uniqe ID for each basin. These IDs are also used to get
       !! the name of the basin from the table in routine routing_reg_names.
       !!
       !!
       CALL flinget(fid, 'basins', iml, jml, lml, tml, 1, 1, basins)
       !!
       !! Topoind : is the topographic index for the retention time of the water in the
       !! grid box. It has been computed with the following formula : 1000 x sqrt(d^3/Dz)
       !! where d is the distance of the river from the current grid box to the next one
       !! as indicated by the variable trip.
       !! Dz the hight difference between between the two grid boxes.
       !! All these variables are in meters.
       !! Furthermore we have to limit the height difference to 1m in order to avoid any unpleasant
       !! surprises. If dz < 1m then dz=1.
       !
       ! Depend of OPTION we read topoindex here
       !
       !Config Key = TOPOINDEX
       !Config Desc = Options to calculate topoindex before routing_reg_getgrid
       !Config Def = 2
       !Config Help = This flag allows to calculate topoindex with different
       !options
       !Config Units = [-]
       !
       option = 3
       CALL getin('TOPOINDEX', option)
       !
       ! Options to calculate topoindex:
       ! = 1 : constant topoindex (=1000.)
       ! = 2 : calculate with the OLD method (1000 x sqrt(d^3/dz)
       ! = 3 : accumulate the "topoind" from routing.nc along the flow (as Agnes DUCHARNE's suggestion)
       ! with these options you should see sub-options in routing_reg_globalize
       !
       IF ( option .EQ. 2 .OR. option .EQ. 3 ) THEN
       !   CALL flinget(fid, 'topoind_h', iml, jml, lml, tml, 1, 1, topoindex)
          CALL flinget(fid, 'topoind', iml, jml, lml, tml, 1, 1, topoindex)
       ELSEIF ( option .EQ. 1 ) THEN
          topoindex=1000.
       ELSE
          WRITE(numout,*) "ERROR: Can not read topoindex"
       ENDIF
       !
       !! Finds,in each grid box, the distance to the outflow point ... this defines the order in which
       !! the water will go through the sub-basins and grid boxes.
       !
       hierarchy(:,:) = undef_sechiba
       CALL flinget(fid, 'disto', iml, jml, lml, tml, 1, 1, hierarchy)
       !
       ! Trung: Get flow accumulation value for improving routing_reg_linkup
       !
       fac(:,:) = undef_sechiba
       CALL flinget(fid, 'fac', iml, jml, lml, tml, 1, 1, fac)
       !
       CALL flinclo(fid)
       !
       ! Update (11 feb 2016):
       ! for topoind, min_topoind = 0.00
       ! for topoind_h, min_topoind = 0.00
       !
       min_topoind = MINVAL(topoindex, MASK=topoindex .LT. undef_sechiba-un)
       !
       DO ip=1,iml
          DO jp=1,jml
             IF ( trip(ip,jp) < 1.e10 .AND. topoindex(ip,jp) > 1.e10) THEN
                WRITE(numout,*) 'trip exists but not topoind :'
                WRITE(numout,*) 'ip, jp :', ip, jp
                WRITE(numout,*) 'trip, topoind : ', trip(ip,jp), topoindex(ip,jp)
                CALL ipslerr_p(3,'routing_reg_basins','trip exists but not topoind','','')
             ENDIF
          ENDDO
       ENDDO
       !
    ENDIF
    !
    ! Deallocate on LOCAL (we can not do it before)
    !
    IF (ALLOCATED(lon_rel)) DEALLOCATE(lon_rel)
    IF (ALLOCATED(lat_rel)) DEALLOCATE(lat_rel)
    !
    CALL bcast(min_topoind)
    !
    ! Use scattermap() here
    !
    ALLOCATE (trip_loc(nbp_loc,nbvmax), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for trip_loc','','')
    CALL scattermap(trip, sub_index_g, sub_area_g, trip_loc)
    IF (ALLOCATED(trip)) DEALLOCATE(trip)
    !
    ALLOCATE (basins_loc(nbp_loc,nbvmax), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basins_loc','','')
    CALL scattermap(basins, sub_index_g, sub_area_g, basins_loc)
    !
    ! Oops, we still need basins for a little thing here
    !
    IF (is_root_prc) THEN
       max_basins = MAXVAL(basins, MASK=basins .LT. 1.e10)
       invented_basins = max_basins
    ENDIF
    !
    CALL bcast(max_basins)
    IF (ALLOCATED(basins)) DEALLOCATE(basins)
    !
    ALLOCATE (topoindex_loc(nbp_loc,nbvmax), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for topoindex_loc','','')
    CALL scattermap(topoindex, sub_index_g, sub_area_g, topoindex_loc)
    IF (ALLOCATED(topoindex)) DEALLOCATE(topoindex)
    !
    ALLOCATE (hierarchy_loc(nbp_loc,nbvmax), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for hierarchy_loc','','')
    CALL scattermap(hierarchy, sub_index_g, sub_area_g, hierarchy_loc)
    IF (ALLOCATED(hierarchy)) DEALLOCATE(hierarchy)
    !
    ALLOCATE (fac_loc(nbp_loc,nbvmax), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for fac_loc','','')
    CALL scattermap(fac, sub_index_g, sub_area_g, fac_loc)
    IF (ALLOCATED(fac)) DEALLOCATE(fac)
    !
    IF (ALLOCATED(sub_index_g)) DEALLOCATE(sub_index_g)
    IF (ALLOCATED(sub_area_g)) DEALLOCATE(sub_area_g)
    !
    ! All below ALLOCATE will be LOCAL
    !
    ALLOCATE (area_bx(nbxmax,nbxmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for area_bx','','')
    ALLOCATE (hierarchy_bx(nbxmax,nbxmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for hierarchy_bx','','')
    ALLOCATE (fac_bx(nbxmax,nbxmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for fac_bx','','')
    ALLOCATE (lon_bx(nbxmax,nbxmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for lon_bx','','')
    ALLOCATE (lat_bx(nbxmax,nbxmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for lat_bx','','')
    ALLOCATE (lshead_bx(nbxmax,nbxmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for lshead_bx','','')
    ALLOCATE (topoind_bx(nbxmax,nbxmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for topoind_bx','','')
    ALLOCATE (trip_bx(nbxmax,nbxmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for trip_bx','','')
    ALLOCATE (basin_bx(nbxmax,nbxmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_bx','','')
    !
    ALLOCATE (coast_pts(nbvmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for coast_pts','','')
    ALLOCATE (basin_inbxid(nbvmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_inbxid','','')
    ALLOCATE (basin_outlet(nbvmax,2), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_outlet','','')
    ALLOCATE (basin_outtp(nbvmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_outtp','','')
    ALLOCATE (basin_sz(nbvmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_sz','','')
    ALLOCATE (basin_pts(nbvmax,nbvmax,2), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_pts','','')
    ALLOCATE (basin_bxout(nbvmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_bxout','','')
    ALLOCATE (basin_bbout(nbvmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_bbout','','')
    ALLOCATE (lshead(nbvmax), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for lshead','','')
    !
    ! Some variables need to be LOCAL
    !
    ALLOCATE (basin_count_loc(nbp_loc), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_count_loc','','')
    ALLOCATE (basin_notrun_loc(nbp_loc), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_notrun_loc','','')
    ALLOCATE (basin_area_loc(nbp_loc,nwbas), basin_hierarchy_loc(nbp_loc,nwbas), basin_fac_loc(nbp_loc,nwbas), basin_topoind_loc(nbp_loc,nwbas), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_area_loc','','')
    ALLOCATE (basin_id_loc(nbp_loc,nwbas), basin_flowdir_loc(nbp_loc,nwbas), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_id_loc','','')
    ALLOCATE (basin_lshead_loc(nbp_loc,nwbas), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_lshead_loc','','')
    ALLOCATE (basin_coor_loc(nbp_loc,nwbas,2), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_coor_loc','','')
    ALLOCATE (basin_type_loc(nbp_loc,nwbas), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_type_loc','','')
    ALLOCATE (outflow_grid_loc(nbp_loc,nwbas), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for outflow_grid_loc','','')
    ALLOCATE (outflow_basin_loc(nbp_loc,nwbas), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for outflow_basin_loc','','')
    ALLOCATE (nbcoastal_loc(nbp_loc), coastal_basin_loc(nbp_loc,nwbas), stat=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for nbcoastal_loc','','')
    !
    ! Order all sub points in each grid_box and find the sub basins
    !
    ! before we start we set the maps to empty
    !
    basin_count_loc(:) = 0
    basin_notrun_loc(:) = 0
    basin_id_loc(:,:) = undef_int
    basin_coor_loc(:,:,:) = undef_int
    basin_type_loc(:,:) = undef_int
    nbcoastal_loc(:) = 0
    outflow_basin_loc(:,:) = undef_int
    !
    IF ( timesimple ) THEN
       timeint=time() ; stime=ctime(timeint)
       WRITE(*,'(I3,1X,A59,A26)') mpi_rank, '##> GET in routing_reg_getgrid/findbasins/globalize at :', stime
    ENDIF
    !
    ! Try to parallel here
    !
    DO ib = 1, nbp_loc
       !
       ! Extracts from the global high resolution fields the data for the current grid box.
       !
       ! Temporarily skip max_basins_loc !!
       !
       lon_bx = 0 ; lat_bx = 0
       !
       CALL routing_reg_getgrid(nbp_loc, ib, sub_pts, sub_index, sub_area, max_basins, min_topoind, &
            & lon_rel_loc, lat_rel_loc, lalo, resolution, contfrac, trip_loc, basins_loc, topoindex_loc, fac_loc, hierarchy_loc, &
            & nbi, nbj, area_bx, trip_bx, basin_bx, topoind_bx, fac_bx, hierarchy_bx, lon_bx, lat_bx, lshead_bx)
       !
       !! Finds the basins: returns the list of all points which are within the same basin of the grid box.
       !
       !
       CALL routing_reg_findbasins(ib, nbi, nbj, trip_bx, basin_bx, fac_bx, hierarchy_bx, topoind_bx, lshead_bx, &
            & diaglalo, nb_basin, basin_inbxid, basin_outlet, basin_outtp, basin_sz, basin_bxout, basin_bbout, &
            & basin_pts, lshead, coast_pts, lon_bx, lat_bx)
       !
       ! Deal with the case where nb_basin=0 for this grid box. In this case all goes into coastal flow.
       !
       IF ( debug .AND. routing_diagbox(ib,diaglalo) ) THEN
          WRITE(numout,*) '===================== IB = :', ib
          WRITE(numout,*) "sub_pts(ib) :", sub_pts(ib), "sub_area(ib,:) :",sub_area(ib,1:2)
          WRITE(numout,*) 'LON LAT of GCM :', lalo(ib,2), lalo(ib,1)
          DO ii=1,NbNeighb
             WRITE(numout,*) 'Neighbor options :', ii, routing_nextgrid(ib,ii), 'heading =',NINT(headings(ib,ii))
          ENDDO
          WRITE(numout,*) 'Resolution :', resolution(ib,1:2)
          WRITE(fmt,"('(',I3,'I6)')") nbi
          WRITE(numout,*) '-------------> trip ', trip_bx(1,1)
          DO jp=1,nbj
             WRITE(numout,fmt) trip_bx(1:nbi,jp)
          ENDDO
          WRITE(numout,*) '-------------> lshead '
          DO jp=1,nbj
             WRITE(numout,fmt) NINT(lshead_bx(1:nbi,jp))
          ENDDO
          WRITE(numout,*) '-------------> basin ',basin_bx(1,1)
          DO jp=1,nbj
             WRITE(numout,fmt) basin_bx(1:nbi,jp)
          ENDDO
          WRITE(numout,*) '-------------> hierarchy ',hierarchy_bx(1,1)
          DO jp=1,nbj
             WRITE(numout,fmt) INT(hierarchy_bx(1:nbi,jp)/1000.)
          ENDDO
          WRITE(numout,*) '-------------> topoindex ',topoind_bx(1,1)
          DO jp=1,nbj
             WRITE(numout,fmt) INT(topoind_bx(1:nbi,jp)/1000.)
          ENDDO
          !
          WRITE(numout,*) '------------> The basins we retain'
          DO jp=1,nb_basin
             WRITE(numout,*) 'index, size, bxout, coast :', basin_inbxid(jp), basin_sz(jp),&
                  & basin_bxout(jp), lshead(jp), coast_pts(jp)
          ENDDO
          !
       ENDIF
       !
       !! Puts the basins found for the current grid box in the context of the global map.
       !
       CALL routing_reg_globalize(nbp_loc, ib, neighbours, area_bx, trip_bx, hierarchy_bx, fac_bx, topoind_bx, min_topoind,&
            & nb_basin, basin_inbxid, basin_outlet, basin_outtp, basin_sz, basin_pts, basin_bxout, basin_bbout, lshead, coast_pts, nwbas, &
            & basin_count_loc, basin_notrun_loc, basin_area_loc, basin_hierarchy_loc, basin_fac_loc, basin_topoind_loc, basin_id_loc, basin_coor_loc, &
            & basin_type_loc, basin_flowdir_loc, basin_lshead_loc, outflow_grid_loc, outflow_basin_loc, nbcoastal_loc, coastal_basin_loc)
       !
       !
       !
       IF ( debug .AND. routing_diagbox(ib, diaglalo)) THEN
          WRITE(numout,*) 'GLOBAL information after routing_reg_globalize for grid ', ib
          DO jp=1,basin_count_loc(ib)
             WRITE(numout,*) 'Basin Nb = ', jp, ' ID = ', basin_id_loc(ib, jp)
             WRITE(numout,*) 'Basin area : ', basin_area_loc(ib, jp)
             WRITE(numout,*) 'Basin flowdir :', basin_flowdir_loc(ib, jp)
             WRITE(numout,*) 'Basin large scale heading :', basin_lshead_loc(ib, jp)
             WRITE(numout,*) 'Basin hierarchy :', basin_hierarchy_loc(ib, jp)
             WRITE(numout,*) 'Basin topoindex :', basin_topoind_loc(ib, jp)
             WRITE(numout,*) 'Basin outflow grid :', outflow_grid_loc(ib,jp)
             WRITE(numout,*) 'Basin outflow basin :', outflow_basin_loc(ib,jp)
          ENDDO
       ENDIF
       !
    ENDDO
    !
    IF ( timesimple ) THEN
       timeint=time() ; stime=ctime(timeint)
       WRITE(*,'(I3,1X,A59,A26)') mpi_rank, '##> FINISH routing_reg_getgrid/findbasins/globalize at :', stime
    ENDIF
    !
    ! Keep this ALLOCATE here for overcoming the memory error in ROOT CPU
    !
    IF (is_root_prc) THEN
       ALLOCATE (basin_count(nbp_glo), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_count','','')
       ALLOCATE (basin_notrun(nbp_glo), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_notrun','','')
       ALLOCATE (basin_area(nbp_glo,nwbas), basin_hierarchy(nbp_glo,nwbas), basin_fac(nbp_glo,nwbas), basin_topoind(nbp_glo,nwbas), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_area','','')
       ALLOCATE (fetch_basin(nbp_glo,nwbas), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for fetch_basin','','')
       ALLOCATE (basin_id(nbp_glo,nwbas), basin_flowdir(nbp_glo,nwbas), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_id','','')
       ALLOCATE (basin_lshead(nbp_glo,nwbas), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_lshead','','')
       ALLOCATE (basin_coor(nbp_glo,nwbas,2), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_coor','','')
       ALLOCATE (basin_type(nbp_glo,nwbas), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_type','','')
       ALLOCATE (outflow_grid(nbp_glo,nwbas),outflow_basin(nbp_glo,nwbas), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for outflow_grid','','')
       !
       ! Trung: In routing_reg_basins (about dimension of inflow_number)
       ! Trung: Try to fix the memory issue: change nwbas to nbxmax*4 or *8
       ! Trung: It's not good to keep fix number (4 or 8) here.
       !
       ALLOCATE (inflow_number(nbp_glo,nbxmax*8), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for inflow_number','','')
       ALLOCATE (inflow_basin(nbp_glo,nbxmax*8,nbxmax*8), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins',&
            &              'Pb in allocate for inflow_basin on root_prc','','')
       ALLOCATE (inflow_grid(nbp_glo,nbxmax*8,nbxmax*8), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins',&
            &              'Pb in allocate for inflow_grid','','')
       !
       ALLOCATE (nbcoastal(nbp_glo), coastal_basin(nbp_glo,nwbas), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for nbcoastal','','')
       !
    ELSE
       ALLOCATE (basin_count(1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_count','','')
       ALLOCATE (basin_notrun(1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_notrun','','')
       ALLOCATE (basin_area(1,1), basin_hierarchy(1,1), basin_fac(1,1), basin_topoind(1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_area','','')
       ALLOCATE (fetch_basin(1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for fetch_basin','','')
       ALLOCATE (basin_id(1,1), basin_flowdir(1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_id','','')
       ALLOCATE (basin_coor(1,1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_coor','','')
       ALLOCATE (basin_type(1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_type','','')
       ALLOCATE (outflow_grid(1,1),outflow_basin(1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for outflow_grid','','')
       ALLOCATE (inflow_number(1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for inflow_number','','')
       ALLOCATE (inflow_basin(1,1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for inflow_basin on other prc','','')
       ALLOCATE (inflow_grid(1,1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for inflow_grid','','')
       ALLOCATE (nbcoastal(1), coastal_basin(1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for nbcoastal','','')
       ALLOCATE (basin_lshead(1,1), stat=ALLOC_ERR)
       IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_basins','Pb in allocate for basin_lshead','','')
    ENDIF
    !
    !
    !
    IF (is_root_prc) THEN
       basin_id(:,:) = undef_int
       basin_coor(:,:,:) = undef_int
       basin_type(:,:) = undef_int
       basin_count(:) = 0
       basin_notrun(:) = 0
       nbcoastal(:) = 0
    ENDIF
    !
    ! Gather all output of routing_reg_globalize
    !
    CALL gather(basin_count_loc, basin_count)
    CALL gather(basin_notrun_loc, basin_notrun)
    CALL gather(basin_area_loc, basin_area)
    CALL gather(basin_hierarchy_loc, basin_hierarchy)
    CALL gather(basin_fac_loc, basin_fac)
    CALL gather(basin_topoind_loc, basin_topoind)
    CALL gather(basin_id_loc, basin_id)
    CALL gather(basin_coor_loc, basin_coor)
    CALL gather(basin_type_loc, basin_type)
    CALL gather(basin_flowdir_loc, basin_flowdir)
    CALL gather(basin_lshead_loc, basin_lshead)
    CALL gather(outflow_grid_loc, outflow_grid)
    CALL gather(outflow_basin_loc, outflow_basin)
    CALL gather(nbcoastal_loc, nbcoastal)
    CALL gather(coastal_basin_loc, coastal_basin)
    !
    ! DEALLOCATE to save memory here
    !
    IF (ALLOCATED(basin_count_loc)) DEALLOCATE(basin_count_loc)
    IF (ALLOCATED(basin_notrun_loc)) DEALLOCATE(basin_notrun_loc)
    IF (ALLOCATED(basin_area_loc)) DEALLOCATE(basin_area_loc)
    IF (ALLOCATED(basin_hierarchy_loc)) DEALLOCATE(basin_hierarchy_loc)
    IF (ALLOCATED(basin_fac_loc)) DEALLOCATE(basin_fac_loc)
    IF (ALLOCATED(basin_topoind_loc)) DEALLOCATE(basin_topoind_loc)
    IF (ALLOCATED(basin_id_loc)) DEALLOCATE(basin_id_loc)
    IF (ALLOCATED(basin_coor_loc)) DEALLOCATE(basin_coor_loc)
    IF (ALLOCATED(basin_type_loc)) DEALLOCATE(basin_type_loc)
    IF (ALLOCATED(basin_flowdir_loc)) DEALLOCATE(basin_flowdir_loc)
    IF (ALLOCATED(basin_lshead_loc)) DEALLOCATE(basin_lshead_loc)
    IF (ALLOCATED(outflow_grid_loc)) DEALLOCATE(outflow_grid_loc)
    IF (ALLOCATED(outflow_basin_loc)) DEALLOCATE(outflow_basin_loc)
    IF (ALLOCATED(nbcoastal_loc)) DEALLOCATE(nbcoastal_loc)
    IF (ALLOCATED(coastal_basin_loc)) DEALLOCATE(coastal_basin_loc)
    !
    ! Do linkup & fetch & truncate on ROOT processor (global variables)
    !
    IF (is_root_prc) THEN
       !
       ! Makes the connections between the basin and ensures global coherence.
       !
       !
       IF ( timesimple ) THEN
          timeint=time() ; stime=ctime(timeint)
          WRITE(*,'(I3,1X,A40,A26)') mpi_rank, '##> GET in routing_reg_linkup at :', stime
       ENDIF
       !
       CALL routing_reg_linkup(nbp_glo, neighbours_g, nwbas, basin_count, basin_area, basin_id, basin_flowdir, &
            & basin_lshead, basin_hierarchy, basin_fac, diaglalo, outflow_grid, outflow_basin, inflow_number, inflow_grid, &
            & inflow_basin, nbcoastal, coastal_basin, invented_basins)
       !
       !
       !
       WRITE(numout,*) 'The maximum number of basins in any grid :', MAXVAL(basin_count), debug
       !
       IF ( debug ) THEN
          DO ib=1,nbp_glo
             IF ( routing_diagbox_g(ib, diaglalo) ) THEN
                WRITE(numout,*) 'After routing_reg_linkup information for grid ', ib
                WRITE(numout,*) 'Coordinates : ', lalo_g(ib,2), lalo_g(ib,1) 
                DO jp=1,basin_count(ib)
                   WRITE(numout,*) jp, ' Basin ID : ', basin_id(ib, jp)
                   WRITE(numout,*) 'Basin outflow_grid :', outflow_grid(ib, jp)
                   WRITE(numout,*) 'Basin outflow_basin:', outflow_basin(ib, jp)
                   WRITE(numout,*) 'Basin basin_flowdir :', basin_flowdir(ib, jp)
                   WRITE(numout,*) 'Basin hierarchy :', basin_hierarchy(ib, jp)
                ENDDO
             ENDIF
          ENDDO
       ENDIF
       !
       !! Computes the fetch of each basin, upstream area in known.
       !
       IF ( timesimple ) THEN
          timeint=time() ; stime=ctime(timeint)
          WRITE(*,'(I3,1X,A40,A26)') mpi_rank, '##> GET in routing_reg_fetch at :', stime
       ENDIF
       !
       CALL routing_reg_fetch(nbp_glo, resolution_g, contfrac_g, nwbas, basin_count, basin_area, basin_id, basin_hierarchy, basin_fac,&
            &                 outflow_grid, outflow_basin, fetch_basin)
       !
       !
       !! Reduces the number of basins per grid to the value chosen by the user.
       !
       !
       IF ( timesimple ) THEN
          timeint=time() ; stime=ctime(timeint)
          WRITE(*,'(I3,1X,A40,A26)') mpi_rank, '##> GET in routing_reg_truncate at :', stime
       ENDIF
       !
       CALL routing_reg_truncate(nbp_glo, resolution_g, contfrac_g, nwbas, basin_count, basin_notrun, basin_area, basin_topoind,&
            & fetch_basin, basin_id, basin_coor, basin_type, basin_flowdir, outflow_grid, outflow_basin, inflow_number,&
            & inflow_grid, inflow_basin)
       !
    ENDIF
    !
    ! Put IF (ALLOCATED) here, too lazy, so put in on LOCAL
    !
    IF (ALLOCATED(sub_area)) DEALLOCATE(sub_area)
    IF (ALLOCATED(sub_index)) DEALLOCATE(sub_index)
    IF (ALLOCATED(sub_pts)) DEALLOCATE(sub_pts)
    !
    IF (ALLOCATED(basin_count)) DEALLOCATE(basin_count)
    IF (ALLOCATED(basin_notrun)) DEALLOCATE(basin_notrun)
    IF (ALLOCATED(basin_area)) DEALLOCATE(basin_area)
    IF (ALLOCATED(basin_hierarchy)) DEALLOCATE(basin_hierarchy)
    IF (ALLOCATED(basin_topoind)) DEALLOCATE(basin_topoind)
    IF (ALLOCATED(fetch_basin)) DEALLOCATE(fetch_basin)
    IF (ALLOCATED(basin_id)) DEALLOCATE(basin_id)
    IF (ALLOCATED(basin_coor)) DEALLOCATE(basin_coor)
    IF (ALLOCATED(basin_type)) DEALLOCATE(basin_type)
    IF (ALLOCATED(basin_flowdir)) DEALLOCATE(basin_flowdir)
    IF (ALLOCATED(outflow_grid)) DEALLOCATE(outflow_grid)
    IF (ALLOCATED(outflow_basin)) DEALLOCATE(outflow_basin)
    IF (ALLOCATED(inflow_number)) DEALLOCATE(inflow_number)
    IF (ALLOCATED(inflow_basin)) DEALLOCATE(inflow_basin)
    IF (ALLOCATED(inflow_grid)) DEALLOCATE(inflow_grid)
    IF (ALLOCATED(nbcoastal)) DEALLOCATE(nbcoastal)
    IF (ALLOCATED(coastal_basin)) DEALLOCATE(coastal_basin)
    !
    ! Scatter all variables
    !
    CALL scatter(routing_area_glo,routing_area_loc)
    CALL scatter(topo_resid_glo,topo_resid_loc)
    CALL scatter(route_togrid_glo,route_togrid_loc)
    CALL scatter(route_tobasin_glo,route_tobasin_loc)
    CALL scatter(route_nbintobas_glo,route_nbintobas_loc)
    CALL scatter(global_basinid_glo,global_basinid_loc)
    !
    CALL scatter(route_outlet_glo,route_outlet_loc)
    CALL scatter(route_type_glo,route_type_loc)
    CALL scatter(origin_nbintobas_glo, origin_nbintobas_loc)
    !
    IF ( timesimple ) THEN
       IF (is_root_prc) THEN
          timeint=time() ; stime=ctime(timeint)
          WRITE(*,'(I3,1X,A40,A26)') mpi_rank, '##> FINISH in routing_reg_basins at :', stime
       ENDIF
    ENDIF
    !
  END SUBROUTINE routing_reg_basins


!! ================================================================================================================================
!! SUBROUTINE : routing_reg_getgrid
!!
!>\BRIEF This subroutine extracts from the global high resolution fields
!! the data for the current grid box we are dealing with.
!!
!! DESCRIPTION (definitions, functional, design, flags) :
!! Convention for trip on the input :
!! The trip field follows the following convention for the flow of the water :
!! trip = 1 : flow = N
!! trip = 2 : flow = NE
!! trip = 3 : flow = E
!! trip = 4 : flow = SE
!! trip = 5 : flow = S
!! trip = 6 : flow = SW
!! trip = 7 : flow = W
!! trip = 8 : flow = NW
!! trip = 97 : return flow into the ground
!! trip = 98 : coastal flow (diffuse flow into the oceans) These values are created here
!! trip = 99 : river flow into the oceans
!!
!! On output, the grid boxes of the basin map which flow out of the GCM grid are identified
!! by numbers larger than 100 :
!! trip = 101 : flow = N out of the coarse grid
!! trip = 102 : flow = NE out of the coarse grid
!! trip = 103 : flow = E out of the coarse grid
!! trip = 104 : flow = SE out of the coarse grid
!! trip = 105 : flow = S out of the coarse grid
!! trip = 106 : flow = SW out of the coarse grid
!! trip = 107 : flow = W out of the coarse grid
!! trip = 108 : flow = NW out of the coarse grid
!! Inside the grid the convention remains the same as above (ie between 1 and 99).:\n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_getgrid(nbpt, ib, sub_pts, sub_index, sub_area, max_basins, min_topoind, &
       & lon_rel, lat_rel, lalo, resolution, contfrac, trip, basins, topoindex, fac, hierarchy, &
       & nbi, nbj, area_bx, trip_bx, basin_bx, topoind_bx, fac_bx, hierarchy_bx, lon_bx, lat_bx, lshead_bx)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    INTEGER(i_std), INTENT(in) :: ib !! Current basin (unitless)
    INTEGER(i_std), INTENT(in) :: sub_pts(nbpt) !! Number of high resolution points on this grid (unitless)
    INTEGER(i_std), INTENT(in) :: sub_index(nbpt,nbvmax,2) !! Indices of the points we need on the fine grid (unitless)
    REAL(r_std), INTENT(inout) :: max_basins !! The current maximum of basins
    REAL(r_std), INTENT(in) :: min_topoind !! The current minimum of topographic index (m)
    REAL(r_std), INTENT(in) :: sub_area(nbpt,nbvmax) !! Area on the fine grid (m^2)
    !
    REAL(r_std), INTENT(in) :: lon_rel(nbpt,nbvmax) !!
    REAL(r_std), INTENT(in) :: lat_rel(nbpt,nbvmax) !! coordinates of the fine grid
    !
    REAL(r_std), INTENT(in) :: lalo(nbpt,2) !! Vector of latitude and longitudes (beware of the order !)
    REAL(r_std), INTENT(in) :: resolution(nbpt,2) !! The size of each grid box in X and Y (m)
    REAL(r_std), INTENT(in) :: contfrac(nbpt) !! Fraction of land in each grid box (unitless;0-1)
    !
    REAL(r_std), INTENT(inout) :: trip(nbpt,nbvmax) !! The trip field (unitless)
    REAL(r_std), INTENT(inout) :: basins(nbpt,nbvmax) !! data on the fine grid
    REAL(r_std), INTENT(inout) :: topoindex(nbpt,nbvmax) !! Topographic index of the residence time (m)
    REAL(r_std), INTENT(inout) :: hierarchy(nbpt,nbvmax) !! data on the fine grid
    REAL(r_std), INTENT(inout) :: fac(nbpt,nbvmax) !! data on the fine grid
    !
!! OUTPUT VARIABLES
    INTEGER(i_std), INTENT(out) :: nbi, nbj !! Number of point in x and y within the grid (unitless)
    REAL(r_std), INTENT(out) :: area_bx(nbxmax,nbxmax) !! Area of each small box in the grid box (m^2)
    REAL(r_std), INTENT(out) :: hierarchy_bx(nbxmax,nbxmax) !! Level in the basin of the point
    REAL(r_std), INTENT(out) :: fac_bx(nbxmax,nbxmax) !! Flow accumulation
    REAL(r_std), INTENT(out) :: lon_bx(nbxmax,nbxmax) !!
    REAL(r_std), INTENT(out) :: lat_bx(nbxmax,nbxmax) !!
    REAL(r_std), INTENT(out) :: lshead_bx(nbxmax,nbxmax) !! Large scale heading for outflow points.
    REAL(r_std), INTENT(out) :: topoind_bx(nbxmax,nbxmax) !! Topographic index of the residence time for each of the smaller boxes (m)
    INTEGER(i_std), INTENT(out) :: trip_bx(nbxmax,nbxmax) !! The trip field for each of the smaller boxes (unitless)
    INTEGER(i_std), INTENT(out) :: basin_bx(nbxmax,nbxmax) !!
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: ip, jp, ll(1), iloc, jloc !! Indices (unitless)
    INTEGER(i_std) :: ipp, jpp
    INTEGER(i_std), DIMENSION(8,2) :: inc
    REAL(r_std) :: cenlon, cenlat, dlon, dlat, deslon, deslat, facti, factj
    REAL(r_std) :: lonstr(nbxmax*nbxmax) !!
    REAL(r_std) :: latstr(nbxmax*nbxmax) !!
    !

!_ ================================================================================================================================
    !
    ! The routing code (i=1, j=2)
    !
    inc(1,1) = 0
    inc(1,2) = -1
    inc(2,1) = 1
    inc(2,2) = -1
    inc(3,1) = 1
    inc(3,2) = 0
    inc(4,1) = 1
    inc(4,2) = 1
    inc(5,1) = 0
    inc(5,2) = 1
    inc(6,1) = -1
    inc(6,2) = 1
    inc(7,1) = -1
    inc(7,2) = 0
    inc(8,1) = -1
    inc(8,2) = -1
    !
    ! Set everything to undef to locate easily empty points
    !
    trip_bx(:,:) = undef_int
    basin_bx(:,:) = undef_int
    topoind_bx(:,:) = undef_sechiba
    area_bx(:,:) = undef_sechiba
    hierarchy_bx(:,:) = undef_sechiba
    fac_bx(:,:) = undef_sechiba
    lon_bx(:,:) = undef_sechiba
    lat_bx(:,:) = undef_sechiba
    lshead_bx(:,:) = undef_sechiba
    cenlon = zero
    cenlat = zero
    !
    IF ( sub_pts(ib) > 0 ) THEN
       !
       DO ip=1,sub_pts(ib)
          IF ( ip >nbxmax*nbxmax ) THEN
             CALL ipslerr_p(3,'routing_reg_getgrid','nbxmax too small when filling lonstr',&
                  &           'Please change method to estimate nbxmax','')
          ENDIF
          lonstr(ip) = lon_rel(ib, ip)
          latstr(ip) = lat_rel(ib, ip)
       ENDDO
       !
       ! Get the size of the area and order the coordinates to go from North to South and West to East
       !
       CALL routing_sortcoord(sub_pts(ib), lonstr, 'WE', nbi)
       CALL routing_sortcoord(sub_pts(ib), latstr, 'NS', nbj)
       !
       ! Verify dimension and allocated space
       !
       IF ( nbi > nbxmax .OR. nbj > nbxmax ) THEN
          WRITE(numout,*) "size of area : nbi=",nbi,"nbj=",nbj, "nbxmax=", nbxmax
          CALL ipslerr_p(3,'routing_reg_getgrid','nbxmax too small','Please change method to estimate nbxmax','')
       ENDIF
       !
       ! Transfer the data in such a way that (1,1) is the North Western corner and
       ! (nbi, nbj) the South Eastern.
       !
       DO ip=1,sub_pts(ib)
          ll = MINLOC(ABS(lonstr(1:nbi) - lon_rel(ib, ip)))
          iloc = ll(1)
          ll = MINLOC(ABS(latstr(1:nbj) - lat_rel(ib, ip)))
          jloc = ll(1)
          !
          trip_bx(iloc, jloc) = NINT(trip(ib, ip))
          basin_bx(iloc, jloc) = NINT(basins(ib, ip))
          area_bx(iloc, jloc) = sub_area(ib, ip)
          topoind_bx(iloc, jloc) = topoindex(ib, ip)
          hierarchy_bx(iloc, jloc) = hierarchy(ib, ip)
          fac_bx(iloc, jloc) = fac(ib, ip)
          lon_bx(iloc, jloc) = lon_rel(ib, ip)
          lat_bx(iloc, jloc) = lat_rel(ib, ip)
          cenlon = cenlon + lon_rel(ib, ip)/sub_pts(ib)
          cenlat = cenlat + lat_rel(ib, ip)/sub_pts(ib)
       ENDDO
       !
    ELSE
       !
       ! This is the case where the model invented a continental point
       !
       nbi = 1
       nbj = 1
       iloc = 1
       jloc = 1
       trip_bx(iloc, jloc) = 98
       basin_bx(iloc, jloc) = NINT(max_basins + 1)
       max_basins = max_basins + 1
       area_bx(iloc, jloc) = resolution(ib,1)*resolution(ib,2)*contfrac(ib)
       topoind_bx(iloc, jloc) = min_topoind
       ! Be careful here: We deal with basin_hierarchy(ib,ij) = min_topoind later!
       hierarchy_bx(iloc, jloc) = min_topoind
       fac_bx(iloc, jloc) = zero
       lon_bx(iloc, jloc) = lalo(ib,2)
       lat_bx(iloc, jloc) = lalo(ib,1)
       !
    ENDIF
    !
    ! Tag in trip all the outflow conditions. The table is thus :
    ! trip = 100+n : Outflow into another grid box
    ! trip = 99 : River outflow into the ocean
    ! trip = 98 : This will be coastal flow (not organized as a basin)
    ! trip = 97 : return flow into the soil (local)
    !
    DO jp=1,nbj
       DO ip=1,nbi
          !
          ! Compute the destination of the flow on the high resolution grid (if possible !) to see if it still within the points
          ! belonging to the grid we are working on.
          !
          IF ( trip_bx(ip,jp) < 97 ) THEN
             !
             ipp = ip+inc(trip_bx(ip,jp),1)
             jpp = jp+inc(trip_bx(ip,jp),2)
             !
             ! Check if the indices are outside of the box, then we have a point on the domain border
             !
             IF ( ipp < 1 .OR. ipp > nbi .OR. jpp < 1 .OR. jpp > nbj ) THEN
                IF (routing_nextgrid(ib,trip_bx(ip,jp)) < -1 ) THEN
                   trip_bx(ip,jp) = 98
                ELSE
                   trip_bx(ip,jp) = trip_bx(ip,jp) + 100
                ENDIF
             ELSE
                !
                ! It can also be a border point if the neighbour is not defined
                ! 
                IF ( basin_bx(ipp,jpp) > undef_int-1 ) THEN
                   IF (routing_nextgrid(ib,trip_bx(ip,jp)) < -1 ) THEN
                      trip_bx(ip,jp) = 98
                   ELSE
                      trip_bx(ip,jp) = trip_bx(ip,jp) + 100
                   ENDIF
                ENDIF
             ENDIF
          ELSE IF ( trip_bx(ip,jp) > 100 .AND. trip_bx(ip,jp) < 109 ) THEN
             WRITE(numout,*) 'WARNING : Point flows our of routing.nc file '
             WRITE(numout,*) 'WARNING : Point : basin = ', basin_bx(ip,jp)
             WRITE(numout,*) 'WARNING : Point : coord = ', lon_bx(ip, jp), lat_bx(ip, jp)
             WRITE(numout,*) 'WARNING : Please consider using a larger domaine for the routing.nc file'
             trip_bx(ip,jp) = 98
          ENDIF
          !
       ENDDO
    ENDDO
    !
    ! Compute the large scale flow direction for each outflow point (trip_bx > 100). This
    ! is done by walking into the direction indicated by the small scale trip. The distance of this walk
    ! will depend how far we are from the corner of the polygon. If we are in
    ! the middle of a segment we walk less far than when we are on either corner.
    !
    dlon=ABS(corners_g(ib,NbSegments,1)-corners_g(ib,1,1))
    dlat=ABS(corners_g(ib,NbSegments,2)-corners_g(ib,1,2))
    DO ip=1,NbSegments-1
       dlon = MAX(dlon,ABS(corners_g(ib,ip+1,1)-corners_g(ib,ip,1)))
       dlat = MAX(dlat,ABS(corners_g(ib,ip+1,2)-corners_g(ib,ip,2)))
    ENDDO
    !
    DO jp=1,nbj
       DO ip=1,nbi
          !
          IF ( trip_bx(ip,jp) < undef_int ) THEN
             lshead_bx(ip,jp) = zero
             IF ( trip_bx(ip,jp) > 100 ) THEN
                IF ( nbi > 1 ) THEN
                   facti = REAL(ABS((nbi-1)/2.0-(ip-1)), r_std)/((nbi-1)/2.0)
                ELSE
                   facti = un
                ENDIF
                deslon = lon_bx(ip,jp) + inc(trip_bx(ip,jp)-100,1)*dlon/2.0*facti
                IF ( nbj > 1 ) THEN
                   factj = REAL(ABS((nbj-1)/2.0-(jp-1)), r_std)/((nbj-1)/2.0)
                ELSE
                   factj = un
                ENDIF
                deslat = lat_bx(ip,jp) - inc(trip_bx(ip,jp)-100,2)*dlat/2.0*factj
                lshead_bx(ip,jp) = MOD(haversine_heading(deslon, deslat, cenlon, cenlat)+180.0, 360.0)
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    !
  END SUBROUTINE routing_reg_getgrid
!
!
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_globalize
!!
!>\BRIEF This subroutine puts the basins found for grid box in the global map.
!! Connection can only be made later when all information is together.
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!! One of the outputs is basin_flowdir. Its convention is 1-8 for the directions from North to North
!! West going through South. The negative values will be -3 for return flow, -2 for coastal flow
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_globalize(nbpt, ib, neighbours, area_bx, trip_bx, hierarchy_bx, fac_bx, topoind_bx, min_topoind,&
       & nb_basin, basin_inbxid, basin_outlet, basin_outtp, basin_sz, basin_pts, basin_bxout, basin_bbout, lshead, coast_pts, &
       & nwbas, basin_count, basin_notrun, basin_area, basin_hierarchy, basin_fac, basin_topoind, basin_id, basin_coor, basin_type, &
       & basin_flowdir, basin_lshead, outflow_grid, outflow_basin, nbcoastal, coastal_basin)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT (in) :: nbpt !! Domain size (unitless)
    INTEGER(i_std), INTENT (in) :: ib !! Current basin (unitless)
    INTEGER(i_std), INTENT(in)  :: neighbours(nbpt,NbNeighb)!! Vector of neighbours for each grid point
                                                                         !! (1=North and then clockwise)
!! LOCAL VARIABLES
    REAL(r_std), DIMENSION(nbxmax,nbxmax)    :: area_bx      !! Area of each small box in the grid box (m^2)
    INTEGER(i_std), DIMENSION(nbxmax,nbxmax) :: trip_bx      !! The trip field for each of the smaller boxes (unitless)
    REAL(r_std), DIMENSION(nbxmax,nbxmax)    :: hierarchy_bx !! Level in the basin of the point
    REAL(r_std), DIMENSION(nbxmax,nbxmax)    :: fac_bx !! Level in the basin of the point
    REAL(r_std), DIMENSION(nbxmax,nbxmax)    :: topoind_bx   !! Topographic index of the residence time for each of the smaller boxes (m)
    REAL(r_std)                              :: min_topoind  !! The current minimum of topographic index (m)
    INTEGER(i_std)                           :: nb_basin     !! Number of sub-basins (unitless)
    INTEGER(i_std), DIMENSION(nbvmax)        :: basin_inbxid, basin_sz !! ID of basin, number of points in the basin
    REAL(r_std), DIMENSION(nbvmax,2)         :: basin_outlet !! Outlet coordinate of each subgrid basin
    REAL(r_std), DIMENSION(nbvmax)           :: basin_outtp  !! 
    INTEGER(i_std), DIMENSION(nbvmax,nbvmax,2) :: basin_pts  !! Points in each basin
    INTEGER(i_std), DIMENSION(nbvmax)        :: basin_bxout  !! outflow direction
    INTEGER(i_std), DIMENSION(nbvmax)        :: basin_bbout  !! outflow sub-basin
    REAL(r_std), DIMENSION(nbvmax)           :: lshead       !! Large scale heading of outflow.
    INTEGER(i_std)                           :: coast_pts(nbvmax) !! The coastal flow points (unitless)
    ! global maps
    INTEGER(i_std) :: nwbas !!
    INTEGER(i_std), DIMENSION(nbpt) :: basin_count         !!
    INTEGER(i_std), DIMENSION(nbpt) :: basin_notrun        !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: basin_id      !!
    REAL(r_std), DIMENSION(nbpt,nwbas,2) :: basin_coor     !!
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_type       !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: basin_flowdir !! Water flow directions in the basin (unitless)
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_area       !!
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_hierarchy  !!
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_fac  !!
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_topoind    !! Topographic index of the residence time for a basin (m)
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_lshead     !! Large scale heading out of the grid box.
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: outflow_grid  !! Type of outflow on the grid box (unitless)
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: outflow_basin !! 
    INTEGER(i_std), DIMENSION(nbpt) :: nbcoastal           !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: coastal_basin !!
    !
    INTEGER(i_std) :: ij, iz !! Indices (unitless)
    CHARACTER(LEN=4) :: hierar_method = 'OUTP' !!
    !
    ! To calculate topoindex in new method
    !
    INTEGER(i_std) :: option !! Option to calculate topoindex
    REAL(r_std) :: area_frac !! Area fraction for calculate topoindex
    !
    LOGICAL, PARAMETER :: debug = .FALSE. !! Add new logical variable to debug
    !
!_ ================================================================================================================================
    !
    ! Get option for calculating topoindex
    !
    !Config Key = TOPOINDEX
    !Config Desc = Options to calculate topoindex
    !Config Def = 3
    !Config Help = This flag allows to calculate topoindex with different options
    !Config Units = [-]
    !
    option = 3
    CALL getin('TOPOINDEX', option)
    !
    ! Checking option: until now (May 2016) there is only 3 acceptable options
    ! = 1 : topoindex .EQ. 1000. everywhere
    ! = 2 : simple average as the method in the older routing scheme
    ! = 3 : accumulate the "topoind" from routing.nc along the flow (as Agnes DUCHARNE's suggestion)
    !
    IF ( option .LT. 1 .OR. option .GT. 3 ) THEN
       WRITE(numout,*) ' You chose wrong option for calculating topoindex: ', option
       WRITE(numout,*) ' It should be: 1, 2 or 3'
       STOP 'routing_reg_globalize'
    ENDIF
    !
    DO ij=1, nb_basin
       !
       ! Count the basins and keep their ID
       !
       basin_count(ib) = basin_count(ib)+1
       if (basin_count(ib) > nwbas) then
          WRITE(numout,*) 'ib=',ib
          WRITE(numout,*) 'nwbas=',nwbas
          WRITE(numout,*) 'basin_count(ib)=',basin_count(ib)
          !
          call ipslerr_p(3,'routing_reg_globalize', &
               & 'Problem with basin_count : ', &
               & 'It is greater than number of allocated basin nwbas.', &
               & '(stop to count basins)')
       endif
       basin_id(ib,basin_count(ib)) = basin_inbxid(ij)
       !
       basin_coor(ib,basin_count(ib),1) = basin_outlet(ij,1)
       basin_coor(ib,basin_count(ib),2) = basin_outlet(ij,2)
       !
       basin_type(ib,basin_count(ib)) = basin_outtp(ij)
       !
       ! Transfer the list of basins which flow into the ocean as coastal flow.
       !
       IF ( basin_id(ib,basin_count(ib)) .LT. 0) THEN
          nbcoastal(ib) = basin_sz(ij)
          coastal_basin(ib,1:nbcoastal(ib)) = coast_pts(1:nbcoastal(ib))
       ENDIF
       !
       !
       ! Compute the area of the basin
       !
       basin_area(ib,ij) = zero
       basin_hierarchy(ib,ij) = zero
       basin_fac(ib,ij) = zero
       !
       SELECT CASE (hierar_method)
          !
          CASE("MINI")
             basin_hierarchy(ib,ij) = undef_sechiba
          !
       END SELECT
       basin_topoind(ib,ij) = zero
       !
       DO iz=1,basin_sz(ij)
          !
          basin_area(ib,ij) = basin_area(ib,ij) + area_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))
          !
          ! There are a number of ways to determine the hierarchy of the entire basin.
          ! We allow for three here :
          ! - Take the mean value
          ! - Take the minimum value within the basin
          ! - Take the value at the outflow point
          ! Probably taking the value of the outflow point is the best solution.
          !
          SELECT CASE (hierar_method)
             !
             CASE("MEAN")
                ! Mean hierarchy of the basin
                basin_hierarchy(ib,ij) = basin_hierarchy(ib,ij) + &
                     & hierarchy_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))
                basin_fac(ib,ij) = basin_fac(ib,ij) + &
                     & fac_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))
             CASE("MINI")
                ! The smallest value of the basin
                IF ( hierarchy_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2)) .LT. basin_hierarchy(ib,ij)) THEN
                   basin_hierarchy(ib,ij) = hierarchy_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))
                   ! Trung: We should take the "fac" value at the same point
                   basin_fac(ib,ij) = fac_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))
                ENDIF
             CASE("OUTP")
                ! Value at the outflow point
                IF ( ( trip_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2)) .GT. 100 ) &
                &  .OR. ( trip_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2)) .EQ. -4 ) ) THEN
                   basin_hierarchy(ib,ij) = hierarchy_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))
                ENDIF
                ! Trung: Be careful of statement " .LT. 0"
                IF ( ( trip_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2)) .GE. 97 ) &
                &  .OR. ( trip_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2)) .EQ. -4 ) ) THEN
                   basin_fac(ib,ij) = fac_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))
                ENDIF
             CASE DEFAULT
                WRITE(numout,*) 'Unknown method for computing the hierarchy of the basin'
                CALL ipslerr_p(3,'routing_reg_globalize','Unknown method for computing the hierarchy of the basin','','')
          END SELECT
          !
       ENDDO
       !
       SELECT CASE (hierar_method)
          !
          CASE("MEAN")
             basin_hierarchy(ib,ij) = basin_hierarchy(ib,ij)/REAL(basin_sz(ij),r_std)
             basin_fac(ib,ij) = basin_fac(ib,ij)/REAL(basin_sz(ij),r_std)
          !
       END SELECT
       !
       ! Calculating topoindex depends on options
       !
       basin_topoind(ib,ij) = zero
       ! Constant value (1000.)
       IF ( option .EQ. 1 ) THEN
          basin_topoind(ib,ij) = 1000.
       ENDIF
       ! Simple average (as the old method)
       IF ( option .EQ. 2 ) THEN
          DO iz=1,basin_sz(ij)
             basin_topoind(ib,ij) = basin_topoind(ib,ij) + topoind_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))
          ENDDO
          basin_topoind(ib,ij) = basin_topoind(ib,ij)/REAL(basin_sz(ij),r_std)
       ENDIF
       ! Minus outlet hierarchy
       IF ( option .EQ. 3 ) THEN
          DO iz=1,basin_sz(ij)
             area_frac = area_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))/REAL(basin_area(ib,ij))
             basin_topoind(ib,ij) = basin_topoind(ib,ij) + topoind_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))*area_frac
          ENDDO
          !
          ! Trung: In routing_reg_globalize:
          ! Trung: Checking negative value of basin_topoind
          !
          IF ( debug ) THEN
             IF ( basin_topoind(ib,ij) .LT. 0. ) THEN
                WRITE (numout,*) 'In routing_reg_globalize: We got negative value in ',ib,ij
                WRITE (numout,*) 'Start checking ... '
                WRITE (numout,*) 'basin_area ', REAL(basin_area(ib,ij))
                WRITE (numout,*) ' ==================> '
                DO iz=1,basin_sz(ij)
                   area_frac = area_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))/REAL(basin_area(ib,ij))
                   WRITE (numout,*) 'iz = ',iz
                   WRITE (numout,*) 'topoind_bx :', topoind_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))
                   WRITE (numout,*) 'area_bx ', area_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))
                   WRITE (numout,*) 'area_frac ', area_frac
                   WRITE (numout,*) 'topoind_bx * area_frac ', topoind_bx(basin_pts(ij,iz,1),basin_pts(ij,iz,2))*area_frac
                ENDDO
             ENDIF
          ENDIF
       ENDIF
       !
       ! To make sure that it has the lowest number if this is an outflow point we reset basin_hierarchy
       !
       IF (basin_bxout(ij) .LT. 0 .AND. basin_bxout(ij) .NE. -4) THEN
          IF ( option .NE. 1 ) THEN
             basin_hierarchy(ib,ij) = min_topoind
             !basin_fac(ib,ij) = min_topoind
          ELSE
             basin_hierarchy(ib,ij) = 0.00
             !basin_fac(ib,ij) = 0.00
          ENDIF
          basin_topoind(ib,ij) = min_topoind
       ENDIF
       !
       !
       ! Keep the outflow boxes and basin
       !
       basin_flowdir(ib,ij) = basin_bxout(ij)
       IF (basin_bxout(ij) .GT. 0) THEN
          basin_lshead(ib,ij) = lshead(ij)
          outflow_grid(ib,ij) = routing_nextgrid(ib,basin_bxout(ij))
       ELSE
          basin_lshead(ib,ij) = undef_sechiba
          outflow_grid(ib,ij) = basin_bxout(ij)
       ENDIF
       !
       outflow_basin(ib,ij) = undef_int
       IF (basin_bxout(ij) .EQ. -4) THEN
          outflow_basin(ib,ij) = basin_bbout(ij)
       ENDIF
       !
       !
    ENDDO
    !
    ! Get the number of sub-grid basin before truncation
    !
    basin_notrun = basin_count
    !
  END SUBROUTINE routing_reg_globalize
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_linkup
!!
!>\BRIEF This subroutine makes the connections between the basins and ensure global coherence.
!!
!! DESCRIPTION (definitions, functional, design, flags) :
!! The convention for outflow_grid is :
!! outflow_grid >    : The grid bow in which the basin is supposed to flow.
!! outflow_grid = -1 : River flow
!! outflow_grid = -2 : Coastal flow
!! outflow_grid = -3 : Return flow
!! outflow_grid = -4 : Flows into a basin in the same grid
!! For outflow_basin we have the following conventions :
!! outflow_basin = basin id in the case of the basin not flowing out of the current grid (i.e. outflow_grid=-4).
!! Else outflow_basin = undef_int and the objective of this routine is to find what that basin is. This work
!! essentially occurs in the routine routing_reg_bestsubbasin. But for that we need to find the grid box where 
!! we will look for the right basin. This operation is performed here in 5 successive steps. The order correspond
!! to the solution we would prefer.
!! 1.0 : We will look in the grid provided by outflow_grid for the right basin. This neighbour has been obtained
!! from the small scale flow direction given in routing.nc.
!! 2.0 : Try the grid box given by the large scale flow direction.
!! 3.0 : Have another attempt at the outflow_grid by looking at the neighbour just to the right and the left.
!! 4.0 : Here we look at half of the neighbouring grid boxes around the large scale flow direction.
!! 5.0 : If all the above failed then we look within the current grid box if we find a suitable outflow basin.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_linkup(nbpt, neighbours, nwbas, basin_count, basin_area, basin_id, basin_flowdir, &
       & basin_lshead, basin_hierarchy, basin_fac, diaglalo, outflow_grid, outflow_basin, inflow_number, inflow_grid, &
       & inflow_basin, nbcoastal, coastal_basin, invented_basins)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT (in) :: nbpt !! Domain size (unitless)
    INTEGER(i_std), DIMENSION(nbpt,NbNeighb), INTENT (in) :: neighbours !!
    REAL(r_std), INTENT(in) :: invented_basins !!
    !
    INTEGER(i_std) :: nwbas !!
    INTEGER(i_std), DIMENSION(nbpt) :: basin_count !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: basin_id !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: basin_flowdir !! Water flow directions in the basin (unitless)
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_lshead  !! Large scale flow direction out of the basin.
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_area !!
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_hierarchy !!
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_fac !!
    REAL(r_std), DIMENSION(:,:), INTENT(in) :: diaglalo !! Point (in Lat/Lon) where diagnostics will be printed.
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: outflow_grid !! Type of outflow on the grid box (unitless)
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: outflow_basin !!
    !
    INTEGER(i_std), DIMENSION(nbpt,nbxmax*8) :: inflow_number !!
    INTEGER(i_std), DIMENSION(nbpt,nbxmax*8,nbxmax*8) :: inflow_basin !!
    INTEGER(i_std), DIMENSION(nbpt,nbxmax*8,nbxmax*8) :: inflow_grid !!
    !
    INTEGER(i_std), DIMENSION(nbpt) :: nbcoastal !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: coastal_basin !!
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: sp, sb, sbl, inp, outdm1, outdp1 !! Indices (unitless)
    INTEGER(i_std) :: spa, sba, sbint
    INTEGER(i_std) :: i, nbocean, it
    REAL(r_std)    :: wocean
    INTEGER(i_std) :: dp1, dm1, dm1i, dls, bp1, bm1, bls !! Indices (unitless)
    INTEGER(i_std) :: dop, bop, nbtotest !!
    INTEGER(i_std) :: fbas(nwbas), nbfbas !!
    REAL(r_std)    :: ang, angp1, angm1, bopqual, blsqual, bp1qual, bm1qual, crit
    REAL(r_std)    :: fbas_hierarchy(nwbas) !!
    INTEGER(i_std) :: ff(1) !!
    INTEGER(i_std), DIMENSION(NbNeighb) :: gridstotest, gridbasin
    REAL(r_std), DIMENSION(NbNeighb) :: diffangle, gridangle
    INTEGER(i_std), DIMENSION(nbpt,5) :: solved
    INTEGER(i_std) :: unsolved, testbasinid
    REAL(r_std), ALLOCATABLE, DIMENSION(:) :: hatoutflow
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: minhloc
    !
    !
!! PARAMETERS
    LOGICAL, PARAMETER :: debug = .FALSE. !! (true/false)
    !
!_ ================================================================================================================================
    ! 
    !
    testbasinid = -93063
    !
    IF ( debug ) WRITE (numout,*) 'SIZE inflow_grid:',SIZE(inflow_grid,1),SIZE(inflow_grid,2),SIZE(inflow_grid,3)
    IF ( debug ) WRITE (numout,*) 'SIZE inflow_basin:',SIZE(inflow_basin,1),SIZE(inflow_basin,2),SIZE(inflow_basin,3)
    CALL FLUSH(numout)
    !
    inflow_number(:,:) = 0
    !
    ! Variable to keep track of how many points were solved at each stage
    !
    solved(:,:) = 0
    !
    ! Some preparatory work. We need the minimum hierarchy for each basin and its location.
    ! This comes from the fact that for coastal basins it is often difficult to decide if they
    ! first flow into another grid or directly into the ocean. So this information will be
    ! useful.
    !
    ALLOCATE(hatoutflow(INT(invented_basins)))
    ALLOCATE(minhloc(INT(invented_basins),2))
    hatoutflow(:) = undef_sechiba
    DO sp=1,nbpt
       DO sb=1,basin_count(sp)
          IF ( basin_id(sp,sb) < INT(invented_basins) .AND. basin_id(sp,sb) > 0 ) THEN
             IF ( hatoutflow(basin_id(sp,sb)) > basin_hierarchy(sp,sb) ) THEN
                hatoutflow(basin_id(sp,sb)) = basin_hierarchy(sp,sb)
                minhloc(basin_id(sp,sb),1) = sp
                minhloc(basin_id(sp,sb),2) = sb
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    !
    ! 1.0 Follow the flow direction as given by the high resolution basin description. This case also treats the
    ! basins which remain in the current grid box, rivers, coastal and return flows.
    !
    DO sp=1,nbpt
       !
       DO sb=1,basin_count(sp)
          !
          !
          ! We only work on this point if it does not flow into the ocean
          ! or flow to another sub-basin in the same grid box.
          ! At this point any of the outflows is designated by a negative values in
          ! outflow_grid
          !
          IF ( outflow_grid(sp,sb) < 0 ) THEN
             IF ( outflow_grid(sp,sb) .EQ. -4 ) THEN
                ! Flow into a basin of the same grid
                bop = outflow_basin(sp,sb)
                CALL routing_updateflow(sp, sb, sp, bop, nbpt, nwbas, nbxmax*8, outflow_grid, outflow_basin, &
                     &                  inflow_number, inflow_grid, inflow_basin)
                IF ( outflow_basin(sp,sb) == bop ) THEN
                   solved(sp,1) = solved(sp,1) + 1
                ENDIF
                !
             ELSE IF ( outflow_grid(sp,sb) .EQ. -3 ) THEN
                ! Return flow
                ! Nothing to do but just remember it is done.
                solved(sp,1) = solved(sp,1) + 1
             ELSE IF ( outflow_grid(sp,sb) .EQ. -2 ) THEN
                ! Coastal flow
                ! Nothing to do but just remember it is done.
                solved(sp,1) = solved(sp,1) + 1
             ELSE IF ( outflow_grid(sp,sb) .EQ. -1 ) THEN
                ! River flow
                ! Nothing to do but just remember it is done.
                solved(sp,1) = solved(sp,1) + 1
             ENDIF
          ELSE IF ( outflow_grid(sp,sb) .GT. 0 ) THEN
             !
             inp = outflow_grid(sp,sb)
             !
             CALL routing_reg_bestsubbasin(sp, sb, basin_id(sp,sb), basin_hierarchy(sp,sb), basin_fac(sp,sb), &
                  &                    basin_flowdir(sp,sb), invented_basins, &
                  &                    nbpt, nwbas, inp, basin_count, basin_id, basin_hierarchy, basin_fac, &
                  &                    basin_flowdir, nbcoastal, coastal_basin, bop, bopqual)
             !
             ! Trung: Because I think it's for nothing, I commented this line:
             !ang = routing_anglediff_g(sp, inp, basin_flowdir(sp,sb))
             !
             ! If the basin is suitable (bop < undef_int) then take it
             !
             IF ( bop .LT. undef_int ) THEN
                !
                CALL routing_updateflow(sp, sb, inp, bop, nbpt, nwbas, nbxmax*8, outflow_grid, outflow_basin, &
                     &                  inflow_number, inflow_grid, inflow_basin)
                IF ( outflow_basin(sp,sb) == bop ) THEN
                   solved(sp,1) = solved(sp,1) + 1
                ENDIF
                !
             ENDIF
             !
          ENDIF
          !
          IF ( debug .AND. routing_diagbox_g(sp,diaglalo) ) THEN
             WRITE(numout,*) "Linkup 1.0 - In routing_reg_linkup working on sp & sb:",sp, sb
             WRITE(numout,*) "Linkup 1.0 - Coords : ", lalo_g(sp,2), lalo_g(sp,1)
             WRITE(numout,*) "Linkup 1.0 - outflow_flowdir = ", basin_flowdir(sp,sb)
             WRITE(numout,*) "Linkup 1.0 - Large scale heading =", basin_lshead(sp,sb)
             WRITE(numout,*) "Linkup 1.0 - Hierarchy =", basin_hierarchy(sp,sb)
             WRITE(numout,*) "Linkup 1.0 - Basin % of grid =", basin_area(sp,sb)/area_g(sp)*100
             WRITE(numout,*) "Linkup 1.0 - outflow_grid =", outflow_grid(sp,sb)
             IF (  outflow_grid(sp,sb) > 0 ) THEN
                WRITE(numout,*) "Linkup 1.0 - Coords outflow: ", lalo_g(outflow_grid(sp,sb),2), lalo_g(outflow_grid(sp,sb),1)
             ENDIF
             WRITE(numout,*) "Linkup 1.0 - outflow_basin = ", outflow_basin(sp,sb)
             IF ( outflow_grid(sp,sb) > 0 .AND. outflow_basin(sp,sb) < undef_int ) THEN
                WRITE(numout,*) "Linkup 1.0 - Outflow Hierarchy =", basin_hierarchy(outflow_grid(sp,sb),outflow_basin(sp,sb))
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    !
    ! 2.0 : Do we have a valid outflow basin ? If not we need to test other options.
    ! The reason for not finding the outflow basin could be that the outflow_grid correspons to a
    ! small scale feature not corresponding to the orientation of the grid. So we use the large
    ! scale heading of the flow out of the grid in order to test another neighbour.
    !
    DO sp=1,nbpt
       !
       DO sb=1,basin_count(sp)
          !
          IF ( outflow_basin(sp,sb) .EQ. undef_int .AND. &
               & outflow_grid(sp,sb) > 0 .AND. basin_flowdir(sp,sb) > 0) THEN
             !
             DO i=1,NbNeighb
                diffangle(i) = ABS(haversine_diffangle(headings_g(sp,i), basin_lshead(sp,sb)))
             ENDDO
             ff = MINLOC(diffangle)
             dls = neighbours_g(sp,ff(1))
             !
             IF ( dls .LE. -1 ) THEN
                ! This flows out of the domain so put the basin into coastal flow
                outflow_grid(sp,sb) = -2
                outflow_basin(sp,sb) = undef_int
                solved(sp,2) = solved(sp,2)+1
                IF ( debug .AND. routing_diagbox_g(sp,diaglalo) ) THEN
                   WRITE(numout,*) "Linkup 2.0 - ", sp, sb, "becomes a coastal flow basin."
                ENDIF
             ELSE
                IF ( dls > 0 ) THEN
                   CALL routing_reg_bestsubbasin(sp, sb, basin_id(sp,sb), basin_hierarchy(sp,sb), basin_fac(sp,sb), basin_flowdir(sp,sb), invented_basins, &
                        &                    nbpt, nwbas, dls, basin_count, basin_id, basin_hierarchy, basin_fac, &
                        &                    basin_flowdir, nbcoastal, coastal_basin, bls, blsqual)
                ELSE
                   bls = undef_int
                   blsqual = 0
                ENDIF
             ENDIF
             !
             IF ( dls > 0 .AND. bls < undef_int ) THEN
                !
                CALL routing_updateflow(sp, sb, dls, bls, nbpt, nwbas, nbxmax*8, outflow_grid, outflow_basin, &
                     &                  inflow_number, inflow_grid, inflow_basin)
                !
             ENDIF
             !
             IF ( debug .AND. routing_diagbox_g(sp,diaglalo) ) THEN
                WRITE(numout,*) "Linkup 2.0 - sp, sb = ", sp, sb
                WRITE(numout,*) "Linkup 2.0 - Large scale heading =", basin_lshead(sp,sb)
                WRITE(numout,*) "Linkup 2.0 - Large scale test =", dls, bls
                WRITE(numout,*) "Linkup 2.0 - outflow_grid =", outflow_grid(sp,sb)
                WRITE(numout,*) "Linkup 2.0 - outflow_basin = ", outflow_basin(sp,sb)
             ENDIF
             !
             IF ( outflow_basin(sp,sb) < undef_int ) solved(sp,2) = solved(sp,2)+1
             !
          ENDIF
          !
       ENDDO
    ENDDO
    !
    !
    ! 3.0 Look within the general direction of the indicated flow to see if we find a matching basin.
    !
    ! In case we did not find the correct basin we start to look
    ! around. We find two options for the outflow direction (dp1 & dm1) and the corresponding 
    ! basin index (bp1 & bm1).
    ! These options are only acceptable if the angle to the original direction is less or equal
    ! to 45 degrees.
    !

    DO sp=1,nbpt
       !
       DO sb=1,basin_count(sp)
          !
          nbocean = 0
          !
          IF ( outflow_basin(sp,sb) .EQ. undef_int .AND. &
               & outflow_grid(sp,sb) > 0 .AND. basin_flowdir(sp,sb) > 0) THEN
             !
             ! Try to neighboring directions on the model grid.
             !
             dp1 = routing_nextgrid_g(sp, basin_flowdir(sp,sb), +1)
             angp1 = routing_anglediff_g(sp, dp1, basin_flowdir(sp,sb))
             dm1 = routing_nextgrid_g(sp, basin_flowdir(sp,sb), -1)
             angm1 = routing_anglediff_g(sp, dm1, basin_flowdir(sp,sb))
             !
             ! How many are ocean points ?
             !
             IF ( dp1 .LE. -1 ) nbocean = nbocean + 1
             IF ( dm1 .LE. -1 ) nbocean = nbocean + 1
             !
             ! Compute bp1 and bm1
             !
             IF ( dp1 .GT. 0 ) THEN
                IF ( angp1 <= 45.0 ) THEN
                   CALL routing_reg_bestsubbasin(sp, sb, basin_id(sp,sb), basin_hierarchy(sp,sb), basin_fac(sp,sb), basin_flowdir(sp,sb), invented_basins, &
                        &                    nbpt, nwbas, dp1, basin_count, basin_id, basin_hierarchy, basin_fac, &
                        &                    basin_flowdir, nbcoastal, coastal_basin, bp1, bp1qual)
                ELSE
                   bp1 = undef_int 
                   bp1qual = 0
                ENDIF
             ELSE
                bp1 = undef_int 
                bp1qual = 0
             ENDIF
             IF ( dm1 .GT. 0 ) THEN
                IF ( angm1 <= 45.0 ) THEN
                   CALL routing_reg_bestsubbasin(sp, sb, basin_id(sp,sb), basin_hierarchy(sp,sb), basin_fac(sp,sb), basin_flowdir(sp,sb), invented_basins, &
                        &                    nbpt, nwbas, dm1, basin_count, basin_id, basin_hierarchy, basin_fac, &
                        &                    basin_flowdir, nbcoastal, coastal_basin, bm1, bm1qual)
                ELSE
                   bm1 = undef_int
                   bm1qual = 0
                ENDIF
             ELSE
                bm1 = undef_int
                bm1qual = 0
             ENDIF
             !
             ! Decide between dp1 and dm1 which is the better solution ! 
             !
             dop = undef_int
             IF ( bp1 < undef_int .OR. bm1 < undef_int ) THEN
                IF ( bp1 < undef_int ) THEN
                   dop = dp1
                   bop = bp1
                ELSE IF ( bm1 < undef_int ) THEN
                   dop = dm1
                   bop = bm1
                ELSE
                   IF ( bp1qual > bm1qual ) THEN
                      dop = dp1
                      bop = bp1
                   ELSE IF ( bp1qual > bm1qual ) THEN
                      dop = dm1
                      bop = bm1
                   ELSE
                      IF ( angm1 < angp1 ) THEN 
                         dop = dm1
                         bop = bm1
                      ELSE
                         dop = dp1
                         bop = bp1
                      ENDIF
                   ENDIF
                ENDIF
             ENDIF
             !
             ! Now that we know where we want the water to flow to we write the
             ! the information in the right fields.
             !
             IF ( dop > 0 .AND. dop < undef_int .AND. bop < undef_int ) THEN
                !
                CALL routing_updateflow(sp, sb, dop, bop, nbpt, nwbas, nbxmax*8, outflow_grid, outflow_basin, &
                     &                  inflow_number, inflow_grid, inflow_basin)
                !
             ENDIF
             !
             ! If one of the outflow options is an ocean point then we should use it.
             ! We are a little more stringent with the angle.
             !
             IF ( outflow_basin(sp,sb) .EQ. undef_int .AND. &
                  & outflow_grid(sp,sb) > 0 .AND. basin_flowdir(sp,sb) > 0) THEN
                IF ( nbocean >= 1 ) THEN
                   IF ( (dp1 .LE. -1 .AND. angp1 < 45.0 ) .OR. &
                        & (dm1 .LE. -1 .AND. angm1 < 45.0 ) ) THEN
                      outflow_grid(sp,sb) = -2
                      outflow_basin(sp,sb) = undef_int
                      solved(sp,3) = solved(sp,3)+1
                      IF ( debug .AND. routing_diagbox_g(sp,diaglalo) ) THEN
                         WRITE(numout,*) "Linkup 3.0 - ", sp, sb, "becomes a coastal flow basin."
                      ENDIF
                   ENDIF
                ENDIF
             ENDIF
             !
             IF ( debug .AND. routing_diagbox_g(sp,diaglalo) ) THEN
                WRITE(numout,*) "Linkup 3.0 - In routing_reg_linkup working on sp & sb:", sp, sb
                WRITE(numout,*) "Linkup 3.0 - outflow_flowdir = ", basin_flowdir(sp,sb)
                WRITE(numout,*) "Linkup 3.0 - Tested dp1 & dm1 : ", dp1, dm1
                WRITE(numout,*) "Linkup 3.0 - Angles : ",angp1, angm1
                WRITE(numout,*) "Linkup 3.0 - Test output bp1 & bm1 = ", bp1, bm1
                WRITE(numout,*) "Linkup 3.0 - Selected bop & dop = ", bop, dop
                WRITE(numout,*) "Linkup 3.0 - outflow_grid =", outflow_grid(sp,sb)
                WRITE(numout,*) "Linkup 3.0 - outflow_basin = ", outflow_basin(sp,sb)
             ENDIF
             !
             IF ( outflow_basin(sp,sb) < undef_int ) solved(sp,3) = solved(sp,3)+1
             !
          ENDIF
          !
       ENDDO
    ENDDO
    !
    ! 4.0 If we still have not found the correct basin so we look into other directions
    ! East and West of the large scale flow direction we have. We work this with the
    ! the smallest angle to the large scale flow direction first. Remember the grid
    ! in the large scale flow direction was already tested (in 2.0).
    !
    nbtotest = INT(NbNeighb/2.0)
    !
    DO sp=1,nbpt
       !
       DO sb=1,basin_count(sp)
          !
          nbocean = 0
          wocean = zero
          !
          IF ( outflow_basin(sp,sb) .EQ. undef_int .AND. &
               & outflow_grid(sp,sb) > 0 .AND. basin_flowdir(sp,sb) > 0) THEN
             !
             ! Try to neighboring directions on the model grid.
             !
             DO i=1,NbNeighb
                diffangle(i) = ABS(haversine_diffangle(headings_g(sp,i), basin_lshead(sp,sb)))
             ENDDO
             !
             ! The minimum was already done above. So we delete from the list
             !
             ff = MINLOC(diffangle)
             dls = neighbours_g(sp,ff(1))
             IF ( dls .LE. -1 ) THEN
                nbocean = nbocean + 1
                wocean = wocean + diffangle(ff(1))
             ENDIF
             diffangle(ff(1)) = undef_sechiba
             !
             DO it=1,nbtotest
                ff = MINLOC(diffangle)
                gridstotest(it) = neighbours_g(sp,ff(1))
                gridangle(it) = diffangle(ff(1))
                !
                ! neighbours outside of the domain are considered as ocean points.
                !
                IF ( gridstotest(it) .LE. -1 ) THEN
                   nbocean = nbocean + 1
                   wocean = wocean + diffangle(ff(1))
                ENDIF
                diffangle(ff(1)) = undef_sechiba
             ENDDO
             !
             ! Now work through our list of points
             !
             DO it=1,nbtotest
                IF ( outflow_basin(sp,sb) .EQ. undef_int ) THEN
                   dls = gridstotest(it)
                   !
                   IF ( dls .GT. 0 ) THEN
                      dop = dls
                      CALL routing_reg_bestsubbasin(sp, sb, basin_id(sp,sb), basin_hierarchy(sp,sb), basin_fac(sp,sb), basin_flowdir(sp,sb), &
                           &                    invented_basins, nbpt, nwbas, dls, basin_count, basin_id, &
                           &                    basin_hierarchy, basin_fac, basin_flowdir, nbcoastal, coastal_basin, bop, bopqual)
                   ELSE
                      bop = undef_int 
                      bopqual = 0
                   ENDIF
                   gridbasin(it) = bop
                   !
                   ! Now that we know where we want the water to flow to we write the
                   ! the information into the right fields.
                   !
                   IF ( dls > 0 .AND. dls < undef_int .AND. bop < undef_int ) THEN
                      !
                      CALL routing_updateflow(sp, sb, dls, bop, nbpt, nwbas, nbxmax*8, outflow_grid, outflow_basin, &
                           &                  inflow_number, inflow_grid, inflow_basin)
                      !
                   ENDIF
                ENDIF
             ENDDO
             !
             IF ( debug .AND. routing_diagbox_g(sp,diaglalo) ) THEN
                WRITE(numout,*) "Linkup 4.0 - In routing_reg_linkup working on sp & sb:", sp, sb
                WRITE(numout,*) "Linkup 4.0 - Large scale heading =", basin_lshead(sp,sb)
                WRITE(numout,*) "Linkup 4.0 - Tested gridstotest : ", gridstotest(1:nbtotest)
                WRITE(numout,*) "Linkup 4.0 - Angles of tested : ", gridangle(1:nbtotest)
                WRITE(numout,*) "Linkup 4.0 - Tested outflow basin = ", gridbasin(1:nbtotest)
                WRITE(numout,*) "Linkup 4.0 - outflow_grid =", outflow_grid(sp,sb)
                WRITE(numout,*) "Linkup 4.0 - outflow_basin = ", outflow_basin(sp,sb)
                WRITE(numout,*) "Linkup 4.0 - Number of ocean options : ", nbocean
                IF (nbocean > 0 ) WRITE(numout,*) "Linkup 4.0 - ocean direction = ", wocean/nbocean
                IF ( basin_id(sp,sb) < invented_basins ) THEN
                   WRITE(numout,*) "Linkup 4.0 - Hmin for basin :", hatoutflow(basin_id(sp,sb))
                   WRITE(numout,*) "Linkup 4.0 - Hmin location :", minhloc(basin_id(sp,sb),1), minhloc(basin_id(sp,sb),2)
                ENDIF
             ENDIF
             !
             IF ( outflow_basin(sp,sb) < undef_int ) solved(sp,4) = solved(sp,4)+1
             !
          ENDIF
          !
          ! If we have not found anything, but we have an ocean point as neighbour, then go for it.
          ! This would be coastal flow then.
          !
          IF ( outflow_basin(sp,sb) .EQ. undef_int .AND. &
               & outflow_grid(sp,sb) > 0 .AND. basin_flowdir(sp,sb) > 0) THEN
             !
             IF ( basin_id(sp,sb) < invented_basins ) THEN
                crit = (basin_hierarchy(sp,sb)-hatoutflow(basin_id(sp,sb)))/(basin_hierarchy(sp,sb)+1)
             ELSE
                crit = zero
             ENDIF
             !
             ! If we have at least 2 ocean points within the examined points then ouflow points
             ! we put this basin into the ocean. If hierarchy is lower than the 
             ! average at other outflow points, then we relax this condition.
             !
             IF ( crit > 0.1 ) THEN
                IF ( nbocean >= nbtotest/2.0 ) THEN
                   !
                   ! These ocean points should not be too far away from the large scale direction.
                   ! In this phase we should also consider the size of the basin. If it small then we can 
                   ! neglect its and put it into the ocean.
                   !
                   IF ( wocean/nbocean < 90.0 .OR. (basin_area(sp,sb)/area_g(sp)*100 < 0.5) ) THEN
                      outflow_grid(sp,sb) = -2
                      outflow_basin(sp,sb) = undef_int
                      solved(sp,4) = solved(sp,4)+1
                      IF ( debug .AND. routing_diagbox_g(sp,diaglalo) ) THEN
                         WRITE(numout,*) "Linkup 4.0 - ", sp, sb, "becomes a coastal flow basin."
                      ENDIF
                   ENDIF
                ENDIF
             ELSE
                !
                ! Here we are with a basin that has a hierarchy close to that of the outflow point. So
                ! we can be more relaxed about the criteria.
                !
                IF ( nbocean >= 1 ) THEN
                   !
                   ! These ocean points should not be too far away from the large scale direction.
                   !
                   IF ( wocean/nbocean < 90.0 .OR. (basin_area(sp,sb)/area_g(sp)*100 < 0.5) ) THEN
                      outflow_grid(sp,sb) = -2
                      outflow_basin(sp,sb) = undef_int
                      solved(sp,4) = solved(sp,4)+1
                      IF ( debug .AND. routing_diagbox_g(sp,diaglalo) ) THEN
                         WRITE(numout,*) "Linkup 4.0 - ", sp, sb, "becomes a coastal flow basin. Relaxed condition"
                      ENDIF
                   ENDIF
                ENDIF
             ENDIF
             !
          ENDIF
          !
          !
       ENDDO
    ENDDO
    !
    ! 5.0
    !
    ! We probably have not yet solved all points. So we go through all points and basins again
    ! to see if we cannot find some local solution as a last resort.
    !
    !
    DO sp=1,nbpt
       !
       DO sb=1,basin_count(sp)
          !
          IF ( outflow_basin(sp,sb) .EQ. undef_int .AND. &
               & outflow_grid(sp,sb) > 0 .AND. basin_flowdir(sp,sb) > 0) THEN
             !
             ! Last resort, we look if a larger basin (i.e. we already have solved !) within this grid has 
             ! a lower or the same hierarchy (within 1%).
             !
             sbint = undef_int
             !
             DO sba=1,basin_count(sp)
                IF ( sba .NE. sb ) THEN
                   IF ( basin_id(sp,sb) == basin_id(sp,sba) ) THEN
                      !
                      ! Only look for suitable hierrachy if we have a solution for the sub-basin : it has an outflow grid
                      ! and basin or flows into the ocean.
                      !
                      IF ( (outflow_grid(sp,sba) > 0 .AND. outflow_basin(sp,sba) < undef_int) .OR. &
                           & outflow_grid(sp,sba) .EQ. -1 .OR. outflow_grid(sp,sba) .EQ. -2 ) THEN
                         !
                         ! calculation of the criterion for considering that the hierarchy are close enough. 
                         ! Watch out hierarchy can be zero. 1 is considered the 
                         ! minimum here.
                         !
                         ! Trung: If current sub-basin (sp,sb) is the closest
                         !        point to the outlet of river, the value of
                         !        basin_hierarchy(sp,sb) is always lower than
                         !        basin_hierarchy(sp,sba). It means that crit
                         !        is negative and statement IF (crit < 0.1) is
                         !        not useful. => I changed it a little bit.
                         !
                         crit = (basin_hierarchy(sp,sb)-basin_hierarchy(sp,sba))/(basin_hierarchy(sp,sb)+1)
                         IF ( crit < 0.01 .AND. crit >= zero ) THEN
                            ! Trung: Put condition of flow accumulation here to
                            !        avoid error of circulation in routing_reg_fetch 
                            IF (basin_fac(sp,sba) .GE. basin_fac(sp,sb)) THEN
                               sbint = sba
                            ENDIF
                         ENDIF
                      ENDIF
                   ENDIF
                ENDIF
             ENDDO
             !
             ! If this grid contains the lowest hierarchy of this basin ID, we can also merge without second thought.
             !
             IF ( basin_id(sp,sb) < invented_basins ) THEN
                IF ( sp == minhloc(basin_id(sp,sb),1) ) THEN
                   sbint = minhloc(basin_id(sp,sb),2) 
                ENDIF
             ENDIF
             !
             ! Trung: Be careful when changing value of basin_hierarchy here !
             !
             IF ( sbint < undef_int ) THEN
                basin_hierarchy(sp,sb) = basin_hierarchy(sp,sbint)
                CALL routing_updateflow(sp, sb, sp, sbint, nbpt, nwbas, nbxmax*8, outflow_grid, outflow_basin, &
                     &                  inflow_number, inflow_grid, inflow_basin)
             ELSE
                ! 
                ! Trung: Well, if you come to this step, it should be a special
                !        (or curious, I said) case. For example, when processing
                !        Sulina branch of river Danube with the forcing data
                !        from E2OFD (/bdd/ORCHIDEE_Forcing/BC/OOL/OL2/E2OFD/) 
                !        in resolution of 0.25 degree. The grid box [45.125N, 29.375E]
                !        is a ocean point in the middle of 7 land points ("a
                !        hole"). But HydroSHEDS shows that Sulina branch flows
                !        through this grid cell (from [45.125N, 29.125E] to [45.125N, 29.625E]). 
                ! Trung: If the "JUMP" does not work, it means a worse case.
                !        Or it means a worse case.
                !        The outlet of this river falls in the ocean point of
                !        forcing data. For example: with 0.25 degree forcing 
                !        data of E2OFD, the grid point [ 45.625N ; 13.875E ] 
                !        lost the outlet of basin ID 69666.
                !        We need to use this point (which should be the last point 
                !        of this river with shortest hierarchy and highest flow accumulation)
                !        to become new outlet.
                !
                !        This is not good solution because there can be few
                !        outlet for one basin ID. 
                !        But I need the model works now !!! => so, come back
                !        here later !
                !
                WRITE (numout,*) "Linkup : Made a NEW OUTLET at sp & sb: ",sp,sb
                ! Coastal flow or river flow is both ok here
                outflow_grid(sp,sb) = -2
                basin_hierarchy(sp,sb) = 0.00
             ENDIF
             !
             ! See if we could find the same basin in this grid with a lower hierarchy
             !
             IF ( debug .AND. routing_diagbox_g(sp,diaglalo) ) THEN
                   IF ( basin_id(sp,sb) == basin_id(sp,sba) ) THEN
                      WRITE(numout,*) "Linkup 5.0 - In routing_reg_linkup working on sp & sb :", sp,sb
                      WRITE(numout,*) "Linkup 5.0 - Do we have an internal solution here :", outflow_basin(sp,sb)
                      WRITE(numout,*) "Linkup 5.0 - ID = ", basin_id(sp,sb), basin_id(sp,sba)
                      WRITE(numout,*) "Linkup 5.0 - H  = ", basin_hierarchy(sp,sb), basin_hierarchy(sp,sba)
                      WRITE(numout,*) "Linkup 5.0 - Area = ", basin_area(sp,sb), basin_area(sp,sba)
                      WRITE(numout,*) "Linkup 5.0 - outflow_grid =", outflow_grid(sp,sb)
                      WRITE(numout,*) "Linkup 5.0 - outflow_basin = ", outflow_basin(sp,sb)
                   ENDIF
             ENDIF
             !
             IF ( outflow_basin(sp,sb) < undef_int ) solved(sp,5) = solved(sp,5)+1
             !
          ENDIF
          !
       ENDDO
    ENDDO
    !
    IF ( debug .AND. testbasinid > 0 ) THEN
       DO sp=1,nbpt
          DO sb=1,basin_count(sp)
             IF ( basin_id(sp,sb) == testbasinid ) THEN
                WRITE(numout,*) "Linkup TEST : ", basin_id(sp,sb), "@", sp, sb
                WRITE(numout,*) "Linkup TEST H= ", basin_hierarchy(sp,sb)
                WRITE(numout,*) "Linkup TEST FAC= ", basin_fac(sp,sb)
                WRITE(numout,*) "Linkup TEST outflow_grid ", outflow_grid(sp,sb)
                WRITE(numout,*) "Linkup TEST outflow_basin ", outflow_basin(sp,sb)
                WRITE(numout,*) "Linkup TEST basin_lshead ", basin_lshead(sp,sb)
                WRITE(numout,*) "Linkup TEST basin_flowdir ", basin_flowdir(sp,sb)
             ENDIF
          ENDDO
       ENDDO
    ENDIF
    !
    ! 4.0 We hopefully have solved all the points but here we check it and print sole diagnostics.
    !
    ! Trung: I prefer to write the percent number here in real than integer
    !        because for large domain, 0.01% is really amount.
    !
    WRITE(numout,'(A16,I10,A8,I10,A14)') "Linkup : Out of ", SUM(basin_count), " basins ", SUM(solved), " were resolved."
    DO sb=1,5
       WRITE(numout,'(A9,7X,F10.4,A27,I2.2,A2)') "Linkup : ",REAL(SUM(solved(:,sb)), r_std)/SUM(basin_count)*100.0, &
            &          " % of basins solved by case ", sb," ."
    ENDDO
    !
    DO sp=1,nbpt
       !
       IF ( debug ) THEN
          DO sb=1,5
             WRITE(numout,*) sp," Solutions found for case ", sb," : ",&
                  &          NINT(REAL(solved(sp,sb), r_std)/basin_count(sp)*100.0)," %"
          ENDDO
          IF ( SUM(solved(sp,:)) < basin_count(sp) ) THEN
             WRITE(numout,*) sp,"No solution found for ", basin_count(sp)-SUM(solved(sp,:))," basins."
          ENDIF
       ENDIF
       !
       DO sb=1,basin_count(sp)
          !
          ! OK this is it, we give up :-)
          !
          IF ( outflow_basin(sp,sb) .EQ. undef_int .AND. &
               & outflow_grid(sp,sb) > 0 .AND. basin_flowdir(sp,sb) > 0 ) THEN
             WRITE(numout,*) 'We could not find the basin into which we need to flow, Linkup FAILED'
             WRITE(numout,*) 'Linkup FAILED : Grid point ', sp, ' and basin ', sb
             WRITE(numout,*) 'Linkup FAILED : Coordinates : ', lalo_g(sp,2), lalo_g(sp,1) 
             WRITE(numout,*) 'Linkup FAILED : Outflow direction :', basin_flowdir(sp,sb)
             WRITE(numout,*) 'Linkup FAILED : Large scale outflow direction :',basin_lshead(sp,sb)
             WRITE(numout,*) 'Linkup FAILED : Outlfow grid :', outflow_grid(sp,sb)
             WRITE(numout,*) 'Linkup FAILED : Outlfow basin :',outflow_basin(sp,sb)
             WRITE(numout,*) 'Linkup FAILED : basin ID:',basin_id(sp,sb)
             WRITE(numout,*) 'Linkup FAILED : Hierarchy :', basin_hierarchy(sp,sb)
             WRITE(numout,*) 'Linkup FAILED : Flow accumulation :', basin_fac(sp,sb)
             CALL ipslerr_p(3,'routing_reg_linkup', &
                  "We could not find the basin into which we need to flow",'Try with debug=.TRUE.','stop routing_reg_linkup')
          ENDIF
          !
       ENDDO
       !
    ENDDO
    !
    ! Check for each outflow basin that it exists
    !
    DO sp=1,nbpt
       DO sb=1,basin_count(sp)
          !
          inp = outflow_grid(sp,sb)
          sbl = outflow_basin(sp,sb)
          IF ( inp .GE. 0 ) THEN
             IF ( basin_count(inp) .LT. sbl ) THEN
                WRITE(numout,*) 'Point :', sp, ' Basin :', sb
                WRITE(numout,*) 'Flows into point :', inp, ' basin :', sbl
                WRITE(numout,*) 'But it flows into now here as basin count = ', basin_count(inp)
                !
                WRITE(numout,*) 'Input sub-basin :'
                WRITE(numout,*) 'Outflow direction :', basin_flowdir(sp,sb)
                WRITE(numout,*) 'Outlfow grid :', outflow_grid(sp,sb)
                WRITE(numout,*) 'Outlfow basin :',outflow_basin(sp,sb)
                WRITE(numout,*) 'Basin ID:',basin_id(sp,sb)
                WRITE(numout,*) 'Hierarchy :', basin_hierarchy(sp,sb)
                WRITE(numout,*) 'Output grid box:'
                WRITE(numout,*) 'Basin ID:',basin_id(inp,basin_count(inp))
                WRITE(numout,*) 'Hierarchy :', basin_hierarchy(inp,basin_count(inp))
                !
                CALL ipslerr_p(3,'routing_reg_linkup','Problem with outflow','','')
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    !
  END SUBROUTINE routing_reg_linkup
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_fetch
!!
!>\BRIEF This subroutine computes the fetch of each basin. This means that for each basin we
!! will know how much area is upstream. It will help decide how to procede with the
!! the truncation later and allow to set correctly in outflow_grid the distinction
!! between coastal and river flow.
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_fetch(nbpt, resolution, contfrac, nwbas, basin_count, basin_area, basin_id, basin_hierarchy, basin_fac,&
       & outflow_grid, outflow_basin, fetch_basin)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    !
    REAL(r_std), DIMENSION(nbpt,2), INTENT(in) :: resolution !! The size of each grid box in X and Y (m)
    REAL(r_std), DIMENSION(nbpt), INTENT(in) :: contfrac !! Fraction of land in each grid box (unitless;0-1)
    !
    INTEGER(i_std) :: nwbas !!
    INTEGER(i_std), DIMENSION(nbpt), INTENT(in) :: basin_count !!
    REAL(r_std), DIMENSION(nbpt,nwbas), INTENT(inout) :: basin_area !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas), INTENT(in) :: basin_id !!
    REAL(r_std), DIMENSION(nbpt,nwbas), INTENT(in)       :: basin_hierarchy
    REAL(r_std), DIMENSION(nbpt,nwbas), INTENT(in)       :: basin_fac
    INTEGER(i_std), DIMENSION(nbpt,nwbas), INTENT(inout) :: outflow_grid !! Type of outflow on the grid box (unitless)
    INTEGER(i_std), DIMENSION(nbpt,nwbas), INTENT(in) :: outflow_basin !!
!
!! OUTPUT VARIABLES
    REAL(r_std), DIMENSION(nbpt,nwbas), INTENT(out) :: fetch_basin !!
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: ib, ij, ff(1), it, itt, igrif, ibasf, nboutflow !! Indices (unitless)
    REAL(r_std) :: contarea !!
    REAL(r_std) :: totbasins !!
    REAL(r_std), DIMENSION(nbpt*nbvmax) :: tmp_area !!
    INTEGER(i_std), DIMENSION(nbpt*nbvmax,2) :: tmpindex !!
    !
    INTEGER(i_std), PARAMETER :: maxiterations=10000

!_ ================================================================================================================================
    !
    !
    ! Normalize the area of all basins
    !
    DO ib=1,nbpt
       !
       totbasins = SUM(basin_area(ib,1:basin_count(ib)))
       contarea = resolution(ib,1)*resolution(ib,2)*contfrac(ib)
       !
       DO ij=1,basin_count(ib)
          basin_area(ib,ij) = basin_area(ib,ij)/totbasins*contarea
       ENDDO
       !
    ENDDO
    !
    ! Compute the area upstream of each basin
    !
    fetch_basin(:,:) = zero
    !
    !
    DO ib=1,nbpt
       !
       DO ij=1,basin_count(ib)
          !
          fetch_basin(ib, ij) = fetch_basin(ib, ij) + basin_area(ib,ij)
          !
          igrif = outflow_grid(ib,ij)
          ibasf = outflow_basin(ib,ij)
          !
          itt = 0
          !
          DO WHILE (igrif .GT. 0)
             !
             fetch_basin(igrif,ibasf) = fetch_basin(igrif,ibasf) + basin_area(ib, ij)
             it = outflow_grid(igrif, ibasf)
             ibasf = outflow_basin(igrif, ibasf)
             igrif = it
             itt = itt + 1
             IF ( itt .GT. maxiterations) THEN
                WRITE(numout,&
                     "('Grid ',I5, ' and basin ',I5, ' did not converge after iteration ',I5)") ib, ij, itt
                WRITE(numout, "('==> We flow through grid ',I5,' and basin ',I5)") igrif, ibasf
                WRITE(numout,*) '==> Basin ID :', basin_id(igrif,ibasf), "Hierarchy :", basin_hierarchy(igrif,ibasf)
                WRITE(numout,*) "==> Flow accumulation :", basin_fac(igrif,ibasf)
                WRITE(numout,*) "==> Coordinates : ", lalo_g(igrif,2), lalo_g(igrif,1)
                !
                IF ( itt .GT. maxiterations+20) THEN
                   CALL ipslerr_p(3,'routing_reg_fetch','Problem...','','')
                ENDIF
             ENDIF
          ENDDO
          !
       ENDDO
       !
    ENDDO
    !
    WRITE(numout,*) 'The smallest FETCH :', MINVAL(fetch_basin)
    WRITE(numout,*) 'The largest FETCH :', MAXVAL(fetch_basin)
    !
    ! Now we set for the 'num_largest' largest basins the outflow condition as stream flow
    ! (i.e. outflow_grid = -1) and all other outflow basins are set to coastal flow
    ! (i.e. outflow_grid = -2). The return flow is not touched.
    !
    nboutflow = 0
    !
    DO ib=1,nbpt
       !
       DO ij=1,basin_count(ib)
          !
          ! We do not need any more the river flow flag as we are going to reset it.
          !
          IF ( outflow_grid(ib,ij) .EQ. -1) THEN
             outflow_grid(ib,ij) = -2
          ENDIF
          !
          IF ( outflow_grid(ib,ij) .EQ. -2) THEN
             !
             nboutflow = nboutflow + 1
             tmp_area(nboutflow) = fetch_basin(ib,ij)
             tmpindex(nboutflow,1) = ib
             tmpindex(nboutflow,2) = ij
             !
          ENDIF
          !
       ENDDO
    ENDDO
    !
    DO ib=1, num_largest
       ff = MAXLOC(tmp_area(1:nboutflow))
       outflow_grid(tmpindex(ff(1),1), tmpindex(ff(1),2)) = -1
       tmp_area(ff(1)) = zero
    ENDDO
    !
  END SUBROUTINE routing_reg_fetch
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_truncate
!!
!>\BRIEF This subroutine reduces the number of basins per grid to the value chosen by the user.
!! It also computes the final field which will be used to route the water at the
!! requested truncation.
!!
!! DESCRIPTION (definitions, functional, design, flags) :
!! Truncate if needed and find the path closest to the high resolution data.
!!
!! The algorithm :
!!
!! We only go through this procedure only as many times as there are basins to take out at most.
!! This is important as it allows the simplifications to spread from one grid to the other.
!! The for each step of the iteration and at each grid point we check the following options for
!! simplifying the pathways of water :
!! 1) If the basin of a grid flows into another basin of the same grid. Kill the one which only
!! served as a transition
!! 2) If in one grid box we have a number of basins which flow into the ocean as coastal flow.
!! We kill the smallest one and put into the largest basin. There is no need to manage many
!! basins going into the ocean as coastal flows.
!! 3) If we have streams run in parallel from one gird box to the others (that is these are
!! different basins) we will put the smaller one in the larger one. This may hapen at any
!! level of the flow but in theory it should propagate downstream.
!! 4) If we have two basins with the same ID but flow into different grid boxes we sacrifice
!! the smallest one and route it through the largest.
!!
!! Obviously if any of the options find something then we skip the rest and take out the basin.:\n
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_truncate(nbpt, resolution, contfrac, nwbas, basin_count, basin_notrun, basin_area, basin_topoind,&
       & fetch_basin, basin_id, basin_coor, basin_type, basin_flowdir, outflow_grid, outflow_basin, inflow_number,&
       & inflow_grid, inflow_basin)
    !
    IMPLICIT NONE
    !
!! PARAMETERS
    !
    INTEGER(i_std), PARAMETER :: pickmax = 2000 !!

!! INPUT VARIABLES
    INTEGER(i_std) :: nbpt !! Domain size (unitless)
    !
    REAL(r_std), DIMENSION(nbpt,2) :: resolution !! The size of each grid box in X and Y (m)
    REAL(r_std), DIMENSION(nbpt), INTENT(in) :: contfrac !! Fraction of land in each grid box (unitless;0-1)
    !
    INTEGER(i_std) :: nwbas !!
    INTEGER(i_std), DIMENSION(nbpt) :: basin_count !!
    INTEGER(i_std), DIMENSION(nbpt) :: basin_notrun !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: basin_id !!
    REAL(r_std), DIMENSION(nbpt,nwbas,2) :: basin_coor !!
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_type !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: basin_flowdir !! Water flow directions in the basin (unitless)
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_area !!
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_topoind !! Topographic index of the residence time for a basin (m)
    REAL(r_std), DIMENSION(nbpt,nwbas) :: fetch_basin !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: outflow_grid !! Type of outflow on the grid box (unitless)
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: outflow_basin !!
    !
    ! Changed nwbas to nbxmax*4 and *8 to save the memory
    !
    INTEGER(i_std), DIMENSION(nbpt,nbxmax*8) :: inflow_number !!
    INTEGER(i_std), DIMENSION(nbpt,nbxmax*8,nbxmax*8) :: inflow_basin !!
    INTEGER(i_std), DIMENSION(nbpt,nbxmax*8,nbxmax*8) :: inflow_grid !!
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: ib, ij, ibf, ijf, igrif, ibasf, cnt, pold, bold, ff(2) !! Indices (unitless)
    INTEGER(i_std) :: ii, kbas, sbas, ik, iter, ibt, obj !! Indices (unitless)
    REAL(r_std), DIMENSION(nbpt,nbasmax) :: floflo !!
    REAL(r_std), DIMENSION(nbpt) :: gridarea !!
    REAL(r_std), DIMENSION(nbpt) :: gridbasinarea !!
    REAL(r_std) :: ratio !!
    INTEGER(i_std), DIMENSION(pickmax,2) :: largest_basins !!
    INTEGER(i_std), DIMENSION(pickmax) :: tmp_ids !!
    INTEGER(i_std) :: multbas !!
    INTEGER(i_std) :: iml(1) !! X resolution of the high resolution grid
    INTEGER(i_std), DIMENSION(pickmax) :: multbas_sz !!
    REAL(r_std), DIMENSION(pickmax) :: tmp_area !!
    INTEGER(i_std), DIMENSION(pickmax,pickmax) :: multbas_list !!
    !
    INTEGER(i_std) :: nbtruncate !!
    INTEGER(i_std), SAVE, ALLOCATABLE, DIMENSION(:) :: indextrunc !!
!$OMP THREADPRIVATE(indextrunc)
    ! 
    LOGICAL        :: trunc_equa = .TRUE. !! Logical to choose if equality truncation is activated or not (true/false)
    !

!_ ================================================================================================================================
    !
    !
    ! Read option for truncation method
    !
    trunc_equa = .TRUE.
    CALL getin('ROUTING_TRUNCEQUA', trunc_equa)
    !
    IF ( .NOT. ALLOCATED(indextrunc)) THEN
       ALLOCATE(indextrunc(nbpt))
    ENDIF
    !
    ! We have to go through the grid as least as often as we have to reduce the number of basins
    ! For good measure we add 3 more passages.
    !
    !
    DO iter = 1, MAXVAL(basin_count) - nbasmax +3
       !
       ! Get the points over which we wish to truncate
       !
       nbtruncate = 0
       DO ib = 1, nbpt
          IF ( basin_count(ib) .GT. nbasmax ) THEN
             nbtruncate = nbtruncate + 1
             indextrunc(nbtruncate) = ib
          ENDIF
       ENDDO
       !
       ! Go through the basins which need to be truncated.
       !
       DO ibt=1,nbtruncate
          !
          ib = indextrunc(ibt)
          !
          ! Check if we have basin which flows into a basin in the same grid
          ! kbas = basin we will have to kill
          ! sbas = basin which takes over kbas
          !
          kbas = 0
          sbas = 0
          !
          ! 1) Merge two basins which flow into the ocean as coastal or return flow
          ! (outflow_grid = -2 or -3). Well obviously only if we have more than 1 and
          ! have not found anything yet!
          !
          IF ( (COUNT(outflow_grid(ib,1:basin_count(ib)) .EQ. -2) .GT. 1 .OR.&
               & COUNT(outflow_grid(ib,1:basin_count(ib)) .EQ. -3) .GT. 1) .AND.&
               & kbas*sbas .EQ. 0) THEN
             !
             multbas = 0
             multbas_sz(:) = 0
             !
             IF ( COUNT(outflow_grid(ib,1:basin_count(ib)) .EQ. -2) .GT. 1 ) THEN
                obj = -2
             ELSE
                obj = -3
             ENDIF
             !
             ! First we get the list of all basins which go out as coastal or return flow (obj)
             !
             DO ij=1,basin_count(ib)
                IF ( outflow_grid(ib,ij) .EQ. obj ) THEN
                   multbas = multbas + 1
                   multbas_sz(multbas) = ij
                   tmp_area(multbas) = fetch_basin(ib,ij)
                ENDIF
             ENDDO
             ! 
             ! Test the new truncation priority
             !
             IF ( trunc_equa ) THEN
                !
                ! Take the smallest to be transfered to the second smallest
                !
                iml = MINLOC(tmp_area(1:multbas), MASK = tmp_area(1:multbas) .GT. zero)
                kbas = multbas_sz(iml(1))
                tmp_area(iml(1)) = -1
                iml = MINLOC(tmp_area(1:multbas), MASK = tmp_area(1:multbas) .GT. zero)
                sbas = multbas_sz(iml(1))
                !
             ELSE
                !
                ! Take the smallest to be transfered to the largest
                !
                iml = MAXLOC(tmp_area(1:multbas), MASK = tmp_area(1:multbas) .GT. zero)
                sbas = multbas_sz(iml(1))
                iml = MINLOC(tmp_area(1:multbas), MASK = tmp_area(1:multbas) .GT. zero)
                kbas = multbas_sz(iml(1))
                !
             ENDIF
          ENDIF
          !
          ! 2) If we have basins flowing into the same grid but different basins then we put them
          ! together. Obviously we first work with the grid which has most streams running into it
          ! and putting the smallest in the largests catchments.
          !
          IF ( kbas*sbas .EQ. 0) THEN
             !
             tmp_ids(1:basin_count(ib)) = outflow_grid(ib,1:basin_count(ib))
             multbas = 0
             multbas_sz(:) = 0
             !
             ! First obtain the list of basins which flow into the same basin
             !
             DO ij=1,basin_count(ib)
                IF ( outflow_grid(ib,ij) .GT. 0 .AND.&
                     & COUNT(tmp_ids(1:basin_count(ib)) .EQ. outflow_grid(ib,ij)) .GT. 1) THEN
                   multbas = multbas + 1
                   DO ii=1,basin_count(ib)
                      IF ( tmp_ids(ii) .EQ. outflow_grid(ib,ij)) THEN
                         multbas_sz(multbas) = multbas_sz(multbas) + 1
                         multbas_list(multbas,multbas_sz(multbas)) = ii
                         tmp_ids(ii) = -99
                      ENDIF
                   ENDDO
                ELSE
                   tmp_ids(ij) = -99
                ENDIF
             ENDDO
             !
             ! Did we come up with any basins to deal with this way ?
             !
             IF ( multbas .GT. 0 ) THEN
                !
                iml = MAXLOC(multbas_sz(1:multbas))
                ik = iml(1)
                !
                ! Take the smallest and largest of these basins !
                !
                DO ii=1,multbas_sz(ik)
                   tmp_area(ii) = fetch_basin(ib, multbas_list(ik,ii))
                ENDDO
                !
                IF ( trunc_equa ) THEN
                  iml = MINLOC(tmp_area(1:multbas_sz(ik)), MASK = tmp_area(1:multbas_sz(ik)) .GT. zero)
                  kbas = multbas_list(ik,iml(1))
                  tmp_area(iml(1)) = -1
                  iml = MINLOC(tmp_area(1:multbas_sz(ik)), MASK = tmp_area(1:multbas_sz(ik)) .GT. zero)
                  sbas = multbas_list(ik,iml(1))
                ELSE
                  iml = MAXLOC(tmp_area(1:multbas_sz(ik)), MASK = tmp_area(1:multbas_sz(ik)) .GT. zero)
                  sbas = multbas_list(ik,iml(1))
                  iml = MINLOC(tmp_area(1:multbas_sz(ik)), MASK = tmp_area(1:multbas_sz(ik)) .GT. zero)
                  kbas = multbas_list(ik,iml(1))
                ENDIF
                !
             ENDIF
             !
          ENDIF
          !
          ! 3) If we have twice the same basin we put them together even if they flow into different
          ! directions. If one of them goes to the ocean it takes the advantage.
          !
          IF ( kbas*sbas .EQ. 0) THEN
             !
             tmp_ids(1:basin_count(ib)) = basin_id(ib,1:basin_count(ib))
             multbas = 0
             multbas_sz(:) = 0
             !
             ! First obtain the list of basins which have sub-basins in this grid box.
             ! (these are identified by their IDs)
             !
             DO ij=1,basin_count(ib)
                IF ( COUNT(tmp_ids(1:basin_count(ib)) .EQ. basin_id(ib,ij)) .GT. 1) THEN
                   multbas = multbas + 1
                   DO ii=1,basin_count(ib)
                      IF ( tmp_ids(ii) .EQ. basin_id(ib,ij)) THEN
                         multbas_sz(multbas) = multbas_sz(multbas) + 1
                         multbas_list(multbas,multbas_sz(multbas)) = ii
                         tmp_ids(ii) = -99
                      ENDIF
                   ENDDO
                ELSE
                   tmp_ids(ij) = -99
                ENDIF
             ENDDO
             !
             ! We are going to work on the basin with the largest number of sub-basins.
             ! (IF we have a basin which has subbasins !)
             !
             IF ( multbas .GT. 0 ) THEN
                !
                iml = MAXLOC(multbas_sz(1:multbas))
                ik = iml(1)
                !
                ! If one of the basins goes to the ocean then it is going to have the priority
                !
                tmp_area(:) = zero
                IF ( COUNT(outflow_grid(ib,multbas_list(ik,1:multbas_sz(ik))) .LT. 0) .GT. 0) THEN
                   DO ii=1,multbas_sz(ik)
                      IF ( outflow_grid(ib,multbas_list(ik,ii)) .LT. 0 .AND. sbas .EQ. 0 ) THEN
                         sbas = multbas_list(ik,ii)
                      ELSE
                         tmp_area(ii) = fetch_basin(ib, multbas_list(ik,ii))
                      ENDIF
                   ENDDO
                   ! take the smallest of the subbasins
                   iml = MINLOC(tmp_area(1:multbas_sz(ik)), MASK = tmp_area(1:multbas_sz(ik)) .GT. zero)
                   kbas = multbas_list(ik,iml(1))
                ELSE
                   !
                   ! Else we take simply the largest and smallest
                   !
                   DO ii=1,multbas_sz(ik)
                      tmp_area(ii) = fetch_basin(ib, multbas_list(ik,ii))
                   ENDDO
                   !
                   IF ( trunc_equa ) THEN
                     iml = MINLOC(tmp_area(1:multbas_sz(ik)), MASK = tmp_area(1:multbas_sz(ik)) .GT. zero)
                     kbas = multbas_list(ik,iml(1))
                     tmp_area(iml(1)) = -1
                     iml = MINLOC(tmp_area(1:multbas_sz(ik)), MASK = tmp_area(1:multbas_sz(ik)) .GT. zero)
                     sbas = multbas_list(ik,iml(1))
                   ELSE
                     iml = MAXLOC(tmp_area(1:multbas_sz(ik)), MASK = tmp_area(1:multbas_sz(ik)) .GT. zero)
                     sbas = multbas_list(ik,iml(1))
                     iml = MINLOC(tmp_area(1:multbas_sz(ik)), MASK = tmp_area(1:multbas_sz(ik)) .GT. zero)
                     kbas = multbas_list(ik,iml(1))
                   ENDIF
                   !
                ENDIF
                !
                !
             ENDIF
          ENDIF
          !
          ! Then we call routing_reg_killbas to clean up the basins in this grid
          !
          IF ( kbas .GT. 0 .AND. sbas .GT. 0 ) THEN
             CALL routing_reg_killbas(nbpt, ib, kbas, sbas, nwbas, basin_count, basin_area, basin_topoind,&
                  & fetch_basin, basin_id, basin_coor, basin_type, basin_flowdir, outflow_grid, outflow_basin, inflow_number,&
                  & inflow_grid, inflow_basin)
          ENDIF
          !
       ENDDO
       !
       !
    ENDDO
    !
    ! We only merge a basin which flows into a basin of the same grid if it's
    ! necessary.
    !
    IF ( COUNT(basin_count .GT. nbasmax) .GT. 0 ) THEN
       !
       ! Get the points over which we wish to truncate
       !
       nbtruncate = 0
       DO ib = 1, nbpt
          IF ( basin_count(ib) .GT. nbasmax ) THEN
             nbtruncate = nbtruncate + 1
             indextrunc(nbtruncate) = ib
          ENDIF
       ENDDO
       !
       ! Go through the basins which need to be truncated.
       !
       DO ibt=1,nbtruncate
          !
          ib = indextrunc(ibt)
          !
          ! Check if we have basin which flows into a basin in the same grid
          ! kbas = basin we will have to kill
          ! sbas = basin which takes over kbas
          !
          kbas = 0
          sbas = 0
          !
          ! 4) Can we find a basin which flows into a basin of the same grid ?
          !
          DO ij=1,basin_count(ib)
             DO ii=1,basin_count(ib)
                IF ( outflow_grid(ib,ii) .EQ. ib .AND. outflow_basin(ib, ii) .EQ. ij .AND. kbas*sbas .NE. 0) THEN
                   kbas = ii
                   sbas = ij
                ENDIF
             ENDDO
          ENDDO
          !
          ! Then we call routing_reg_killbas to clean up the basins in this grid
          !
          IF ( kbas .GT. 0 .AND. sbas .GT. 0 ) THEN
             CALL routing_reg_killbas(nbpt, ib, kbas, sbas, nwbas, basin_count, basin_area, basin_topoind,&
                  & fetch_basin, basin_id, basin_coor, basin_type, basin_flowdir, outflow_grid, outflow_basin, inflow_number,&
                  & inflow_grid, inflow_basin)
          ENDIF
          !
       ENDDO
       !
    ENDIF
    !
    ! If there are any grids left with too many basins we need to take out the big hammer !
    ! We will only do it if this represents less than 5% of all points.
    !
    IF ( COUNT(basin_count .GT. nbasmax) .GT. 0 ) THEN
       !
       !
       IF ( COUNT(basin_count .GT. nbasmax)/nbpt*100 .GT. 5 ) THEN
          WRITE(numout,*) 'We have ', COUNT(basin_count .GT. nbasmax)/nbpt*100, '% of all points which do not yet'
          WRITE(numout,*) 'have the right trunctaction. That is too much to apply a brutal method'
          DO ib = 1, nbpt
             IF ( basin_count(ib) .GT. nbasmax ) THEN
                !
                WRITE(numout,*) 'We did not find a basin which could be supressed. We will'
                WRITE(numout,*) 'not be able to reduce the truncation in grid ', ib
                DO ij=1,basin_count(ib)
                   WRITE(numout,*) 'grid, basin nb and id :', ib, ij, basin_id(ib,ij)
                   WRITE(numout,*) 'Outflow grid and basin ->', outflow_grid(ib,ij), outflow_basin(ib, ij)
                ENDDO
             ENDIF
          ENDDO
          CALL ipslerr_p(3,'routing_reg_truncate','No basin found which could be supressed.','','')
       ELSE
          !
          !
          DO ib = 1,nbpt
             DO WHILE ( basin_count(ib) .GT. nbasmax )
                !
                WRITE(numout,*) 'HAMMER, ib, basin_count :', ib, basin_count(ib)
                !
                ! Here we simply put the smallest basins into the largest ones. It is really a brute force
                ! method but it will only be applied if everything has failed.
                !
                DO ii = 1,basin_count(ib)
                   tmp_area(ii) = fetch_basin(ib, ii)
                ENDDO
                !
                IF ( trunc_equa ) THEN
                  iml = MINLOC(tmp_area(1:basin_count(ib)), MASK = tmp_area(1:basin_count(ib)) .GT. 0.)
                  kbas = iml(1)
                  tmp_area(iml(1)) = -1
                  iml = MINLOC(tmp_area(1:basin_count(ib)), MASK = tmp_area(1:basin_count(ib)) .GT. 0.)
                  sbas =iml(1)
                ELSE
                  iml = MAXLOC(tmp_area(1:basin_count(ib)), MASK = tmp_area(1:basin_count(ib)) .GT. 0.)
                  sbas =iml(1)
                  iml = MINLOC(tmp_area(1:basin_count(ib)), MASK = tmp_area(1:basin_count(ib)) .GT. 0.)
                  kbas = iml(1)
                ENDIF
                !
                IF ( kbas .GT. 0 .AND. sbas .GT. 0 ) THEN
                   CALL routing_reg_killbas(nbpt, ib, kbas, sbas, nwbas, basin_count, basin_area, basin_topoind,&
                        & fetch_basin, basin_id, basin_coor, basin_type, basin_flowdir, outflow_grid, outflow_basin, inflow_number,&
                        & inflow_grid, inflow_basin)
                ENDIF
             ENDDO
          ENDDO
          !
       ENDIF
       !
       !
    ENDIF
    !
    ! Now that we have reached the right truncation (resolution) we will start
    ! to produce the variables we will use to route the water.
    !
    DO ib=1,nbpt
       !
       ! For non existing basins the route_tobasin variable is put to zero. This will allow us
       ! to pick up the number of basin afterwards.
       !
       route_togrid_glo(ib,:) = ib
       route_tobasin_glo(ib,:) = 0
       routing_area_glo(ib,:) = zero
       route_nbintobas_glo(ib) = basin_count(ib)
       origin_nbintobas_glo(ib) = basin_notrun(ib)
       !
    ENDDO
    !
    ! Transfer the info into the definitive variables
    !
    DO ib=1,nbpt
       DO ij=1,basin_count(ib)
          !
          routing_area_glo(ib,ij) = basin_area(ib,ij)
          topo_resid_glo(ib,ij) = basin_topoind(ib,ij)
          global_basinid_glo(ib,ij) = basin_id(ib,ij)
          route_outlet_glo(ib,ij,1) = basin_coor(ib,ij,1)
          route_outlet_glo(ib,ij,2) = basin_coor(ib,ij,2)
          !
          route_type_glo(ib,ij) = basin_type(ib,ij)
          !
          route_togrid_glo(ib,ij) = outflow_grid(ib,ij)
          route_tobasin_glo(ib,ij) = outflow_basin(ib,ij)
          !
       ENDDO
    ENDDO
    !
    !
    ! Set the new convention for the outflow conditions
    ! Now it is based in the outflow basin and the outflow grid will
    ! be the same as the current.
    ! returnflow to the grid : nbasmax + 1
    ! coastal flow : nbasmax + 2
    ! river outflow : nbasmax + 3
    !
    ! Here we put everything here in coastal flow. It is later where we will
    ! put the largest basins into river outflow.
    !
    DO ib=1,nbpt
       DO ij=1,basin_count(ib)
          ! River flows
          IF ( route_togrid_glo(ib,ij) .EQ. -1 ) THEN
             route_tobasin_glo(ib,ij) = nbasmax + 2
             route_togrid_glo(ib,ij) = ib
          ! Coastal flows
          ELSE IF ( route_togrid_glo(ib,ij) .EQ. -2 ) THEN
             route_tobasin_glo(ib,ij) = nbasmax + 2
             route_togrid_glo(ib,ij) = ib
          ! Return flow
          ELSE IF ( route_togrid_glo(ib,ij) .EQ. -3 ) THEN
             route_tobasin_glo(ib,ij) = nbasmax + 1
             route_togrid_glo(ib,ij) = ib
          ENDIF
       ENDDO
    ENDDO
    !
    ! A second check on the data. Just make sure that each basin flows somewhere.
    !
    DO ib=1,nbpt
       DO ij=1,basin_count(ib)
          ibf = route_togrid_glo(ib,ij)
          ijf = route_tobasin_glo(ib,ij)
          IF ( ijf .GT. basin_count(ibf) .AND. ijf .LE. nbasmax) THEN
             WRITE(numout,*) 'Second check'
             WRITE(numout,*) 'point :', ib, ' basin :', ij
             WRITE(numout,*) 'Flows into point :', ibf, ' basin :', ijf
             WRITE(numout,*) 'But it flows into now here as basin count = ', basin_count(ibf)
             CALL ipslerr_p(3,'routing_reg_truncate','Problem with routing..','','')
          ENDIF
       ENDDO
    ENDDO
    !
    ! Verify areas of the continents
    !
    floflo(:,:) = zero
    gridarea(:) = contfrac(:)*resolution(:,1)*resolution(:,2)
    DO ib=1,nbpt
       gridbasinarea(ib) = SUM(routing_area_glo(ib,:))
    ENDDO
    !
    DO ib=1,nbpt
       DO ij=1,basin_count(ib)
          cnt = 0
          igrif = ib
          ibasf = ij
          DO WHILE (ibasf .LE. nbasmax .AND. cnt .LT. nbasmax*nbpt)
             cnt = cnt + 1
             pold = igrif
             bold = ibasf
             igrif = route_togrid_glo(pold, bold)
             ibasf = route_tobasin_glo(pold, bold)
             IF ( ibasf .GT. basin_count(igrif) .AND. ibasf .LE. nbasmax) THEN
                WRITE(numout,*) 'We should not be here as the basin flows into the pampa'
                WRITE(numout,*) 'Last correct point :', pold, bold
                WRITE(numout,*) 'It pointed to in the new variables :', route_togrid_glo(pold, bold),route_tobasin_glo(pold, bold)
                WRITE(numout,*) 'The old variables gave :', outflow_grid(pold, bold), outflow_basin(pold, bold)
                WRITE(numout,*) 'Where we ended up :', igrif,ibasf
                CALL ipslerr_p(3,'routing_reg_truncate','Problem with routing..','','')
             ENDIF
          ENDDO
          !
          IF ( ibasf .GT. nbasmax ) THEN
             floflo(igrif,bold) = floflo(igrif,bold) + routing_area_glo(ib,ij)
          ELSE
             WRITE(numout,*) 'The flow did not end up in the ocean or in the grid cell.'
             WRITE(numout,*) 'For grid ', ib, ' and basin ', ij
             WRITE(numout,*) 'The last grid was ', igrif, ' and basin ', ibasf
             CALL ipslerr_p(3,'routing_reg_truncate','Problem with routing..','','')
          ENDIF
       ENDDO
    ENDDO
    !
    DO ib=1,nbpt
       IF ( gridbasinarea(ib) > zero ) THEN
          ratio = gridarea(ib)/gridbasinarea(ib)
          routing_area_glo(ib,:) = routing_area_glo(ib,:)*ratio
       ELSE
          WRITE(numout,*) 'gridbasinarea(ib) <= zero. We should stop here :', ib
       ENDIF
    ENDDO
    !
    WRITE(numout,*) 'Sum of area of all outflow areas :',SUM(routing_area_glo)
    WRITE(numout,*) 'Surface of all continents :', SUM(gridarea)
    !
    ! Redo the the distinction between river outflow and coastal flow. We can not
    ! take into account the return flow points.
    !
    ibf = 0
    DO ib=1, pickmax
       ff = MAXLOC(floflo)
       ! tdo - To take into account rivers that do not flow to the oceans
       IF ( route_tobasin_glo(ff(1), ff(2)) .GT. nbasmax ) THEN
          ibf = ibf + 1
          largest_basins(ibf,:) = ff(:)
       ENDIF
       floflo(ff(1), ff(2)) = zero
    ENDDO
    !
    ! Put the largest basins into river flows.
    !
    IF ( ibf .LT. num_largest) THEN
       WRITE(numout,*) 'Not enough basins to choose the ', num_largest, 'largest'
       CALL ipslerr_p(3,'routing_reg_truncate','Not enough basins','','')
    ENDIF
    !
    !
    !
    DO ib=1, num_largest
       route_tobasin_glo(largest_basins(ib,1),largest_basins(ib,2)) = nbasmax + 3
    ENDDO
    !
    WRITE(numout,*) 'NUMBER OF RIVERS :', COUNT(route_tobasin_glo .GE. nbasmax + 3)
    !
  END SUBROUTINE routing_reg_truncate
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_killbas
!!
!>\BRIEF The aim of this subroutine is to kill a basin (that is put into another larger one).
!! When we do this we need to be careful and change all associated variables.
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_killbas(nbpt, ib, tokill, totakeover, nwbas, basin_count, basin_area, basin_topoind,&
       & fetch_basin, basin_id, basin_coor, basin_type, basin_flowdir, outflow_grid, outflow_basin, inflow_number,&
       & inflow_grid, inflow_basin)
    !
    !
    IMPLICIT NONE
    !
    INTEGER(i_std) :: tokill !!
    INTEGER(i_std) :: totakeover !!
    INTEGER(i_std) :: nbpt !! Domain size (unitless)
    INTEGER(i_std) :: ib !! Current basin (unitless)
    !
    INTEGER(i_std) :: nwbas !!
    INTEGER(i_std), DIMENSION(nbpt) :: basin_count !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: basin_id !!
    REAL(r_std), DIMENSION(nbpt,nwbas,2) :: basin_coor !!
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_type !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: basin_flowdir !! Water flow directions in the basin (unitless)
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_area !!
    REAL(r_std), DIMENSION(nbpt,nwbas) :: basin_topoind !! Topographic index of the residence time for a basin (m)
    REAL(r_std), DIMENSION(nbpt,nwbas) :: fetch_basin !!
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: outflow_grid !! Type of outflow on the grid box (unitless)
    INTEGER(i_std), DIMENSION(nbpt,nwbas) :: outflow_basin !!
    !
    ! Saving memory: changed nwbas to nbxmax*4 and *8
    !
    INTEGER(i_std), DIMENSION(nbpt,nbxmax*8) :: inflow_number !!
    INTEGER(i_std), DIMENSION(nbpt,nbxmax*8,nbxmax*8) :: inflow_basin !!
    INTEGER(i_std), DIMENSION(nbpt,nbxmax*8,nbxmax*8) :: inflow_grid !!
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: inf, ibs, ing, inb, ibasf, igrif, it !! Indices (unitless)
    LOGICAL :: doshift !! (true/false)

!_ ================================================================================================================================
    !
    ! Update the information needed in the basin "totakeover"
    ! For the moment only area
    !
    !
    basin_area(ib, totakeover) = basin_area(ib, totakeover) + basin_area(ib, tokill)
    basin_topoind(ib, totakeover) = (basin_topoind(ib, totakeover) + basin_topoind(ib, tokill))/2.0
    !
    ! Add the fetch of the basin will kill to the one which gets the water
    !
    fetch_basin(ib, totakeover) = fetch_basin(ib, totakeover) + fetch_basin(ib, tokill)
    igrif = outflow_grid(ib,totakeover)
    ibasf = outflow_basin(ib,totakeover)
    !
    inf = 0
    DO WHILE (igrif .GT. 0)
       fetch_basin(igrif,ibasf) = fetch_basin(igrif,ibasf) + fetch_basin(ib, tokill)
       it = outflow_grid(igrif, ibasf)
       ibasf = outflow_basin(igrif, ibasf)
       igrif = it
       inf = inf + 1
    ENDDO
    !
    ! Take out the basin we have just rerouted from the fetch of the basins in which it used to flow.
    !
    igrif = outflow_grid(ib,tokill)
    ibasf = outflow_basin(ib,tokill)
    !
    DO WHILE (igrif .GT. 0)
       fetch_basin(igrif,ibasf) = fetch_basin(igrif,ibasf) - fetch_basin(ib, tokill)
       it = outflow_grid(igrif, ibasf)
       ibasf = outflow_basin(igrif, ibasf)
       igrif = it
    ENDDO
    !
    ! Redirect the flows which went into the basin to be killed before we change everything
    !
    DO inf = 1, inflow_number(ib, tokill)
       outflow_basin(inflow_grid(ib, tokill, inf), inflow_basin(ib, tokill, inf)) = totakeover
       inflow_number(ib, totakeover) = inflow_number(ib, totakeover) + 1
       inflow_grid(ib, totakeover, inflow_number(ib, totakeover)) = inflow_grid(ib, tokill, inf)
       inflow_basin(ib, totakeover, inflow_number(ib, totakeover)) = inflow_basin(ib, tokill, inf)
    ENDDO
    !
    ! Take out the basin to be killed from the list of inflow basins of the downstream basin
    ! (In case the basin does not flow into an ocean or lake)
    !
    IF ( outflow_grid(ib,tokill) .GT. 0) THEN
       !
       ing = outflow_grid(ib, tokill)
       inb = outflow_basin(ib, tokill)
       doshift = .FALSE.
       !
       DO inf = 1, inflow_number(ing, inb)
          IF ( doshift ) THEN
             inflow_grid(ing, inb, inf-1) = inflow_grid(ing, inb, inf)
             inflow_basin(ing, inb, inf-1) = inflow_basin(ing, inb, inf)
          ENDIF
          IF ( inflow_grid(ing, inb, inf) .EQ. ib .AND. inflow_basin(ing, inb, inf) .EQ. tokill) THEN
             doshift = .TRUE.
          ENDIF
       ENDDO
       !
       ! This is only to allow for the last check
       !
       inf = inflow_number(ing, inb)
       IF ( inflow_grid(ing, inb, inf) .EQ. ib .AND. inflow_basin(ing, inb, inf) .EQ. tokill) THEN
          doshift = .TRUE.
       ENDIF
       !
       IF ( .NOT. doshift ) THEN
          WRITE(numout,*) 'Strange we did not find the basin to kill in the downstream basin'
          CALL ipslerr_p(3,'routing_reg_killbas','Basin not found','','')
       ENDIF
       inflow_number(ing, inb) = inflow_number(ing, inb) - 1

    ENDIF
    !
    ! Now remove from the arrays the information of basin "tokill"
    !
    basin_id(ib, tokill:basin_count(ib)-1) = basin_id(ib, tokill+1:basin_count(ib))
    basin_coor(ib, tokill:basin_count(ib)-1,1) = basin_coor(ib, tokill+1:basin_count(ib),1)
    basin_coor(ib, tokill:basin_count(ib)-1,2) = basin_coor(ib, tokill+1:basin_count(ib),2)
    basin_type(ib, tokill:basin_count(ib)-1) = basin_type(ib, tokill+1:basin_count(ib))
    basin_flowdir(ib, tokill:basin_count(ib)-1) = basin_flowdir(ib, tokill+1:basin_count(ib))
    basin_area(ib, tokill:basin_count(ib)-1) = basin_area(ib, tokill+1:basin_count(ib))
    basin_area(ib, basin_count(ib):nwbas) = zero
    basin_topoind(ib, tokill:basin_count(ib)-1) = basin_topoind(ib, tokill+1:basin_count(ib))
    basin_topoind(ib, basin_count(ib):nwbas) = zero
    fetch_basin(ib, tokill:basin_count(ib)-1) = fetch_basin(ib, tokill+1:basin_count(ib))
    fetch_basin(ib, basin_count(ib):nwbas) = zero
    !
    ! Before we remove the information from the outflow fields we have to correct the corresponding inflow fields
    ! of the grids into which the flow goes
    !
    DO ibs = tokill+1,basin_count(ib)
       ing = outflow_grid(ib, ibs)
       inb = outflow_basin(ib, ibs)
       IF ( ing .GT. 0 ) THEN
          DO inf = 1, inflow_number(ing, inb)
             IF ( inflow_grid(ing,inb,inf) .EQ. ib .AND. inflow_basin(ing,inb,inf) .EQ. ibs) THEN
                inflow_basin(ing,inb,inf) = ibs - 1
             ENDIF
          ENDDO
       ENDIF
    ENDDO
    outflow_grid(ib, tokill:basin_count(ib)-1) = outflow_grid(ib, tokill+1:basin_count(ib))
    outflow_basin(ib, tokill:basin_count(ib)-1) = outflow_basin(ib, tokill+1:basin_count(ib))
    !
    ! Basins which moved down also need to redirect their incoming flows.
    !
    DO ibs=tokill+1, basin_count(ib)
       DO inf = 1, inflow_number(ib, ibs)
          outflow_basin(inflow_grid(ib, ibs, inf), inflow_basin(ib, ibs, inf)) = ibs-1
       ENDDO
    ENDDO
    !
    ! Shift the inflow basins
    !
    DO it = tokill+1,basin_count(ib)
       inflow_grid(ib, it-1, 1:inflow_number(ib,it)) = inflow_grid(ib, it, 1:inflow_number(ib,it))
       inflow_basin(ib, it-1, 1:inflow_number(ib,it)) = inflow_basin(ib, it, 1:inflow_number(ib,it))
       inflow_number(ib,it-1) = inflow_number(ib,it)
    ENDDO
    !
    basin_count(ib) = basin_count(ib) - 1
    !
  END SUBROUTINE routing_reg_killbas
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_names
!!
!>\BRIEF This subroutine lists the name of the largest basins which are explicitly listed in the basin
!! description file used by ORCHIDEE.
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_names(numlar, basin_names)
    !
    IMPLICIT NONE
    !
    ! Arguments
    !
    INTEGER(i_std), INTENT(in) :: numlar !!
    CHARACTER(LEN=*), INTENT(inout) :: basin_names(numlar) !! Name of the basins (unitless)
!! PARAMETERS
    INTEGER(i_std), PARAMETER :: listleng=6530 !!
    !
!! LOCAL VARIABLES
    INTEGER(i_std) :: lenstr, i !!
    CHARACTER(LEN=60), DIMENSION(listleng) :: list_names !!
    CHARACTER(LEN=60) :: tmp_str !!

!_ ================================================================================================================================
    !

    !
    ! Update (12 May 2016 ):
    ! The basin description file used by ORCHIDEE (so call routing.nc) now used
    ! information from HydroSHEDS data [1]. It only contains IDs of basins
    ! without river name. So we need to compare these IDs with river name from
    ! GRDC [2] data to get the corrected names. Until now, I only corrected 
    ! some rivers in the Mediterranean region.
    ! Of course, because of complexity when detecting name of river 
    ! based on observation station location, there are maybe some mistakes.
    ! I'll finish this correction as soon as possible.
    ! There is 7000 IDs but maximum IDs which has river name is 6530 
    ! and only 819 river names are available.
    ! Uncorrected river name is marked by '(uncor)' or separated by '-'
    ! 
    ! Refs:
    ! 1. http://hydrosheds.cr.usgs.gov/index.php
    ! 2. /bdd/ORCHIDEE_Forcing/Validation/GRDC/GRDC_Monthly_June14.nc
    !
    ! Some rivers belong to old namelist which I can not find out:
    ! "Chari"
    ! "Erg Iguidi (Sahara)"
    ! "Great Artesian Basin"
    ! "Shatt el Arab"
    ! "Yukon"
    ! "Chott Jerid"
    ! "Jubba"
    ! "Wadi al Farigh"
    ! "Kolyma"
    ! "Neva"
    ! "Taymyr"
    ! "Pyasina"
    ! "Peel"
    ! "Penzhina"
    ! "Barito"
    ! "Colville"
    ! "Glama"
    ! .......

    lenstr = LEN(basin_names(1))
    !
    ! Get the temporary name for all rivers
    !
    DO i = 1, listleng
       WRITE (list_names(i),'(A5,I4.4)') 'GRDC_',i
    ENDDO
    !
    ! The river name which contains "(uncor)" or '-' should be rechecked.
    !
    list_names(1) = "Amazon"
    list_names(2) = "Congo"
    list_names(3) = "Mississippi"
    list_names(4) = "Nile"
    list_names(5) = "Parana"
    list_names(6) = "Niger"
    list_names(7) = "Amur"
    list_names(8) = "Chang Jiang (Yangtze)"
    list_names(9) = "Ganges"
    list_names(10) = "Yenisei"
    list_names(11) = "Volga"
    list_names(12) = "Zambezi"
    list_names(13) = "St Lawrence"
    list_names(14) = "Nelson"
    list_names(15) = "Komadugu Yobe (Tchad)"
    list_names(16) = "Orinoco"
    list_names(17) = "Tarim"
    list_names(18) = "Orange"
    list_names(19) = "Indus"
    list_names(20) = "Karasu"
    list_names(21) = "Danube"
    list_names(22) = "Murray"
    list_names(23) = "Mekong"
    list_names(24) = "Araguaia"
    list_names(25) = "Huang He (Yellow)"
    list_names(26) = "Wabi Shebelle (uncor)"
    list_names(27) = "Georgina"
    list_names(28) = "Ob"
    list_names(29) = "Columbia"
    list_names(30) = "Cacuchi"
    list_names(31) = "Sao Francisco"
    list_names(32) = "Colorado (Texas)"
    list_names(33) = "Amu Darya"
    list_names(34) = "Irtysh"
    list_names(35) = "Mackenzie"
    list_names(36) = "Dnepr (uncor)"
    list_names(37) = "Rio Grande (US)"
    list_names(39) = "Lena"
    list_names(40) = "Senegal"
    list_names(41) = "Don"
    list_names(42) = "Volta"
    list_names(43) = "Limpopo"
    list_names(44) = "Karatal Sharyn (uncor)"
    list_names(45) = "Zhujiang"
    list_names(46) = "Irrawaddy"
    list_names(47) = "Balsas"
    list_names(48) = "Syr Darya Naryn (uncor)"
    list_names(50) = "Godavari"
    list_names(52) = "Churchill (Hud)"
    list_names(53) = "Tobol"
    list_names(54) = "Colorado (Arg)"
    list_names(55) = "KASHAF RUD (uncor)"
    list_names(56) = "Uruguay"
    list_names(57) = "Slaween (Nujiang)"
    list_names(58) = "Magdalena"
    list_names(59) = "BOLSHOY NIMNIR (uncor)"
    list_names(60) = "Krishna"
    list_names(62) = "Fraser"
    list_names(63) = "LEBIYOU (uncor)"
    list_names(65) = "Hai"
    list_names(66) = "Bolshoy (IK)"
    list_names(69) = "Bug (uncor)"
    list_names(70) = "Liao"
    list_names(71) = "Kura"
    list_names(72) = "Rufiji"
    list_names(74) = "Rhine"
    list_names(76) = "MBAGALA ROVUMA"
    list_names(77) = "Essequibo"
    list_names(79) = "Wabi (Ghibe)"
    list_names(80) = "BELEN ABAUCAN PICHANAS CONLARA"
    list_names(81) = "KOBDO SAGSAIN GOL"
    list_names(82) = "DESAGUADERO"
    list_names(83) = "Chao Phraya"
    list_names(84) = "Kara (Turgay)"
    list_names(85) = "Fitzroy Dawson"
    list_names(87) = "Hong"
    list_names(88) = "Elbe"
    list_names(89) = "Mahanadi (Mahahadi)"
    list_names(91) = "Aux Melezes Caniapiscau"
    list_names(92) = "Albany"
    list_names(93) = "Sanaga"
    list_names(95) = "Sali Dulce Primero"
    list_names(96) = "BURDEKIN BOGIE BROKEN MISTAKE CREEK"
    list_names(97) = "Sacramento"
    list_names(98) = "Grijalva"
    list_names(99) = "San Juan Capistrano Rio Santiago"
    list_names(100) = "Olekma Nyukzha"
    list_names(101) = "Neisse (uncor)"
    list_names(102) = "Loire"
    list_names(104) = "Negro Arg"
    list_names(107) = "Alabama"
    list_names(108) = "PLACERES DEL ORO"
    list_names(110) = "Brazos (Tex)"
    list_names(111) = "KUNENE CUANDO COLUI"
    list_names(112) = "Flinders"
    list_names(113) = "Hudson"
    list_names(114) = "Hayes"
    list_names(115) = "Colorado (Ari)"
    list_names(119) = "Runde"
    list_names(120) = "Severn (Can)"
    list_names(121) = "Victoria"
    list_names(123) = "Bandama"
    list_names(124) = "Guir"
    list_names(125) = "Esla (uncor)"
    list_names(126) = "Grande Pontois"
    list_names(127) = "NARMADA"
    list_names(128) = "Rhone"
    list_names(129) = "FITZROY LEOPOLD"
    list_names(130) = "Minija (uncor)"
    list_names(131) = "Awash (uncor)"
    list_names(132) = "TANA"
    list_names(133) = "CHUBUT GUALJAINA SENGUERR"
    list_names(137) = "CHURCHILL ASHUANIPI"
    list_names(142) = "PERIBONCA"
    list_names(145) = "CHARA TYANYA ZHUYA"
    list_names(146) = "NURA CHURUBAY"
    list_names(147) = "SALADO"
    list_names(148) = "PANUCO LOS HULES CALABOZO"
    list_names(149) = "Daugava Obsha"
    list_names(150) = "Qom"
    list_names(151) = "Ebro"
    list_names(154) = "Comoe (uncor)"
    list_names(155) = "DOCE MANHUACU PIRACICABA"
    list_names(158) = "SEPIK"
    list_names(159) = "NDEMBERA SONGWE MTEMBWA"
    list_names(163) = "CAUVERY"
    list_names(164) = "ROPER"
    list_names(165) = "ASHBURTON"
    list_names(167) = "Kizilirmak (Kanak Cayi)"
    list_names(168) = "Sien (uncor)"
    list_names(169) = "Chu (Sho)"
    list_names(171) = "Sandougou (uncor)"
    list_names(173) = "GASCOYNE"
    list_names(174) = "ARACATIACU BANABUIU JAGUARIBE"
    list_names(175) = "Po"
    list_names(176) = "Dniestr"
    list_names(177) = "Zezdy"
    list_names(178) = "Seine"
    list_names(179) = "YAQUI"
    list_names(181) = "CARREIRO VACACAI"
    list_names(185) = "JEQUITINHONHA ARACUAI"
    list_names(187) = "SUSQUEHANNA CHENANGO CHEMUNG"
    list_names(188) = "CARAGUATA TACUAREMBO NEGRO YI"
    list_names(189) = "Tejo (Jarama)"
    list_names(191) = "Halil"
    list_names(194) = "Erzin"
    list_names(195) = "Guadiana Jabalon"
    list_names(196) = "NOTTAWAY WASWANIPI"
    list_names(197) = "MARONI OELEMARI TAMPOC"
    list_names(198) = "MURCHISON"
    list_names(199) = "WINISK SHAMATTAWA ASHEWEIG"
    list_names(203) = "TAPTI"
    list_names(205) = "KABALEBO"
    list_names(208) = "MITCHELL"
    list_names(209) = "Southern Bug (Inhul)"
    list_names(212) = "Sakarya"
    list_names(214) = "YALU SANGWAN GANG"
    list_names(218) = "FUTUN XI (Minjiang)"
    list_names(221) = "Bouenza (uncor)"
    list_names(222) = "PAROO"
    list_names(225) = "Huai (part)"
    list_names(226) = "Shah Rud (uncor)"
    list_names(227) = "Okpara (uncor)"
    list_names(229) = "POMBA PARAIBA DO SUL PAREIBUNA"
    list_names(231) = "Guadalquivir"
    list_names(232) = "PIRATINI"
    list_names(233) = "Narva Jogi (uncor)"
    list_names(234) = "BETSIBOKA IKOPA"
    list_names(235) = "MEARIM CORDA GRAJAU"
    list_names(236) = "Wadi Ghadun"
    list_names(237) = "Garonne"
    list_names(238) = "DE CONTAS GAVIAO"
    list_names(239) = "JACUIPE PARAGUACU SANTO ANTONIO"
    list_names(240) = "MANANANTANANA MATSIATRA MANGOKY IHOSY"
    list_names(241) = "MADAWASKA SAINT JOHN"
    list_names(242) = "PENNER"
    list_names(244) = "DALY DRY"
    list_names(245) = "Moulouya"
    list_names(247) = "NECHES SABINE"
    list_names(248) = "DE GREY"
    list_names(249) = "HUMBOLDT CARSON (CALIF)"
    list_names(253) = "KAMCHATKA"
    list_names(254) = "PACAJA"
    list_names(255) = "Cross"
    list_names(256) = "ITAPECURU CODOZINHO CORRENTES"
    list_names(257) = "Ord"
    list_names(260) = "Kuban"
    list_names(261) = "Maritsa"
    list_names(263) = "Zarrineh Rud"
    list_names(265) = "SKEENA BABINE"
    list_names(269) = "CHATTAHOOCHEE FLINT APALACHICOLA"
    list_names(270) = "DEASE STIKINE ISKUT"
    list_names(271) = "SEAL NORTH SEAL SOUTH SEAL"
    list_names(273) = "ATTAWAPISKAT PINEIMUTA"
    list_names(275) = "Zia (Dades)"
    list_names(276) = "Capim"
    list_names(277) = "Norman"
    list_names(281) = "MAHAJILO TSIRIBIHINA MANIA"
    list_names(282) = "FORTESCUE MARILLANA CREEK"
    list_names(287) = "SANDRIVIER INCOMATI SABIE"
    list_names(289) = "DORING"
    list_names(290) = "DEL ORO RAMOS"
    list_names(291) = "YADKIN ROCKY"
    list_names(292) = "EASTMAIN"
    list_names(293) = "ELM FORK TRINITY (TEXAS)"
    list_names(295) = "Luan He"
    list_names(296) = "MANICOUAGAN (RIVIERE)"
    list_names(299) = "Rajang"
    list_names(301) = "OLIFANTSRIVIER TOUWSRIVIER"
    list_names(303) = "Weser (uncor)"
    list_names(304) = "Chelif (uncor)"
    list_names(305) = "Talas"
    list_names(306) = "Nueces"
    list_names(308) = "Gregory"
    list_names(309) = "Fuchu jiang"
    list_names(311) = "Jordan (uncor)"
    list_names(313) = "Terek (Tergi)"
    list_names(314) = "Kuma (Trib Caspian sea)"
    list_names(316) = "AMAPARI ARAGUARI"
    list_names(319) = "Rupert"
    list_names(320) = "FEUILLES (RIVIERE AUX) GOUDALIE"
    list_names(321) = "Zayandeh"
    list_names(323) = "VIEJO SAN JUAN (NICARAGUA/COSTA RICA)"
    list_names(324) = "BALEINE, GRANDE RIVIERE DE LA DENYS, RIVIERE"
    list_names(325) = "PIRANHASACU PIANCO"
    list_names(326) = "George"
    list_names(328) = "Mkomazi (Pangani)"
    list_names(329) = "GILBERT"
    list_names(331) = "PINDARE ZUTIUA"
    list_names(333) = "Shur (Jahrom)"
    list_names(334) = "CATAWBA BROAD"
    list_names(336) = "PAPALOAPAN SAN JUAN (MEXICO)"
    list_names(341) = "Sebou (uncor)"
    list_names(343) = "DAMODAR"
    list_names(344) = "ALTAMAHA"
    list_names(351) = "Yesilirmak"
    list_names(352) = "ITAPICURU ITAPICURUMIRIM"
    list_names(354) = "TURGEON (RIVIERE) HARRICANAW"
    list_names(355) = "RIOSUCIO ATRATO QUITO"
    list_names(357) = "Oum er Rbia (El Abid)"
    list_names(361) = "GURUPI"
    list_names(362) = "HUDSON SACANDAGA WALLKILL"
    list_names(363) = "CHINIPAS FUERTE"
    list_names(364) = "SOUTRIVIER (ZAWESTERN CAPE) HEUNINGKLIPRIVIER"
    list_names(365) = "SEVIER"
    list_names(367) = "MAHI"
    list_names(368) = "CHAUNG"
    list_names(372) = "DEL CARMEN"
    list_names(373) = "DONG JIANG"
    list_names(374) = "BURNETT CADARGA CREEK BARKER CREEK"
    list_names(379) = "WILLOCHRA CREEK"
    list_names(381) = "MEUSE NIERS OURTHE CHIERS"
    list_names(382) = "QUEVEDO ZAPOTAL VINCES DAULE"
    list_names(384) = "LEICHHARDT"
    list_names(388) = "PURARI"
    list_names(389) = "BALEINE, GRANDE RIVIERE DE LA GEORGE"
    list_names(390) = "CATOLE GRANDE PRADO"
    list_names(395) = "PUNGOE"
    list_names(396) = "WILLIAMSON SPRAGUE KLAMATH"
    list_names(397) = "Ntem"
    list_names(400) = "UMPULUZI, UMPILUSI USUSHWANA MAPUTO, USUTU"
    list_names(401) = "POTOMAC MONOCACY SHENANDOAH"
    list_names(403) = "TEEBUSRIVIER GROOTVIS VLEKPOORTRIVIER"
    list_names(405) = "SABARMATI"
    list_names(410) = "Cavally (uncor)"
    list_names(414) = "KHAN SWAKOP"
    list_names(415) = "UGAB"
    list_names(420) = "BUFFELSRIVIER TUGELA MOOIRIVIER"
    list_names(423) = "CONNECTICUT PASSUMPSIC CHICOPEE"
    list_names(424) = "JELAI LIPIS PAHANG"
    list_names(428) = "MAE NAM KHWAE NOI QUAE YAI MAE KLONG"
    list_names(429) = "SOFIA BEMARIVO"
    list_names(430) = "DELAWARE SCHUYLKILL"
    list_names(431) = "Buzi"
    list_names(433) = "Tano"
    list_names(439) = "Nyong"
    list_names(441) = "PINACANAUAN CAGAYAN MAGAT MARANG"
    list_names(442) = "TARRA ZULIA"
    list_names(446) = "BROAD SAVANNAH"
    list_names(448) = "SANTA CRUZ"
    list_names(451) = "BAKER"
    list_names(452) = "GUADALUPE SAN MARCOS SANDIES CREEK"
    list_names(454) = "SUWANNEE SANTA FE"
    list_names(457) = "OYAPOCK"
    list_names(465) = "SAN PEDRO"
    list_names(470) = "KARGAT CHULYM SUMA"
    list_names(472) = "STAATEN"
    list_names(473) = "ROANOKE"
    list_names(475) = "HAN GANG"
    list_names(476) = "Nyanga"
    list_names(479) = "JUQUIA RIBEIRA DO IGUAPE"
    list_names(481) = "Messalo"
    list_names(482) = "ST JOHNS OCKLAWAHA"
    list_names(483) = "Bubu (uncor)"
    list_names(485) = "COCO"
    list_names(486) = "KENNEBEC ANDROSCOGGIN"
    list_names(487) = "Vardar"
    list_names(490) = "Corubal"
    list_names(491) = "Buyuk Menderes"
    list_names(494) = "BIOBIO"
    list_names(499) = "Luzege (Vezere)"
    list_names(501) = "Asi Nehri"
    list_names(502) = "KOCASU ORHANELI CAYI"
    list_names(503) = "GUAYAPE"
    list_names(504) = "SILETI KEDEY"
    list_names(507) = "DALINGHE"
    list_names(509) = "NAKTONG"
    list_names(511) = "PRA"
    list_names(512) = "Mejerda (Mellegue)"
    list_names(513) = "Mono"
    list_names(516) = "CAPE FEAR"
    list_names(521) = "Blackwood"
    list_names(522) = "Tutoh"
    list_names(525) = "Patia"
    list_names(529) = "CLARENCE ORARA NYMBOIDA"
    list_names(530) = "Coruhsuyu"
    list_names(531) = "MAE NAM PRACHIN BURI"
    list_names(532) = "HUMUYA AGUA CALIENTE"
    list_names(534) = "PENOBSCOT"
    list_names(535) = "PEARL BOGUE CHITTO"
    list_names(536) = "CHIKASAWHAY LEAF PASCAGOULA"
    list_names(540) = "ESMERALDAS TOACHI"
    list_names(544) = "VOELRIVIER"
    list_names(545) = "JUCAR"
    list_names(547) = "GOULBURN HUNTER"
    list_names(548) = "COATZACOALCOS"
    list_names(549) = "COPPENAME"
    list_names(550) = "AMARGOSA"
    list_names(551) = "CEYHAN NEHRI"
    list_names(554) = "KOR"
    list_names(557) = "PURIFICACION CORONA"
    list_names(560) = "GOEKSUN SEYHAN"
    list_names(562) = "Clutha"
    list_names(565) = "KLAAS SMITSRIVIER SWARTKEIRIVIER"
    list_names(566) = "BROADBACK"
    list_names(568) = "SUNGAI KAMPARKIRI"
    list_names(570) = "DIANI ST PAUL"
    list_names(571) = "CAPERTEE NEPEAN"
    list_names(579) = "MZINTLAVA MZIMVUBU TINARIVIER"
    list_names(580) = "NASS"
    list_names(583) = "PARAIBA TAPEROA"
    list_names(584) = "SHIDERTI"
    list_names(585) = "MAKONA MOA"
    list_names(586) = "SUBARNAREKHA"
    list_names(588) = "LITTLE MECATINA"
    list_names(589) = "TADONG BIRYUGANG NAMGANG"
    list_names(591) = "KARASUK BURLA"
    list_names(594) = "LICUARE"
    list_names(597) = "MOISIE (RIVIERE)"
    list_names(599) = "SCHELDE"
    list_names(600) = "OUTARDES (RIVIERE AUX)"
    list_names(607) = "Seli"
    list_names(614) = "TUMA GRANDE DE MATAGALPA"
    list_names(616) = "VERDE (ATOYAK)"
    list_names(617) = "James"
    list_names(620) = "Lempa"
    list_names(621) = "MACARTHUR"
    list_names(626) = "Chira"
    list_names(629) = "Ruvu"
    list_names(630) = "TOCUYO"
    list_names(631) = "MANGORO ONIVE"
    list_names(634) = "BARIMA"
    list_names(640) = "LIELUPE MUSA"
    list_names(643) = "TAKU"
    list_names(646) = "DRIN DRINIT TE ZI"
    list_names(652) = "KISSIMMEE"
    list_names(653) = "MANANARA"
    list_names(663) = "KUISEB GAUB"
    list_names(664) = "CAMAQUA"
    list_names(665) = "GAVE DE PAU GAVE DOLORON"
    list_names(666) = "STRUMA"
    list_names(668) = "ABAHUAB"
    list_names(671) = "GEDIZ"
    list_names(672) = "KONKOURE"
    list_names(675) = "SIL MINHO"
    list_names(676) = "TIBER"
    list_names(682) = "BADIRAGUATO TAMAZULA HUMAYA"
    list_names(686) = "MOTAGUA"
    list_names(688) = "VAZABARRIS"
    list_names(692) = "MILAN KINABATANGAN"
    list_names(693) = "KULUNDA"
    list_names(695) = "NATASHQUAN (RIVIERE)"
    list_names(697) = "AGA TONE AGATSUMA"
    list_names(700) = "HOANIB"
    list_names(704) = "MUNIM"
    list_names(707) = "SOLO (BENGAWAN SOLO) BENGAWAN SOLO"
    list_names(708) = "BERBICE"
    list_names(712) = "DRYSDALE"
    list_names(715) = "MUCURI"
    list_names(716) = "BALEINE PETITE RIVIERE"
    list_names(723) = "TAMANA"
    list_names(729) = "ZEROUD"
    list_names(730) = "MAIPO"
    list_names(732) = "MOTALA STROEM"
    list_names(733) = "ITAJAIACU"
    list_names(736) = "DELEGATE SNOWY (SE AU)"
    list_names(743) = "PERAK KINTA"
    list_names(749) = "ACARAU GROAIRAS ARACATIACU"
    list_names(750) = "CANDELARIA"
    list_names(751) = "NASKAUPI"
    list_names(757) = "ADIGE"
    list_names(759) = "WAIKATO"
    list_names(760) = "ISHIKARI SORACHI YUBARI"
    list_names(767) = "KAZIKUMUKHSKOYE KOYSU"
    list_names(769) = "SEWA"
    list_names(770) = "APODI DO CARMO"
    list_names(772) = "DURACK"
    list_names(774) = "MALYY UZEN"
    list_names(783) = "CUARTO"
    list_names(784) = "SINU"
    list_names(802) = "PONNAIYAR"
    list_names(803) = "RAPEL"
    list_names(805) = "AMITE"
    list_names(806) = "BRISBANE"
    list_names(815) = "RIONI"
    list_names(821) = "NORMANBY"
    list_names(822) = "MANAMBOLO"
    list_names(824) = "TURIACU"
    list_names(825) = "PLYUSSA LUGA"
    list_names(827) = "ROGUE"
    list_names(829) = "PORALI"
    list_names(830) = "MAE NAM TA PI"
    list_names(831) = "Thames LEE KENNET"
    list_names(833) = "GLENELG"
    list_names(836) = "PALENA"
    list_names(839) = "KELANTAN"
    list_names(847) = "MERRIMACK"
    list_names(853) = "SINALOA"
    list_names(860) = "MANDRARE"
    list_names(868) = "OMARURU"
    list_names(870) = "BREE"
    list_names(875) = "NOTTOWAY"
    list_names(876) = "GEBA CASAMANCE"
    list_names(882) = "AMECA"
    list_names(883) = "SILVIES"
    list_names(884) = "NKAM WOURI"
    list_names(889) = "EMS SUIR"
    list_names(890) = "TUIN GOL"
    list_names(891) = "UMPQUA"
    list_names(895) = "ARCHER"
    list_names(902) = "UPSALQUITCH RESTIGOUCHE"
    list_names(907) = "CHOCTAWHATCHEE"
    list_names(912) = "VENTA"
    list_names(913) = "LIMARI"
    list_names(914) = "LITTLE SOUTHWEST MIRAMICHI"
    list_names(915) = "SUCK INNY BROSNA SHANNON"
    list_names(917) = "TRENT DERWENT"
    list_names(919) = "NEUSE CONTENTNEA"
    list_names(927) = "GORGAN RUD"
    list_names(929) = "KARASUK"
    list_names(933) = "MICO"
    list_names(936) = "OGEECHEE"
    list_names(938) = "FUTALEUFU"
    list_names(941) = "MACLEAY"
    list_names(945) = "MURRAY (SOUTH WEST AU) WILLIAMS"
    list_names(946) = "EXPLOITS"
    list_names(952) = "BRANTAS"
    list_names(955) = "SALINAS"
    list_names(960) = "SEVERN TEME AVON"
    list_names(961) = "ESCAMBIA"
    list_names(964) = "BANDAMA AGNEBY"
    list_names(966) = "OZAT"
    list_names(967) = "AVON THOMSON"
    list_names(977) = "CHUCUNAQUE TUIRA"
    list_names(987) = "DALAKI"
    list_names(992) = "VECHTE"
    list_names(993) = "SHINANO, CHIKUMA UONO SAI"
    list_names(994) = "TUNDIK"
    list_names(997) = "MIRA"
    list_names(1000) = "INNUKSUAC"
    list_names(1006) = "ERMENEK CAYIR GOEKSU"
    list_names(1009) = "MINAB"
    list_names(1016) = "TEHUANTEPEC"
    list_names(1020) = "SOUTH ESK"
    list_names(1022) = "CALCASIEU"
    list_names(1027) = "KITAKAMI"
    list_names(1028) = "MAHAWELI GANGA"
    list_names(1031) = "SAN LORENZO"
    list_names(1037) = "NICKERIE"
    list_names(1041) = "KANAIRIKTOK"
    list_names(1047) = "BOUREGREG GROU"
    list_names(1051) = "HUASCO"
    list_names(1053) = "SWARTMFOLOZI"
    list_names(1057) = "GEUM (GEUMGANG)"
    list_names(1064) = "ARMERIA"
    list_names(1079) = "PAMPANGA"
    list_names(1084) = "SARAMACCA"
    list_names(1085) = "TAURI"
    list_names(1090) = "BIG COULEE"
    list_names(1092) = "EEL (CALIF)"
    list_names(1095) = "WALKER"
    list_names(1097) = "QUINTO"
    list_names(1098) = "SWALE_DERWENT_OUSE"
    list_names(1106) = "MKUZE"
    list_names(1112) = "GAUJA"
    list_names(1114) = "PADAS"
    list_names(1115) = "BOUSELAM"
    list_names(1127) = "KISO NAGARA IBI"
    list_names(1138) = "MARY (AUQUEENSLAND)"
    list_names(1143) = "HERBERT"
    list_names(1147) = "BONITO"
    list_names(1148) = "SATILLA"
    list_names(1149) = "RHUMEL"
    list_names(1155) = "MOORABOOL BARWON"
    list_names(1159) = "HOPKINS"
    list_names(1170) = "CURU"
    list_names(1175) = "MENARANDRA"
    list_names(1177) = "KING EDWARD CARSON MORGAN"
    list_names(1178) = "TOKACHI TOSHIBETSU"
    list_names(1182) = "ANKOBRA"
    list_names(1205) = "WAIAU"
    list_names(1208) = "BEDFORD OUSE"
    list_names(1212) = "SKAGIT SAUK"
    list_names(1218) = "TAR"
    list_names(1219) = "MONITOR CREEK SOUNDING CREEK"
    list_names(1220) = "PONTAX"
    list_names(1226) = "MANNING"
    list_names(1241) = "SELI"
    list_names(1248) = "PAMPANA"
    list_names(1257) = "GREAT SCARCIES"
    list_names(1269) = "AGANO (AGAN)"
    list_names(1277) = "SETA KATSURA UJI KIZU YODO"
    list_names(1286) = "UPPER HUMBER"
    list_names(1290) = "VOHITRA IAROKA"
    list_names(1295) = "CAPIBARIBE"
    list_names(1299) = "EAST ALLIGATOR"
    list_names(1300) = "MAGPIE"
    list_names(1303) = "PAPAGAYO"
    list_names(1308) = "DEMERARA"
    list_names(1309) = "ROBE"
    list_names(1311) = "SHAGHALALY"
    list_names(1314) = "TYMI"
    list_names(1319) = "BAYOU NEZPIQUE MERMENTAU"
    list_names(1321) = "EDISTO"
    list_names(1322) = "SAN JACINTO"
    list_names(1323) = "MARY (AUSTRALIA, TIMOR SEA)"
    list_names(1326) = "WENLOCK"
    list_names(1328) = "SALO WALANAE"
    list_names(1341) = "TAFNA ISSER"
    list_names(1344) = "ADELAIDE"
    list_names(1347) = "SIO"
    list_names(1350) = "MBEI"
    list_names(1352) = "ONGARUE WHANGANUI"
    list_names(1359) = "COAHUAYANA"
    list_names(1374) = "YAQUE DEL NORTE"
    list_names(1380) = "SANTA CATARINA"
    list_names(1382) = "MANANJARI"
    list_names(1390) = "PIAXTLA"
    list_names(1396) = "SHOALHAVEN"
    list_names(1402) = "MOGAMI"
    list_names(1407) = "VAIGAI"
    list_names(1432) = "ALIAKMON"
    list_names(1436) = "UNA (BRASIL ATLANTIC) UNA"
    list_names(1438) = "VIJOSA"
    list_names(1442) = "RICHMOND"
    list_names(1453) = "AGNO (PHLUZON)"
    list_names(1457) = "UMBELUZI"
    list_names(1459) = "MONDEGO"
    list_names(1467) = "FITZMAURICE"
    list_names(1474) = "SINNAMARY"
    list_names(1498) = "BULLER"
    list_names(1520) = "TSAUCHAB"
    list_names(1522) = "PERIYAR"
    list_names(1525) = "PAERNU JOGI"
    list_names(1533) = "LAGAN (SWEDEN)"
    list_names(1541) = "LINTA"
    list_names(1545) = "Somme"
    list_names(1546) = "SADO"
    list_names(1551) = "MBASHE"
    list_names(1555) = "DEVOLLIT OSUMIT"
    list_names(1556) = "MARACACUME"
    list_names(1564) = "CANETE"
    list_names(1571) = "CHARNLEY"
    list_names(1572) = "MUAR"
    list_names(1574) = "LULULEDI"
    list_names(1603) = "LAVACA"
    list_names(1605) = "MESTA (NESTOS)"
    list_names(1607) = "PANTEPAC"
    list_names(1624) = "PEACE (FLORIDA)"
    list_names(1637) = "HOMATHKO"
    list_names(1653) = "LOWER BANN"
    list_names(1657) = "BALUARTE"
    list_names(1662) = "OCHLOCKONEE"
    list_names(1666) = "TAMIRAPARNI"
    list_names(1674) = "ACHELOOS"
    list_names(1687) = "MATAURA"
    list_names(1691) = "YUNA"
    list_names(1694) = "TESHIO"
    list_names(1697) = "DE ACAPONETA"
    list_names(1702) = "BARROW NORE"
    list_names(1705) = "MARIBYRNONG"
    list_names(1713) = "PIRIA (PIRIABAS)"
    list_names(1718) = "CHEHALIS"
    list_names(1723) = "BELLA COOLA"
    list_names(1727) = "LOKOUNDJE"
    list_names(1730) = "KAMCHIYA"
    list_names(1734) = "ABUKUMA"
    list_names(1745) = "MAEVARANO"
    list_names(1748) = "PALLINUP"
    list_names(1754) = "TOLLENSE"
    list_names(1769) = "AUDE"
    list_names(1775) = "TUNULIK"
    list_names(1784) = "YAQUE DEL SUR"
    list_names(1786) = "GANDER"
    list_names(1802) = "TENRYU"
    list_names(1823) = "TUMMEL TAY"
    list_names(1824) = "AVACHA"
    list_names(1830) = "RAPPAHANNOCK"
    list_names(1832) = "LLOBREGAT"
    list_names(1837) = "KEEP"
    list_names(1841) = "HOUSATONIC"
    list_names(1842) = "GENERAL COTO BRUS TERRABA"
    list_names(1845) = "TWEED (SCOTLAND) TEVIOT"
    list_names(1852) = "ABRA"
    list_names(1896) = "POJUCA"
    list_names(1898) = "KALI NADI"
    list_names(1899) = "CACHOEIRA"
    list_names(1900) = "SEOMJINGANG"
    list_names(1909) = "WITHLACOOCHEE"
    list_names(1915) = "CUYAMA"
    list_names(1919) = "OMONO TAMA"
    list_names(1932) = "SWARTDORINGRIVIER"
    list_names(1933) = "FRANKLAND"
    list_names(1945) = "LABUK"
    list_names(1949) = "JUBONES"
    list_names(1950) = "WONNANGATTA MITCHELL (SE AU)"
    list_names(1955) = "MOSQUITO CREEK DRAIN M"
    list_names(1973) = "HELGE A"
    list_names(1977) = "MSANGASI"
    list_names(1986) = "TERENGGANU"
    list_names(1987) = "EMAN"
    list_names(1988) = "SNOHOMISH"
    list_names(2006) = "TUBARAO"
    list_names(2010) = "MKOMAZI"
    list_names(2015) = "COREAU"
    list_names(2019) = "PIRANJI"
    list_names(2029) = "ERNE"
    list_names(2045) = "MORONDAVA"
    list_names(2064) = "WARREN"
    list_names(2089) = "SAN JUAN DE MICAY"
    list_names(2090) = "MUDA"
    list_names(2095) = "MUNGO"
    list_names(2096) = "SATA CLARA"
    list_names(2099) = "POTENGI"
    list_names(2105) = "MUNDAU"
    list_names(2111) = "CURIMATAU"
    list_names(2113) = "APPOMATTOX"
    list_names(2127) = "WYE (WALES)"
    list_names(2128) = "AI"
    list_names(2130) = "LAR RUD"
    list_names(2134) = "ISSER"
    list_names(2166) = "YONESHIRO"
    list_names(2237) = "SAINT CROIX ST CROIX"
    list_names(2261) = "RUSSIAN"
    list_names(2262) = "RANGITIKEI"
    list_names(2263) = "GONO (GO)"
    list_names(2270) = "SQUAMISH"
    list_names(2273) = "LOGAN"
    list_names(2328) = "SHETUCKET"
    list_names(2334) = "VILLA NUEVA"
    list_names(2358) = "COUFFO"
    list_names(2365) = "WAIRAU"
    list_names(2369) = "MHLATUZE"
    list_names(2384) = "ALEXIS"
    list_names(2387) = "FUJI"
    list_names(2394) = "MAE NAM TRANG"
    list_names(2403) = "HASTINGS"
    list_names(2406) = "SKJERN A"
    list_names(2414) = "BE ER SHEVA"
    list_names(2421) = "MARACAPUCU"
    list_names(2439) = "SUIR"
    list_names(2449) = "YELLOW"
    list_names(2454) = "ARA (HONSHU PACIFIC COAST)"
    list_names(2464) = "ARACATIACU"
    list_names(2465) = "SOCA INSONZO"
    list_names(2466) = "YOSHINO"
    list_names(2469) = "IVONDRO"
    list_names(2470) = "IPOJUCA"
    list_names(2498) = "BOESMANS"
    list_names(2519) = "MAHAVAVY"
    list_names(2524) = "MALWATU OYA"
    list_names(2542) = "ANTIGUA"
    list_names(2558) = "SERGIPE"
    list_names(2560) = "SANTA MARIA (PANAMA)"
    list_names(2588) = "TER"
    list_names(2596) = "MAMANGUAPE"
    list_names(2605) = "BLACKWATER (MUNSTER)"
    list_names(2606) = "KALGAN"
    list_names(2621) = "SUGUT"
    list_names(2645) = "CHANGUINOLA"
    list_names(2690) = "CORRIB"
    list_names(2720) = "RANGITAIKI"
    list_names(2729) = "RAMENA SAMBIRANO"
    list_names(2749) = "SPEY (SCOTLAND)"
    list_names(2752) = "ETAMAMIOU (RIVIERE)"
    list_names(2754) = "CLYDE (SCOTLAND)"
    list_names(2768) = "OFANTO"
    list_names(2770) = "KALU GANGA"
    list_names(2773) = "STOUR (S ENGLAND)"
    list_names(2792) = "ATERNO"
    list_names(2794) = "MOURNE"
    list_names(2808) = "KUZURYU"
    list_names(2820) = "TAMBO"
    list_names(2841) = "YONGSANGANG"
    list_names(2842) = "TYNE"
    list_names(2853) = "TRACUNHAEM"
    list_names(2854) = "SHKUMBINIT"
    list_names(2857) = "SIXAOLA"
    list_names(2871) = "VAR"
    list_names(2897) = "CAZONES"
    list_names(2900) = "CHIKUGO"
    list_names(2908) = "BOBOS TECOLUTLA"
    list_names(2935) = "JUBA"
    list_names(2937) = "REVENTAZON"
    list_names(2957) = "GUDENA"
    list_names(2967) = "BOYNE"
    list_names(2978) = "KEISKAMMA"
    list_names(2983) = "JINTSU"
    list_names(3010) = "JARDINE"
    list_names(3022) = "HURUNUI"
    list_names(3044) = "CEARAMIRIM"
    list_names(3051) = "MTATA"
    list_names(3061) = "KERT"
    list_names(3074) = "TAKAHASHI"
    list_names(3105) = "KOEPRUE"
    list_names(3113) = "HUON"
    list_names(3119) = "BERNAM"
    list_names(3158) = "REGA"
    list_names(3178) = "SEBAOU"
    list_names(3233) = "UNUK"
    list_names(3272) = "PORTNEUF (RIVIERE)"
    list_names(3298) = "IWAKI"
    list_names(3329) = "KOKTAS"
    list_names(3359) = "PAZ"
    list_names(3368) = "GRANDE DE SAN MIGUEL"
    list_names(3373) = "EDEN (N ENGLAND)"
    list_names(3376) = "SHINGU (KUMANO)"
    list_names(3412) = "LEYRE"
    list_names(3418) = "KELANI GANGA"
    list_names(3497) = "NENE"
    list_names(3503) = "YOSHII"
    list_names(3511) = "KENT"
    list_names(3516) = "VISKAN"
    list_names(3542) = "HAUGHTON"
    list_names(3556) = "OYODO"
    list_names(3566) = "NOOKSACK"
    list_names(3582) = "BARRON"
    list_names(3600) = "GRANDE DE TARCOLES"
    list_names(3621) = "WATARI"
    list_names(3625) = "DEE (SCOTLAND)"
    list_names(3626) = "LANGAT"
    list_names(3709) = "MABECHI"
    list_names(3711) = "MOY"
    list_names(3712) = "CHIRIQUI"
    list_names(3731) = "MAZAFRAN"
    list_names(3749) = "PASCOE"
    list_names(3784) = "IMERA MERIDIONALE"
    list_names(3791) = "GRAVATAI"
    list_names(3802) = "DEE (WALES)"
    list_names(3888) = "DRAIN L"
    list_names(3904) = "TOKORO"
    list_names(3955) = "KAKO"
    list_names(3959) = "RIBBLE"
    list_names(3969) = "KUMA GAWA"
    list_names(3987) = "SANTA MARGARITA"
    list_names(3995) = "SELANGOR"
    list_names(3997) = "DUNGUN"
    list_names(4013) = "NISQUALLY"
    list_names(4031) = "CALLIOPE"
    list_names(4051) = "SMITH"
    list_names(4075) = "SHO"
    list_names(4093) = "JOHOR"
    list_names(4102) = "YURA"
    list_names(4130) = "PLATANI"
    list_names(4206) = "WHAKATANE"
    list_names(4219) = "KINO"
    list_names(4240) = "SLANE"
    list_names(4260) = "THYAMIS"
    list_names(4281) = "CORURIPE"
    list_names(4327) = "AYENSU"
    list_names(4333) = "JALAUR"
    list_names(4476) = "PIONEER"
    list_names(4478) = "RIMOUSKI"
    list_names(4480) = "SHIRIBETSU"
    list_names(4503) = "ASAHI"
    list_names(4517) = "OTA"
    list_names(4545) = "MEDWAY"
    list_names(4585) = "SENDAI (KYUSHU)"
    list_names(4596) = "TARWIN"
    list_names(4633) = "BZIPI"
    list_names(4636) = "NIYODO"
    list_names(4679) = "ALSTERAN"
    list_names(4682) = "ORB"
    list_names(4706) = "TULLY"
    list_names(4716) = "LAHAVE"
    list_names(4799) = "YUBETSU"
    list_names(4819) = "Kuji"
    list_names(4821) = "ST MARYS (NOVA SCOTIA)"
    list_names(4838) = "SAN LUIZ REY"
    list_names(5005) = "SUCHIATE"
    list_names(5061) = "ABASHIRI"
    list_names(5088) = "MOTU"
    list_names(5162) = "PUYALLUP"
    list_names(5208) = "SARU"
    list_names(5242) = "TYWI"
    list_names(5287) = "DON (SCOTLAND)"
    list_names(5319) = "NEN JIANG"
    list_names(5342) = "OI"
    list_names(5499) = "TEST"
    list_names(5524) = "TAMA"
    list_names(5553) = "BAY DU NORD"
    list_names(5560) = "MU"
    list_names(5567) = "LIFFEY"
    list_names(5699) = "TABASARA"
    list_names(5711) = "QUEETS"
    list_names(5714) = "Nouvelle"
    list_names(5794) = "SENDAI (HONSHU)"
    list_names(5949) = "ARA (HONSHU JAPAN SEA COAST)"
    list_names(6000) = "Supsa"
    list_names(6064) = "Takatsu"
    list_names(6245) = "York"
    list_names(6304) = "Besos"
    list_names(6529) = "Rio de Acaponeta"
    !
    basin_names(:) = '    '
    !
    DO i=1,numlar
       tmp_str = list_names(i)
       basin_names(i) = tmp_str(1:MIN(lenstr,LEN_TRIM(tmp_str)))
    ENDDO
    !
  END SUBROUTINE routing_reg_names
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_irrigmap
!!
!>\BRIEF This subroutine interpolates the 0.5x0.5 degree based map of irrigated areas to the resolution of the model.
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_irrigmap (nbpt, index, lalo, neighbours, resolution, contfrac, &
       & init_irrig, irrigated, init_flood, floodplains, init_swamp, swamp, hist_id, hist2_id)
    !
    IMPLICIT NONE
    !
!! PARAMETERS
    INTEGER(r_std), PARAMETER :: ilake = 1 !! Number of type of lakes area (unitless)
    INTEGER(r_std), PARAMETER :: idam = 2 !! Number of type of dams area (unitless)
    INTEGER(r_std), PARAMETER :: iflood = 3 !! Number of type of floodplains area (unitless)
    INTEGER(r_std), PARAMETER :: iswamp = 4 !! Number of type of swamps area (unitless)
    INTEGER(r_std), PARAMETER :: isal = 5 !! Number of type of salines area (unitless)
    INTEGER(r_std), PARAMETER :: ipond = 6 !! Number of type of ponds area (unitless)
    INTEGER(r_std), PARAMETER :: ntype = 6 !! Number of types of flooded surfaces (unitless)

!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    INTEGER(i_std), INTENT(in) :: index(nbpt) !! Index on the global map.
    REAL(r_std), INTENT(in) :: lalo(nbpt,2) !! Vector of latitude and longitudes (beware of the order !)
    INTEGER(i_std), INTENT(in) :: neighbours(nbpt,NbNeighb)!! Vector of neighbours for each grid point
    REAL(r_std), INTENT(in) :: resolution(nbpt,2) !! The size of each grid box in X and Y (m)
    REAL(r_std), INTENT(in) :: contfrac(nbpt) !! Fraction of land in each grid box (unitless;0-1)
    INTEGER(i_std), INTENT(in) :: hist_id !! Access to history file (unitless)
    INTEGER(i_std), INTENT(in) :: hist2_id !! Access to history file 2 (unitless)
    LOGICAL, INTENT(in) :: init_irrig !! Logical to initialize the irrigation (true/false)
    LOGICAL, INTENT(in) :: init_flood !! Logical to initialize the floodplains (true/false)
    LOGICAL, INTENT(in) :: init_swamp !! Logical to initialize the swamps (true/false)
    !
!! OUTPUT VARIABLES
    REAL(r_std), INTENT(out) :: irrigated(:) !! Irrigated surface in each grid box (m^2)
    REAL(r_std), INTENT(out) :: floodplains(:) !! Surface which can be inundated in each grid box (m^2)
    REAL(r_std), INTENT(out) :: swamp(:) !! Surface which can be swamp in each grid box (m^2)
    !
!! LOCAL VARIABLES
    ! Interpolation variables
    !
    INTEGER(i_std) :: nbpmax, nix, njx, fopt !!
    CHARACTER(LEN=30) :: callsign !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:) :: resol_lu !! Resolution read on the map
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: mask !! Mask to exclude some points (unitless)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: irrsub_area !! Area on the fine grid (m^2)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:,:) :: irrsub_index !! Indices of the points we need on the fine grid (unitless)
    INTEGER :: ALLOC_ERR !!
    LOGICAL :: ok_interpol = .FALSE. !! Flag for interpolation (true/false)
    !
    CHARACTER(LEN=80) :: filename !! Name of the netcdf file (unitless)
    INTEGER(i_std) :: iml, jml, lml, tml, fid, ib, ip, jp, itype !! Indices (unitless)
    REAL(r_std) :: lev(1), date, dt, coslat !!
    INTEGER(i_std) :: itau(1) !!
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: latrel !! Latitude
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: lonrel !! Longitude
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:) :: irrigated_frac !! Irrigated fraction of the grid box (unitless;0-1)
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:,:) :: flood_fracmax !! Maximal flooded fraction of the grid box (unitless;0-1)
    REAL(r_std) :: area_irrig !! Irrigated surface in the grid box (m^2)
    REAL(r_std) :: area_flood(ntype) !! Flooded surface in the grid box (m^2)
!!$ REAL(r_std) :: irrigmap(nbpt)
!!$ REAL(r_std) :: floodmap(nbpt)
!!$ REAL(r_std) :: swampmap(nbpt)

!_ ================================================================================================================================

    !
    !Config Key = IRRIGATION_FILE
    !Config Desc = Name of file which contains the map of irrigated areas
    !Config Def = floodplains.nc
    !Config If = DO_IRRIGATION OR DO_FLOODPLAINS
    !Config Help = The name of the file to be opened to read the field
    !Config with the area in m^2 of the area irrigated within each
    !Config 0.5 0.5 deg grid box. The map currently used is the one
    !Config developed by the Center for Environmental Systems Research
    !Config in Kassel (1995).
    !Config Units = [FILE]
    !
    filename = 'floodplains.nc'
    CALL getin_p('IRRIGATION_FILE',filename)
    !
    IF (is_root_prc) THEN
       CALL flininfo(filename,iml, jml, lml, tml, fid)
       CALL flinclo(fid)
    ENDIF
    CALL bcast(iml)
    CALL bcast(jml)
    CALL bcast(lml)
    CALL bcast(tml)
    !
    !
    ALLOCATE (latrel(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_irrigmap','Pb in allocate for latrel','','')

    ALLOCATE (lonrel(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_irrigmap','Pb in allocate for lonrel','','')

    ALLOCATE (irrigated_frac(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_irrigmap','Pb in allocate for irrigated_frac','','')

    ALLOCATE (flood_fracmax(iml,jml,ntype), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_irrigmap','Pb in allocate for flood_fracmax','','')

    IF (is_root_prc) CALL flinopen(filename, .FALSE., iml, jml, lml, lonrel, latrel, lev, tml, itau, date, dt, fid)
    CALL bcast(lonrel)
    CALL bcast(latrel)
    CALL bcast(lev)
    CALL bcast(itau)
    CALL bcast(date)
    CALL bcast(dt)

    !
    IF (is_root_prc) CALL flinget(fid, 'irrig', iml, jml, lml, tml, 1, 1, irrigated_frac)
    CALL bcast(irrigated_frac)
    IF (is_root_prc) CALL flinget(fid, 'lake', iml, jml, lml, tml, 1, 1, flood_fracmax(:,:,ilake))
    IF (is_root_prc) CALL flinget(fid, 'dam', iml, jml, lml, tml, 1, 1, flood_fracmax(:,:,idam))
    IF (is_root_prc) CALL flinget(fid, 'flood', iml, jml, lml, tml, 1, 1, flood_fracmax(:,:,iflood))
    IF (is_root_prc) CALL flinget(fid, 'swamp', iml, jml, lml, tml, 1, 1, flood_fracmax(:,:,iswamp))
    IF (is_root_prc) CALL flinget(fid, 'saline', iml, jml, lml, tml, 1, 1, flood_fracmax(:,:,isal))
    CALL bcast(flood_fracmax)
    !
    IF (is_root_prc) CALL flinclo(fid)
    !
    ! Set to zero all fraction which are less than 0.5%
    !
    DO ip=1,iml
       DO jp=1,jml
          !
          IF ( irrigated_frac(ip,jp) .LT. undef_sechiba-un) THEN
             irrigated_frac(ip,jp) = irrigated_frac(ip,jp)/100.
             IF ( irrigated_frac(ip,jp) < 0.005 ) irrigated_frac(ip,jp) = zero
          ENDIF
          !
          DO itype=1,ntype
             IF ( flood_fracmax(ip,jp,itype) .LT. undef_sechiba-1.) THEN
                flood_fracmax(ip,jp,itype) = flood_fracmax(ip,jp,itype)/100
                IF ( flood_fracmax(ip,jp,itype) < 0.005 ) flood_fracmax(ip,jp,itype) = zero
             ENDIF
          ENDDO
          !
       ENDDO
    ENDDO
    !
    WRITE(numout,*) 'lonrel : ', MAXVAL(lonrel), MINVAL(lonrel)
    WRITE(numout,*) 'latrel : ', MAXVAL(latrel), MINVAL(latrel)
    WRITE(numout,*) 'irrigated_frac : ', MINVAL(irrigated_frac, MASK=irrigated_frac .GT. 0), &
         & MAXVAL(irrigated_frac, MASK=irrigated_frac .LT. undef_sechiba)
    WRITE(numout,*) 'flood_fracmax : ', MINVAL(flood_fracmax, MASK=flood_fracmax .GT. 0), &
         & MAXVAL(flood_fracmax, MASK=flood_fracmax .LT. undef_sechiba)
    !
    ! Consider all points a priori
    !
    ALLOCATE(resol_lu(iml,jml,2), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_irrigmap','Pb in allocate for resol_lu','','')

    ALLOCATE(mask(iml,jml), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_irrigmap','Pb in allocate for mask','','')
    mask(:,:) = 0

    DO ip=1,iml
       DO jp=1,jml
          !
          ! Exclude the points where we are close to the missing value.
          !
!MG This condition cannot be applied in floodplains/swamps configuration because
! the same mask would be used for the interpolation of irrigation, floodplains and swamps maps.
! IF ( irrigated_frac(ip,jp) < undef_sechiba ) THEN
             mask(ip,jp) = 1
! ENDIF
          !
          ! Resolution in longitude
          !
          coslat = MAX( COS( latrel(ip,jp) * pi/180. ), mincos )
          IF ( ip .EQ. 1 ) THEN
             resol_lu(ip,jp,1) = ABS( lonrel(ip+1,jp) - lonrel(ip,jp) ) * pi/180. * R_Earth * coslat
          ELSEIF ( ip .EQ. iml ) THEN
             resol_lu(ip,jp,1) = ABS( lonrel(ip,jp) - lonrel(ip-1,jp) ) * pi/180. * R_Earth * coslat
          ELSE
             resol_lu(ip,jp,1) = ABS( lonrel(ip+1,jp) - lonrel(ip-1,jp) )/2. * pi/180. * R_Earth * coslat
          ENDIF
          !
          ! Resolution in latitude
          !
          IF ( jp .EQ. 1 ) THEN
             resol_lu(ip,jp,2) = ABS( latrel(ip,jp) - latrel(ip,jp+1) ) * pi/180. * R_Earth
          ELSEIF ( jp .EQ. jml ) THEN
             resol_lu(ip,jp,2) = ABS( latrel(ip,jp-1) - latrel(ip,jp) ) * pi/180. * R_Earth
          ELSE
             resol_lu(ip,jp,2) = ABS( latrel(ip,jp-1) - latrel(ip,jp+1) )/2. * pi/180. * R_Earth
          ENDIF
          !
       ENDDO
    ENDDO
    !
    ! The number of maximum vegetation map points in the GCM grid is estimated.
    ! Some lmargin is taken.
    !
    IF (is_root_prc) THEN
       nix=INT(MAXVAL(resolution_g(:,1))/MAXVAL(resol_lu(:,:,1)))+2
       njx=INT(MAXVAL(resolution_g(:,2))/MAXVAL(resol_lu(:,:,2)))+2
       nbpmax = nix*njx*2
    ENDIF
    CALL bcast(nbpmax)
    !
    callsign = 'Irrigation map'
    ok_interpol = .FALSE.
    !
    WRITE(numout,*) "Projection arrays for ",callsign," : "
    WRITE(numout,*) "nbpmax = ",nbpmax, nix, njx

    ALLOCATE(irrsub_index(nbpt, nbpmax, 2), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_irrigmap','Pb in allocate for irrsub_index','','')
    irrsub_index(:,:,:)=0

    ALLOCATE(irrsub_area(nbpt, nbpmax), STAT=ALLOC_ERR)
    IF (ALLOC_ERR /= 0) CALL ipslerr_p(3,'routing_reg_irrigmap','Pb in allocate for irrsub_area','','')
    irrsub_area(:,:)=zero
    !
    CALL aggregate_p(nbpt, lalo, neighbours, resolution, contfrac, &
         & iml, jml, lonrel, latrel, mask, callsign, &
         & nbpmax, irrsub_index, irrsub_area, ok_interpol)
    !
    WHERE (irrsub_area < 0) irrsub_area=zero
    !
    ! Test here if not all sub_area are larger than 0 if so, then we need to increase nbpmax
    !
    DO ib=1,nbpt
       !
       area_irrig = 0.0
       area_flood = 0.0
       !
       DO fopt=1,COUNT(irrsub_area(ib,:) > zero)
          !
          ip = irrsub_index(ib, fopt, 1)
          jp = irrsub_index(ib, fopt, 2)
          !
          IF (irrigated_frac(ip,jp) .LT. undef_sechiba-1.) THEN
             area_irrig = area_irrig + irrsub_area(ib,fopt)*irrigated_frac(ip,jp)
          ENDIF
          !
          DO itype=1,ntype
             IF (flood_fracmax(ip,jp,itype) .LT. undef_sechiba-1.) THEN
                area_flood(itype) = area_flood(itype) + irrsub_area(ib,fopt)*flood_fracmax(ip,jp,itype)
             ENDIF
          ENDDO
       ENDDO
       !
       ! Put the total irrigated and flooded areas in the output variables
       !
       IF ( init_irrig ) THEN
          irrigated(ib) = MIN(area_irrig, resolution(ib,1)*resolution(ib,2)*contfrac(ib))
          IF ( irrigated(ib) < 0 ) THEN
             WRITE(numout,*) 'We have a problem here : ', irrigated(ib)
             WRITE(numout,*) 'resolution :', resolution(ib,1), resolution(ib,2)
             WRITE(numout,*) area_irrig
             CALL ipslerr_p(3,'routing_reg_irrigmap','Problem with irrigated...','','')
          ENDIF
!!$ ! Compute a diagnostic of the map.
!!$ IF(contfrac(ib).GT.zero) THEN
!!$ irrigmap (ib) = irrigated(ib) / ( resolution(ib,1)*resolution(ib,2)*contfrac(ib) )
!!$ ELSE
!!$ irrigmap (ib) = zero
!!$ ENDIF
          !
       ENDIF
       !
       IF ( init_flood ) THEN
          floodplains(ib) = MIN(area_flood(iflood)+area_flood(idam)+area_flood(isal), &
               & resolution(ib,1)*resolution(ib,2)*contfrac(ib))
          IF ( floodplains(ib) < 0 ) THEN
             WRITE(numout,*) 'We have a problem here : ', floodplains(ib)
             WRITE(numout,*) 'resolution :', resolution(ib,1), resolution(ib,2)
             WRITE(numout,*) area_flood
             CALL ipslerr_p(3,'routing_reg_irrigmap','Problem with floodplains..','','')
          ENDIF
!!$ ! Compute a diagnostic of the map.
!!$ IF(contfrac(ib).GT.zero) THEN
!!$ floodmap(ib) = floodplains(ib) / ( resolution(ib,1)*resolution(ib,2)*contfrac(ib) )
!!$ ELSE
!!$ floodmap(ib) = 0.0
!!$ ENDIF
       ENDIF
       !
       IF ( init_swamp ) THEN
          swamp(ib) = MIN(area_flood(iswamp), resolution(ib,1)*resolution(ib,2)*contfrac(ib))
          IF ( swamp(ib) < 0 ) THEN
             WRITE(numout,*) 'We have a problem here : ', swamp(ib)
             WRITE(numout,*) 'resolution :', resolution(ib,1), resolution(ib,2)
             WRITE(numout,*) area_flood
             CALL ipslerr_p(3,'routing_reg_irrigmap','Problem with swamp...','','')
          ENDIF
!!$ ! Compute a diagnostic of the map.
!!$ IF(contfrac(ib).GT.zero) THEN
!!$ swampmap(ib) = swamp(ib) / ( resolution(ib,1)*resolution(ib,2)*contfrac(ib) )
!!$ ELSE
!!$ swampmap(ib) = zero
!!$ ENDIF
       ENDIF
       !
       !
    ENDDO
    !
    !

    IF ( init_irrig ) WRITE(numout,*) "Diagnostics irrigated :", MINVAL(irrigated), MAXVAL(irrigated)
    IF ( init_flood ) WRITE(numout,*) "Diagnostics floodplains :", MINVAL(floodplains), MAXVAL(floodplains)
    IF ( init_swamp ) WRITE(numout,*) "Diagnostics swamp :", MINVAL(swamp), MAXVAL(swamp)
!
! No compensation is done for overlapping floodplains, swamp and irrig. At least overlapping will not
! happen between floodplains and swamp alone
! IF ( init_irrig .AND. init_flood ) THEN
! DO ib = 1, nbpt
! surp = (floodplains(ib)+swamp(ib)+irrigated(ib)) / (resolution(ib,1)*resolution(ib,2)*contfrac(ib))
! IF ( surp .GT. un ) THEN
! floodplains(ib) = floodplains(ib) / surp
! swamp(ib) = swamp(ib) / surp
! irrigated(ib) = irrigated(ib) / surp
! ENDIF
! ENDDO
! ENDIF
    !
    DEALLOCATE (irrsub_area)
    DEALLOCATE (irrsub_index)
    !
    DEALLOCATE (mask)
    DEALLOCATE (resol_lu)
    !
    DEALLOCATE (lonrel)
    DEALLOCATE (latrel)
    !
  END SUBROUTINE routing_reg_irrigmap
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_waterbal
!!
!>\BRIEF This subroutine checks the water balance in the routing module.
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_waterbal(nbpt, firstcall, floodout, runoff, drainage, returnflow, &
               & reinfiltration, irrigation, riverflow, coastalflow)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbpt !! Domain size (unitless)
    LOGICAL, INTENT(in) :: firstcall !! Controls behaviour (true/false)
    REAL(r_std), INTENT(in) :: floodout(nbpt) !! Grid-point flow out of floodplains (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: runoff(nbpt) !! Grid-point runoff (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: drainage(nbpt) !! Grid-point drainage (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: returnflow(nbpt) !! The water flow from lakes and swamps which returns to the grid box.
                                                       !! This water will go back into the hydrol or hydrolc module to allow re-evaporation (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: reinfiltration(nbpt) !! Water flow from ponds and floodplains which returns to the grid box (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: irrigation(nbpt) !! Irrigation flux. This is the water taken from the reservoirs and beeing put into the upper layers of the soil (kg/m^2/dt)
    REAL(r_std), INTENT(in) :: riverflow(nbpt) !! Outflow of the major rivers. The flux will be located on the continental grid but this should be a coastal point (kg/dt)
    REAL(r_std), INTENT(in) :: coastalflow(nbpt) !! Outflow on coastal points by small basins. This is the water which flows in a disperse way into the ocean (kg/dt)
    !
    ! We sum-up all the water we have in the warious reservoirs
    !
    REAL(r_std), SAVE :: totw_flood !! Sum of all the water amount in the floodplains reservoirs (kg)
!$OMP THREADPRIVATE(totw_flood)
    REAL(r_std), SAVE :: totw_stream !! Sum of all the water amount in the stream reservoirs (kg)
!$OMP THREADPRIVATE(totw_stream)
    REAL(r_std), SAVE :: totw_fast !! Sum of all the water amount in the fast reservoirs (kg)
!$OMP THREADPRIVATE(totw_fast)
    REAL(r_std), SAVE :: totw_slow !! Sum of all the water amount in the slow reservoirs (kg)
!$OMP THREADPRIVATE(totw_slow)
    REAL(r_std), SAVE :: totw_lake !! Sum of all the water amount in the lake reservoirs (kg)
!$OMP THREADPRIVATE(totw_lake)
    REAL(r_std), SAVE :: totw_pond !! Sum of all the water amount in the pond reservoirs (kg)
!$OMP THREADPRIVATE(totw_pond)
    REAL(r_std), SAVE :: totw_in !! Sum of the water flow in to the routing scheme
!$OMP THREADPRIVATE(totw_in)
    REAL(r_std), SAVE :: totw_out !! Sum of the water flow out to the routing scheme
!$OMP THREADPRIVATE(totw_out)
    REAL(r_std), SAVE :: totw_return !!
!$OMP THREADPRIVATE(totw_return)
    REAL(r_std), SAVE :: totw_irrig !!
!$OMP THREADPRIVATE(totw_irrig)
    REAL(r_std), SAVE :: totw_river !!
!$OMP THREADPRIVATE(totw_river)
    REAL(r_std), SAVE :: totw_coastal !!
!$OMP THREADPRIVATE(totw_coastal)
    REAL(r_std) :: totarea !! Total area of basin (m^2)
    REAL(r_std) :: area !! Total area of routing (m^2)
    INTEGER(i_std) :: ig !!
    !
    ! Just to make sure we do not get too large numbers !
    !
!! PARAMETERS
    REAL(r_std), PARAMETER :: scaling = 1.0E+6 !!
    REAL(r_std), PARAMETER :: allowed_err = 50. !!

!_ ================================================================================================================================
    !
    IF ( firstcall ) THEN
       !
       totw_flood = zero
       totw_stream = zero
       totw_fast = zero
       totw_slow = zero
       totw_lake = zero
       totw_pond = zero
       totw_in = zero
       !
       DO ig=1,nbpt
          !
          totarea = SUM(routing_area_loc(ig,:))
          !
          totw_flood = totw_flood + SUM(flood_reservoir(ig,:)/scaling)
          totw_stream = totw_stream + SUM(stream_reservoir(ig,:)/scaling)
          totw_fast = totw_fast + SUM(fast_reservoir(ig,:)/scaling)
          totw_slow = totw_slow + SUM(slow_reservoir(ig,:)/scaling)
          totw_lake = totw_lake + lake_reservoir(ig)/scaling
          totw_pond = totw_pond + pond_reservoir(ig)/scaling
          !
          totw_in = totw_in + (runoff(ig)*totarea + drainage(ig)*totarea - floodout(ig)*totarea)/scaling
          !
       ENDDO
       !
    ELSE
       !
       totw_out = zero
       totw_return = zero
       totw_irrig = zero
       totw_river = zero
       totw_coastal = zero
       area = zero
       !
       DO ig=1,nbpt
          !
          totarea = SUM(routing_area_loc(ig,:))
          !
          totw_flood = totw_flood - SUM(flood_reservoir(ig,:)/scaling)
          totw_stream = totw_stream - SUM(stream_reservoir(ig,:)/scaling)
          totw_fast = totw_fast - SUM(fast_reservoir(ig,:)/scaling)
          totw_slow = totw_slow - SUM(slow_reservoir(ig,:)/scaling)
          totw_lake = totw_lake - lake_reservoir(ig)/scaling
          totw_pond = totw_pond - pond_reservoir(ig)/scaling
          !
          totw_return = totw_return + (reinfiltration(ig)+returnflow(ig))*totarea/scaling
          totw_irrig = totw_irrig + irrigation(ig)*totarea/scaling
          totw_river = totw_river + riverflow(ig)/scaling
          totw_coastal = totw_coastal + coastalflow(ig)/scaling
          !
          area = area + totarea
          !
       ENDDO
       totw_out = totw_return + totw_irrig + totw_river + totw_coastal
       !
       ! Now we have all the information to balance our water
       !
       IF ( ABS((totw_flood + totw_stream + totw_fast + totw_slow + totw_lake + totw_pond) - &
            & (totw_out - totw_in)) > allowed_err ) THEN
          WRITE(numout,*) 'WARNING : Water not conserved in routing. Limit at ', allowed_err, ' 10^6 kg'
          WRITE(numout,*) '--Water-- change : flood stream fast ', totw_flood, totw_stream, totw_fast
          WRITE(numout,*) '--Water-- change : slow, lake ', totw_slow, totw_lake
          WRITE(numout,*) '--Water>>> change in the routing res. : ', totw_flood + totw_stream + totw_fast + totw_slow + totw_lake
          WRITE(numout,*) '--Water input : ', totw_in
          WRITE(numout,*) '--Water output : ', totw_out
          WRITE(numout,*) '--Water output : return, irrig ', totw_return, totw_irrig
          WRITE(numout,*) '--Water output : river, coastal ',totw_river, totw_coastal
          WRITE(numout,*) '--Water>>> change by fluxes : ', totw_out - totw_in, ' Diff [mm/dt]: ', &
               & ((totw_flood + totw_stream + totw_fast + totw_slow + totw_lake) - (totw_out - totw_in))/area

          ! Stop the model
          CALL ipslerr_p(3, 'routing_reg_waterbal', 'Water is not conserved in routing.','','')
       ENDIF
       !
    ENDIF
    !
  END SUBROUTINE routing_reg_waterbal

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_writeaggr
!!
!> BRIEF This subroutine writes output from aggregate step for saving time of experiment
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_writeaggr(nbagg, nmagg, sindex, sarea)
    !
    USE netcdf
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbagg !! Domain size (unitless)
    INTEGER(i_std), INTENT(in) :: nmagg !! The maximum number of basins we can handle at any time during the generation of the maps (unitless)
    INTEGER(i_std), DIMENSION(nbagg,nmagg,2), INTENT(in) :: sindex !! Indices of the points we need on the fine grid (unitless)
    REAL(r_std), DIMENSION(nbagg,nmagg), INTENT(in) :: sarea !! Area on the fine grid (m^2)
!! LOCAL VARIABLES
    CHARACTER(LEN=30), PARAMETER :: file_aggr = "out_aggregate.nc"
    INTEGER(i_std), PARAMETER :: NDIMS = 2
    INTEGER(i_std) :: ncid, ind01id, ind02id, areaid
    INTEGER(i_std) :: dimids(NDIMS), x_dimid, y_dimid

    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: sindex_tmp1 !! Indices of the points we need on the fine grid (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: sindex_tmp2 !! Indices of the points we need on the fine grid (unitless)

    !
    ALLOCATE (sindex_tmp1(nbagg,nmagg))
    ALLOCATE (sindex_tmp2(nbagg,nmagg))
    !
    sindex_tmp1(:,:) = sindex(:,:,1)
    sindex_tmp2(:,:) = sindex(:,:,2)
    !
    CALL routing_reg_check( NF90_CREATE(file_aggr, NF90_CLOBBER, ncid) )

    CALL routing_reg_check( NF90_DEF_DIM(ncid, "X", nmagg, x_dimid) )
    CALL routing_reg_check( NF90_DEF_DIM(ncid, "Y", nbagg, y_dimid) )

    dimids = (/ y_dimid, x_dimid /)

    CALL routing_reg_check( NF90_DEF_VAR(ncid, "index1", NF90_INT, dimids, ind01id) )
    CALL routing_reg_check( NF90_DEF_VAR(ncid, "index2", NF90_INT, dimids, ind02id) )
    CALL routing_reg_check( NF90_DEF_VAR(ncid, "area", NF90_FLOAT, dimids, areaid) )

    CALL routing_reg_check( NF90_ENDDEF(ncid) )

    CALL routing_reg_check( NF90_PUT_VAR(ncid, ind01id, sindex_tmp1) )
    CALL routing_reg_check( NF90_PUT_VAR(ncid, ind02id, sindex_tmp2) )
    CALL routing_reg_check( NF90_PUT_VAR(ncid, areaid, sarea) )

    CALL routing_reg_check( NF90_CLOSE(ncid) )
    !
    IF ( ALLOCATED(sindex_tmp1) ) DEALLOCATE (sindex_tmp1)
    IF ( ALLOCATED(sindex_tmp2) ) DEALLOCATE (sindex_tmp2)
    !
  END SUBROUTINE routing_reg_writeaggr
  !
  SUBROUTINE routing_reg_check(status)
  !
  INTEGER(i_std), INTENT(in) :: status
  !
  IF (status /= NF90_NOERR) THEN
     WRITE (*,*) TRIM(NF90_STRERROR(status))
     STOP 'routing_reg_check'
  ENDIF
  END SUBROUTINE routing_reg_check

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_readaggr
!!
!> BRIEF This subroutine reads input instead of aggregate step for saving time of experiment
!!
!! DESCRIPTION (definitions, functional, design, flags) : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

SUBROUTINE routing_reg_readaggr(nbagg, nmagg, sindex, sarea)
    !
    USE netcdf
    IMPLICIT NONE
    !
!! INPUT/OUTPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbagg !! Domain size (unitless)
    INTEGER(i_std), INTENT(in) :: nmagg !! The maximum number of basins we can handle at any time during the generation of the maps (unitless)
    INTEGER(i_std), DIMENSION(nbagg,nmagg,2), INTENT(inout) :: sindex !! Indices of the points we need on the fine grid (unitless)
    REAL(r_std), DIMENSION(nbagg,nmagg), INTENT(inout) :: sarea !! Area on the fine grid (m^2)
!! LOCAL VARIABLES
    CHARACTER(LEN=30), PARAMETER :: file_aggr = "out_aggregate.nc"
    INTEGER(i_std), PARAMETER :: NDIMS = 2
    INTEGER(i_std) :: ncid, ind01id, ind02id, areaid

    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: sindex_tmp1 !! Indices of the points we need on the fine grid (unitless)
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:) :: sindex_tmp2 !! Indices of the points we need on the fine grid (unitless)
    !
    ALLOCATE (sindex_tmp1(nbagg,nmagg))
    ALLOCATE (sindex_tmp2(nbagg,nmagg))
    !
    CALL routing_reg_check( NF90_OPEN(file_aggr, NF90_NOWRITE, ncid ) )

    CALL routing_reg_check( NF90_INQ_VARID(ncid, "index1", ind01id) )
    CALL routing_reg_check( NF90_INQ_VARID(ncid, "index2", ind02id) )
    CALL routing_reg_check( NF90_INQ_VARID(ncid, "area", areaid) )

    CALL routing_reg_check( NF90_GET_VAR(ncid, ind01id, sindex_tmp1) )
    CALL routing_reg_check( NF90_GET_VAR(ncid, ind02id, sindex_tmp2) )
    CALL routing_reg_check( NF90_GET_VAR(ncid, areaid, sarea) )

    ! Should we need to check the data here ?!
    CALL routing_reg_check( NF90_CLOSE(ncid) )
    !
    sindex(:,:,1) = sindex_tmp1(:,:)
    sindex(:,:,2) = sindex_tmp2(:,:)
    !
    IF ( ALLOCATED(sindex_tmp1) ) DEALLOCATE (sindex_tmp1)
    IF ( ALLOCATED(sindex_tmp2) ) DEALLOCATE (sindex_tmp2)
    !
  END SUBROUTINE routing_reg_readaggr

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_findbasins
!!
!>\BRIEF This subroutine finds the basins and does some clean up.
!! The aim is to return the list off all points which are within the
!! same basin of the grid box. This is the faster version of the old routing_reg_findbasins
!!
!! DESCRIPTION (definitions, functional, design, flags) :
!! We will also collect all points which directly flow into the ocean in one basin
!! Make sure that we do not have a basin with two outflows and other exceptions.
!! At this stage no effort is made to come down to the truncation of the model.
!!
!! Convention for trip \n
!! ------------------- \n
!! Inside of the box : \n
!! trip = 1 : flow = N \n
!! trip = 2 : flow = NE \n
!! trip = 3 : flow = E \n
!! trip = 4 : flow = SE \n
!! trip = 5 : flow = S \n
!! trip = 6 : flow = SW \n
!! trip = 7 : flow = W \n
!! trip = 8 : flow = NW \n
!! trip = 97 : return flow into the ground \n
!! trip = 98 : coastal flow (diffuse flow into the oceans) These values are created here \n
!! trip = 99 : river flow into the oceans \n
!!
!! Out flow from the grid : \n
!! trip = 101 : flow = N out of the coarse grid \n
!! trip = 102 : flow = NE out of the coarse grid \n
!! trip = 103 : flow = E out of the coarse grid \n
!! trip = 104 : flow = SE out of the coarse grid \n
!! trip = 105 : flow = S out of the coarse grid \n
!! trip = 106 : flow = SW out of the coarse grid \n
!! trip = 107 : flow = W out of the coarse grid \n
!! trip = 108 : flow = NW out of the coarse grid! \n
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_findbasins(ib, nbi, nbj, trip, basin, fac, hierarchy, topoind, lshead, diaglalo, &
       & nb_basin, basin_inbxid, basin_outlet,  basin_outtp, basin_sz, basin_bxout, basin_bbout, basin_pts, &
       & basin_lshead, coast_pts, lontmp, lattmp)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: ib !! 
    INTEGER(i_std), INTENT(in) :: nbi !! Number of point in x within the grid (unitless)
    INTEGER(i_std), INTENT(in) :: nbj !! Number of point in y within the grid (unitless)
    REAL(r_std), INTENT(in) :: hierarchy(:,:) !!
    REAL(r_std), INTENT(in) :: fac(:,:) !!
    REAL(r_std), INTENT(in) :: lshead(:,:)
    REAL(r_std), DIMENSION(:,:), INTENT(in) :: diaglalo !! Point (in Lat/Lon) where diagnostics will be printed.
    !
    ! Modified
    !
    INTEGER(i_std), INTENT(inout) :: trip(:,:) !! The trip field (unitless)
    INTEGER(i_std), INTENT(inout) :: basin(:,:) !!
    REAL(r_std), INTENT(inout) :: topoind(:,:) !! Topographic index of the residence time (m)
    !
    ! For debug and get coordinate of river outlet
    !
    REAL(r_std), INTENT(in) :: lontmp(:,:) !! Longitude
    REAL(r_std), INTENT(in) :: lattmp(:,:) !! Latitude
    !
!! OUTPUT VARIABLES
    INTEGER(i_std), INTENT(out) :: nb_basin               !! Number of sub-basins (unitless)
    INTEGER(i_std), INTENT(out) :: basin_inbxid(nbvmax)   !!
    REAL(r_std), INTENT(out)    :: basin_outlet(nbvmax,2) !!
    REAL(r_std), INTENT(out)    :: basin_outtp(nbvmax)    !!
    INTEGER(i_std), INTENT(out) :: basin_sz(nbvmax)       !!
    INTEGER(i_std), INTENT(out) :: basin_bxout(nbvmax)    !!
    REAL(r_std), INTENT(out)    :: basin_lshead(nbvmax)   !!
    INTEGER(i_std), INTENT(out) :: basin_bbout(nbvmax)    !!
    INTEGER(i_std), INTENT(out) :: basin_pts(nbvmax, nbvmax, 2) !!
    INTEGER(i_std), INTENT(out) :: coast_pts(nbvmax)      !! The coastal flow points (unitless)
    !
!! LOCAL VARIABLES
    LOGICAL, PARAMETER :: debug=.FALSE.
    CHARACTER(LEN=7) :: fmt !!
    CHARACTER(LEN=9) :: fmtr !!
    !
    LOGICAL :: newpoint !! (true/false)
    INTEGER(i_std), DIMENSION(8,2) :: inc !!
    INTEGER(i_std), DIMENSION(nbi,nbj) :: outtmp !!
    !
    INTEGER(i_std) :: nbb, p, cnt, ibas !! To count number of subbasin
    INTEGER(i_std) :: il, jl, ill, jll, ij, iz!!
    INTEGER(i_std) :: totsz, checksz !!
    !
    INTEGER(i_std) :: tbname(nbvmax) !!
    INTEGER(i_std) :: tsz(nbvmax) !!
    INTEGER(i_std) :: tpts(nbvmax,nbvmax,2) !!
    INTEGER(i_std) :: toutdir(nbvmax) !!
    INTEGER(i_std) :: toutbas(nbvmax) !!
    INTEGER(i_std) :: toutloc(nbvmax,2) !!
    REAL(r_std) :: tlon(nbvmax) !!
    REAL(r_std) :: tlat(nbvmax) !!
    REAL(r_std) :: touttp(nbvmax) !!
    REAL(r_std) :: toutlshead(nbvmax) !!
    !
    INTEGER(i_std) :: tmpsz(nbvmax) !!
    INTEGER(i_std) :: ip, jp, jpp(1), ipb !!
    INTEGER(i_std) :: ie, je !!
    INTEGER(i_std) :: sortind(nbvmax) !!
    !
    INTEGER(i_std) :: itrans !!
    INTEGER(i_std) :: trans(nbvmax) !!
    !
    INTEGER(i_std) :: new_nb !! Number of sub-basins (unitless)
    INTEGER(i_std) :: mp !! Number of dividing sub-basin in mainstream (unitless)
    INTEGER(i_std) :: new_bname(nbvmax) !!
    INTEGER(i_std) :: new_outdir(nbvmax) !!
    REAL(r_std)    :: new_heading(nbvmax)!!
    INTEGER(i_std) :: new_outbas(nbvmax) !!
    INTEGER(i_std) :: new_outloc(nbvmax,2) !!
    REAL(r_std)    :: new_lon(nbvmax) !!
    REAL(r_std)    :: new_lat(nbvmax) !!
    INTEGER(i_std) :: new_sz(nbvmax) !!
    INTEGER(i_std) :: new_pts(nbvmax, nbvmax, 2) !!
    INTEGER(i_std) :: oldorder, neworder !!
    !
    INTEGER(i_std) :: option !! Option to calculate topoindex

!_ ================================================================================================================================
    !
    IF ( debug .AND. routing_diagbox(ib, diaglalo) ) THEN
       WRITE(numout,*) "Point: ", ib
       WRITE(numout,*) "Coord:", lalo(ib,2), lalo(ib,1)
       WRITE(numout,*) "Per: ", nbi, nbj
       !
       WRITE(fmt,"('(',I3,'I6)')") nbi
       WRITE(numout,*) '+++++++++++++++++++ TRIP IN FINDBASINS ++++++++++++++++++++'
       DO je=1,nbj
          WRITE(numout,fmt) trip(1:nbi,je)
       ENDDO
    ENDIF
    !
    ! The routing code (i=1, j=2)
    !
    inc(1,1) = 0
    inc(1,2) = -1
    inc(2,1) = 1
    inc(2,2) = -1
    inc(3,1) = 1
    inc(3,2) = 0
    inc(4,1) = 1
    inc(4,2) = 1
    inc(5,1) = 0
    inc(5,2) = 1
    inc(6,1) = -1
    inc(6,2) = 1
    inc(7,1) = -1
    inc(7,2) = 0
    inc(8,1) = -1
    inc(8,2) = -1
    !
    ! 1.0 Find number of outflow point (>= 97)
    ! So basically, we get nb_basin, basin_bxout, basin_inbxid
    ! (with nbb, toutdir, tbname)
    !
    nbb = 0
    tbname(:) = undef_int
    toutdir(:) = 0
    toutloc(:,:) = 0
    outtmp(:,:) = -1
    !
    toutbas(:) = undef_int
    touttp(:) = undef_int
    toutlshead(:) = undef_sechiba
    !
    DO ip=1,nbi
       DO jp=1,nbj
          !
          !
          IF ( basin(ip,jp) .LT. undef_int) THEN
             ! Because, there is maybe difference between undef_int and undef in routing.nc
             !IF ( trip(ip,jp) .GE. 97 .AND. trip(ip,jp) .LT. undef_int ) THEN
             IF ( trip(ip,jp) .GE. 97 .AND. trip(ip,jp) .LT. 109 ) THEN
                nbb = nbb + 1
                IF ( nbb .GT. nbvmax ) CALL ipslerr_p(3,'routing_reg_findbasins','nbvmax too small','first section','')
                tbname(nbb) = basin(ip,jp)
                toutdir(nbb) = trip(ip,jp)-100
                toutlshead(nbb) = lshead(ip,jp)
                touttp(nbb) = trip(ip,jp)
                toutloc(nbb,1) = ip
                toutloc(nbb,2) = jp
                tlat(nbb) = lattmp(ip,jp)
                tlon(nbb) = lontmp(ip,jp)
                outtmp(ip,jp) = nbb
             ENDIF
          ENDIF
       ENDDO
    ENDDO
    !
    ! 2.0 Follow the flow of the water to get size and gather point for each
    ! sub-basin. So we get basin_sz, basin_pts (with tsz, tpts)
    !
    totsz = 0
    tsz(:) = 0
    tpts(:,:,:) = 0
    !
    DO ip=1,nbi
       DO jp=1,nbj
         IF ( basin(ip,jp) .LT. undef_int) THEN
         ! Think about basin .LT. undef_int here
         !IF ( trip(ip,jp) .GT. 0 .AND. trip(ip,jp) .LT. undef_int ) THEN
         IF ( trip(ip,jp) .GT. 0 .AND. trip(ip,jp) .LT. 109 ) THEN
           p = trip(ip,jp)
           cnt = 0
           il = ip ; jl = jp
           newpoint = .TRUE.
           DO WHILE ( p .GT. 0 .AND. p .LT. 9 .AND. cnt .LT. nbi*nbj .AND. newpoint )
              cnt = cnt + 1
              ill = il + inc(p,1)
              jll = jl + inc(p,2)
              il = ill ; jl = jll
              p = trip(il,jl)
              IF ( outtmp(il,jl) .NE. -1 ) newpoint = .FALSE.
           ENDDO
           !
           !
           IF ( cnt .EQ. nbi*nbj) THEN
              IF ( debug ) THEN
                 WRITE(numout,*) "Error: cnt .EQ. nbi*nbj "
                 WRITE(numout,*) "Point: ", ip, jp
                 WRITE(numout,*) "Per: ", nbi, nbj
                 WRITE(numout,*) "Number of outlet: ", nbb
                 WRITE(numout,*) '+++++++++++++++++++ OUTLET IN FINDBASINS ++++++++++++++++++++'
                 WRITE(fmt,"('(',I3,'I6)')") nbi
                 DO je=1,nbj
                    WRITE(numout,fmt) outtmp(1:nbi,je)
                 ENDDO
                 !
                 WRITE(numout,*) '+++++++++++++++++++ TRIP IN FINDBASINS ++++++++++++++++++++'
                 DO je=1,nbj
                    WRITE(numout,fmt) trip(1:nbi,je)
                 ENDDO
                 !
                 WRITE(numout,*) '+++++++++++++++++++ BASIN IDs IN FINDBASINS ++++++++++++++++++++'
                 DO je=1,nbj
                    WRITE(numout,fmt) basin(1:nbi,je)
                 ENDDO
                 !
                 WRITE(numout,*) '+++++++++++++++++++ LONGITUDE IN FINDBASINS ++++++++++++++++++++'
                 WRITE(fmtr,"('(',I3,'F8.1)')") nbi
                 DO je=1,nbj
                    WRITE(numout,fmtr) lontmp(1:nbi,je)
                 ENDDO
                 !
                 WRITE(numout,*) '+++++++++++++++++++ LATITUDE IN FINDBASINS ++++++++++++++++++++'
                 WRITE(fmtr,"('(',I3,'F8.1)')") nbi
                 DO je=1,nbj
                    WRITE(numout,fmtr) lattmp(1:nbi,je)
                 ENDDO
                 !
                 WRITE(numout,*) '+++++++++++++++++++ HIERARCHY IN FINDBASINS ++++++++++++++++++++'
                 WRITE(fmtr,"('(',I3,'F8.1)')") nbi
                 DO je=1,nbj
                    WRITE(numout,fmtr) hierarchy(1:nbi,je)
                 ENDDO
                 CALL FLUSH(numout)
              ENDIF
              CALL ipslerr_p(3,'routing_reg_findbasins','We could not route point','','')
           ENDIF
           !
           IF ( outtmp(il,jl) .NE. -1 ) THEN
              ibas = outtmp(il,jl)
              tsz(ibas) = tsz(ibas) + 1
              IF ( tsz(ibas) .GT. nbvmax ) CALL ipslerr_p(3,'routing_reg_findbasins','nbvmax too small','second section','')
              tpts(ibas, tsz(ibas), 1) = ip
              tpts(ibas, tsz(ibas), 2) = jp
              outtmp(ip,jp) = outtmp(il,jl)
           ELSE
              IF ( debug ) THEN
                 WRITE(numout,*) "Error: outtmp(il,jl) .EQ. -1 "
                 WRITE(numout,*) "Point: ", ip, jp
                 WRITE(numout,*) "Per: ", nbi, nbj
                 WRITE(numout,*) "Number of outlet: ", nbb
                 WRITE(numout,*) '+++++++++++++++++++ OUTLET IN FINDBASINS ++++++++++++++++++++'
                 WRITE(fmt,"('(',I3,'I6)')") nbi
                 DO je=1,nbj
                    WRITE(numout,fmt) outtmp(1:nbi,je)
                 ENDDO
                 !
                 WRITE(numout,*) '+++++++++++++++++++ TRIP IN FINDBASINS ++++++++++++++++++++'
                 DO je=1,nbj
                    WRITE(numout,fmt) trip(1:nbi,je)
                 ENDDO
                 !
                 WRITE(numout,*) '+++++++++++++++++++ BASIN IDs IN FINDBASINS ++++++++++++++++++++'
                 DO je=1,nbj
                    WRITE(numout,fmt) basin(1:nbi,je)
                 ENDDO
                 !
                 WRITE(numout,*) '+++++++++++++++++++ LONGITUDE IN FINDBASINS ++++++++++++++++++++'
                 WRITE(fmtr,"('(',I3,'F8.1)')") nbi
                 DO je=1,nbj
                    WRITE(numout,fmtr) lontmp(1:nbi,je)
                 ENDDO
                 !
                 WRITE(numout,*) '+++++++++++++++++++ LATITUDE IN FINDBASINS ++++++++++++++++++++'
                 WRITE(fmtr,"('(',I3,'F8.1)')") nbi
                 DO je=1,nbj
                    WRITE(numout,fmtr) lattmp(1:nbi,je)
                 ENDDO
                 !
                 WRITE(numout,*) '+++++++++++++++++++ HIERARCHY IN FINDBASINS ++++++++++++++++++++'
                 WRITE(fmtr,"('(',I3,'F8.1)')") nbi
                 DO je=1,nbj
                    WRITE(numout,fmtr) hierarchy(1:nbi,je)
                 ENDDO
              ENDIF
              CALL FLUSH(numout)
              WRITE (numout,*) ' outtmp(il,jl):', outtmp(il,jl)
              WRITE (numout,*) ' trip(il,jl):', trip(il,jl)
              CALL ipslerr_p(3,'routing_reg_findbasins','We could not find right outlet','','')
           ENDIF
           !
           totsz = totsz + 1
         ENDIF
         ! Testing this IF
         ENDIF
       ENDDO
    ENDDO
    !
    IF (  debug .AND. routing_diagbox(ib, diaglalo) ) THEN
       WRITE(numout,*) "=== To check grid before dividing sub-basin ==="
       WRITE(numout,*) "Number of points: ", nbi, nbj
       WRITE(numout,*) "Number of outlet: ", nbb
       WRITE(numout,*) '+++++++++++++++++++ OUTLET IN FINDBASINS ++++++++++++++++++++'
       WRITE(fmt,"('(',I3,'I6)')") nbi
       DO je=1,nbj
          WRITE(numout,fmt) outtmp(1:nbi,je)
       ENDDO
       !
       WRITE(numout,*) '+++++++++++++++++++ TRIP IN FINDBASINS ++++++++++++++++++++'
       DO je=1,nbj
          WRITE(numout,fmt) trip(1:nbi,je)
       ENDDO
    ENDIF
    !
    ! 4.0 Use the level 1 of the Pfafstetter coding system to divide the large
    ! sub-grid basin. The large sub-grid basin here is defined by the area 
    ! larger than 9% of grid box area. This threshold should be discussed later.
    !
    ! Make a loop through all sub-grid basins to find if there is big sub-grid basin.
    !
    ! We need at least one subbasin with size greater than 4 (to have at least one
    ! tributary). If you run ORCHIDEE with high resolution, such as 5 km, you
    ! don't need to divide the subbasin too small.
    !
    ! Check if there is still subbasin with area larger than [maxpercent] % of total area
    !
    DO WHILE ( COUNT(tsz(:)/REAL(totsz) >= maxpercent/REAL(100) ) > 0 .AND. COUNT(tsz(:) >= 4 ) > 0 )
       !
       cnt = 0   
       DO ibas = 1, nbb
          ! Only take the subbasin is bigger than 4 points
          IF (( tsz(ibas)/REAL(totsz) .GE. maxpercent/REAL(100) ) .AND. ( tsz(ibas) .GE. 4 )) THEN
             cnt = cnt + 1
             trans(cnt) = ibas
          ENDIF
       ENDDO
       !
       ! cnt should be greater or equal 1 here
       DO ipb = 1, cnt 
          !
          ibas = trans(ipb)
          !
          CALL routing_reg_divbas(nbi, nbj, ibas, toutloc(ibas,1), toutloc(ibas,2), tsz(ibas), toutbas(ibas), toutdir(ibas), & 
             & toutlshead(ibas), tpts, trip, basin, fac, lontmp, lattmp, & 
             & new_nb, mp, new_bname, new_outdir, new_heading, new_outbas, new_lat, new_lon, new_outloc, new_sz, new_pts)
          !
          ! If the dividing step is ok 
          !
          IF ( new_nb .NE. 0 ) THEN
             !
             ! If we have split the basin then we need to change the old one
             !
             tbname(ibas) = new_bname(1)
             toutdir(ibas) = new_outdir(1)
             toutlshead(ibas) = new_heading(1)
             !
             IF ( new_outbas(1) .NE. undef_int ) THEN
                IF ( mp .NE. 1 ) THEN
                   toutbas(ibas) = new_outbas(1)+nbb-1
                ELSE
                   toutbas(ibas) = new_outbas(1)
                ENDIF
             ENDIF
             !
             toutloc(ibas,:) = new_outloc(1,:)
             tlat(ibas) = new_lat(1)
             tlon(ibas) = new_lon(1)
             !
             touttp(ibas) = trip(new_outloc(1,1),new_outloc(1,2))
             !
             tsz(ibas) = new_sz(1)
             tpts(ibas,:,:) = new_pts(1,:,:)
             DO p = 1, tsz(ibas) 
                outtmp(tpts(ibas,p,1),tpts(ibas,p,2)) = ibas
             ENDDO
             !
             !  Add the new sub-grid basins to the end of the list
             !
             IF ( nbb+new_nb-1 .LE. nbvmax) THEN
                DO ip = 2, new_nb
                   tbname(nbb+ip-1) = new_bname(ip)
                   toutdir(nbb+ip-1) = new_outdir(ip)
                   toutlshead(nbb+ip-1) = new_heading(ip)
                   !
                   ! Trung: Should come back here soon (small bug can come from
                   ! Trung: this point)
                   !
                   IF ( new_outbas(ip) .EQ. 1 .AND. ip .NE. mp ) THEN
                      toutbas(nbb+ip-1) = ibas
                   ELSE IF ( ip .NE. mp ) THEN 
                      toutbas(nbb+ip-1) = new_outbas(ip)+nbb-1
                   ELSE
                      toutbas(nbb+ip-1) = new_outbas(ip)
                   ENDIF
                   !
                   toutloc(nbb+ip-1,:) = new_outloc(ip,:)
                   tlat(nbb+ip-1) = new_lat(ip)
                   tlon(nbb+ip-1) = new_lon(ip)
                   !
                   touttp(nbb+ip-1) = trip(new_outloc(ip,1),new_outloc(ip,2))
                   !
                   tsz(nbb+ip-1) = new_sz(ip)
                   tpts(nbb+ip-1,:,:) = new_pts(ip,:,:)
                   DO p = 1, tsz(nbb+ip-1) 
                      outtmp(tpts(nbb+ip-1,p,1),tpts(nbb+ip-1,p,2)) = nbb+ip-1
                   ENDDO
                ENDDO
                nbb = nbb+new_nb-1
             ELSE
                WRITE(numout,*) 'Increase nbvmax. It is too small to contain all the basins (routing_reg_findbasins)'
                CALL ipslerr_p(3,'routing_reg_findbasins','Increase nbvmax.','It is too small to contain all the basins','')
             ENDIF
             !
          ELSE
             !
             ! Print some information if the division failed
             !
             WRITE(numout,*) 'Can not divide sub-basins (routing_reg_findbasins): ',ibas
             CALL ipslerr_p(3,'routing_reg_findbasins','new_nb = 0','Something is wrong with routing_reg_divbas','')
             ! 
          ENDIF
       ENDDO
       !
    ENDDO
    !
    ! 3.0 Gather coastal points
    !
    itrans = 0
    coast_pts(:) = undef_int
    !
    ! Get all the points we can collect (negative value of IDs in routing.nc)
    !
    DO ip=1,nbb
       IF ( tsz(ip) .EQ. 1 .AND. trip(tpts(ip,1,1),tpts(ip,1,2)) .EQ. 98 .AND. tbname(ip) .LT. 0 ) THEN
          itrans = itrans + 1
          trans(itrans) = ip
       ENDIF
    ENDDO
    !
    ! Put everything in the first basin (beware of tbname(trans(1)) can be negative)
    !
    IF ( itrans .GT. 1) THEN
       ipb = trans(1)
       coast_pts(tsz(ipb)) = tbname(ipb)
       tbname(ipb) = -1
       DO ip=2,itrans
          tsz(ipb) = tsz(ipb) + 1
          coast_pts(tsz(ipb)) = tbname(trans(ip))
          tsz(trans(ip)) = 0
          tpts(ipb, tsz(ipb), 1) = tpts(trans(ip), 1, 1)
          tpts(ipb, tsz(ipb), 2) = tpts(trans(ip), 1, 2)
          !
          outtmp(tpts(ipb, tsz(ipb), 1),tpts(ipb, tsz(ipb), 2)) = ipb
       ENDDO
    ENDIF
    !
    ! A.0 To check each grid when testing this subroutine
    !
    IF (  debug .AND. routing_diagbox(ib, diaglalo) ) THEN
       WRITE(numout,*) "=== To check grid after dividing sub-basin ==="
       WRITE(numout,*) "Number of points: ", nbi, nbj
       WRITE(numout,*) "Number of outlet: ", nbb
       WRITE(numout,*) '+++++++++++++++++++ OUTLET IN FINDBASINS ++++++++++++++++++++'
       WRITE(fmt,"('(',I3,'I6)')") nbi
       DO je=1,nbj
          WRITE(numout,fmt) outtmp(1:nbi,je)
       ENDDO
       !
       WRITE(numout,*) '+++++++++++++++++++ TRIP IN FINDBASINS ++++++++++++++++++++'
       DO je=1,nbj
          WRITE(numout,fmt) trip(1:nbi,je)
       ENDDO
       !
       WRITE(numout,*) '+++++++++++++++++++ BASIN IDs IN FINDBASINS ++++++++++++++++++++'
       DO je=1,nbj
          WRITE(numout,fmt) basin(1:nbi,je)
       ENDDO
       !
       WRITE(numout,*) '+++++++++++++++++++ LONGITUDE IN FINDBASINS ++++++++++++++++++++'
       WRITE(fmtr,"('(',I3,'F8.1)')") nbi
       DO je=1,nbj
          WRITE(numout,fmtr) lontmp(1:nbi,je)
       ENDDO
       !
       WRITE(numout,*) '+++++++++++++++++++ LATITUDE IN FINDBASINS ++++++++++++++++++++'
       WRITE(fmtr,"('(',I3,'F8.1)')") nbi
       DO je=1,nbj
          WRITE(numout,fmtr) lattmp(1:nbi,je)
       ENDDO
       !
       WRITE(numout,*) '+++++++++++++++++++ HIERARCHY IN FINDBASINS ++++++++++++++++++++'
       WRITE(fmtr,"('(',I3,'F8.1)')") nbi
       DO je=1,nbj
          WRITE(numout,fmtr) hierarchy(1:nbi,je)
       ENDDO
       WRITE(numout,*) "=== ====================================== ==="
       !
       WRITE(numout,*) '+++++++++++++++++++ FAC IN FINDBASINS ++++++++++++++++++++'
       WRITE(fmtr,"('(',I3,'F8.1)')") nbi
       DO je=1,nbj
          WRITE(numout,fmtr) fac(1:nbi,je)
       ENDDO
       WRITE(numout,*) "=== ====================================== ==="
    ENDIF
    !
    ! 5.0 Sort the output by size of the various basins.
    !
    nb_basin = COUNT(tsz(1:nbb) .GT. 0)
    tmpsz(:) = -1
    tmpsz(1:nbb) = tsz(1:nbb)
    DO ip=1,nbb
       jpp = MAXLOC(tmpsz(:))
       IF ( tsz(jpp(1)) .GT. 0) THEN
          sortind(ip) = jpp(1)
          tmpsz(jpp(1)) = -1
       ENDIF
    ENDDO
    basin_inbxid(1:nb_basin) = tbname(sortind(1:nb_basin))
    basin_outlet(1:nb_basin,1) = tlat(sortind(1:nb_basin))
    basin_outlet(1:nb_basin,2) = tlon(sortind(1:nb_basin))
    basin_outtp(1:nb_basin) = touttp(sortind(1:nb_basin))
    basin_sz(1:nb_basin) = tsz(sortind(1:nb_basin))
    basin_pts(1:nb_basin,:,:) = tpts(sortind(1:nb_basin),:,:)
    basin_bxout(1:nb_basin) = toutdir(sortind(1:nb_basin))
    basin_lshead(1:nb_basin) = toutlshead(sortind(1:nb_basin))
    !
    ! Correct toutbas after sorting
    !
    basin_bbout(1:nb_basin) = undef_int
    DO ip = 1, nb_basin
       IF ( toutbas(sortind(ip)) .NE. undef_int ) THEN
          neworder = 0
          oldorder = toutbas(sortind(ip))
          DO ie = 1, nb_basin
             IF ( sortind(ie) .EQ. oldorder ) THEN
                neworder = ie
             ENDIF
          ENDDO
          IF ( neworder .NE. 0 ) THEN
             basin_bbout(ip) = neworder 
          ELSE
             WRITE(numout,*) 'ip, sortind(ip) :', ip, sortind(ip)
             WRITE(numout,*) 'toutbas(sortind(ip)) :', toutbas(sortind(ip))
             WRITE(numout,*) 'Found new order :', neworder
             !
             WRITE(numout,*) 'nb_basin & nbb :', nb_basin, nbb
             WRITE(numout,*) 'sortind(1:nb_basin) :', sortind(1:nb_basin)
             WRITE(numout,*) 'tsz(sortind(1:nb_basin)) :', tsz(sortind(1:nb_basin))
             WRITE(numout,*) 'tsz(1:nbb) :', tsz(1:nbb)
             WRITE(numout,*) 'toutbas(sortind(1:nb_basin)) :', toutbas(sortind(1:nb_basin))
             WRITE(numout,*) 'toutdir(sortind(1:nb_basin)) :', toutdir(sortind(1:nb_basin))
             WRITE(numout,*) 'tbname(sortind(1:nb_basin)) :', tbname(sortind(1:nb_basin))
             WRITE(numout,*) 'toutbas(1:nbb) :', toutbas(1:nbb)
             !
             CALL ipslerr_p(3,'routing_reg_findbasins','We got problem when sorting toutbas','','') 
          ENDIF
       ENDIF
    ENDDO 
    !
    IF (  debug .AND. routing_diagbox(ib, diaglalo) ) THEN
       WRITE(numout,*) "Grid box: ", ib
       WRITE(numout,*) "Number of points: ", nbi, nbj
       WRITE(numout,*) mpi_rank, "=== To check coastal point simplify ==="
       WRITE(numout,*) "Number of outlet: "
       WRITE(numout,*) " + Before gather: ", nbb
       WRITE(numout,*) " + After gather: ", nb_basin
       WRITE(numout,*) mpi_rank, "=== To check dividing sub-basin process ==="
       ip = COUNT(basin_bxout(1:nb_basin) .GE. -3 .AND. basin_bxout(1:nb_basin) .LE. -1 )
       WRITE(numout,*) "Outlet to lake, ocean: ", ip
       ip = COUNT(basin_bxout(1:nb_basin) .GE. 1 .AND. basin_bxout(1:nb_basin) .LE. 8 )
       WRITE(numout,*) "Outlet to other grid box: ", ip
       ip = COUNT(basin_bxout(1:nb_basin) .EQ. -4)
       WRITE(numout,*) "Outlet in the same grid box: ", ip
       ip = COUNT(basin_bbout(1:nb_basin) .NE. undef_int)
       WRITE(numout,*) "Should be the same value as above: ", ip
    ENDIF
    !
    ! 6.0 Make simple verification
    ! What is the size of the region behind each outflow point ?
    ! We can only check if we have at least as many outflows as basins
    !
    checksz = 0
    DO ip=1,nbb
       checksz = checksz + tsz(ip)
    ENDDO
    IF ( checksz .NE. totsz) THEN
       WRITE(numout,*) 'Water got lost while I tried to find basins'
       WRITE(numout,*) checksz, ' /= ', totsz
       CALL ipslerr_p(3,'routing_reg_findbasins','Water got lost while I tried to find basins','','')
    ENDIF
    !
    ip = COUNT(trip(1:nbi,1:nbj) .GE. 97 .AND. trip(1:nbi,1:nbj) .LT. undef_int)
    ip = ip + COUNT(trip(1:nbi,1:nbj) .EQ. -4)
    IF ( ip .LT. nb_basin ) THEN
       WRITE(numout,*) 'We found some coastal point here :', ip
       WRITE(numout,*) 'nb_basin :', nb_basin
       WRITE(numout,*) 'Basin sized :', basin_sz(1:nb_basin)
    ENDIF
    !
    ! 7.0 If we calculate topoindex with option 3
    !     we need to accumulate the topoind_bx along the flow
    !
    ! Get option for calculating topoindex
    !
    !Config Key = TOPOINDEX
    !Config Desc = Options to calculate topoindex
    !Config Def = 3
    !Config Help = This flag allows to calculate topoindex with different options
    !Config Units = [-]
    !
    option = 3
    CALL getin('TOPOINDEX', option)
    !
    ! Checking option: until now (May 2016) there is only 3 acceptable options
    ! = 1 : topoindex .EQ. 1000. everywhere
    ! = 2 : simple average as the method in the older routing scheme
    ! = 3 : accumulate the "topoind" from routing.nc along the flow (as Agnes DUCHARNE's suggestion)
    !
    IF ( option .LT. 1 .OR. option .GT. 3 ) THEN
       WRITE(numout,*) ' You chose wrong option for calculating topoindex: ', option
       WRITE(numout,*) ' It should be: 1, 2 or 3'
       STOP 'routing_reg_globalize'
    ENDIF
    !
    ! Work only using option number 3
    !
    IF ( option .EQ. 3 ) THEN
       ! Loop for all sub-basins and each point belongs to that sub-basin
       DO ij=1,nb_basin
          DO iz=1,basin_sz(ij)
             ! Get the coordinate of this point
             ip = basin_pts(ij,iz,1)
             jp = basin_pts(ij,iz,2)
             ! Start to accumulate "topoind" if this is not a outlet point
             IF ( trip(ip,jp) .GT. 0 .AND. trip(ip,jp) .LT. 97 ) THEN
                p = trip(ip,jp)
                cnt = 0
                il = ip ; jl = jp
                DO WHILE ( p .GT. 0 .AND. p .LT. 9 .AND. cnt .LT. nbi*nbj )
                   cnt = cnt + 1
                   ill = il + inc(p,1)
                   jll = jl + inc(p,2)
                   ! Of course, if you are careful, you will want to check if
                   ! this point still belongs to current sub-basin. Here I
                   ! assume that the sub-routine routing_reg_divbas worked well.
                   ! So we don't need to check.
                   topoind(basin_pts(ij,iz,1),basin_pts(ij,iz,2))=topoind(basin_pts(ij,iz,1),basin_pts(ij,iz,2))+topoind(ill,jll)
                   !
                   il = ill ; jl = jll
                   p = trip(il,jl)
                ENDDO
             ENDIF
             !
          ENDDO
       ENDDO
    ENDIF
    !
  END SUBROUTINE routing_reg_findbasins
  !
!! ================================================================================================================================
!! SUBROUTINE : routing_reg_divbas
!!
!>\BRIEF This subroutine divide the large sub-grid basins to smaller ones. It is
!! based on The Pfafstetter Coding System for Watershed Identification. You can
!! find information here (for example):
!! http://ponce.sdsu.edu/pfafstetter_system_revised_presentation.html
!!
!! DESCRIPTION (definitions, functional, design, flags) :
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_divbas(nbi, nbj, ibas, iloc, jloc, tsz, tout, toutd, headd, &
                & tpts, trip, basin, fac, lon, lat,      &
                & new_nb, oic, new_bname, new_outdir, new_head, new_outbas, new_lat, new_lon, new_outloc, new_sz, new_pts)
    !
    IMPLICIT NONE
    !
!! INPUT VARIABLES
    INTEGER(i_std), INTENT(in) :: nbi !! Number of point in x within the grid (unitless)
    INTEGER(i_std), INTENT(in) :: nbj !! Number of point in y within the grid (unitless)
    INTEGER(i_std), INTENT(in) :: ibas !! Order of the basin will be divided 
    INTEGER(i_std), INTENT(in) :: iloc, jloc !! Outlet location
    !
    INTEGER(i_std), INTENT(in) :: tsz !! Size of sub-basin
    INTEGER(i_std), INTENT(in) :: tout !! Outlet type
    INTEGER(i_std), INTENT(in) :: toutd !! Outlet direction
    REAL(r_std), INTENT(in)    :: headd
    INTEGER(i_std), INTENT(in) :: tpts(:,:,:) !!
    INTEGER(i_std), INTENT(in) :: basin(:,:) !!
    REAL(r_std), INTENT(in) :: fac(:,:) !!
    REAL(r_std), INTENT(in) :: lon(:,:), lat(:,:) !!
    !
!! MODIFIED VARIABLES
    INTEGER(i_std), INTENT(inout) :: trip(:,:) !!
    !
!! OUTPUT VARIABLES
    INTEGER(i_std), INTENT(out) :: new_nb !! Number of sub-basins (unitless)
    INTEGER(i_std), INTENT(out) :: oic
    INTEGER(i_std), INTENT(out) :: new_bname(nbvmax) !!
    INTEGER(i_std), INTENT(out) :: new_outdir(nbvmax) !!
    REAL(r_std), INTENT(out)    :: new_head(nbvmax) !!
    INTEGER(i_std), INTENT(out) :: new_outbas(nbvmax) !!
    INTEGER(i_std), INTENT(out) :: new_outloc(nbvmax,2) !!
    REAL(r_std), INTENT(out) :: new_lon(nbvmax) !!
    REAL(r_std), INTENT(out) :: new_lat(nbvmax) !!
    INTEGER(i_std), INTENT(out) :: new_sz(nbvmax) !!
    INTEGER(i_std), INTENT(out) :: new_pts(nbvmax, nbvmax, 2) !!
    !
!! LOCAL VARIABLES
    REAL(r_std), PARAMETER :: flag=-9999. !!
    LOGICAL, PARAMETER :: debug=.FALSE.
    LOGICAL, PARAMETER :: checkgrid=.FALSE.
    CHARACTER(LEN=7) :: fmt !!
    CHARACTER(LEN=9) :: fmtr !!
    CHARACTER(LEN=11) :: afmt !!
    CHARACTER(LEN=13) :: afmtr !!
    !
    !
    REAL(r_std), DIMENSION(nbi,nbj) :: factmp !!
    INTEGER(i_std), DIMENSION(nbi,nbj) :: triptmp !!
    INTEGER(i_std), DIMENSION(nbi,nbj) :: triptemp !!
    INTEGER(i_std), DIMENSION(nbi,nbj) :: basintmp !!
    REAL(r_std), DIMENSION(nbi,nbj) :: lontmp, lattmp !!
    !
    INTEGER(i_std) :: il, jl, ill, jll !!
    INTEGER(i_std) :: ie, je !!
    INTEGER(i_std) :: p, cnt, k, l, ic, ik !!
    INTEGER(i_std) :: ip, isz, checksz !!
    INTEGER(i_std) :: ff(1) !!
    !
    LOGICAL :: okpoint=.TRUE.
    !
    ! Number of neighbours on the HydroShed grid (regular Lat/Lon)
    !
    INTEGER(i_std), PARAMETER :: nbne=8
    INTEGER(i_std), DIMENSION(nbne,2) :: inc !!
    !
    ! Explanation for dimension of below arrays:
    !
    ! + Firstly, I suggest that you should read about the Pfafstetter Coding
    ! System, here for example:
    ! http://ponce.sdsu.edu/pfafstetter_system_revised_presentation.html
    !
    !    * Note: It will be easier if you keep in mind that the original
    !    information given in grid box of nbi*nbj. But here, to deal with main
    !    stem and tributaries, I need to store them to cnt*nbne (exactly,
    !    cnt*[nbne-2]). Where, cnt is the length of main stem and nbne is number
    !    of neighbour points, and usually 2 neighbour points will belongs to
    !    main stem.
    !
    ! + You will see steps for coding the sub-basin are:
    !
    !    - From the sub-basin outlet, trace upstream along the main stem of the
    !    river, and get the tributaries. To store location and flow accumulation
    !    of points belongs to main stream and tributaries:
    !
    INTEGER(i_std) :: main_loc(nbvmax,2), tri_loc(nbne,nbvmax,2)
    REAL(r_std) :: main_fac(nbvmax), tri_fac(nbne,nbvmax)
    REAL(r_std) :: tmptri_fac(nbne)  ! Flow accumulation of all neighbour points 
    INTEGER(i_std) :: sortedtrifac(nbne) ! And sort of tmptri_fac(nbne)
    REAL(r_std) :: alltri_fac(nbvmax) ! Sort flow accumulation of all tributaries
    INTEGER(i_std) :: alltri_loc1(nbvmax), alltri_loc2(nbvmax) ! And their location
    INTEGER(i_std) :: sortedallfac(nbvmax) ! Sort alltri_fac(nbvmax) 
    ! 
    !     *  Note: again, there are few dimensions should be exactly [nbne-1]
    !     or [nbne-2]. Because when you follow upstream the mainstream, there are
    !     always at least ONE neighbour points will not belong to TRIBUTARY.
    !     But to make your life easier, I put all "nbne" here.
    !
    !    - Identify the 4 greatest tributaries:
    ! 
    REAL(r_std) :: tmpmain_fac(4) ! Flow accumulations of 4 greatest tributaries
    INTEGER(i_std) :: tmpmain_loc(4,2), sortedmainfac(4) ! And their locations and sorted values
    INTEGER(i_std) :: numtri(4), usetri_loc(4,4,2) ! Counting and store location when processing these 4 points
    !
    !    - Inter-basins are the watersheds that contribute flow to the main
    !    stem: there can be 5 inter-basins. So we need 9 dividing points:
    !
    INTEGER(i_std) :: divloc(9,2) ! Location of dividing points (maximum is 9)
    ! 

!_ ================================================================================================================================

    !
    ! The routing code (i=1, j=2)
    !
    inc(1,1) = 0
    inc(1,2) = -1
    inc(2,1) = 1
    inc(2,2) = -1
    inc(3,1) = 1
    inc(3,2) = 0
    inc(4,1) = 1
    inc(4,2) = 1
    inc(5,1) = 0
    inc(5,2) = 1
    inc(6,1) = -1
    inc(6,2) = 1
    inc(7,1) = -1
    inc(7,2) = 0
    inc(8,1) = -1
    inc(8,2) = -1
    !
    ! 0.0 Get the information of all the points belongs to the subbasin
    ! which need to divide. Flag for flow accumulation is -9999 (the value of
    ! flow accumulation should be always non negative).
    !
    factmp(:,:) = flag
    basintmp(:,:) = 0
    lontmp(:,:) = 0
    lattmp(:,:) = 0
    triptmp(:,:) = -1
    triptemp(:,:) = -1
    !
    DO isz = 1, tsz
       il = tpts(ibas, isz, 1)
       jl = tpts(ibas, isz, 2)
       !
       factmp(il,jl) = fac(il,jl)
       triptmp(il,jl) = trip(il,jl)
       triptemp(il,jl) = trip(il,jl)
       basintmp(il,jl) = basin(il,jl)
       lontmp(il,jl) = lon(il,jl)
       lattmp(il,jl) = lat(il,jl)
    ENDDO
    !
    ! Print out information of grid box
    !
    IF ( checkgrid ) THEN
       ! 
       WRITE(numout,*) " Routing_reg_divbas: Grid before dividing "
       WRITE(numout,*) " Size: ", nbi, nbj
       WRITE(fmt,"('(',I3,'I6)')") nbi
       WRITE(fmtr,"('(',I3,'F8.1)')") nbi
       !
       WRITE(numout,*) '+++++++++++++++++++ TRIP IN DIVBAS ++++++++++++++++++++'
       DO je=1,nbj
          WRITE(numout,fmt) triptmp(1:nbi,je)
       ENDDO
       !
       WRITE(numout,*) '+++++++++++++++++++ BASIN IDs IN DIVBAS ++++++++++++++++++++'
       DO je=1,nbj
          WRITE(numout,fmt) basintmp(1:nbi,je)
       ENDDO
       !
       WRITE(numout,*) '+++++++++++++++++++ LONGITUDE IN DIVBAS ++++++++++++++++++++'
       DO je=1,nbj
          WRITE(numout,fmtr) lontmp(1:nbi,je)
       ENDDO
       !
       WRITE(numout,*) '+++++++++++++++++++ LATITUDE IN DIVBAS ++++++++++++++++++++'
       DO je=1,nbj
          WRITE(numout,fmtr) lattmp(1:nbi,je)
       ENDDO
       !
       WRITE(numout,*) '+++++++++++++++++++ FAC IN DIVBAS ++++++++++++++++++++'
       DO je=1,nbj
          WRITE(numout,fmtr) factmp(1:nbi,je)
       ENDDO
       CALL FLUSH(numout)
    ENDIF
    !
    ! 1.0 If the first version of this subroutine doesn't work well,
    ! we need to write the validation step of input arrays here (for
    ! example, the input sub-basin should only has 1 outlet).
    ! (October 2016: it still works well :))
    !
    IF ( triptmp(iloc,jloc) .LT. -4 .OR. triptmp(iloc,jloc) .GE. 109 ) THEN
       CALL ipslerr_p(3,'routing_reg_divbas','The TRIP of outlet is wrong','','')
    ENDIF
    IF ( debug ) THEN
       WRITE(numout,*) 'Basin number: ', ibas
       WRITE(numout,*) 'Basin size: ', tsz
       WRITE(numout,*) 'Basin out: ', tout, toutd
    ENDIF
    !
    ! 2.0 From the outlet of the subbasin, trace back to find the main stream 
    ! and identify the 4 greatest tributaries. If there are less than 4
    ! tributraries, take them all.
    !
    ! Trace back upstream from the outlet of the sub-basin
    ! and define the mainstream and all tributaries
    !
    main_loc(:,:) = 0
    main_fac(:) = flag
    tri_loc(:,:,:) = 0
    tri_fac(:,:) = flag
    !
    ! Start!
    !
    cnt = 0
    okpoint = .TRUE.
    il = iloc ; jl = jloc
    DO WHILE ( il .GT. 0 .AND. il .LE. nbi .AND. jl .GT. 0 .AND. jl .LE. nbj &
             & .AND. cnt .LT. nbi*nbj .AND. okpoint )
       !
       ! Count the length of the mainstream
       !
       cnt = cnt + 1
       main_loc(cnt,1) = il
       main_loc(cnt,2) = jl
       main_fac(cnt) = factmp(il,jl)
       !
       ! Look for [nbne] neighbour points to find the tributaries
       ! October 2016: so far, nbne = 8 (for 8 directions of TRIP)
       !
       l = 0
       DO k = 1, nbne
          ill = il + inc(k,1)
          jll = jl + inc(k,2)
          ! If this point is still in the grid box
          IF ( ill .GT. 0 .AND. ill .LE. nbi .AND. jll .GT. 0 .AND. jll .LE. nbj ) THEN
             ! If this point still belongs to the current sub-basin
             IF ( triptmp(ill,jll) .GT. 0 .AND. triptmp(ill,jll) .LE. nbne ) THEN
                ie = ill + inc(INT(triptmp(ill,jll)),1)
                je = jll + inc(INT(triptmp(ill,jll)),2)
                ! If this neighbour point contributes water for current point
                IF ( ie .EQ. il .AND. je .EQ. jl ) THEN
                   ! Store all points here
                   l = l + 1
                   tri_fac(l,cnt) = factmp(ill,jll) 
                   tri_loc(l,cnt,1) = ill
                   tri_loc(l,cnt,2) = jll
                   ! Mark the processed points here
                   triptmp(ill,jll) = -2
                ENDIF
             ENDIF
          ENDIF
       ENDDO
       !
       ! Only go further if we have found tributaries (there is still neighbour
       ! point flows into the current point)
       !
       IF ( l > 0 ) THEN
          !
          ! Sort the FAC of all neighbour points
          tmptri_fac(:) = flag
          sortedtrifac(:) = 0
          !
          tmptri_fac(:) = tri_fac(:,cnt)
          DO k = 1, l
             ff = MAXLOC(tmptri_fac)
             sortedtrifac(k) = ff(1)
             tmptri_fac(ff(1)) = flag
          ENDDO
          ! Continue trace upstream: the neighbour point with highest flow
          ! accumulation will belongs to main stream. We move to this point
          ! (change value of il, jl).
          IF ( tri_fac(sortedtrifac(1),cnt) .NE. flag ) THEN
             il = tri_loc(sortedtrifac(1),cnt,1)       
             jl = tri_loc(sortedtrifac(1),cnt,2)
             !
             IF ( checkgrid ) THEN
                WRITE(numout,*) "cnt = ", cnt, " fac = ", tri_fac(sortedtrifac(1),cnt)
             ENDIF
             !
             tri_fac(sortedtrifac(1),cnt) = flag
          ELSE
             okpoint = .FALSE.
          ENDIF
       ELSE
          okpoint = .FALSE.
       ENDIF
       !
       IF ( cnt .EQ. nbi*nbj) THEN
          IF ( debug ) THEN
             WRITE(numout,*) "Error: cnt .EQ. nbi*nbj "
             WRITE(numout,*) "Point: ", il, jl
             WRITE(numout,*) "Per: ", nbi, nbj
             WRITE(fmt,"('(',I3,'I6)')") nbi
             WRITE(fmtr,"('(',I3,'F8.1)')") nbi
             !
             WRITE(numout,*) '+++++++++++++++++++ TRIP IN DIVBAS ++++++++++++++++++++'
             DO je=1,nbj
                WRITE(numout,fmt) triptemp(1:nbi,je)
             ENDDO
             !
             WRITE(numout,*) '+++++++++++++++++++ BASIN IDs IN DIVBAS ++++++++++++++++++++'
             DO je=1,nbj
                WRITE(numout,fmt) basintmp(1:nbi,je)
             ENDDO
             !
             WRITE(numout,*) '+++++++++++++++++++ LONGITUDE IN DIVBAS ++++++++++++++++++++'
             DO je=1,nbj
                WRITE(numout,fmtr) lontmp(1:nbi,je)
             ENDDO
             !
             WRITE(numout,*) '+++++++++++++++++++ LATITUDE IN DIVBAS ++++++++++++++++++++'
             DO je=1,nbj
                WRITE(numout,fmtr) lattmp(1:nbi,je)
             ENDDO
             !
             WRITE(numout,*) '+++++++++++++++++++ FAC IN DIVBAS ++++++++++++++++++++'
             DO je=1,nbj
                WRITE(numout,fmtr) factmp(1:nbi,je)
             ENDDO
             CALL FLUSH(numout)
          ENDIF
          CALL ipslerr_p(3,'routing_reg_divbas','We could not route point','','')
       ENDIF
       !
    ENDDO
    IF ( debug ) WRITE (numout,*) 'Length of mainstream: ', cnt
    !
    ! 3.0 Sort the flow accumulation of the tributaries and find the 
    ! divided location.
    !
    alltri_fac(:) = flag
    alltri_loc1(:) = 0
    alltri_loc2(:) = 0
    divloc(:,:) = 0
    !
    ip = 0
    DO k = 1, cnt
       ! Actually, we should only DO l = 1, nbne-2
       ! Because, there are usually 2 neighbour points belong to main stream
       ! and maximum (nbne-2) neighbour points can be tributaries.
       ! DO l = 1, nbne-2 
       DO l = 1, nbne
          IF ( tri_fac(l,k) .NE. flag ) THEN
             ip = ip + 1
             alltri_fac(ip) = tri_fac(l,k)
             ! Attention: here we store location of each tributary in main stream
             ! ( k <= cnt ) and its direction compared with main stream ( l <= nbne )
             alltri_loc1(ip) = k
             alltri_loc2(ip) = l
          ENDIF
       ENDDO
    ENDDO
    IF ( debug ) WRITE (numout,*) 'Number of tributaries: ', ip
    ! If we have at least one tributary
    IF ( ip .GT. 0 ) THEN
       ! Original output
       oic = 0
       ! 
       sortedallfac(:) = 0
       DO k = 1, ip
          ff = MAXLOC(alltri_fac)
          sortedallfac(k) = ff(1)
          alltri_fac(ff(1)) = flag
       ENDDO
       ! Get the points in the mainstream
       tmpmain_fac(:) = flag
       tmpmain_loc(:,:) = 0
       numtri(:) = 0
       usetri_loc(:,:,:) = 0
       ! Using l to count the actual number of points in main stream need to use
       ! for dividing (not always equal 4 as ideal case).
       l = 0
       ! According to Pfafstetter coding, we only look for 4 greatest
       ! tributaries:
       DO k = 1, 4
          !
          IF ( sortedallfac(k) .NE. 0 ) THEN
             !
             ik = alltri_loc1(sortedallfac(k))
             il = alltri_loc2(sortedallfac(k))
             ! Starting store location and flow accumulation of points in main stream:
             IF ( COUNT ( tmpmain_fac(:) .EQ. main_fac(ik) ) .EQ. 0 ) THEN
                l = l + 1
                tmpmain_loc(l,1) = main_loc(ik,1)
                tmpmain_loc(l,2) = main_loc(ik,2)
                tmpmain_fac(l) = main_fac(ik)
                !
                numtri(l) = 1
                usetri_loc(numtri(l),l,1) = tri_loc(il,ik,1) 
                usetri_loc(numtri(l),l,2) = tri_loc(il,ik,2)
                !
             ELSE
                ! If this point in main stream have more than 1 tributary
                ! belongs to top 4 greatest tributaries
                numtri(l) = numtri(l) + 1
                usetri_loc(numtri(l),l,1) = tri_loc(il,ik,1) 
                usetri_loc(numtri(l),l,2) = tri_loc(il,ik,2)
                !
             ENDIF
          ENDIF
       ENDDO
       ! Sort the mainstream points
       sortedmainfac(:) = 0
       DO k = 1, l
          ff = MAXLOC(tmpmain_fac)
          sortedmainfac(k) = ff(1)
          tmpmain_fac(ff(1)) = flag
       ENDDO
       !
       ic = 0
       DO k = l, 1, -1
          IF ( tmpmain_loc(sortedmainfac(k),1) .NE. 0 ) THEN
             ic = ic + 1
             divloc(ic,1) = tmpmain_loc(sortedmainfac(k),1)
             divloc(ic,2) = tmpmain_loc(sortedmainfac(k),2)
             !
             new_outbas(ic) = ic + 1
             !
          ENDIF
       ENDDO
       ! If the last divide point is not the original outlet
       IF ( divloc(ic,1) .NE. iloc .OR. divloc(ic,2) .NE. jloc ) THEN
          ic = ic + 1
          divloc(ic,1) = iloc
          divloc(ic,2) = jloc
       ENDIF
       !
       new_outbas(ic) = tout
       ! Number of mainstream sub-basin
       oic = ic 
       !
       DO k = l, 1, -1
          !
          !DO il = 1, nbne-2
          ! Maximum 4 tributaries: 
          DO il = 1, 4
             IF ( usetri_loc(il,sortedmainfac(k),1) .NE. 0 ) THEN
                ic = ic + 1
                divloc(ic,1) = usetri_loc(il,sortedmainfac(k),1)
                divloc(ic,2) = usetri_loc(il,sortedmainfac(k),2)
                !
                !new_outbas(ic) = new_outbas(l-k+1)
                new_outbas(ic) = l-k+1
                !
             ENDIF
          ENDDO
       ENDDO
       ! Save number of dividing points
       l = ic
       !       
    ! If we don't have any tributary
    ELSE
       oic = 2
       l = 2
       cnt = cnt/2 + 1
       divloc(1,1) = main_loc(cnt,1)
       divloc(1,2) = main_loc(cnt,2)
       divloc(2,1) = iloc
       divloc(2,2) = jloc
       !
       new_outbas(1) = 2
       new_outbas(2) = tout
       !
    ENDIF
    IF ( debug ) WRITE (numout,*) 'Number of new sub-basin: ', l, oic
    !
    ! Now, cut it ! Cut the sub-basin ! Release The Kraken !
    !
    new_nb = l
    !
    DO ik = 1, l
       ! A small trick here
       IF ( oic .EQ. 2 .AND. l .EQ. 2 ) THEN
          k = ik
       ELSE
          IF ( ik .LE. (l - oic) ) THEN
             k = ik + oic 
          ELSE
             k = ik - (l - oic)
          ENDIF
       ENDIF
       ! I'm so stupid careful here with this IF
       IF ( divloc(k,1) .NE. 0 .AND. divloc(k,2) .NE. 0 ) THEN
          !
          new_sz(k) = 1 
          new_pts(k,new_sz(k),1) = divloc(k,1) 
          new_pts(k,new_sz(k),2) = divloc(k,2)
          !
          new_bname(k) = basin(divloc(k,1),divloc(k,2))
          new_outloc(k,1) = divloc(k,1)
          new_outloc(k,2) = divloc(k,2)
          new_lon(k) = lon(divloc(k,1),divloc(k,2))
          new_lat(k) = lat(divloc(k,1),divloc(k,2))
          !
          ! Change the TRIP here for routing_reg_globalize (don't change the
          ! original outlet before dividing). Change new_outdir for
          ! routing_reg_linkup.
          !
          IF ( k .NE. oic ) THEN
             trip(divloc(k,1), divloc(k,2)) = -4
             new_outdir(k) = -4
             new_head(k) = undef_sechiba
          ELSE
             new_outdir(k) = toutd
             new_head(k) = headd
          ENDIF
          !
          !
          triptemp(divloc(k,1), divloc(k,2)) = -1
          !   
          DO isz = 1, tsz
             il = tpts(ibas, isz, 1)
             jl = tpts(ibas, isz, 2)
             IF ( triptemp(il,jl) .GT. 0 .AND. triptemp(il,jl) .LT. 9 ) THEN
                p = triptemp(il,jl)
                cnt = 0
                ie = il ; je = jl
                okpoint = .TRUE.
                DO WHILE ( p .GT. 0 .AND. p .LT. 9 .AND. cnt .LT. nbi*nbj .AND. okpoint )
                   cnt = cnt + 1
                   ill = ie + inc(p,1)
                   jll = je + inc(p,2)
                   ie = ill ; je = jll
                   p = trip(ie,je)
                   IF ( ie .EQ. divloc(k,1) .AND. je .EQ. divloc(k,2) ) THEN
                      okpoint = .FALSE.
                      new_sz(k) = new_sz(k) + 1 
                      new_pts(k,new_sz(k),1) = il 
                      new_pts(k,new_sz(k),2) = jl
                      triptemp(tpts(ibas, isz, 1), tpts(ibas, isz, 2)) = -1
                   ENDIF
                ENDDO
             ENDIF
          ENDDO   
       ENDIF
    ENDDO 
    !
    ! 4.0 Make simple verification
    !
    IF ( new_nb .GE. 0 ) THEN
       checksz = 0
       DO ip=1,new_nb
          checksz = checksz + new_sz(ip)
       ENDDO
       !  
       IF ( checksz .NE. tsz) THEN
          WRITE(numout,*) ' Water got lost while I tried to divide basins'
          WRITE(numout,*) ' Number of new sub-basin : ', new_nb
          ip = COUNT(divloc(:,1) .NE. 0)
          WRITE(numout,*) ' Number of mainstrem sub-basin : ', oic
          WRITE(numout,*) ' Number of tributaries : ', ip - oic
          WRITE(numout,*) checksz, ' /= ', tsz
          !
          WRITE(afmt,"('(A10,', I3,'I6)')") ip
          WRITE(afmtr,"('(A10,', I3,'F8.1)')") ip
          !
          WRITE(numout,afmt) 'new_sz = ', new_sz(1:ip)
          WRITE(numout,*) 'new_sz(oic) = ', new_sz(oic)
          WRITE(numout,afmt) 'new_bname', new_bname(1:ip)
          WRITE(numout,afmt) 'new_outdir', new_outdir(1:ip)
          WRITE(numout,afmt) 'new_outbas', new_outbas(1:ip)
          WRITE(numout,afmt) 'new_outloc1', new_outloc(1:ip,1)
          WRITE(numout,afmt) 'new_outloc2', new_outloc(1:ip,2)
          WRITE(numout,afmtr) 'new_lon', new_lon(1:ip)
          WRITE(numout,afmtr) 'new_lat', new_lat(1:ip)
          !
          ip = COUNT(triptemp(1:nbi,1:nbj) .EQ. -1)
          WRITE(numout,*) 'Unprocessed points :', nbi*nbj-ip
          !
          WRITE(fmt,"('(',I3,'I6)')") nbi
          WRITE(fmtr,"('(',I3,'F8.1)')") nbi
          !
          WRITE(numout,*) '+++++++++++++++++++ TRIP IN DIVBAS ++++++++++++++++++++'
          DO je=1,nbj
             WRITE(numout,fmt) triptemp(1:nbi,je)
          ENDDO
          !
          CALL ipslerr_p(3,'routing_reg_divbas','Water got lost while I tried to divide sub-basins','','')
       ENDIF
    ENDIF
    !
    ip = COUNT(triptemp(1:nbi,1:nbj) .EQ. -1)
    IF ( ip .NE. nbi*nbj ) THEN
       WRITE(*,*) 'Unprocessed points :', nbi*nbj-ip
    ENDIF
    !
  END SUBROUTINE routing_reg_divbas
  !
  !! ================================================================================================================================
!! SUBROUTINE : routing_reg_riverloc
!!
!>\BRIEF This subroutine provides the coordinate of the estuaries of the num_largest rivers for
!!       passing it on to the ocean model.  
!!
!! DESCRIPTION (definitions, functional, design, flags) :
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_riverloc (nstart, nlen, lonout, latout)
    !!
    !! Arguments
    !!
    INTEGER(i_std), INTENT(out) :: nstart, nlen               !! skip and length for the position of the rivers of this proc on the
                                                              !! global list of river estuaries.
    REAL(r_std), INTENT(out)    :: lonout(num_largest)        !! Latitude of outflow point.
    REAL(r_std), INTENT(out)    :: latout(num_largest)        !! Longitude of outflow point.
    !!
    nstart = outflow_offset_loc
    nlen = outflow_nb_loc
    lonout(:) = outflow_lalo_loc(:,2)
    latout(:) = outflow_lalo_loc(:,1)
    !!
  END SUBROUTINE routing_reg_riverloc

!! ================================================================================================================================
!! SUBROUTINE : routing_reg_riverflow
!!
!>\BRIEF This subroutine provides the outflow of the num_largest rivers so that it can be passed onto the ocean model.
!!
!! DESCRIPTION (definitions, functional, design, flags) :
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCES : None
!!
!! FLOWCHART : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE routing_reg_riverflow (nboutflow, tmplon, tmplat, tmpoutflow)
    !!
    !! Arguments
    !!
    INTEGER(i_std), INTENT(out) :: nboutflow                  !! Number of rivers on this processor
    REAL(r_std), INTENT(out)    :: tmplon(num_largest), tmplat(num_largest) !! Location of outlet
    REAL(r_std), INTENT(out)    :: tmpoutflow(num_largest)    !! outflow in m³/s of the river for the rivers on this proc.
    !!
    !!
    nboutflow = outflow_nb_loc
    tmplon(1:nboutflow) = outflow_lalo_loc(1:nboutflow,2)
    tmplat(1:nboutflow) = outflow_lalo_loc(1:nboutflow,1)
    tmpoutflow(1:nboutflow) = outflow_loc(1:nboutflow)/mille/dt_sechiba
    !!
  END SUBROUTINE routing_reg_riverflow
  !
END MODULE routing_reg
